<TeXmacs|1.0.1>

<style|generic>

<\body>
  <\expand|make-title>
    SmartFlow 2 Specifications<title|>
  </expand>

  This documents is a technical review of the decision made by the NIST
  Smartspace group staff, made of Vincent Stanford, Martial Michel, Julien
  Aube, Olivier Galibert and Christophe Kern.

  <strong|This document is intended for the NIST Smartspace staff only>.

  It will present the planned internal functions and data structures of the
  new version of the SmartFlow system thereafter called "SmartFlow 2". The
  purpose of this new version is to elaborate a solution to the identified
  problems of the Smartflow 1 currently used.

  The following constrains must be respected :

  <\expand|itemize-minus>
    <item>The API must be easy to use for an inexperimented user

    <item>The code created with the "sfgen" tool must be reused as-is.

    <item>The library must be as safe as possible, meaning that the library
    must not lead to a crash if the API is used as documented, if necessary
    at the expense of performance (up to some limits).
  </expand>

  To respect these constrains, some assumption have been made :

  <\expand|itemize-minus>
    <item>The processing power of computers and the bandwith of networks are
    supposed to continue their current improvement. It mean in a given
    development situation were two ways of achieving the goal are possible,
    the safest must be choosen even if it implie some performance degradation
    (This is particulary noticeable in the memory subsystem and in error
    condition checking).

    <item>The system will run on a POSIX-compliant OS, presumably Linux but
    the porting to another system should be made as easy as possible by not
    using specific OS methods, or if it become necessary, make sure to use
    encapsulation. *-BSD unix ports come in mind.

    <item>The system will be coded in C++ using POSIX libraries (pthread,
    sockets, unix processes), standart user tools (SSH, NFS). No porting are
    planned for other languages, however the library API must be done so that
    an "Encapsulation" layer for other high-level object languages can be
    possible (Java, Python...). <strong|The Smartspace staff do not take any
    responsabilities for supporting these language in the future>.
  </expand>

  <\expand|make-title>
    The general conception<title|>
  </expand>

  The SmartFlow 2 is now conceived as a modular system : It will be created
  with several modules dynamically loaded by a "core system". This system
  will be made of these components :

  <\expand|itemize-minus>
    <item>Core base :

    <\expand|itemize-dot>
      <item>Plugin loading/unloading management and registration

      <item>Command-line argument parsing

      <item>Preference plugin selection, create an instance of a "preference
      object"

      <item>Loading the necessary plugins selected for initial connection
    </expand>
  </expand>

  <\expand|itemize-minus>
    <item>Preference plugin (necessary) :

    <\expand|itemize-dot>
      <item>Parse the command-line for arguments not recognized by the core

      <item>Read one or more config file in a given format (XML, ini...)

      <item>Create an object to store these preferences for a later use by
      the other plugins
    </expand>
  </expand>

  <\expand|itemize-minus>
    <item>Authentication (optional) :

    <\expand|itemize-dot>
      <item>The smartfow can be used to requiere an authentication from the
      client before accepting it. In this case, some kind of "authentication
      tokens" can be passed.

      <item>By default, no authentication will be necessary, and a "alway
      accepted" authentication layer will be used.
    </expand>
  </expand>

  <\expand|itemize-minus>
    <item>Crossbar Discovering plugin (necessary) :

    <\expand|itemize-dot>
      <item>Find other Smartflow Nodes and establish connection with them
      (using the Transport plugin below) Eventually, a plugin can be made to
      avoid connection to other servers, for local use. Exemples of plugins :
      NFS-shared file, broadcast "a la DHCP", connection to a "Smartflow DNS
      server".....
    </expand>
  </expand>

  <\expand|itemize-minus>
    <item>Message Transport plugin (necessary) :

    <\expand|itemize-dot>
      <item>Transport messages to other clients either local or remote. It is
      the task of this module to maintain a list of administrative
      informations to help other plugins to locate the information they need
      (ie the list of flows)

      <item>The current planned way to achieve this result is to spawn a
      "server" on the local machine and take care of that business, this
      server exit when it has no clients anymore. Only one server per node
      per smartflow system can be active. One input/output of the client is
      connected to the active server.
    </expand>

    <item>Message plugin (necessary) :

    <\expand|itemize-dot>
      <item>A way to define all the administrative messages exchanged between
      servers on distant nodes, between clients and server on the same node,
      and routing them to the correct subsystem inside a client.

      <item>The current plan is to use binary messages, with a tag, and a
      system flag.

      <item>This plugin must also take care of the cration of the clients
      endpoints, either implicit or requested by the client, the filtering,
      the call-backs...
    </expand>

    <item>Flow Transport plugin (necessary):

    <\expand|itemize-dot>
      <item>Take care of the flows connection transport and opening, using
      the information provided by the Crossbar Transport plugin through a
      system Message.

      <item>Currently planned : Simple socket connections
    </expand>

    <item>Flow plugins (optional, for exemple in an management interface):

    <\expand|itemize-dot>
      <item>Specific to a particular data type to transport, it provide
      standard methods (ie GetBuffer() ) but can also provide some
      type-specific methods as well as methods to make data format
      conversion.
    </expand>
  </expand>

  These subsystems will be loaded in a predetermined order : First the Core
  (by running the client) that will try to find a usable Preference module
  and load it. The preferences are then read from this plugin, thus determing
  the other plugins to load for each subsystem : Authentication, Crossbar
  discovering, the Transport systems and the Messages plugin. Then the client
  can be connected by contacting the other clients/nodes. Finally, the Flows
  can be created, by loading the appropriate Flow plugins.

  <\expand|make-title>
    Client API<title|>
  </expand>

  This chapter present an client API for the SmartFlow 2. It doesn't present
  the internal object hierarchy yet, this will be presented in a later
  chapter.

  All the methods here return a error code or 0 if no problems occured (For
  now, this value is a long int, however you shouldn't assume this. If this
  is changed, the "==" and "!=" operators will be redefined so that the code
  using them will still work).

  If an error is triggered, the static method sfSmartFlow::perror() return
  the text of the error, without a trailing <inactive|<hybrid|n>>.

  <postscript|UserClasses.eps||||||>

  <\expand|make-title>
    The SmartFlow Class<title|>
  </expand>

  <\code>
    sfSmartFlow::init(int argc, char **argv, string application,string user)
  </code>

  Initialise the SmartFlow system. The "application" provide the particular
  instance of a running smartflow to connect to, \ the "user" will be useful
  if authentication is used. Default values are set if arguments are NULL.
  The user-provided argc/argv is parsed for a valid string that could select
  a "sfPreference" interface. If found, this interface is loaded.

  <\code>
    sfSmartFlow::Authenticate(string method, authenticateInfos authinfos)
  </code>

  If the SmartFlow requiere it, an authentication can be provided by the
  client with this method. The argument "method" give the name of the
  authentication plugin to use, and any token can be provided with the
  "authinfos" arguments (depandant of the used plugin). The plugin can also
  use the "application name" and the "user name" to make its mind about the
  authorization. A system where all clients must authenticate agains each
  others with dedicated messages is also planned, since the actual
  authentication is done when the run() method is called and not when this
  method is called.

  <\code>
    static sfSmartFlow::perror(sfError error,string & text, unsigned long
    size)
  </code>

  This method return in "text" the error message associated with "error",
  trucated to "size" if necessary. "text" must be allocated. This method
  alway succed.

  <\code>
    static sfSmartFlow::printError(sfError error)
  </code>

  Print a textual message on the standart error output corresponding to the
  "error" code.

  <\code>
    sfSmartFlow::Preferences(string method, preferencesInfos options)
  </code>

  <\code>
    sfSmartFlow::CrossbarDiscover(string method, crossbardiscoveryInfos
    options)
  </code>

  <\code>
    sfSmartFlow::MessageTransport(string method, messagetransportInfos
    options)
  </code>

  <\code>
    sfSmartFlow::FlowTransport(string method, flowtransportInfos options)
  </code>

  These methods select different plugins at runtime. They must be called
  after init() but before run(). They are not necessary to run the SmartFlow,
  they are only provided as convenience. The "method" argument give the
  plugin to load and the "option" argument, if any, give parameters in a
  plugin-specific syntax.

  <\code>
    sfSmartFlow::run()
  </code>

  Try to connect to the Smartflow system using all the specified plugins.
  Select plugins in this order :\ 

  <\expand|enumerate-numeric>
    <item>Command-line

    <item>By the configuration file

    <item>Provided by the client with the above methods

    <item>SmartFlow compiled defaults
  </expand>

  If this call is successful, none of the above method is usable anymore,
  including this one. However, the following methods become usable. Note that
  an authentication error will be throwed here rather than in the
  authenticate() method.\ 

  <\code>
    sfSmartFlow::CreateFlow(string char *FlowType, sfFlow **
    flow,INPUT\|OUTPUT,string arguments=NULL,NEXT\|CURRENT=NEXT, long
    *group=NULL, unsigned long int hist_size=0)
  </code>

  This method create a flow of "FlowType", by loading the appropriate plugin.
  It return a Flow object for consuming or providing data in "flow" and 0 as
  return value if all goes well, or NULL in "flow" and an error as return
  value. <strong|This is a major change form the SmartFlow 1:> If "group" is
  null, the next free group is automatically choosen. It mean that opening an
  already opened output flow <strong|*SUCCEED*>, but with another group
  number.

  The NEXT or CURRENT value define the buffers that will be returned by an
  input flow each time the buffer is updated : With the NEXT value, a client
  get all the buffers that are produced, limited in number by "hist_size"
  (For "hist_size", if 0 is given then the default flow history size is
  used). On the other hand, the "CURRENT" value here mean that each time the
  buffer is updated, the most recent value is returned. In this case, buffers
  may have been lost if the provider is faster than the consumer.

  For an input flow, the NULL value in "group" mean "connect to the next
  clientless flow of this type". If there is only one provider for this flow,
  the connection is made to it, even if there is already clients to it. If
  there is no provider, the flow is opened with no group number, waiting for
  the next provider with any group number. If there is more than one
  provider, the connection is made to the next clientless output flow. If
  none are available, this is an error as the system will not pickup a flow
  at random.\ 

  Anyway, a integer number can be given in the variable pointed to by "group"
  to return to the old behaviour.

  This behaviour has been made to simplify the management of the SmartFlow
  while editing the client's map in the graphical user interface.

  <\code>
    sfSmartflow::DestroyFlow( Flow * flow)
  </code>

  Disconnect the client from the flow. Also kill "dependants" clients if
  possible (Filters clients come in mind, or network clients)

  <\code>
    sfSmartFlow::CreateMessage(Message ** message,unsigned long tag=0,
    MessageContent *content=NULL, unsigned long size=0, clientAddress
    destination=NULL )
  </code>

  <\code>
    sfSmartFlow::DestroyMessage(Message * message)
  </code>

  Make a new (empty) message. By default this message has no destination and
  thus cannot be sent (an error is throwed if you try).

  <\code>
    sfSmartFlow::CreateMessageHandler(MessageHandler ** handler, long int
    TagFilter=0x0fff)

    sfSmartFlow::DestroyMessageHandler(MessageHandler * handler)
  </code>

  Listen to messages that match with TagFilter. By default, listen only to
  user messages (the one with the most significant byte to 0.) However, you
  can force the reception of the system message by setting the most
  significant byte to 1. (ie a tag value of 0xffff receive ALL messages,
  0x0000 receive no messages, 0xf000 receive all system message only, ...)

  <\code>
    sfSmartFlow::stop()

    sfSmartFlow::close()
  </code>

  These method are the reverse of the run() and init() methods, in that
  order. A call to close() without calling stop() first is valid because
  close() call to stop() if the system is still connected.

  <\expand|make-title>
    The other classes<title|>
  </expand>

  \ 

  The only way to create the following classes is to use the sfSmartFlow
  class to get an instance. Once you have it, you can "cast" it if necessary
  to the correct type, for exemple to have access to special Flow methods.
  This technic is reserved for advanced users has it *BREAKS* the
  encapsulation of classes.

  <\expand|make-title>
    The Flow Class<title|>
  </expand>

  \ 

  <\code>
    sfFlow::getBufferInstance(Buffer ** buffer) \ [input or ouput flows]
  </code>

  Return a pointer to an instance of the Buffer that has been allocated for
  this flow. For a given Flow object instance, there will be only one Buffer
  allocated, the one returned by this call. The return is 0 or an error code,
  in this case the returned buffer will be NULL.

  <\code>
    sfFlow::UpdateBuffer(NEXT\|CURRENT=0) [input flows only]
  </code>

  Update the content of the buffer returned by the above method. Of course,
  you must not have another thread playing with this buffer while calling
  this method, unless the other thread respect the lock() and unlock()
  convention on it. this function will block if no new buffers is available.
  When this method return, the Buffer is guaranted to contain a new block of
  data.

  The NEXT or CURRENT value is optional. If it is not provided, the default
  value from the flow creation is used. If the value used in the
  "CreateFlow()" method is NEXT, then the argument here can be NEXT (the
  default) or CURRENT, to get the most recent one. But if the flow was
  created with the "CURRENT" argument, it is not legal to specify here the
  "NEXT" value.

  There is a specific version of this method to get a specific buffer by
  number relative to a given buffer. It is not documented here as the exact
  prototype has yet to be determined.

  <\code>
    sfFlow::SendBuffer() [output flows only]
  </code>

  Send the content of the Buffer to the other clients witing for it. The
  method will make a copy of the buffer, and return quickly, so when the
  method return you can reuse the Buffer memory space immediatly for reading
  or writing. Be careful however to not modify the buffer with another thread
  while it is read by the method, use the lock() methods of the buffer if
  necessary. If you do such nasty things, the results are unpredictables
  since the library has no ways to known that the buffer is modified while it
  does the read operation on the memory space.

  <\code>
    sfFlow::QueueData(void * data, unsigned long number)
  </code>

  This method "bypass" the Buffer class for an output flow by queuing a data
  chunk directly. The data argument is a pointer from where the data can be
  retrieved by the library. You also have to provide the "number" of blocks
  pointed to by the "data" pointer. (these blocks must then be contigious in
  memory). For exemple, if this flow has a block size of 1024 bytes, and
  "number" is 4, then data must point to a memory space of at leat 4096
  bytes. In this particular case, the library cannot take care of the locking
  problems concerning the memory area "data", so the user have to take care
  of possible race conditions by himself if multithread is used.

  <\code>
    sfFlow::GetData(void *data, unsigned long maxnumber, unsigned long *
    number, NEXT\|CURRENT)
  </code>

  As for the QueueBuffers() method above, this one provide a way to get some
  data directly from the library without the encapsulation of a Buffer class.
  "data" must point to a memory space of (buffer size of the flow) x
  maxnumber bytes. The actual number of buffer copied is returned in
  "number". The constant value "NEXT" return buffers from the older, whereas
  the value "CURRENT" return them from the newer. The buffers are however
  alway return in the most recent to the less recent one. ie: If we have an
  history of 20 buffers ( 20 is the most recent, 1 the older ), the call
  GetBuffers(data,5,&n,NEXT) will return in data the 5,4,3,2,1 buffers in
  that order but \ \ GetBuffers(data,5,&n,CURRENT) will return buffers
  20,19,18,17,16. Again, locking problems on data are not managed in this
  method.

  <\code>
    sfFlow::getType(string & type)
  </code>

  Give the type of the flow ( Audio, Video, Data...). The program should know
  that already, but it is not a big deal to add, so.... Read-only by clients.

  <\code>
    sfFlow::getArguments( string & arguments )
  </code>

  Get the arguments that are currently used by the flow. If no arguments are
  given at the creation of an input flow, then this method return the
  arguments used by the provider. Read only by clients once the flow is
  online.

  <\code>
    sfFlow::getGroup(unsigned long ** group)
  </code>

  Get the current group on wich this flow is currently sending or receiving
  data. If the flow is an input flow waiting for a provider without a
  specific group specified , NULL is returned in *group. Read only by
  clients.

  <\code>
    sfFlow::getProviderName(string & name)
  </code>

  Get the unique adress of the flow provider, to send him a message for
  exemple. When this function return, if name[0] is NULL then no provider is
  currently connected. If this method is called from the provider itself, no
  result will be returned since the adress of the current client is never
  returned. Read only by clients.

  <\code>
    sfFlow::getConsumerName(list\<less\>string\<gtr\> & nameList)
  </code>

  Get the unique adress of each listeners of the current flow. If this flow
  is a consumer itself, its own address is not returned. Read only by
  clients.\ 

  <\expand|make-title>
    The Buffer Class<title|>
  </expand>

  <\code>
    sfBuffer::lock()

    sfBuffer::unlock()

    sfBuffer::trylock()

    sfBuffer::islocked()
  </code>

  These function are only to be use you you want to play with multithreading
  in your client. This can happen if two thread could access the buffer
  returned by getBufferInstance() above. In THIS case ONLY, you should lock()
  the buffer before reading or writing it, and release the lock after. This
  is what the library will do in this circumsances. This is not necessary for
  everyday usage anyway. The library will ONLY play with this buffer in
  functions UpdateBuffer(), SendBuffer(), and more generally in any other
  method name that terminate by Buffer() in the Flow class.

  <\code>
    sfBuffer::size()
  </code>

  Get the size in byte of the data pointed to by the return of the data()
  method

  <\code>
    sfBuffer::data()
  </code>

  This method return the data themselves. (b-\<gtr\>data()[19] = 5 put the
  value '5' in the 20th byte of the buffer). For a given buffer, you can rely
  on the fact that the data will always be on the same place. ( data() will
  alway return the same value for a given buffer instance). If data() return
  NULL, something goes insanely wrong, and you should terminate the progam
  immediatly and report this as a bug ( The buffer object should not have
  been allocated in the first place ).

  <\expand|make-title>
    The Message Class<title|>
  </expand>

  All setXXX() methods have their pendant getXXX() method as well. The
  setXXX() methods check for the validity of arguments in a syntaxic range
  only : For exemple, it will not detect if a destination string is
  non-existent : You will get this information only when you try to send the
  message.

  <\code>
    sfMessage::setDestination(char * destination)
  </code>

  The destination must exist, otherwise an error will be throwed when the
  message is sent. A special value send the message to every clients :
  BROADCAST. An empty address mean nobody, this is the default value when
  creating a message but it is not a valid address (You will get an error
  when trying to send the message with such a destination). The destination
  can also be a client-created endpoint (see below).

  sfMessage::setSender(char * sender)

  Normally this function should not be used. It override the normal adressing
  system by forcing a specific source address here. It can be useful in some
  situation, so it is provided anyhow.

  <\code>
    sfMessage::setTag(unsigned long tag)
  </code>

  Set the tag. The API cannot set system message, an error is returned in
  this case (if (tag&0xf000) != 0 ). So the tag can be any value between
  0x0000 and 0x0fff.

  <\code>
    sfMessage::setContent(void * content, unsigned long size)
  </code>

  Put some stuff in a message. The size is limited for performance reasons :
  The messages are made for short piece of information anyway. (The
  limitation will be roughtly the size of the maximal size of a TCP packet ,
  about 1500 bytes.) It is legal for a message to not have any content (This
  is the default and it is used for flow control for exemple)

  <\code>
    sfMessage::Send( KEEP \| DESTROY=KEEP, GET_STATUS_REPORT \| 0 = 0)
  </code>

  Send the message. The KEEP argument mean that the Message object is kept
  after its send, for a reuse or a status check. The DESTROY argument (the
  default) mean that the Message object is destroyed after it has been sent,
  no calls on the object will be accepted but DestroyMessage() (It's a "fire
  and forget" message). The message is sent "as is" when this call is made :
  you can safely modify the content of the message after the call. There is
  also a protection against multithreaded modification of the message during
  its use by the Send() method. The destination is checked at this time and
  an error is returned if it is unknown. This call queue the message and
  return : it does not wait for the message to be received. You can get
  status information on the message if the last argument is
  GET_STATUS_REPORT. In this case, the following method can be used to get
  the status of the message (and the first parameter is implicitly set to
  "KEEP").

  <\code>
    sfMessage::Status( unsigned long int *status )
  </code>

  Get the status of a message : NOT_READY -\<gtr\> Incomplete construction,
  READY -\<gtr\> OK to send, ON_QUEUE -\<gtr\> On the send queue, SENT
  -\<gtr\> currently on the wire, RECEIVED -\<gtr\> The message hit the other
  party queue, DELIVERED -\<gtr\> The message have been extracted from the
  other party queue. Be aware however that the situation at a certain time
  can have changed when the called have returned, since all these status
  information may have to travel through the network. If possible, try to
  avoid the status condition as it uses the network to transport the status.
  If the argument GET_STATUS_REPORT wasn't provided and the KEEP status is
  set, the local status is still available and the status value will increase
  to the SENT state. After sending a message, if the message object is
  modified with the setXXX() method, the status is reinitialised to READY or
  NOT_READY (depending of the alteration made).

  <\expand|make-title>
    The MessageHandler Class<title|>
  </expand>

  This class is used to receive messages by polling or callbacks. <strong|Be
  careful :> The messages callbacks can be called from another threads than
  the main one. All the message callbacks will run from the same thread
  however, serilized. That mean developers must be careful of race condition
  issues in the client if callbacks are used. Also, callbacks functions
  should be kept short and non-blocking (except on semaphores or condition
  variables for memory locking). No problems can occurs when using the
  polling method.

  <\code>
    sfMessageHandler::getNextMessage(Message * message)
  </code>

  If no messages are available, the function block. Otherwise, the available
  message is copied into the memory space pointed to by "message", then the
  message is remove form the queue. If requiered, a Status message is
  returned to the sender labeled "DELIVERED". The "message" argument here
  cannot be null : it must be a valid Message object, its content if any will
  be overwritten by the function (its initial content doesn't matter however,
  so a simple call to sfSmartFlow::CreateMessage() is okay). The message
  object can be reused.\ 

  <\code>
    sfMessageHandler::registerCallback( callback(Message *,void * userdata) )
  </code>

  This function register a function of the prototype "void callback(Message
  *,void *)" to be called when a message occur. A thread is run from now on
  to take care of this task. Only one callback can be registered for a given
  user message. If this function is called two times, then the thread wait
  for a completion of an eventual message processing, then replace the
  callback by the new one.

  If you call getNextMessage() on a MessageHandler that has a registered
  callback, you will get an error.

  The second argument is passed to the callback when it is called, as a
  convenience. It can be used to allow the main thread to known that a new
  message is present, among other things. for this argument however, no
  threading/locking mecanism is provided.

  \;

  \<less\>***** QUESTION HERE *****\<gtr\>

  When we want to return a "list of basic types" (for exemple, a list of
  string), should we use the STL and its "list" types here, ( pro : cleaner,
  cons: more difficult to understand for non-C++ people because it uses
  templates) of a simple "char **", as does the main(argc, argv) function ?

  \<less\>/***** QUESTION HERE *****\<gtr\>

  <\expand|make-title>
    Remarks<title|>
  </expand>

  Concerning the ability to push data in a flow in a different way than the
  one used to retrieve the data ( discussion about the audio flow for exemple
  ) :\ 

  We can create a basic "AudioFlow" plugin, and then later create a
  specialised AudioFlow class that inherit from the original "AudioFlow" (
  -\<gtr\> benefit of all the methods of the AudioFlow ) but implement its
  own buffer-management methods and add a set of specific methods to access
  the data in a specialized way. In the client, the code will look like this
  :

  <\code>
    sfSmartFlow *sf=NULL;

    sfSpecialAudioFlow *f=NULL;

    /* Smartflow initialisation */

    f = (sfSpecialAudioFlow *) sf-\<gtr\>CreateFlow("SpecialAudioFlow",.....);

    f-\<gtr\>MySpecificMethodForSpecialAudioFlow(...).
  </code>

  Then, if the code is proven stable, efficient and useful, we just have to
  replace the "AudioFlow" Plugin by the "SpecialAudioFlow" plugin. The
  clients that use an AudioFlow will see no changes, as all the methods of an
  "AudioFlow" are available, and the "advanced clients" that need to do some
  tricks with audio data can then make them directly from an "AudioFlow"
  plugin.

  Of course, there may be a performance penality with this particular case
  that wouln't have been here if the system was design to allow that kind of
  trick from the begining. I think however that 1/ the system will be simpler
  to design if we only have to deal with fixed-size blocks per flow (improved
  reliability) and 2/ Do we really care of loosing a few percent in
  performance ? The bottleneck is not here anyway.

  With the same idea in mind, if we have to send variable-sized data,
  sentences for exemple for the case of the speech recognition, we define a
  bloc-size of the maximal size of a piece of data, and for each sentence we
  have to send, fill the bloc with the text and pad with '0' (Or send the
  actual size in the first four bytes if the 0-padding technic is not
  appliable). Even if the transported size on the network will be larger than
  necessary, the global efficiency will be greater. ( The network reservation
  cost on an ethernet network is almost the same for 64 bytes or 1500 bytes,
  for exemple)

  <\expand|make-title>
    The internal class hierarchy<title|>
  </expand>

  The following will describe the classes that will make the SmartFlow 2 when
  running. There will be more classes interacting than the one exposed to the
  API.

  There will be some "Managers" classes whose main purpose is to organise,
  tracks, and make all the memory management stuff needed by the system, as
  well as spawning threads, handling network links, and globally providing
  generic low-level services to the classes that make the API and the
  plugins. For exemple, the MessageManager handle the message queue of the
  Smartflow, using the services of the MessageTransport Plugin when sending
  or receiving a message, take care of calling the MessageHandler callbacks
  when appropriate, destroying old messages... Here is a class diagram to
  expose the relationship between classes.

  \;

  <postscript|InternalClasses.eps||||||>

  \;

  There is a manager for each SmartFlow subsystem, plus the "Core Manager"
  that keep track of each created object (to be able to destroy them when
  necessary, if the client didn't made the appropriate calls to close the
  connections, release the buffers,.... It also receive the system message
  and take the appropriate action for each of them.\ 

  Note that in this class diagram, it is not defined HOW the object will
  communicate with each other, and where are the actual code running for this
  particular instance ( In the client, in another thread of the same client
  process, in another process, in the spawned server....) We will now see
  these particular cases.

  <\expand|make-title>
    The Network discovering<title|>
  </expand>

  \;

  <\expand|make-title>
    The Messages transport<title|>
  </expand>

  \;

  <\expand|make-title>
    The System messages<title|>
  </expand>

  \;

  <\expand|make-title>
    The Flow transport<title|>
  </expand>

  \;

  <\expand|make-title>
    The memory management<title|>
  </expand>

  This section describe how the data produced by a client will travel in the
  SmartFlow, in every possible situation.

  When the term "Buffer" is used below, it mean "a Buffer Object" for the
  developper.

  From the client point of view, for an ouput flow, the procedure is :

  - get the "Buffer" address from the library

  - do forever {

  \ \ \ - fill the buffer with data (the memspace is pointed to by the data()
  method of the Buffer object)

  \ \ \ - call "SendBuffer()" to push the buffer. (This call may block)

  }

  For an input flow, here is what to do :

  - get the "Buffer" address from the library

  - do forever {

  \ \ \ - call UpdateBuffer() to pop a buffer from the list (This call may
  block)

  \ \ \ - do what is needed with the buffer data (pointed to by the data()
  method). It is guaranted that the data will not be messed up outside of the
  UpdateBuffer() function. You must still copy the memory area if you want to
  keep the data between two calls to UpdateBuffer().

  }

  Here is a schema of the system.

  <postscript|MemDesc.eps||||||>

  The dashed lines are threads. The "dotted" space is the user's thread, the
  large dashed space is an internal library thread.\ 

  "linked list" : STL Queue (fifo)\ 

  "copy + signal" : Inside the same process : memcpy() + thread condition
  variable. Between process, either read()-write() on special files, or copy
  on sysv shared mem + signal by sysv semaphores. The data is copied into
  each list that have an available memory block. Once the copy is done, the
  call return and the other thread/process can queue the new memory block in
  the linked list.

  "get next Buffer" : Blocking call from the user's thread ( or network
  thread ) to get the next avalable block, which can be either the most
  recent one added (if the user asked for the "CURRENT" buffer) or the oldest
  one (if the client asked for the NEXT buffer). The block is copied into the
  user's thread buffer, and the linked list storage block is recycled.

  The linked list has a fixed size (The history size of the flow). Two
  distincts behaviour are possible when the linked list of one of the client
  become full : Either the last written block in this list is reused, thus
  losing a buffer, or we can force the provider to wait ( but this will
  starve the other clients's linked lists) until a new block become
  available, providing a basic "flow-control" function.

  We can see that the particular case of the network transmission is made on
  the same template, it is only a "threaded" client inside the main process
  whose only task is to read a buffer and send the data over the link to
  subscribed remote clients.\ 

  If the client ask for the CURRENT buffer, the linked list above become a
  list of only two elements (a sort of "double-buffering" system).

  \;

  <\expand|make-title>
    Server connection<title|>
  </expand>

  The following will explain how each client can communicate via Messages to
  the server of his node to get information about flows (when creating a flow
  for exemple, or for management), or other SmartFlow information. It will
  also explain the server spawning method and the takeover mecanism when two
  clients start or if the server crash.

  You will also find here the method used by the server to establich the
  crossbar from a network point of view.

  <\expand|make-title>
    SmartFlow management<title|>
  </expand>

  This section will describe the messages exchanged by the servers of each
  node to ensure that each server has a correct knowledge of the management
  information (mainly the adress of each clients (node and unique
  informations), and the flows associated to each clients) . It also describe
  other protocols, like the one used to create a new flow.

  \;

  \;
</body>

<\initial>
  <\collection>
    <associate|paragraph width|150mm>
    <associate|odd page margin|30mm>
    <associate|page right margin|30mm>
    <associate|page top margin|30mm>
    <associate|reduction page right margin|25mm>
    <associate|reduction page bottom margin|15mm>
    <associate|even page margin|30mm>
    <associate|reduction page left margin|25mm>
    <associate|page bottom margin|30mm>
    <associate|reduction page top margin|15mm>
  </collection>
</initial>

<\references>
  <\collection>
    <associate|gly-1|<tuple|1|?>>
  </collection>
</references>
