<TeXmacs|1.0.1.16>

<style|generic>

<\body>
  Smartflow 2 Implementation notes

  This document describe the current implementation choices that have been
  made today.

  The Smartflow 2 network system will be deployed with 3 programs :

  - One server on each host. This server listen on a fixed dedicated TCP port
  for both local and remote connections. It is created around a single
  select() and thus exchange messages synchronously.\ 

  - On each host, one Hub per flow. The Hub is responsible for flow
  distribution on other hosts and flow coordination on a given host.

  - On each host, zero or one Provider client per flow. This client is
  connected to a Hub and to a server, but never directly to other clients.

  - On one host , zero or more Consumer clients per flow. These clients are
  connected to a Hub and to a server, but never directly to other clients.

  On a given host, flow data are exchanged by shared memory. Implementation
  is choosen as a mmaped() file without backing store, as this kind of shared
  memory is available on most plateforms. Synchronisation is made over local
  connections with the responsible Hub for this flow (no semaphore here, to
  ease cross-plateform compatibility and reduce deadlocking risk if a client
  crash unexpectedely)

  Here is a schematics of the programs running when the smartflow is active
  on two hosts.

  - The flow are exchanged as raw data. No metadata are transported with
  them, on the network and locally in the shared memory buffer.\ 

  - The shared memory has a fixed size for a given flow. Tests will be made
  to determine the best number of independant buffers, but for now we will
  use 2 buffers of the size of one datum for a flow.

  - The Synchronisation link is bidirectional, and carry only elementary
  information such as the "new buffer in SHM" and "Buffer is available for
  reuse" on only one byte.

  - The control link is the most complex one in term of information
  transported. We will use a remote procedure call protocol to exchange data
  over this link. For now, the protocol choosen for this purpose is XML-RPC.

  Main organisation :\ 

  When a client start, it does the following :

  - Try to connect to the localhost server in tcp. If it does not succeed, it
  spawn a server and try again for a while before giving up.

  - Once connected, it uses the remote procedure call protocol to identifie
  himself. The server return an ID number that will uniquely identify the
  client on this host only.

  - Again using the remote procedure call protocol, it give the server its
  flow requierement. (providing or consuming). The server can eventually
  disagree, for exemple if it knows the same flow is already provided
  elsewhere on the network. If it agree, it return to the client a local
  socket adress that will be used to communicate with the hub for this flow.\ 

  - The client connect to the hub and identifie himself as a provider or a
  consumer. The hub return him a shared memory segment to write or read flow
  data respectively.\ 

  - The client can then start to use the shared segment and the local
  connection to exchange flow data. For a consumer, the client wait on the
  local connection. When a datum is available in the SHM, the Hub send him a
  buffer number byte. The client read this buffer, and then return a "buffer
  read" information. If the client is not consuming the data for any reason,
  it must return the "buffer read" signal anyway. For a provider, the client
  copy the datum in the shared memory and then send a "buffer available
  signal". Then it goes on with its task. When the time comes to copy another
  buffer, it first check if the first one is available by doing a
  non-blocking read on the local connection with the hub. If the hub return
  the "buffer read" information, it can reuse the previous buffer. However,
  if the hub hasn't answer yet, then it tries to copy the data in another
  available SHM buffer. If there is no more, then it does a blocking read on
  the local connection, to wait for the hub authorisation to reuse the
  buffer.

  In any clients, if a read return -1, it implie something goes wrong and the
  case must be dealt with. (most likely the other end has close the
  connection). But with this method, we can know something went wrong.

  That way, when a client wants to quit, it just close all its connections
  and shut down.\ 

  \;

  When a hub start, it does the following :

  - It connect to the server and identifie himself. The server return give
  him back an ID.

  - Then it waits. The server will call him back (alway by RPC) for serving a
  flow.

  - When the server contact him with the flow information, it allocate a SHM
  segment of the requiered size (Unless it already serve a flow, in which
  case it refuse the call and inform the server of the error). Then it open
  one local socket in listening mode, a TCP socket in either listening or
  connecting mode ( depending of the direction of the flow) and return the
  adress & ports information back to the server.

  - It then wait for client connections. If a consumer connect, it keep the
  connection open and do nothing. If a Provider connect with the local link,
  then it start forwarding messages to consumers, and collecting their
  acknowledgements.\ 

  - If the provider is remote, the packets from the remote connection start
  to arrive, the Hub is now provider of this flow for this node and start
  acting as such ( copy packet on SHM, send signals to the consumers )

  - If the provider is local and a remote client has connected, then it start
  to act as a client for this node and send packets to the remote peer(s)
  when they arrive from the SHM.

  Either way, a read() returning -1 indicate the peer has disconnected.

  When the provider is disconnected, a hub inform the server and wait for a
  little while. The server may ask him to reconnect to a new data source,
  this way the current consumers may starves but do not dies. If nothing come
  up, the hub close its connections and quit.

  \;

  When a server start it does the following :

  - It create a list of peers, a list of flows, and a list of unused hubs.
  (empty for now)

  - It spawn a hub, enter it in the peer list and in the unused hub list.

  - It open a socket and wait for connections.

  - When a client connect, it enter it in the peer list, and give it back an
  ID.

  - When a client ask for a flow ( in provider or consumer mode), first it
  look in the flow list if this flow is already served by a hub, if yes it
  forward the local socket info to the client. If not, it contact a Hub from
  the unused list and provide him the flow info. If the hub accept, it is
  removed from the unused hub list, and the information about this hub is
  updated in the peer and flow list. Then the server contact all the other
  connected servers and tell them the new flow is available. If only one
  other server disagree, the hub is killed and an error is returned to the
  client (duplicated flow). At last, the hub information is sent back to the
  client, and a new hub is spawn and added to the unused hub list.

  - When a server connect, a replication session is initated so that all
  servers know the info of all others. This is done to speed up the things
  when the server must take a decision whereas a flow already exists or not.

  - If a link is closed, the server immediatly remove all reference to this
  peer form the lists, and forward the info to the remote servers.

  RPC syntax : The RPC system choosen is XML-RPC.\ 

  XML-RPC provide basic types (bool int double strings), binary data coded in
  base64, and multitype such as structures and arrays.

  RPC Call handled by the server :

  RPC Call handled by a hub :

  RPC Call handled by a client :

  \;

  External libraries used by the Smartflow so far :

  - ACE library v 5.3 ( http://www.cs.wustl.edu/~schmidt/ACE.html ) for all
  low-level encapsulation : Sockets, Time-related, Shared Memory, .... This
  library has been successfully compiled on all plateform we intend to use,
  including PDAs, Mac/OS-X, Windows... and provide an API compatible for all
  these plateforms.

  - XML-RPC++ v0.7 : A low-level C++ library to work with the XML-RPC
  protocol in C++ without having to deal with the XML itself or any kind of
  encapsulation. This library is events oriented, so will be the smartflow on
  the messaging side. Not tested on anything else than Linux so far, but the
  library is very tiny, so modification will be very easy if needed at all.

  - QT 3 library will be used for GUI. We intend to reuse the "map" of the
  first smartflow version with dramatic improvements in the drawing
  algorithm, since no other suitable components could be found with a
  compatible licence (non-GPL).

  \;
</body>

<\initial>
  <\collection>
    <associate|paragraph width|150mm>
    <associate|odd page margin|30mm>
    <associate|page right margin|30mm>
    <associate|page top margin|30mm>
    <associate|reduction page right margin|25mm>
    <associate|page type|a4>
    <associate|reduction page bottom margin|15mm>
    <associate|even page margin|30mm>
    <associate|reduction page left margin|25mm>
    <associate|page bottom margin|30mm>
    <associate|reduction page top margin|15mm>
    <associate|language|english>
  </collection>
</initial>
