#!/usr/bin/env perl

use strict;

my $debug = 0;

my $ifilen = shift @ARGV;

open IFILE, "<$ifilen"
  or errorquit("Could not find source file ($ifilen): $!");

####################
# !WARNING! 1 class per file file (for now)
####################
# Note: 1 pass parser

# In header/class
my $isin = "header";
# Currently parsing a public or private section
my $ismode = "public";

# copied as is
my $uinc = "";
my $private = "";

# stored, then written at the end (.h)
my $umembers = "";
my $umethods = "";

# class/publicname
my $class = "";
my $public = "";

# To check for
my $found_invoke = 0;
my $found_init   = 0;
my $found_deinit = 0;

# Outputfile
my $ofileh = "";
my $ofilec= "";
my $ofileh_h = "";
# Text
my $agtxt = "/* Auto-generated */";

# Curl bracket counter
my $cbc = -1;
# C Comment counter
my $ccc = 0;

my $line;
my $linec = 0;
my $rest;
my $tpl = ""; # To Process Line
while ($line = <IFILE>) {
  $linec++;
  chomp($line);
  $rest = "NOT EMPTY";
  dprint("**$line");
  
  # Cleanup: spaces at beg and end of line (for easier parsing)
  $line = cleanline($line);

  # skip empty lines
  next if ($line =~ /^$/);

  # C++ comments
  next if ($line =~ /^\/\//);
  # C comments
  my $tline = cleancomments($line);
  $line = cleanline($tline) if ($tline ne $line); # clean 'comment cleaned'
  # skip empty lines
  next if ($line =~ /^$/);

  $tpl .= " $line";

  # Multi lines are made into a single line for easier processing
  # (when in "class" processing)
  next if (($isin eq "classcore") && ($line !~ /[\;|\:]$/));

  # Last clean up (required by the fact that tpl is at least " ")
  $tpl = cleanerline($tpl);
  # skip empty lines
  next if ($tpl =~ /^$/);

  dprint("#$cbc#$isin#$tpl");

  if ($isin eq "header") {
    $rest = doheader($tpl);
  }


  if ($isin =~ /^class/) {
    # "curly bracket counter" is to avoid processing data after the closing };
    cbc_counter($tpl);
    next if ($cbc == 0);
    $rest = doclass($tpl);
  }

  errorquit("not an empty line left, line $linec", "Src: [$tpl]", "Rest:[$rest]") if ($rest !~ /^$/);

  $tpl = "";
}

close IFILE;
close_c();
close_h();

errorquit("one of the three required prototype (invoke, init, deinit) was not found, included in .h file automaticaly, remember to code them") if ($found_invoke + $found_init + $found_deinit != 3);

okquit();

# END
########################################

sub doheader {
  my $line = shift @_;

  # includes
  if ($line =~ /^\#/) {
    $uinc .= $line;
    $uinc .= "\n";
    $line = "";
  }

  # Switch from header to class
  $isin = "class" if ($line =~ /^class/);

  return $line;
}

sub doclass {
  my $line = shift @_;

  # class definition itself, file creation
  if ($line =~ /^class\s+(.+?)\:\s+public\s+(.+?)$/) {
    $class  = cleanline($1);
    $public = cleanline($2);
    errorquit("Could not extract proper 'class' name", "line=$linec [$line]")
      if ($class =~ /^$/);
    errorquit("Could not extract proper 'public' name", "line=$linec [$line]")
      if ($public =~ /^$/);
    open_h();
    open_c();

    return "";
  }

  # class definition itself, file creation (no public)
  if ($line =~ /^class\s+(.+?)$/) {
    $class  = cleanline($1);
    $public = "";
    errorquit("Could not extract proper 'class' name", "line=$linec [$line]")
      if ($class =~ /^$/);
    open_h();
    open_c();

    return "";
  }

  # Mode change
  if ($line =~ /^private\s*\:/) {
    $ismode = "private";
    return "";
  }
  if ($line =~ /^public\s*\:/) {
    $ismode = "public";
    return "";
  }

  # private
  if ($ismode eq "private") {
    $private .= "   $line\n";
    return "";
  }

  # public

  # Some lines must be present
  # 1) 'int invoke( Message ** reply );'
  if ($line =~ /^int\s+invoke\s*\(\s*Message\s*\*\*\s+reply\s*\)\s*\;/ ) {
    $found_invoke = 1;
    return "";
  }
  # 2) 'int init(void);'
  if ($line =~ /^int\s+init\s*\(\s*void\s*\)\s*\;/ ) {
    $found_init = 1;
    return "";
  }
  # 3) 'int deinit(void);'
  if ($line =~ /^int\s+deinit\s*\(\s*void\s*\)\s*\;/) {
    $found_deinit = 1;
    return "";
  }

  # lines with () are functions
  if ($line =~ /\(.*?\)/) {
    $umethods .= "   $line\n";
    return "";
  }

  # variables declaration (! almost anything left can be that)
  if ($line =~ /^([^\s]+?)\s+([^\s]+?)\s*\;$/) {
    my ($type, $var) = ($1, $2);
    dovar($type, $var);
    return "";
  }
  
  # Special case ... only "{" (first)
  if (($cbc == 1) && ($line =~ /^\{$/)) {
    $isin = "classcore";
    return "";
  }

  return $line;
}

####################

sub open_h {
  $ofileh = "${class}_MessageImpl.h";
  
  open OFILEH, ">$ofileh"
    or errorquit("Could not open output H file ($ofileh): $!");
  
  $ofileh_h = $ofileh;
  $ofileh_h =~ s/\./_/g;
  $ofileh_h = uc ($ofileh_h);

  my $publicm = ($public ne "") ? ",$public" : "";

  print OFILEH<<EOT;
#ifndef $ofileh_h
#define $ofileh_h
$agtxt
#include <Messages/Message.h>
#include <Types/XMLRPC_Value.h>
#include <string>
#include <iostream>

// User Headers
$uinc

class $class : public Message$publicm
{
EOT
  
}

#####

sub open_c {
  $ofilec = "${class}_MessageImpl.cpp";
  
  open OFILEC, ">$ofilec"
    or errorquit("Could not open output C file ($ofilec): $!");
  
  print OFILEC<<EOT;
#include \"$ofileh\"

$agtxt
${class}::${class}()
 : Message(\"${class}\")
{
EOT
  
}
  
  
####################

sub close_h {
  print OFILEH<<EOT;
public:
   /* User members */
$umembers

   /* User methods */
$umethods

   /* Interface-required method */
   virtual int invoke( Message ** reply );
   virtual int init(void);
   virtual int deinit(void);

   $agtxt
   ${class}();
   virtual ~${class}();
   virtual Message *makenew(void);
   virtual int dispose(void);
EOT

  if ($private ne "") {
    print OFILEH<<EOT;

 // 'Private' content
 private:
$private
EOT

  }

  print OFILEH<<EOT;
};
#endif
EOT
  
}

#####

sub close_c {
  print OFILEC<<EOT;
   init();
}
 
$agtxt
${class}::~${class}()
{
   deinit();
}
   
$agtxt
Message * ${class}::makenew(void)
{
   return new ${class}();
}
int ${class}::dispose(void)
{
   return Message::dispose();
}

#include <${class}.cpp>
EOT
  
}

####################
  
sub dovar {
  my ($type, $var) = @_;
    
  my $realtype = $type;
  $realtype =~ s/^.+?\:\://;
  my $dv = "";

  if ($realtype eq "int") {
    $dv = "0";
  } elsif ($realtype eq "double") { 
    $ dv = "0";
  } elsif ($realtype eq "bool") {
    $dv = "false";
  } elsif ($realtype eq "string") {
    $dv = "\"\"";
  } else {
    errorquit("strange type ($type) for var ($var), line $linec", "[$line]");
  }

  print OFILEC<<EOT;
   param_list_.push_back(\&$var);
   $var.setname(\"$var\");
   $var.holder = $dv;

EOT
    
  $umembers .= "   XMLRPCValue\<$type\>  $var;\n"

}


####################

sub cleanline {
  my $var = shift @_;

  $var =~ s/^\s+//;
  $var =~ s/\s+$//;

  return $var;
}

sub cleanerline {
  my $var = shift @_;

  $var = cleanline($var);
  # Erase tabs, mutliple spaces
  while ($var =~ s/\t/ /) {1;}
  while ($var =~ s/\s{2}/ /) {1;}

  return $var;
}


sub cbc_counter {
  my $line = shift @_;

  my $lcbc = 0;

  while ($line =~ s/\{//) { $lcbc++; }
  while ($line =~ s/\}//) { $lcbc--; }

  $cbc = 0 if (($cbc == -1) && ($lcbc != 0));

  $cbc += $lcbc;
}

sub cleancomments {
  my $line = shift @_;
  
  if ($ccc != 0) { # already in a multi-line comment
    if ($line =~ /\*\//) { # See the end of comment ?
      $line =~ s/^.*?\*\///;
      $ccc = 0;
    } else {
      $line = "";
    }

    return $line;
  } 

  # No comment at all, and out of multi-line mode
  return $line if (($line !~ /\/\*/) && ($line !~ /\*\//));
  
  # Beg and End comment ?
  return $line if ($line =~ s/\/\*.*?\*\///);

  # What's left: Beg mutli-line comment
  if ($line =~ s/\/\*.*$//) {
    $ccc = 1;
  }

  return $line;
}

##########

sub okquit {
  doprint("", "Generated: $ofilec", "Generated: $ofileh", "Done.");

  exit 0;
}

sub errorquit {
  doprint("ERROR| ", @_);

  exit 1;
}

##########

sub doprint {
  my $ht = shift @_;

  foreach my $line (@_) {
    print "$ht$line\n";
  }
}

sub dprint {
  return if ($debug == 0);

  doprint("DEBUG| ", @_);
}

