# Check for Qt compiler flags, linker flags, and binary packages
AC_DEFUN([CHECK_QT],
[
AC_REQUIRE([AC_PROG_CXX])
AC_REQUIRE([AC_PATH_X])

AC_MSG_CHECKING([QTDIR])
AC_ARG_WITH([qtdir], [  --with-qtdir=DIR        Qt installation directory [default=$QTDIR]], QTDIR=$withval)
# Check that QTDIR is defined or that --with-qtdir given
if test x"$QTDIR" = x ; then
    QT_SEARCH="/usr/lib/qt41 /usr/local/qt41 /usr/lib/qt4 /usr/local/qt4 /usr/lib/qt /usr/local/qt /opt/qt /opt/qt4 /usr"
    for i in $QT_SEARCH; do
        if test -f $i/include/Qt/qglobal.h -a x$QTDIR = x; then QTDIR=$i; fi
    done
fi
if test "x$is_macosx" = "xyes" ; then
   if test x"$QTDIR" = x ; then
      qt_framework_install=yes
      QT_SEARCH="/Library/Frameworks"
      for i in $QT_SEARCH; do
        if test -f $i/QtCore.framework/Headers/qglobal.h -a x$QTDIR = x; then QTDIR=$i; fi
      done
   else
      qt_framework_install=no
   fi
fi
have_libQT=yes
if test x"$QTDIR" = x ; then
    have_libQT=no
    AC_MSG_WARN([*** QTDIR must be defined, or --with-qtdir option given])
else
AC_MSG_RESULT([$QTDIR])

# Change backslashes in QTDIR to forward slashes to prevent escaping
# problems later on in the build process, mainly for Cygwin build
# environment using MSVC as the compiler
# TODO: Use sed instead of perl
QTDIR=`echo $QTDIR | perl -p -e 's/\\\\/\\//g'`

# Figure out which version of Qt we are using
AC_MSG_CHECKING([Qt version])
if test "x$is_macosx" = "xyes" && test "x$qt_framework_install" = "xyes" ; then
   QT_VER=`grep 'define.*QT_VERSION_STR\W' $QTDIR/QtCore.framework/Headers/qglobal.h | perl -p -e 's/\D//g'`
else
   QT_VER=`grep 'define.*QT_VERSION_STR\W' $QTDIR/include/Qt/qglobal.h | perl -p -e 's/\D//g'`
fi
case "${QT_VER}" in
    4*)
        QT_MAJOR="4"
    ;;
    *)
        #AC_MSG_WARN([*** Don't know how to handle this Qt major version])
    ;;
esac
AC_MSG_RESULT([$QT_VER ($QT_MAJOR)])

# Check that moc is in path
AC_CHECK_PROG(MOC, moc, moc)
if test x$MOC = x ; then
   AC_CHECK_PROG(MOC, moc-qt4, moc-qt4)
   if test x$MOC = x ; then
        AC_MSG_WARN([*** moc must be in path])
        have_libQT=no
   fi
fi

# uic is the Qt user interface compiler
AC_CHECK_PROG(RCC, rcc, rcc)
if test x$RCC = x ; then
        AC_MSG_WARN([*** rcc must be in path])
        have_libQT=no
fi

# Calculate Qt include path
if test "x$is_macosx" = "xyes" && test "x$qt_framework_install" = "xyes" ; then
   QT_CXXFLAGS="-I$QTDIR/QtCore.framework/Headers -I$QTDIR/QtGui.framework/Headers -I$QTDIR/QtXml.framework/Headers -I$QTDIR/Qt3Support.framework/Headers"
else
   QT_CXXFLAGS="-I$QTDIR/include"
fi

# figure out if we're doing a static or dynamic link
case "${host}" in
    *darwin*)
	if test "x$qt_framework_install" = "xyes" ; then
	   QT_IS_DYNAMIC="yes" 
           QT_LIB="-framework QtCore -framework Qt3Support -framework QtXml -framework QtGui"
	   QT_IS_STATIC="no"
	else
	   QT_IS_DYNAMIC=`ls $QTDIR/lib/libQtCore*.dylib 2> /dev/null`
           if test "x$QT_IS_DYNAMIC" = x ; then
	      QT_IS_DYNAMIC="no"
           else
	      QT_IS_DYNAMIC="yes" 
              QT_LIB="-lQtCore -lQt3Support -lQtXml -lQtGui"
	      QT_IS_STATIC="no"
           fi        
           if test x$QT_IS_DYNAMIC = xno; then
	      QT_IS_STATIC=`ls $QTDIR/lib/libQtCore*.a 2> /dev/null`
              if test "x$QT_IS_STATIC" = x;  then
                QT_IS_DYNAMIC="no"
                QT_IS_STATIC="no"
                have_libQT=no
                AC_MSG_WARN([*** Couldn't find any Qt libraries])
              fi
           fi
        fi
       ;;

    *)
	QT_IS_DYNAMIC=`ls $QTDIR/lib/libQtCore*.so 2> /dev/null`
        if test "x$QT_IS_DYNAMIC" = x ; then
            QT_IS_DYNAMIC=`ls $QTDIR/lib64/libQtCore*.so 2> /dev/null`
            if test "x$QT_IS_DYNAMIC" = x ; then
               QT_IS_DYNAMIC="no"
            else
               QT_IS_DYNAMIC="yes" 
               QT_LIB="-lQtCore -lQt3Support -lQtXml -lQtGui"
	       QT_IS_STATIC="no"
            fi
        else
            QT_IS_DYNAMIC="yes" 
            QT_LIB="-lQtCore -lQt3Support -lQtXml -lQtGui"
	    QT_IS_STATIC="no"
        fi        
        if test x$QT_IS_DYNAMIC = xno; then
	    QT_IS_STATIC=`ls $QTDIR/lib/libqt*.a 2> /dev/null`
            if test "x$QT_IS_STATIC" = x;  then
                QT_IS_DYNAMIC="no"
                QT_IS_STATIC="no"
                have_libQT=no
                AC_MSG_WARN([*** Couldn't find any Qt libraries])
            else
                QT_LIB="-lQtCore -lQt3Support -lQtXml -lQtGui"
            fi
        fi
        ;;
esac
AC_MSG_CHECKING([if Qt is static])
AC_MSG_RESULT([$QT_IS_STATIC])
AC_MSG_CHECKING([if Qt is multithreaded])
AC_MSG_RESULT([$QT_IS_MT])

QT_GUILINK=""
QASSISTANTCLIENT_LDADD="-lqassistantclient"
case "${host}" in
    *irix*)
        QT_LIBS="$QT_LIB"
        if test x$QT_IS_STATIC = xyes ; then
            QT_LIBS="$QT_LIBS -L$x_libraries -lXext -lX11 -lm -lSM -lICE"
        fi
        ;;

    *linux*)
        QT_LIBS="$QT_LIB"
        if test x$QT_IS_STATIC = xyes ; then
            QT_LIBS="$QT_LIBS -L$x_libraries -lXext -lX11 -lm -lSM -lICE -ldl -ljpeg"
        fi
        ;;

    *darwin*)
        QT_LIBS="$QT_LIB"
        if test "x$qt_framework_install" = "xyes" ; then
            QT_LIBS="$QT_LIBS -L$x_libraries -lXext -lX11 -lm -lSM -lICE -ldl"
	else
	    QT_LIBS="$QT_LIBS -L$x_libraries -lXext -lX11 -lm -lSM -lICE -ldl -ljpeg"	
	fi
        ;;

    *osf*) 
        # Digital Unix (aka DGUX aka Tru64)
        QT_LIBS="$QT_LIB"
        if test x$QT_IS_STATIC = xyes ; then
            QT_LIBS="$QT_LIBS -L$x_libraries -lXext -lX11 -lm -lSM -lICE"
        fi
        ;;

    *solaris*)
        QT_LIBS="$QT_LIB"
        if test x$QT_IS_STATIC = xyes ; then
            QT_LIBS="$QT_LIBS -L$x_libraries -lXext -lX11 -lm -lSM -lICE -lresolv -lsocket -lnsl"
        fi
        ;;
esac

QT_CXXFLAGS="$QT_CXXFLAGS -D_REENTRANT -DQT_QT3SUPPORT_LIB -DQT3_SUPPORT -DQT_XML_LIB -DQT_GUI_LIB -DQT_CORE_LIB -DQT_SHARED"
if test "x$is_macosx" = "xyes" && test "x$qt_framework_install" = "xyes" ; then
   QT_LDADD="-F$QTDIR $QT_LIBS"
else
   QT_LDADD="-L$QTDIR/lib $QT_LIBS"
fi

if test x$QT_IS_STATIC = xyes ; then
    OLDLIBS="$LIBS"
    LIBS="$QT_LDADD"
    AC_CHECK_LIB(Xft, XftFontOpen, QT_LDADD="$QT_LDADD -lXft")
    LIBS="$OLDLIBS"
fi

AC_MSG_CHECKING([QT_CXXFLAGS])
AC_MSG_RESULT([$QT_CXXFLAGS])
AC_MSG_CHECKING([QT_LDADD])
AC_MSG_RESULT([$QT_LDADD])

AC_SUBST(QT_CXXFLAGS)
AC_SUBST(QT_LDADD)
AC_SUBST(QT_GUILINK)
AC_SUBST(QASSISTANTCLIENT_LDADD)

fi

])


#High level macro that check whether Qt is present
AC_DEFUN([SF_CHECK_LIB_QT],
[
AC_ARG_WITH([qt],
  [AS_HELP_STRING([--without-qt],
  [disable support for qt])],
  [],
  [with_qt=yes])
if test "x$with_qt" = "xyes" ; then
  CHECK_QT
fi
AM_CONDITIONAL([HAVE_LIBQT], [test "x$have_libQT" = "xyes"])
])

#Warning for Qt
AC_DEFUN([SF_WARNING_QT],
[
	if test "x$have_libQT" = "xno"; then
	   AC_MSG_WARN([QT4 library not found (The problem can also be that moc is not in the path). The control center won't be compiled.])
	fi
])