############################################################
# This file contains the macros used by the configure script
############################################################


############################################################
# This macro checks for program such as compiler, linker...
############################################################
AC_DEFUN([SF_CHECK_PROGRAMS],
[
	AC_PROG_LIBTOOL
	AC_PROG_CXX
	AC_PROG_CC
	AC_PROG_CPP
	AC_PROG_INSTALL
	AC_PROG_LN_S
	AC_PROG_MAKE_SET
])

############################################################
# This macro checks for some required system header files
############################################################
AC_DEFUN([SF_CHECK_HEADERS],
[
	AC_HEADER_STDC
	AC_CHECK_HEADERS([arpa/inet.h fcntl.h mach/mach.h malloc.h memory.h netinet/in.h stdint.h stdlib.h string.h sys/time.h unistd.h])
])

############################################################
# This macro checks for for typedefs, structures, 
# and compiler characteristics
############################################################
AC_DEFUN([SF_CHECK_COMPILER_CHARAC],
[
	AC_HEADER_STDBOOL
	AC_C_CONST
	AC_C_INLINE
	AC_TYPE_OFF_T
	AC_TYPE_PID_T
	AC_TYPE_SIZE_T
	AC_HEADER_TIME
	AC_C_VOLATILE
])

############################################################
# This macro checks for standard functions
############################################################
AC_DEFUN([SF_CHECK_FUNCTIONS],
[
	AC_FUNC_ERROR_AT_LINE
	AC_FUNC_FORK
	AC_FUNC_MALLOC
	AC_FUNC_MEMCMP
	AC_FUNC_REALLOC
	AC_TYPE_SIGNAL
	AC_CHECK_FUNCS([bzero clock_gettime gettimeofday memset socket sqrt strerror strstr strtol])
])

############################################################
# This macro saves the linker flags
############################################################
AC_DEFUN([SF_SAVE_LINK_FLAGS],
[
	AC_LANG_PUSH([C++])
	ORIG_LDFLAGS="$LDFLAGS"
	ORIG_LIBS="$LIBS"
])

############################################################
# This macro restores the linker flags
############################################################
AC_DEFUN([SF_RESTORE_LINK_FLAGS],
[
	LDFLAGS="$ORIG_LDFLAGS"
	LIBS="$ORIG_LIBS"
	AC_LANG_POP([C++])
])

############################################################
# This macro detects wether we are running on Darwin
############################################################

AC_DEFUN([SF_CHECK_MACOSX],
[
case "${host}" in
    *darwin*)
        is_macosx=yes
#test which version of MAC OS X we are aorking on :
	cur_target=`sw_vers -productVersion | sed 's/\(10\.[[0-9]]*\).*/\1/'`
	if test ${cur_target} '>' 10.2 ; then
          if test "x$MACOSX_DEPLOYMENT_TARGET" = "x" ; then
	    if test "x$cur_target" = "x10.3" ; then
              MACOSX_DEPLOYMENT_TARGET=10.3
	      LDFLAGS="$LDFLAGS -Wl,-macosx_version_min -Wl,10.3"
            elif test "x$cur_target" = "x10.4" ; then
              MACOSX_DEPLOYMENT_TARGET=10.4
	      LDFLAGS="$LDFLAGS -Wl,-macosx_version_min -Wl,10.4"
	    elif test "x$cur_target" = "x10.5" ; then
              MACOSX_DEPLOYMENT_TARGET=10.4
	      LDFLAGS="$LDFLAGS -Wl,-macosx_version_min -Wl,10.4"
            fi
          elif test "x$MACOSX_DEPLOYMENT_TARGET" = "x10.1" ; then
	    AC_MSG_ERROR(
                  [The environment variable MACOSX_DEPLOYMENT_TARGET is defined to 10.1. The Smartflow2 won't compile. Define this variable to at least 10.3 and try again.]
            )
          elif test "x$MACOSX_DEPLOYMENT_TARGET" = "x10.2" ; then
	    AC_MSG_ERROR(
                  [The environment variable MACOSX_DEPLOYMENT_TARGET is defined to 10.2. The Smartflow2 won't compile. Define this variable to at least 10.3 and try again.]
            )
	  fi
        else
          AC_MSG_ERROR(
                [The version of your system is too old. You must use at least MAC OS X 10.3 in order to compile the Smartflow2.]
          )
        fi
        ;;
    *)
        is_macosx=no
        ;;
esac

if test "x$is_macosx" = "xyes" ; then
  MACOSX_CPPFLAGS="-DMACX"
else
  MACOSX_CPPFLAGS=""
fi
AC_SUBST(MACOSX_CPPFLAGS)
AC_SUBST(MACOSX_DEPLOYMENT_TARGET)
AM_CONDITIONAL([IS_MACOSX], [test "x$is_macosx" = "xyes"])
])

##Misc libraries

##############################################################
#Unused at the moment.
#Test for ICE library
##############################################################
AC_DEFUN([SF_CHECK_LIB_ICE],
[
	AC_CHECK_LIB([ICE], [IceOpenConnection], have_libICE=yes, have_libICE=no)
	AM_CONDITIONAL([HAVE_LIBICE], [test "x$have_libICE" = "xyes"])
])

##############################################################
#Unused at the moment.
#Test for SM library
##############################################################
AC_DEFUN([SF_CHECK_LIB_SM],
[
	AC_CHECK_LIB([SM], [SmcOpenConnection], have_libSM=yes, have_libSM=no)
	AM_CONDITIONAL([HAVE_LIBSM], [test "x$have_libSM" = "xyes"])
])

##############################################################
#Unused at the moment.
#Test for Xi library
##############################################################
AC_DEFUN([SF_CHECK_LIB_XI],
[
	AC_CHECK_LIB([Xi], [XGetExtensionVersion], have_libXi=yes, have_libXi=no)
	AM_CONDITIONAL([HAVE_LIBXI], [test "x$have_libXi" = "xyes"])
])

##############################################################
#Unused at the moment.
#Test for Xmu library
##############################################################
AC_DEFUN([SF_CHECK_LIB_XMU],
[
	AC_CHECK_LIB([Xmu], [XmuLookupStandardColormap], have_libXmu=yes, have_libXmu=no)
	AM_CONDITIONAL([HAVE_LIBXMU], [test "x$have_libXmu" = "xyes"])
])

##############################################################
#Unused at the moment.
#Test for m library
##############################################################
AC_DEFUN([SF_CHECK_LIB_M],
[
 	AC_CHECK_LIB([m], [roundf], have_libm=yes, have_libm=no)
	AM_CONDITIONAL([HAVE_LIBM], [test "x$have_libm" = "xyes"])
])

############################################################
# These macros display warning if necessary
############################################################

#Warning for ICE
AC_DEFUN([SF_WARNING_ICE],
[
if test "x$have_libICE" = "xno"; then
   AC_MSG_WARN([ICE library not found. Clients using it won't be compiled.])
fi
])

#Warning for SM
AC_DEFUN([SF_WARNING_SM],
[
	if test "x$have_libSM" = "xno"; then
	   AC_MSG_WARN([SM library not found. Clients using it won't be compiled.])
	fi
])

#Warning for Xi
AC_DEFUN([SF_WARNING_XI],
[
	if test "x$have_libXi" = "xno"; then
	   AC_MSG_WARN([Xi library not found. Clients using it won't be compiled.])
	fi
])

#Warning for Xmu
AC_DEFUN([SF_WARNING_XMU],
[
	if test "x$have_libXmu" = "xno"; then
	   AC_MSG_WARN([Xmu library not found. Clients using it won't be compiled.])
	fi
])

#Warning for m
AC_DEFUN([SF_WARNING_M],
[
	if test "x$have_libm" = "xno"; then
	   AC_MSG_WARN([m library not found. Clients using it won't be compiled.])
	fi
])

############################################################
# These macro checks if sub-modules of the project are present
############################################################
AC_DEFUN([SF_CHECK_SUBMODULES],
[
	#obsolete...
	#First we check if the module is present by checking the file
	#Is the file should be an argument? Yes!
	#And call recursively the configure file
	#otherwise display a warning saying the module has not been found...
		
	AC_MSG_CHECKING(whether module $1 is present)	
	AC_CHECK_FILE($2/configure, has_module=yes, has_module=no)
	if test "x$has_module" = "xyes"; then
		AC_MSG_RESULT(Module $1 has been found)
		AC_CONFIG_SUBDIRS($2)
	else 
		AC_MSG_RESULT(Module $1 has not been found)	
	fi
])

############################################################
# These macro build the list of Makefiles that need
# to be generated
############################################################
AC_DEFUN([SF_BUILD_MAKEFILE_LIST],
[
	AC_CONFIG_FILES([
	   Makefile
	   src/Makefile
	   src/admin/Makefile
	   src/admin/duplicator/Makefile
	   src/admin/sf2d/Makefile
	   src/smartflow2/Makefile
	   src/utility/Makefile
	   src/flows/Makefile
	   src/flows/Flow_BlockTest/Makefile
	   src/flows/Flow_Java/Makefile
	   src/clients/Makefile
	   src/clients/examples/example_consumer_custom_metadata/Makefile
	   src/clients/examples/example_consumer_iterator/Makefile
	   src/clients/examples/example_consumer_memcpy/Makefile
	   src/clients/examples/example_consumer_popdata/Makefile   
	   src/clients/examples/example_provider_custom_metadata/Makefile   
	   src/clients/examples/example_provider_memcpy/Makefile   
	   src/clients/examples/example_provider_multipush/Makefile   
	   src/clients/examples/example_provider_pushdata/Makefile   
	   src/clients/examples/example_provider_with_complex_exit_func/Makefile   
	   src/clients/examples/example_provider_with_simple_exit_func/Makefile
	   src/tools/Makefile
	   src/tools/sfcontrolcenter/Makefile
	])

	AC_OUTPUT
])

############################################################
# Display warnings telling the user which modules are present
# or not.
############################################################
AC_DEFUN([SF_WARNING_SUBMODULES],
[

])
