#ifndef _LOG_MANAGER_H
#define _LOG_MANAGER_H

#define DEBUG_PREFIX		LM_DEBUG,		ACE_TEXT ("%I[DUPL DEBUG   %T] ") 
#define WARNING_PREFIX		LM_WARNING,		ACE_TEXT ("%I[DUPL WARNING %T] ")
#define ERROR_PREFIX		LM_ERROR,		ACE_TEXT ("%I[DUPL ERROR   %T] ")
#define USER_PREFIX			LM_ERROR,		ACE_TEXT ("%I[DUPL USER    %T] ")
#define ANNOUNCE_PREFIX		LM_ERROR,		ACE_TEXT ("%I[DUPL ANNOUNCE%T] ")

#include "ace/OS.h"
#include "ace/streams.h"
#include "ace/Log_Msg.h"
#include "ace/Log_Msg_Callback.h"
#include "ace/Log_Record.h"
#include "ace/SString.h"
#include "ace/Message_Queue.h"
#include "ace/Thread_Manager.h"

class DUP_Log;
class Log_Callback;
class Log_Manager;

/** \brief Class Log_Callback .
 * This class is used to implement a callback method
 * invoked when a logging operation occured.
 * It is used to propagate the log to the server
 * \author Nist Smartspace Group
 * \date 04-19-2004
 */
class Log_Callback : public ACE_Log_Msg_Callback {
	public:
		Log_Callback(Log_Manager *manager);
		~Log_Callback();
		void log(ACE_Log_Record &log_record);

    private:
		int sendLogToServer();
		void putLogInQueue(ACE_Log_Record &log_record);
		Log_Manager *manager_;
};


/** \brief Class Log_Manager_ACE
 * This class is used to manage the log system.
 * It is used to define the desired severity.
 * It is also used to send the log info to the server, 
 * write them in a file if desired and print them on the stderr
 */
class Log_Manager {
   public:
    
     /** \brief Class constructor. 
     * \warning This method does not initialise the object, the init() method
     * must be used for this purpose.
     * 
     */
    Log_Manager(DUP_Log *logHandler);
    
	/** \brief Class destructor.
     * Take care of any cleanup that is necessary to free the object.
     */
	virtual ~Log_Manager();
    
	/** \brief Class initialiser. Must be called just after the object
     * construction. 
     * \param sf The Smartflow object in which this object will be be registered.
     * \return 0 if the object is correctly initialised, otherwise -1. If possible,
     * an error will be set via Smartflow::seterror() to explain the error.
     */
	virtual int init(ACE_TString flowID, ACE_TString severity, int isSyslogEnabled, int isForwardEnabled, ACE_TString logFilePath);

	DUP_Log * getLogHandler() {return log_handler; }

	ACE_Message_Queue<ACE_SYNCH> *getQueue() { return &logQueue; }

	int isNotificationSent() { return notificationSent;}
	
	void setNotificationSent(int notif) { notificationSent = notif; }
	
	bool forwardLog() { return logForwardEnabled; }
    
private:
	Log_Callback *cb;
	DUP_Log *log_handler;
	ACE_OSTREAM_TYPE *file;
	ACE_Message_Queue<ACE_SYNCH> logQueue;
	int notificationSent;
	bool logForwardEnabled;
};


#endif

