#include "handlers.h"
#include "duplicator.h"
#include "XML_Parser.h"
#include "sf2d_common_ACE.h"

#define buffer_length 512

bool DUP_Consumer::has_read_block(int block_number_) {
	ACE_TRACE(ACE_TEXT("DUP_Consumer::has_read_block"));

	return (this->blocks_read[block_number_]);
}

/**************************************************************/
/**************************************************************/
int DUP_Local_Consumer::handle_input(ACE_HANDLE handle)
{
	ACE_TRACE(ACE_TEXT("DUP_Local_Consumer::handle_input"));
	
	ACE_UNUSED_ARG(handle);

	int nbytes;
	local_block_t buf;
	memset(&buf, 0, sizeof(buf));
	
	nbytes = this->peer().recv_n(&buf, sizeof(buf));
	// check if no bytes were received or an error occurred
	if (nbytes <= 0)
	{
		// if so, return -1 so that the handle_close() hook method gets called
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Local_Consumer::handle_input ] ERROR: no bytes were received or an error occurred\n")));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DUP_Local_Consumer::handle_input ] ERROR: no bytes were received or an error occurred\n")), -1);
	}

	// notify the duplicator that the block has been read
	return TheDuplicator::instance()->block_release(this);
}

int DUP_Local_Provider::handle_input(ACE_HANDLE handle)
{
	ACE_TRACE(ACE_TEXT("DUP_Local_Provider::handle_input"));

	ACE_UNUSED_ARG(handle);
	
	//we don't take any data from the provider if the flow isSynchronized and the start tranfer data request has
	//not been received
	if ( ( TheDuplicator::instance()->isSynchronized() == 1 ) && ( TheDuplicator::instance()->getRequestDataTransfer() == 0 ) && ( TheDuplicator::instance()->isLocalProviderConnected() == 1 ) )  {
			//ACE_DEBUG((LM_DEBUG, "[ DUP_Local_Provider::handle_input ]  sync = %d start = %d\n \n ",TheDuplicator::instance()->isSynchronized(), TheDuplicator::instance()->getRequestDataTransfer()));
#if (!(defined (ACE_WIN32)))
			//Under Linux or Mac, the reactor will invoke this method until we take the data out of the socket.
			//We make the duplicator sleep between of each call, otherwise it will take all the resources of the computer.
			ACE_Time_Value waitingTime(0,1);
	        ACE_OS::sleep(waitingTime);
#endif
			return 0;
	}
	else if ( TheDuplicator::instance()->isLocalProviderConnected() == 0 ) {
		//The reactor processes the events sequentially, meaning if a data is sent
		//to the socket, handle_input() is invoked. If the data is not taken out of the
		//handler, handle_input() is invoked again, even if the close signal is sent to the
		//handler in the meantime. The close signal should be preemptive but it is not.
		//We have to handle this case.
		return -1;
	}
	else {
		int nbytes;
		local_block_t buf;
		memset(&buf, 0, sizeof(buf));

		nbytes = this->peer().recv_n(&buf, sizeof(buf));
		
	// the provider has shutdown or an error occurred
#if (!(defined (ACE_WIN32)))
		if (nbytes == 0) {
			ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Local_Provider::handle_input ] ERRORa: the provider has shutdown or an error occurred\n")));
			return -1;

		}
		else if (nbytes == -1) {
			ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Local_Provider::handle_input ] ERRORb: an error occurred\n")));
			return -1;
		}
#else
		if (nbytes == 0 || nbytes == -1) {
			ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Local_Provider::handle_input ] ERROR: the provider has shutdown or an error occurred\n")));
			return -1;
		}
#endif /* ACE_WIN32 */
		
		// notify the duplicator that the block has been written
		return TheDuplicator::instance()->block_release(this, buf.block_size);
	}
}

int DUP_Remote_Consumer::handle_input(ACE_HANDLE handle)
{
	ACE_TRACE(ACE_TEXT("DUP_Remote_Consumer::handle_input"));

	// a remote consumer should never send data to the duplicator
	// close the connection

	ACE_UNUSED_ARG(handle);

	local_block_t buf;
	memset(&buf, 0, sizeof(buf));

	this->peer().recv_n(&buf, sizeof(buf));

	// return -1 so that the handle_close() hook method gets called
	return -1;
}

int DUP_Remote_Provider::handle_input(ACE_HANDLE handle)
{
	ACE_TRACE(ACE_TEXT("DUP_Remote_Provider::handle_input"));
	
	ACE_UNUSED_ARG(handle);

	if (next_block_number == -1)
	{
		/* overflow !! */
	        //data cannot be consumed, so they stay on the socket 
	        //and this methods is called again and again as fast as possible until data can be retrieved
                //So we suspend the handler until we can get the data
		reactor()->suspend_handler(this);
		overflow = true;
		return 0;
	}
	return recv_data();
}


int DUP_Remote_Provider::recv_data()
{
	ACE_TRACE(ACE_TEXT("DUP_Remote_Provider::recv_data"));

	int nbytes;
	char *membase;
	unsigned int current_pos = 0;

	local_block_t buf;
	memset(&buf, 0, sizeof(buf));

	membase = TheDuplicator::instance()->mem_base(next_block_number);

#if (!(defined (ACE_WIN32)))
	nbytes = this->peer().recv_n(membase, sizeof(ACE_UINT32));
#else
	nbytes = this->peer().recv_n(membase, sizeof(ACE_UINT32));
#endif

	// check if no bytes were received or an error occurred
	if (nbytes <= 0)
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Remote_Provider::recv_data() ] Error receiving the block size from the provider\n")));
		// if so, return -1 so that the handle_close() hook method gets called
		return -1;
	}			

	buf.block_size = ntohl(*((ACE_UINT32 *) membase));
	current_pos = nbytes;

	while (current_pos < (buf.block_size + sizeof(ACE_UINT32)))
	{ 
#if (!(defined (ACE_WIN32)))
		nbytes = this->peer().recv_n(membase + current_pos, buf.block_size + sizeof(ACE_UINT32) - current_pos);
#else
		nbytes = this->peer().recv_n(membase + current_pos, buf.block_size + sizeof(ACE_UINT32) - current_pos);
#endif

		// check if no bytes were received or an error occurred
		if (nbytes <= 0)
		{
			ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Remote_Provider::recv_data() ] Error receiving the data block from the provider\n")));
			// if so, return -1 so that the handle_close() hook method gets called
			return -1;
		}

		current_pos += nbytes;
	}
	next_block_number = -1;

	// notify the duplicator that the block has been written
	int theResult = TheDuplicator::instance()->block_release(this, buf.block_size);
	
	if ( overflow ) {
        	reactor()->resume_handler(this);
        }
        overflow = false;
	
	return theResult;
}


ACE_TString DUP_Server::receive_xml()
{
	ACE_TRACE(ACE_TEXT("DUP_Server::receive_xml"));

	ACE_TString doc;
	char size_[512] = {0};
	
	//first we get the message size
	if ( this->peer().recv_n(size_,sizeof (size_) ) != sizeof (size_) )
	{    
		//what should we do in case of communication error?
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Server::receive_xml() ] ERROR: no bytes were received or an error occurred\n")));
		doc = ACE_TEXT("");
		return doc;
	}

	//we check the message size is correct
	ACE_TString size = ACE_TEXT_CHAR_TO_TCHAR(size_);
	int length = ACE_OS::atoi(size.c_str()); 
	if (length == 0)
	{  	  
		doc = ACE_TEXT("");
		return doc;
	}

	//we allocate a buffer to contain the message
	char *xml = new char[length+1];
	ACE_OS::memset(xml,0,length + 1);

	//then we get the message itself
	if ( this->peer().recv_n(xml, length) != length )
	{
		//what should we do in case of communication error?
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Server::receive_xml() ] ERROR: no bytes were received or an error occurred\n")));
		doc = ACE_TEXT("");
		return doc;
	}
	
	doc = ACE_TEXT_CHAR_TO_TCHAR(xml); // copy of the data into the ACE_TString object

	delete xml;
	return doc;
}

int DUP_Server::handle_input(ACE_HANDLE handle)
{
	ACE_TRACE(ACE_TEXT("DUP_Server::handle_input"));

	ACE_UNUSED_ARG(handle);

	ACE_TString doc = receive_xml();

	if (doc != ACE_TEXT(""))
	{
		if (process_message(doc, handle) != 0)
			return -1;
		else
			return 0;
	}
	else
		return -1;
}

/**************************************************************/
/**************************************************************/
int DUP_Local_Consumer::buffer_action(unsigned char block_number_, unsigned int block_size_)
{
	ACE_TRACE(ACE_TEXT("DUP_Local_Consumer::buffer_action"));

	int nbytes;

	local_block_t buf;
	memset(&buf, 0, sizeof(buf));

	buf.block_number = block_number_;
	buf.block_size = block_size_;

	// send the number and size of the block to the local consumer
	nbytes = this->peer().send_n(&buf, sizeof(buf));

	// check if no bytes were sent or an error occurred
	if (nbytes <= 0)
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Local_Consumer::buffer_action ] ERROR: No bytes were sent to local consumer %x or an error occurred\n"), &this->peer()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DUP_Local_Consumer::buffer_action ] ERROR: No bytes were sent to local consumer %x or an error occurred\n"), &this->peer()), -1);
	}

	return 0;
}

int DUP_Remote_Consumer::buffer_action(unsigned char block_number_, unsigned int block_size_)
{
	ACE_TRACE(ACE_TEXT("DUP_Remote_Consumer::buffer_action"));

	int nbytes;
	int result;
	char *membase;

	membase = TheDuplicator::instance()->mem_base(block_number_);

	// host order to network long
	// *((unsigned long *) (TheDuplicator::instance()->mem_base(block_number_))) = htonl(block_size_);


	*((ACE_UINT32 *) membase) = htonl(block_size_);

	// send the block to remote consumer
	//nbytes = this->peer().send(TheDuplicator::instance()->mem_base(block_number_), block_size_ + sizeof(long));

#if (!(defined (ACE_WIN32)))
	nbytes = this->peer().send_n(membase, block_size_ + sizeof(ACE_UINT32));
#else
	nbytes = this->peer().send_n(membase, block_size_ + sizeof(ACE_UINT32));
#endif

	// check if no bytes were sent or an error occurred
	if (nbytes <= 0)
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Remote_Consumer::buffer_action ] ERROR: No bytes were sent to remote consumer %x or an error occurred\n"), &this->peer()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DUP_Remote_Consumer::buffer_action ] ERROR: No bytes were sent to remote consumer %x or an error occurred\n"), &this->peer()), -1);
	}

	// notify the duplicator that the block has been read
	result = TheDuplicator::instance()->block_release(this);

	if (result == -1)
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Remote_Consumer::buffer_action ] ERROR: Error while releasing the block\n")));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DUP_Remote_Consumer::buffer_action ] ERROR: Error while releasing the block\n")), -1);
	}

	return 0;
}

int DUP_Local_Provider::buffer_action(unsigned char block_number_, unsigned int block_size_)
{
	ACE_TRACE(ACE_TEXT("DUP_Local_Provider::buffer_action"));
	int nbytes;
	local_block_t buf;
	memset(&buf, 0, sizeof(buf));

	buf.block_number = block_number_;
	buf.block_size = block_size_;

	// send the number and size of the block to the local provider
	nbytes = this->peer().send_n(&buf, sizeof(buf));

	// check if no bytes were sent or an error occurred
	if (nbytes <= 0)
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Local_Provider::buffer_action ] ERROR: No bytes were sent to local provider %x or an error occurred\n"), &this->peer()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DUP_Local_Provider::buffer_action ] ERROR: No bytes were sent to local provider %x or an error occurred\n"), &this->peer()), -1);
	}
	
	return 0;
}

int DUP_Remote_Provider::buffer_action(unsigned char block_number_, unsigned int block_size_)
{
	ACE_TRACE(ACE_TEXT("DUP_Remote_Provider::buffer_action"));

	ACE_UNUSED_ARG(block_size_);

	next_block_number = block_number_;

	if (overflow == true)
		return recv_data();
	else
		return 0;
}

int DUP_Server::process_message(ACE_TString xml, ACE_HANDLE handle)
{
	ACE_TRACE(ACE_TEXT("DUP_Server::process_message"));

	ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message]\n")));	
	int type = 0;
	int target = 0;

	XML_Parser parser(xml);
	ACE_TString clientID ;
	ACE_TString clientName ;
	ACE_TString clientGroup ;
	ACE_TString host_name ;

	if (parser.getInfos("type") == "")
		return -1; 
	else
		type = ACE_OS::atoi( parser.getInfos("type").c_str() );

	if (!(parser.getInfos("target") == ""))
		target = ACE_OS::atoi( parser.getInfos("target").c_str() ); 

	switch (type) 
	{
	case REQUEST :
		switch (target) 
		{
		case PAUSE:
			ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message ][PAUSE (from Local_Event_Handler) ]\n")));
			clientName = (parser.getInfos("clientname")).c_str();
			clientGroup = (parser.getInfos("clientgroup")).c_str();
			clientID = clientGroup+":"+clientName;
			TheDuplicator::instance()->pause(clientID);
			break;

		case PAUSEFLOW:
			ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message ][PAUSE (from Peer_Event_Handler)]\n")));
			clientName = (parser.getInfos("clientname")).c_str();
			clientGroup = (parser.getInfos("clientgroup")).c_str();
			clientID = clientGroup+":"+clientName;
			TheDuplicator::instance()->pause(clientID);
			break;

		case PAUSE_DUPLI:
			ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message ][PAUSE_DUPLI]\n")));
			host_name = (parser.getInfos("hostname")).c_str();
			TheDuplicator::instance()->pause_dupProvider_dupConsumer(host_name);
			break;

		case WAKE_UP_FOR_DUPLI:
			ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message ][WAKE_UP_FOR_DUPLI]\n")));
			host_name = (parser.getInfos("hostname")).c_str();
			TheDuplicator::instance()->wake_up_host(host_name);
			break;

		case DIE :
			ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message ][DIE] request from server, killing myself\n")));
			TheDuplicator::instance()->disconnect(this);
     		ACE_HANDLER_t::handle_close(handle, ACE_Event_Handler::ALL_EVENTS_MASK);

			// As long as sf2d hasn't called wait(), the system needs to keep the dead child (duplicator)
			// in the global process list, as "defunct"
			ACE_OS::exit();
			break;

		case RESTART :
			ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message ][RESTART (from Local_Event_Handler)]\n")));
			clientName = (parser.getInfos("clientname")).c_str();
			clientGroup = (parser.getInfos("clientgroup")).c_str();
			clientID = clientGroup+":"+clientName;
			TheDuplicator::instance()->restart(clientID);
			break;

		case RESTARTFLOW :
			ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message ][RESTART (from Peer_Event_Handler)]\n")));
			clientName = (parser.getInfos("clientname")).c_str();
			clientGroup = (parser.getInfos("clientgroup")).c_str();
			clientID = clientGroup+":"+clientName;
			TheDuplicator::instance()->restart(clientID);
			break;

		case START_DATA_TRANSFER :
			ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message ][START_DATA_TRANSFER]\n")));
			TheDuplicator::instance()->setRequestDataTransfer(1);
			//Here we call a method to start the data transfer
			break;

		case CONNECT :
			{
				ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message ][CONNECT]\n")));
				ACE_TString distant_duplicator_address = (parser.getInfos("duplicatoraddress")).c_str();
				ACE_TString distant_duplicator_port = (parser.getInfos("duplicatorport")).c_str();
				ACE_TString distant_duplicator = distant_duplicator_address + ":" + distant_duplicator_port;
				ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message ] Request to connect to %s\n"), distant_duplicator.c_str()));
				TheDuplicator::instance()->connect_to_distant_duplicator(distant_duplicator);
				break;
			}
		case OPEN_PROVIDER_PART :
			{
				ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message ][OPEN_PROVIDER_PART]\n")));
				ACE_TString port_for_remote_consumers = (parser.getInfos("PortForRemoteConsumers")).c_str();
				TheDuplicator::instance()->open_provider_part(port_for_remote_consumers);
				break;
			}
		case CLOSE_PROVIDER_PART :
			ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ DUP_Server::process_message ][CLOSE_PROVIDER_PART]\n")));
			TheDuplicator::instance()->close_provider_part();
			break;
		default:
			ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Server::process_message ] Invalid Request: %d\n"), target));
			break;
		}
		break;
	default :
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Server::process_message ] Invalid Command \n")));
		break;
	}

	return 0; 
}

/* ************************************************************ */
/* ************************************************************ */
int DUP_Local_Consumer::open(void *p)
{
	ACE_TRACE(ACE_TEXT("DUP_Local_Consumer::open"));

	int result;

	int byte_count = 0;
	char *clientID;
	clientID = new char[1024];

	byte_count = this->peer().recv(clientID,1024,0);
	client_id = clientID;
	is_remote_consumer = false;
	is_paused = false;

	for (int i = 0; i < 2; i++) {
		blocks_read[i] = true;
	}

	// try to activate the client handler
	result = ACE_HANDLER_l::open(p);

	if (result == -1)
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Local_Consumer::open ] ERROR: could not open connection with local consumer %x\n"), p));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DUP_Local_Consumer::open ] ERROR: could not open connection with local consumer %x\n"), p), -1);
	}
	else
	{
		ACE_DEBUG((ANNOUNCE_PREFIX ACE_TEXT("[ DUP_Local_Consumer::open ] Local consumer %x connected\n"), p));
	}
	//here we notify the server that a new consumer is connected
	ACE_TString IDClient = clientID;
	ACE_TString IDFlow = TheDuplicator::instance()->getFlowID();
	notifyServerAClientConnected(IDClient, IDFlow);

	return TheDuplicator::instance()->connect(this);
}

int DUP_Local_Provider::open(void *p)
{
	ACE_TRACE(ACE_TEXT("DUP_Local_Provider::open"));

	int result;
	
	//we receive first the synchronization information from the provider
	int byte_count = 0;
	char *isSync;
	isSync = new char[10];

	byte_count = this->peer().recv(isSync,10,0);
	ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ Dup_Local_Provider::open ] received isSync  \"%s\" : %d \n"), isSync, byte_count));
	
	//We need to setup the value isSynchronized of the Duplicator object.

	TheDuplicator::instance()->setSynchronized(ACE_OS::atoi(isSync));
	TheDuplicator::instance()->setLocalProviderConnected(1);
	
	if ( TheDuplicator::instance()->isSynchronized() == 1 )
		//it may need a test to see if it successful
		TheDuplicator::instance()->sendToServerTheNumberOfRemoteDuplicatorsConnected();

	// try to activate the client handler
	result = ACE_HANDLER_l::open(p);

	if (result == -1)
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Local_Provider::open ] ERROR: could not open connection with local provider %x\n"), p));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DUP_Local_Provider::open ] ERROR: could not open connection with local provider %x\n"), p), -1);
	}
	else
	{
		ACE_DEBUG((ANNOUNCE_PREFIX ACE_TEXT("[ DUP_Local_Provider::open ] Local provider %x connected\n"), p));
	}

	return TheDuplicator::instance()->connect(this);
}

int DUP_Remote_Consumer::open(void *p)
{
	ACE_TRACE(ACE_TEXT("DUP_Remote_Consumer::open"));

	int result;

	is_remote_consumer = true;
	is_paused = false;

	for (int i = 0; i < 2; i++)
		blocks_read[i] = true;

	ACE_INET_Addr remote_addr_;

	this->peer().get_remote_addr(remote_addr_);

	host = remote_addr_.get_host_addr();

	// try to activate the client handler

	result = ACE_HANDLER_t::open(p);


	if (result == -1)
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Remote_Consumer::open ] ERROR: could not open connection with remote consumer %x\n"), p));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DUP_Remote_Consumer::open ] ERROR: could not open connection with remote consumer %x\n"), p), -1);
	}
	else
	{
		ACE_DEBUG((ANNOUNCE_PREFIX ACE_TEXT("[ DUP_Remote_Consumer::open ] Remote consumer %x connected\n"), p));
	}
	
	//we have a new duplicator connected, we increase the total of remote consumers connected
	TheDuplicator::instance()->increaseNumberOfRemoteConsumerConnected();

	//then we send to the server the number of dup connected (# of instances of DUP_Remote_Consumer).
	if ( TheDuplicator::instance()->isSynchronized() == 1 )
		TheDuplicator::instance()->sendToServerTheNumberOfRemoteDuplicatorsConnected();

	return TheDuplicator::instance()->connect(this);
}

int DUP_Remote_Provider::open(void *p)
{
	ACE_TRACE(ACE_TEXT("DUP_Remote_Provider::open"));

	int result;
    overflow = false;
	
	// try to activate the client handler
	result = ACE_HANDLER_t::open(p);

	if (result == -1)
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Remote_Provider::open ] ERROR: could not open connection with remote provider %x\n"), p));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DUP_Remote_Provider::open ] ERROR: could not open connection with remote provider %x\n"), p), -1);
	}
	else
	{
		ACE_DEBUG((ANNOUNCE_PREFIX ACE_TEXT("[ DUP_Remote_Provider::open ] Remote provider %x connected\n"), p));
	}

	return TheDuplicator::instance()->connect(this);
}



int DUP_Server::open(void *s)
{
	ACE_TRACE(ACE_TEXT("DUP_Server::open"));

	int result;

	// try to activate the client handler
	result = ACE_HANDLER_t::open(s);

	if (result == -1)
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ DUP_Server::open ] ERROR: could not open connection with the server %x\n"), s));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DUP_Server::open ] ERROR: could not open connection with the server %x\n"), s), -1);
	}
	else
	{
		ACE_DEBUG((ANNOUNCE_PREFIX ACE_TEXT("[ DUP_Server::open ] Server '%x' connected\n"), s));
	}

	return TheDuplicator::instance()->connect(this);
}

/**************************************************************/

/**************************************************************/

int DUP_Local_Consumer::handle_close(ACE_HANDLE handle, ACE_Reactor_Mask close_mask)
{
	ACE_TRACE(ACE_TEXT("DUP_Local_Consumer::handle_close"));

	TheDuplicator::instance()->disconnect(this);
	return ACE_HANDLER_l::handle_close(handle, close_mask);
}

int DUP_Local_Provider::handle_close(ACE_HANDLE handle, ACE_Reactor_Mask close_mask)
{	
	ACE_TRACE(ACE_TEXT("DUP_Local_Provider::handle_close"));
	TheDuplicator::instance()->disconnect(this);
	TheDuplicator::instance()->setSynchronized(0);
	TheDuplicator::instance()->setRequestDataTransfer(0);
	return ACE_HANDLER_l::handle_close(handle, close_mask);
}

int DUP_Remote_Consumer::handle_close(ACE_HANDLE handle, ACE_Reactor_Mask close_mask)
{
	ACE_TRACE(ACE_TEXT("DUP_Remote_Consumer::handle_close"));

	TheDuplicator::instance()->decreaseNumberOfRemoteConsumerConnected();
	TheDuplicator::instance()->disconnect(this);
	return ACE_HANDLER_t::handle_close(handle, close_mask);
}

int DUP_Remote_Provider::handle_close(ACE_HANDLE handle, ACE_Reactor_Mask close_mask)
{
	ACE_TRACE(ACE_TEXT("DUP_Remote_Provider::handle_close"));

	TheDuplicator::instance()->disconnect(this);
	return ACE_HANDLER_t::handle_close(handle, close_mask);
}

int DUP_Server::handle_close(ACE_HANDLE handle, ACE_Reactor_Mask close_mask)
{
	ACE_TRACE(ACE_TEXT("DUP_Server::handle_close"));

	TheDuplicator::instance()->disconnect(this);
	ACE_HANDLER_t::handle_close(handle, close_mask);
	
	// As long as sf2d hasn't called wait(), the system needs to keep the dead child (duplicator)
	// in the global process list, as "defunct"
	ACE_OS::exit();
	return 0;
}

int DUP_Local_Consumer::notifyServerAClientConnected(ACE_TString clientID_, ACE_TString flowID_) {
	ACE_TRACE(ACE_TEXT("DUP_Local_Consumer::notifyServerAClientConnected"));

	//first we create the message
	size_t size_needed = sizeof(CONSUMER_CONNECTED_MESSAGE) + flowID_.length() + clientID_.length() + 1; 
	ACE_TCHAR *theMessage = (ACE_TCHAR *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(theMessage, 0, size_needed);
	
	//char theMessage[MAX_MESSAGE_SIZE]={0};
	ACE_OS::sprintf(theMessage, CONSUMER_CONNECTED_MESSAGE, flowID_.c_str(), clientID_.c_str());

	//then we send it
	int result = TheDuplicator::instance()->sendXMLMessageToServer(theMessage);
	
	ACE_OS::free(theMessage);
	
	return result;
}

DUP_Log::DUP_Log(){
	ACE_TRACE(ACE_TEXT("DUP_Log::DUP_Log"));
	ACE_Reactor::instance()->register_handler(this, ACE_Event_Handler::READ_MASK);
	this->reactor(ACE_Reactor::instance());
}

int DUP_Log::handle_input(ACE_HANDLE handle_) {
	//ACE_TRACE(ACE_TEXT("DUP_Log::handle_input"));
	//ACE_DEBUG((ANNOUNCE_PREFIX ACE_TEXT("[ DUP_Log::handle_input ]\n")));
	//here we dequeue as many message as possible to create a message to send to the server.
	//the log messages needs to be formatted in XML first
	ACE_UNUSED_ARG(handle_);

	ACE_TString messages;
	int result = 0;
	int stop = 0;
	unsigned int nbMessages = 0;
	ACE_Message_Block *mb;
	ACE_TString logMess = LOG_MESSAGE;
	size_t lengthMessage = logMess.length();
	lengthMessage += TheDuplicator::instance()->getFlowID().length();

	char namelocal[256] = "";
	ACE_OS::hostname(namelocal,256);
	std::string serverName;
	serverName = namelocal;

	lengthMessage += serverName.length();
	
	ACE_Message_Queue<ACE_SYNCH> *messageQueue = TheDuplicator::instance()->getLog_Manager()->getQueue();
	if ( messageQueue ) {
		nbMessages = messageQueue->message_count();
		while ( (nbMessages != 0) && !stop ) {
			//ACE_DEBUG((ANNOUNCE_PREFIX ACE_TEXT("[ DUP_Log::handle_input ] size = %d \n"), messageQueue->message_count()));
			//we dequeue the message...
			ACE_Time_Value timeout(ACE_OS::gettimeofday());
			result = messageQueue->dequeue_head(mb, &timeout);
			if ( result == -1 )
				stop = 1;
			else {
				//get its size
				int lengthMB = mb->length();

				if ( (lengthMessage + lengthMB + 1)  > MAX_MESSAGE_SIZE ) {
					stop = 1;
					//we put back the message in the queue
					ACE_Time_Value timeout(ACE_OS::gettimeofday());
					result = messageQueue->enqueue_head(mb, &timeout);
				}
				else {
					//set up its read pointer on the beginning of the message
					mb->rd_ptr(mb->base());
					//Allocate the buffer to copy the message
					ACE_TCHAR *dest = (ACE_TCHAR *) ACE_OS::malloc(lengthMB+1);
					//copy the message into dest
					ACE_OS::memcpy(dest, mb->rd_ptr(), lengthMB);
	
					ACE_TString messTString = dest;
					ACE_TString messInXML = create_XML_message(messTString);
					messages += messInXML;
					
					//we update the size
					lengthMessage += lengthMB;
					
					nbMessages--;

					//we don't need the message block anymore, we delete it
					delete mb;
					mb = NULL;
					//we free the dest variable
					ACE_OS::free(dest);
				}
			}
		}
		if ( messages.length() > 0 ) {

			//we have at least one log message int mess.
			size_t size_needed = sizeof(LOG_MESSAGE) + TheDuplicator::instance()->getFlowID().length() + serverName.length() + messages.length() + 1; 
			ACE_TCHAR *theMessage = (ACE_TCHAR *) ACE_OS::malloc(size_needed);	
			ACE_OS::memset(theMessage, 0, size_needed);
			
			//char theMessage[MAX_MESSAGE_SIZE]={0};
			ACE_OS::sprintf(theMessage,LOG_MESSAGE, TheDuplicator::instance()->getFlowID().c_str(), serverName.c_str(), messages.c_str());
			//now we send it
			//the method putting the id of the flow is in the Duplicator class
			TheDuplicator::instance()->sendXMLMessageToServer(theMessage);
			//TheDuplicator::instance()->sendLogToServer(theMessage);
			
			ACE_OS::free(theMessage);
		}
		if ( nbMessages != 0 ) {
			//we didn't take care of everymessage, we will finish the job later
			//because we don't want to let the reactor blocked for too long..
			//so we notify the reactor to call again this method later
			ACE_Reactor::instance()->notify(TheDuplicator::instance()->getLog_Manager()->getLogHandler(), ACE_Event_Handler::READ_MASK);
		}
		else {
			//here we notify the Log_Callback object that we received the notification
			//so it can notify the reactor to invoke this method later
			TheDuplicator::instance()->getLog_Manager()->setNotificationSent(0);
		}
	}
	return 0;
}

int DUP_Log::handle_timeout(const ACE_Time_Value &currentTime, const void *) {
	ACE_UNUSED_ARG(currentTime);
	ACE_Message_Queue<ACE_SYNCH> *messageQueue = TheDuplicator::instance()->getLog_Manager()->getQueue();
	if ( messageQueue ) {
		unsigned int nbMessages = messageQueue->message_count();
		if ( nbMessages != 0 && TheDuplicator::instance()->getLog_Manager()->isNotificationSent() == 0 ) {
			reactor()->notify(TheDuplicator::instance()->getLog_Manager()->getLogHandler(), ACE_Event_Handler::READ_MASK);
			TheDuplicator::instance()->getLog_Manager()->setNotificationSent(1);
		}
	}
	return 0;
}

void DUP_Log::enableTimer() {
	ACE_Time_Value delay(1);
	ACE_Time_Value interval(1);
	reactor()->schedule_timer(this, 0, delay, interval);
}

ACE_TString DUP_Log::create_XML_message(ACE_TString message_){
	ACE_TRACE(ACE_TEXT("DUP_Log::create_XML_message"));
	ACE_TString doc ;
	doc += ACE_TEXT("<message>");
	doc += ACE_TEXT("<![CDATA[");
	doc += message_;
	doc += ACE_TEXT("]]>");
	doc += ACE_TEXT("</message>\n");
	
	return doc;
}

/*DUP_Signal::DUP_Signal(int signum) {
	ACE_TRACE(ACE_TEXT("DUP_Signal::DUP_Signal"));
	ACE_DEBUG((LM_DEBUG, ACE_TEXT("\n \n \n \n [DUP_Signal::DUP_Signal] \n \n \n \n")));
	ACE_Reactor::instance()->register_handler(signum, this);
}

int DUP_Signal::handle_signal(int signum, siginfo_t *, ucontext_t *) {
	//we don't want to do the job in this method.
	//it is safest to do as few things as possible here because
	//because we are in the signal state
	ACE_DEBUG((LM_DEBUG, ACE_TEXT("\n \n \n \n [DUP_Signal::handle_signal] Signal properly catched \n \n \n \n")));

	//notify is invoked with the default parameter EXCEPT_MASK
	//that call tells the reactor to invoke the handle_exception callback,
	//which won't be in the signal state but in the normal execution flow
	ACE_Reactor::instance()->notify(this);
	return 0;
}

int DUP_Signal::handle_exception(ACE_HANDLE fd) {
	ACE_TRACE(ACE_TEXT("DUP_Signal::handle_exception"));
	//in this method we are in the normal execution flow,
	//not in the signal state anymore
	//here we send the log to the server
	ACE_DEBUG((LM_DEBUG, ACE_TEXT("\n \n \n \n [DUP_Signal::handle_exception] Signal properly catched \n \n \n \n")));

	return 0;
}*/

