#ifndef HANDLERS_H
#define HANDLERS_H

#include "ace/ACE.h"
#include "ace/SString.h"
#include "ace/Svc_Handler.h"
#include "ace/SOCK_Stream.h"

#if (!(defined (ACE_WIN32)))
	#include "ace/LSOCK_Stream.h"
#endif /* ACE_WIN32 */
//#include "ace/MEM_Stream.h"
#include "duplicator_common_ACE.h"

// use the ACE_SOCK_STREAM macro to use this code on systems with and
// without template traits type support
typedef ACE_Svc_Handler<ACE_SOCK_STREAM, ACE_NULL_SYNCH> ACE_HANDLER_t;

#if (defined (ACE_WIN32))
	typedef ACE_Svc_Handler<ACE_SOCK_STREAM, ACE_NULL_SYNCH> ACE_HANDLER_l;
#else

	typedef ACE_Svc_Handler<ACE_LSOCK_STREAM, ACE_NULL_SYNCH> ACE_HANDLER_l;

#endif /* ACE_WIN32 */

/** Common DuplicatorHandlers - specific variables.
\internal
    Not in use at the moment (Only 1 shared memory buffer is in use)
*/
class DUP_Handler
{
	long block_number;
	public:
		void set_block_number(long block_number_) {block_number = block_number_;}
		long get_block_number(void) {return block_number;}
		DUP_Handler() :block_number(-1) { }
} ;

class DUP_Log : public ACE_Event_Handler {
  public:
	  DUP_Log();
	  virtual int handle_input(ACE_HANDLE handle_ = ACE_INVALID_HANDLE);
	  virtual int handle_timeout(const ACE_Time_Value &currentTime, const void * = 0 );
	  void enableTimer();
  private:
	  ACE_TString create_XML_message(ACE_TString message_);
};

/** Handler for server.
*/
class DUP_Server : public ACE_HANDLER_t
{ 
	public:
	    virtual int open(void *p_);
		virtual int handle_close(ACE_HANDLE handle_, ACE_Reactor_Mask close_mask_);
		virtual int handle_input(ACE_HANDLE handle_ = ACE_INVALID_HANDLE);
		virtual int process_message(ACE_TString xml, ACE_HANDLE handle);
		ACE_TString receive_xml();
};


/** Common class for consumers.
\internal
*/
class DUP_Consumer : public DUP_Handler
{
 public:
  char* client_id;
  bool is_remote_consumer;
  bool is_paused;
  bool blocks_read[2];
  ACE_TString host;
  virtual int buffer_action(unsigned char block_number_, unsigned int block_size_) = 0;
  // return true if the block_number has been read by the consumer
  // return false otherwise
  bool has_read_block(int block_number_);
};


/** Common class for providers.
\internal
*/
class DUP_Provider : public DUP_Handler
{
	public:
		virtual int buffer_action(unsigned char block_number_, unsigned int block_size_ = 0) = 0;
};


/** Handler for local consumer.
	\internal
    This class is the Event Handler for a local consumer, data direction
    is DUP -> LocalClient. The local client is supposed to read the buffer
    number and the size of the buffer, and return the same information once
    the block has been consumed.
*/
class DUP_Local_Consumer : public ACE_HANDLER_l, public DUP_Consumer
{ 
  //int nb;
 public:
  virtual int open(void *p_);
  virtual int handle_close(ACE_HANDLE handle_, ACE_Reactor_Mask close_mask_);
  virtual int handle_input(ACE_HANDLE handle_ = ACE_INVALID_HANDLE);
  virtual int buffer_action(unsigned char block_number_, unsigned int block_size_);

  virtual int notifyServerAClientConnected(ACE_TString clientID_, ACE_TString flowID_);
};

/** Handler for local provider.
	\internal
    This class is the Event Handler for a local provider, data direction
    is LocalProvider -> DUP. The local client is supposed to wait for a buffer number
    from the duplicator, fill it and return the buffer number and the actual size of 
	the block filled in it.
*/

class DUP_Local_Provider : public ACE_HANDLER_l, public DUP_Provider
{ 
 public:
  virtual int open(void *p_);
  virtual int handle_close(ACE_HANDLE handle_, ACE_Reactor_Mask close_mask_);
  virtual int handle_input(ACE_HANDLE handle_ = ACE_INVALID_HANDLE);
  virtual int buffer_action(unsigned char block_number_, unsigned int block_size_ = 0);
};

/** Handler for remote consumer.
	\internal
    This class is the Event Handler for a remote consumer, data direction
    is DUP -> RemoteClient. The remote client *opens* the connection. Then
    all buffers are pushed to it.
	the remote client should just read the data and send nothing.
*/
class DUP_Remote_Consumer : public ACE_HANDLER_t, public DUP_Consumer
{
 public:
  virtual int open(void *p_);
  virtual int handle_close(ACE_HANDLE handle_, ACE_Reactor_Mask close_mask_);
  virtual int handle_input(ACE_HANDLE handle_ = ACE_INVALID_HANDLE);
  virtual int buffer_action(unsigned char block_number_, unsigned int block_size_);
};

/** Handler for remote provider.
	\internal
    This class is the Event Handler for a remote provider, data direction
    is RemoteClient -> DUP. The connection is opened by the duplicator.
    Once the connection is opened, the data is read as often as possible.
    This class "holds" a block number for the read, and releases it after 
    each buffer. No data should be sent to the other end.
*/

class DUP_Remote_Provider : public ACE_HANDLER_t, public DUP_Provider
{
		bool overflow;
		int next_block_number;
	public:
		virtual int open(void *p_);
		virtual int handle_close(ACE_HANDLE handle_, ACE_Reactor_Mask close_mask_);
		virtual int handle_input(ACE_HANDLE handle_ = ACE_INVALID_HANDLE);
		virtual int buffer_action(unsigned char block_number_, unsigned int block_size_ = 0);
		virtual int recv_data();
};

#endif


