#include "ace/OS.h"

#include "Acceptor_Event_Handler.h"
#include "Peer_Event_Handler.h"
#include "Log_Manager.h"

#include "list/LastModifications.h"
#include "list/SocketList.h"
#include "list/DuplicatorList.h"
#include "list/Client.h"
#include "list/GlobalList.h"


int Acceptor_Event_Handler::open(const ACE_INET_Addr &local_addr)
{
  ACE_TRACE(ACE_TEXT("Acceptor_Event_Handler::open"));
  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Acceptor_Event_Handler::open ]\n")));
  if (acceptor_.open(local_addr,1) == -1) 
    return -1 ;  
  return reactor()->register_handler(this,ACE_Event_Handler::ACCEPT_MASK);
}

int Acceptor_Event_Handler::handle_close(ACE_HANDLE,ACE_Reactor_Mask)
{
  ACE_TRACE(ACE_TEXT("Acceptor_Event_Handler::handle_close"));
  acceptor_.close();
  delete this;
  return 0;
}

int Peer_Acceptor_Event_Handler::handle_input(ACE_HANDLE)
{
  ACE_TRACE(ACE_TEXT("Acceptor_Event_Handler::handle_input"));
  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Peer_Acceptor_Event_Handler::handle_input ]\n")));
  Peer_Event_Handler *peer_handler =0;
  ACE_INET_Addr peer_addr;
  ACE_NEW_RETURN(peer_handler,Peer_Event_Handler(reactor()),-1);
  if (acceptor_.accept(peer_handler->peer(), &peer_addr) == -1) {
    delete peer_handler;
    return -1;
      }
    else if (peer_handler->open() == -1){
       peer_handler->handle_close();
      return -1;
    }  
    
    /*SocketList *sl = SOCKET_LIST::instance();
	sl->addServer( peer_addr.get_host_name() ,&peer_handler->peer()); 
    //GlobalList *gl = GLOBAL_LIST::instance();
	LastModifications *lm = LAST_MODIFICATIONS::instance();
	ACE_TString modif ;
	modif = add_server_message(peer_addr.get_host_name()).c_str();
	lm->addModifications(modif) ;	*/
	return 0;
}

int Local_Acceptor_Event_Handler::handle_input(ACE_HANDLE){
   
  ACE_TRACE(ACE_TEXT("Local_Acceptor_Event_Handler::handle_input"));
  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Acceptor_Event_Handler::handle_input ]\n"))); 
  Local_Event_Handler *peer_handler =0;
  ACE_NEW_RETURN(peer_handler,Local_Event_Handler(reactor()),-1);
  if (acceptor_.accept(peer_handler->peer()) == -1) {
    delete peer_handler;
    return -1;
      }
    else if (peer_handler->open() == -1){
       peer_handler->handle_close();
      return -1;
    }  
    
    
    return 0;
}

int Local2_Acceptor_Event_Handler::handle_input(ACE_HANDLE)
{
	ACE_TRACE(ACE_TEXT("Local2_Acceptor_Event_Handler::handle_input"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local2_Acceptor_Event_Handler::handle_input ]\n")));   
		
	int result;
	ACE_TCHAR clientname[512] = {0} ;
	ACE_TCHAR clientgroup[512] = {0} ;
	ACE_SOCK_Stream* client_socket = new ACE_SOCK_Stream() ;
	
	result = acceptor_.accept(*client_socket);

	//we now have a method to access the handler, we also need a cleanup method in the handler...
	//track the removeClient/removeSocket code.
	//BIG PB: if it fails here, we don't have any way to know which client is not fully connected.
	//This should be detected by the client...
	if (result == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local2_Acceptor_Event_Handler::handle_input ] ERROR: 'accept' failed\n")));
		
		return 0;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ Local2_Acceptor_Event_Handler::handle_input ] ERROR: 'accept' failed\n")), -1);
	}

	result = client_socket->recv_n(clientname, 512 * sizeof(ACE_TCHAR));

	// check if no bytes were received or an error occurred
	if (result <= 0)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local2_Acceptor_Event_Handler::handle_input ] ERROR Name : no bytes were received or an error occurred\n")));
		return 0;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ Local2_Acceptor_Event_Handler::handle_input ] ERROR Name : no bytes were received or an error occurred\n")), -1);
	}

	result = client_socket->recv_n(clientgroup, 512 * sizeof(ACE_TCHAR));

	// check if no bytes were received or an error occurred
	if (result <= 0)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local2_Acceptor_Event_Handler::handle_input ] ERROR Group: no bytes were received or an error occurred\n")));
		return 0;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ Local2_Acceptor_Event_Handler::handle_input ] ERROR Group: no bytes were received or an error occurred\n")), -1);
	}

	ACE_TString clientID = Client::getIDFromGroupAndName( clientgroup , clientname );  

	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local2_Acceptor_Event_Handler::handle_input ] Server connexion successful for client '%s' on socket '%x' \n"), clientID.c_str(), &client_socket ));
		
	SocketList *sl = SOCKET_LIST::instance();
	sl->addClient(clientID, client_socket);
	
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local2_Acceptor_Event_Handler::handle_input ] END \n"))); 
	return 0;
}

int Duplicator_Acceptor_Event_Handler::handle_input(ACE_HANDLE)
{
	//new version
	ACE_TRACE(ACE_TEXT("Duplicator_Acceptor_Event_Handler::handle_input"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Acceptor_Event_Handler::handle_input ]\n")));
	Duplicator_Event_Handler *peer_handler =0;
	ACE_NEW_RETURN(peer_handler, Duplicator_Event_Handler(reactor()),-1);
	if (acceptor_.accept(peer_handler->peer()) == -1) {
		delete peer_handler;
		return -1;
    }
    else if (peer_handler->open() == -1){
		peer_handler->handle_close();
		return -1;
	}
	return 0;
}
