#include "DuplicatorManager.h"
#include "Peer_Event_Handler.h"
#include "Log_Manager.h"

DuplicatorManager::DuplicatorManager(){
	ACE_TRACE(ACE_TEXT("DuplicatorManager::DuplicatorManager"));
	buffer_length = 512;
	buffer_size = buffer_length * sizeof(ACE_TCHAR);

}

DuplicatorManager::~DuplicatorManager() {}

int DuplicatorManager::send_message(ACE_SOCK_Stream *dupli, const ACE_TCHAR *message) {

	ACE_TRACE(ACE_TEXT("DuplicatorManager::send_message"));
	if ( !dupli ) {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_... ] ERROR: dupli==NULL\n")));
		return -1;	
		//ACE_ERROR_RETURN((LM_DEBUG, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_... ] ERROR: dupli==NULL\n")), -1);
	}

	ACE_NEW_RETURN(buffer, ACE_TCHAR[buffer_length], -1);
	int result;

	// put the message into the buffer
	result = ACE_OS::snprintf(buffer, buffer_length, message);
	if (result >= buffer_length)
	{
		delete buffer;
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_... ] ERROR: snprintf truncated the message (length > %d)\n"), buffer_length));
		return -1;	
		//ACE_ERROR_RETURN((LM_DEBUG, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_... ] ERROR: snprintf truncated the message (length > %d)\n"), buffer_length), -1);
	}
	else if (result < 0)
	{
		delete buffer;
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_... ] ERROR: snprintf output error\n")));
		return -1;	
		//ACE_ERROR_RETURN((LM_DEBUG, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_... ] ERROR: snprintf output error\n")), -1);
	}

	// send the message
	result = dupli->send_n(buffer, buffer_size);
	delete buffer;
	return result;

}

int DuplicatorManager::get_buffer_size() {
	ACE_TRACE(ACE_TEXT("DuplicatorManager::get_buffer_size"));
	return buffer_size;
}


int DuplicatorManager::notify_duplicators_to_die_local(ACE_TString clientID_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorManager::notify_duplicators_to_die_local"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] \n")));
	Flow *f;
	//ACE_SOCK_Stream *dupli = NULL;
	Duplicator_Event_Handler *dupli = NULL;
	GlobalList *gl = GLOBAL_LIST::instance();  
	DuplicatorList *dl = DUPLICATOR_LIST::instance();

	//int buffer_size = get_buffer_size();


	ACE_TCHAR namelocal[256];
	if (ACE_OS::hostname(namelocal,256) ==1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ACE_OS::hostname \n")));
		//ACE_TString createxmlfailure = create_xmlfailure("DuplicatorManager::notify_duplicators_to_die_local : ACE_OS::hostname");
		//send_xml(createxmlfailure.c_str());
		return 0;
	}	
	ACE_TString localaddr = namelocal ;

	//ACE_TCHAR dieMessage[MAX_MESSAGE_SIZE]={0};
	//ACE_OS::sprintf(dieMessage,DIE_MESSAGE);

	// for each flow this client was providing or consuming,
	// check if there are still clients providing or consuming it
	// if not, tell the duplicator to die
	for (FLOW_ITERATOR fi = gl->getFirstFlow(); fi != gl->getLastFlow(); fi++)
	{
		f = GlobalList::getFlowFromIterator(fi);

		// check if client 'clientID_' is the producer of this flow
		if (clientID_ == f->getProducer())
		{
			// check if there are still local consumers for this flow
			if (f->countConsumersOnServer( localaddr ) == 0)
			{
				dupli = dl->getDupEventHandler(f->getID());
				// no more local consumers, tell the duplicator to die
				if ( dupli == NULL )
				{
					ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: dupli==NULL\n")));
					return -1;
					//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: dupli==NULL\n")), -1);
				}
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ]\n")));
				
				ACE_TString dieMessage(DIE_MESSAGE); 
				int result = dupli->send_xml(dieMessage);
				
				//if (result < buffer_size)
				if (result == -1) {
					ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: send not successful\n")));
					return -1;
				}
				else
				{
					ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] Sent 'die' xml message to the duplicator of flow '%s'\n"), f->getID().c_str()));
					dl->remove(f->getID());
				}
			}
			else 
			{
				// There is still at least one local consumer so we just ask the duplicator to close its provider part
				// and we update the duplicator list
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] We notify the duplicator to close its provider part\n")));
				dupli = dl->getDupEventHandler(f->getID());
				if ( dupli == NULL ) {
					ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: dupli==NULL\n")));
					return -1;
					//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: dupli==NULL\n")), -1);
				}
				
				//ACE_TCHAR closeProviderPartRequestMessage[MAX_MESSAGE_SIZE]={0};
				ACE_TString closeProviderPartRequestMessage(CLOSE_PROVIDER_PART_REQUEST_MESSAGE); 
				int result = dupli->send_xml(closeProviderPartRequestMessage);
								
				if (result == -1)
					ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::notify_duplicators_to_die_local] ERROR : Can't send the message to the duplicator\n")));
				dl->remove(f->getID());
				dl->addConsumer(f->getID());
				dl->setDupEventHandler(f->getID(), dupli);
			}
		} // if
		else
		{
			// check if client 'clientID_' is a consumer of this flow
			if (f->containsConsumer(clientID_) == 0)
			{
				// check if it is the last consumer of this flow
				// and there is no producer
				if ((f->countConsumersOnServer( localaddr ) == 1) ) // && (f->getProducerStatus() == CLIENT_STATUS_UNKNOWN))
				{												
					if ( (f->getServerProducerID() != localaddr ) 
						|| ( (f->countConsumers() == 1)  && (f->getProducerStatus() == CLIENT_STATUS_UNKNOWN) )  )
					{
						dupli = dl->getDupEventHandler(f->getID());

						if ( dupli == NULL ) {
							ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: dupli==NULL\n")));
							return -1;
							//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: dupli==NULL\n")), -1);
						}
						// send the buffer to the duplicator

						ACE_TString dieMessage(DIE_MESSAGE);						
						int result = dupli->send_xml(dieMessage);
						
						//if (result < buffer_size)
						if (result == -1) {
							ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: send not successful\n")));
							return -1;
							//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: send not successful\n")), -1);
						}
						else
						{
							ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] Sent 'die' xml message to the duplicator of flow '%s'\n"), f->getID().c_str()));
							dl->remove(f->getID());
						}
					}
				}			
			} // if
		} // else
	} // for

	//delete buffer;
	return 0;
}

int  DuplicatorManager::notify_duplicator_to_die(ACE_TString flowID_, ACE_TString clientID_) {
	ACE_TRACE(ACE_TEXT("DuplicatorManager::notify_duplicator_to_die"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicator_to_die(flowID) ] \n")));
	
	Flow *f = NULL;
	int result = -1;
	//ACE_SOCK_Stream *dupli = NULL;
	Duplicator_Event_Handler *dupli = NULL;
	GlobalList *gl = GLOBAL_LIST::instance();  
	DuplicatorList *dl = DUPLICATOR_LIST::instance();
	
	//ACE_TCHAR dieMessage[MAX_MESSAGE_SIZE]={0};
	//ACE_OS::sprintf(dieMessage,DIE_MESSAGE);

	ACE_TCHAR namelocal[256];
	if (ACE_OS::hostname(namelocal,256) ==1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ACE_OS::hostname \n")));
		//ACE_TString createxmlfailure = create_xmlfailure("DuplicatorManager::notify_duplicators_to_die_local : ACE_OS::hostname");
		//send_xml(createxmlfailure.c_str());
		return 0;
	}	
	ACE_TString localaddr = namelocal ;

	if (gl->findFlow(flowID_) == 0) {
		f = gl->getFlow(flowID_);
		
		// check if client 'clientID_' is the producer of this flow
		if (clientID_ == f->getProducer())
		{
			// check if there are still local consumers for this flow
			if (f->countConsumersOnServer( localaddr ) == 0)
			{
				dupli = dl->getDupEventHandler(f->getID());
				// no more local consumers, tell the duplicator to die
				if ( dupli == NULL ) {
					ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicator_to_die_local ] ERROR: dupli==NULL\n")));
					return -1;
					//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorManager::notify_duplicator_to_die_local ] ERROR: dupli==NULL\n")), -1);
				}
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::notify_duplicator_to_die_local]\n")));
				
				ACE_TString dieMessage(DIE_MESSAGE); 				
				result = dupli->send_xml(dieMessage);
				
				if (result == -1) {
					ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: send not successful\n")));
					return -1;
					//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: send not successful\n")), -1);
				}
				else
				{
					ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] Sent 'die' xml message to the duplicator of flow '%s'\n"), f->getID().c_str()));
					dl->remove(f->getID());
				}
			}
			else 
			{
				// There is still at least one local consumer so we just ask the duplicator to close its provider part
				// and we update the duplicator list
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::notify_duplicators_to_die_local] We notify the duplicator to close its provider part\n")));
				dupli = dl->getDupEventHandler(f->getID());
				if ( dupli == NULL ) {
					ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: dupli==NULL\n")));
					return -1;
					//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: dupli==NULL\n")), -1);
				}
				if (result == -1)
					ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::notify_duplicators_to_die_local] ERROR : Can't get the duplicator socket\n")));
				
				//ACE_TCHAR closeProviderPartRequestMessage[MAX_MESSAGE_SIZE]={0};
				//ACE_OS::sprintf(closeProviderPartRequestMessage, CLOSE_PROVIDER_PART_REQUEST_MESSAGE);
				ACE_TString closeProviderPartRequestMessage(CLOSE_PROVIDER_PART_REQUEST_MESSAGE);

				//result = send_message(dupli, closeProviderPartRequestMessage);
				result = dupli->send_xml(closeProviderPartRequestMessage);
								
				if (result == -1)
					ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::notify_duplicators_to_die_local] ERROR : Can't send the message to the duplicator\n")));
				dl->remove(f->getID());
				dl->addConsumer(f->getID());
				dl->setDupEventHandler(f->getID(), dupli);
			}
		}
		else
		{
		//the client is not a producer
			// check if client 'clientID_' is a consumer of this flow
			if (f->containsConsumer(clientID_) == 0)
			{
				// check if it is the last consumer of this flow
				// and there is no producer
				if ((f->countConsumersOnServer( localaddr ) == 1) ) // && (f->getProducerStatus() == CLIENT_STATUS_UNKNOWN))
				{												
					if ( (f->getServerProducerID() != localaddr ) 
						|| ( (f->countConsumers() == 1)  && (f->getProducerStatus() == CLIENT_STATUS_UNKNOWN) )  )
					{
						dupli = dl->getDupEventHandler(f->getID());

						if ( dupli == NULL ) {
							ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: dupli==NULL\n")));
							return -1;
							//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: dupli==NULL\n")), -1);
						}
						// send the buffer to the duplicator
						ACE_TString dieMessage(DIE_MESSAGE); 
						result = dupli->send_xml(dieMessage);
					
						if (result == -1) {
							ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: send not successful\n")));
							return -1;
							//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] ERROR: send not successful\n")), -1);
						}
						else
						{
							ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_local ] Sent 'die' xml message to the duplicator of flow '%s'\n"), f->getID().c_str()));
							dl->remove(f->getID());
						}
					}
				}			
			} // if
		} // else
	}
	return 0;
}

/*
int DuplicatorManager::notify_duplicators_to_die_peer(ACE_TString clientID_)
{
ACE_DEBUG((LM_DEBUG, "[DuplicatorManager::notify_duplicators_to_die_peer ] \n"));	
Flow *f;
int result;
ACE_SOCK_Stream dupli;
GlobalList *gl = GLOBAL_LIST::instance();  
DuplicatorList *dl = DUPLICATOR_LIST::instance();

//int buffer_size = dm.get_buffer_size();
ACE_TCHAR dieMessage[MAX_MESSAGE_SIZE]={0};
ACE_OS::sprintf(dieMessage,DIE_MESSAGE);


// for each flow this client was providing or consuming,
// check if there are still clients providing or consuming it
// if not, tell the duplicator to die
for (FLOW_ITERATOR fi = gl->getFirstFlow(); fi != gl->getLastFlow(); fi++)
{
f = GlobalList::getFlowFromIterator(fi);

// check if client 'clientID_' was the last consumer of this flow and that the provider was on this server with an unknown status
if ( (f->containsConsumer(clientID_) == 0) && (f->countConsumers() == 1) && (f->getProducerStatus() == CLIENT_STATUS_UNKNOWN) )
{											
ACE_TCHAR namelocal[256];
if (ACE_OS::hostname(namelocal,256) ==1)
{
ACE_DEBUG ((LM_DEBUG,"[ DuplicatorManager::notify_duplicators_to_die_peer ] ACE_OS::hostname \n"));
//ACE_TString createxmlfailure = create_xmlfailure("Peer_Handler_Launch::notify_duplicators_to_die : ACE_OS::hostname");
//send_xml(createxmlfailure.c_str());
return 0;
}	
ACE_TString localaddr = namelocal ;

if (f->getServerProducerID() == localaddr ) 
{												


result = dl->getServerSocket(f->getID(), &dupli);		   
// send the buffer to the duplicator
result =  send_message(dupli,dieMessage);
if (result < buffer_size)
{
ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_peer ] ERROR: send not successful\n")), -1);
}
else
{
ACE_DEBUG((LM_DEBUG, ACE_TEXT("[ DuplicatorManager::notify_duplicators_to_die_peer ] Sent 'die' message to the duplicator of flow '%s'\n"), f->getID().c_str()));
dl->remove(f->getID());
}
}
}			
} // for
return 0;
}
*/
