#include "ace/Thread_Manager.h"

#include "Peer_Event_Handler.h"
#include "Smartflow.h"
#include "XML_Parser.h"
#include "sf2d_common_ACE.h"

#include "list/ClientList.h"
#include "list/Flow.h"
#include "list/FlowList.h"
#include "list/GlobalList.h"
#include "list/DuplicatorList.h"
#include "list/SocketList.h"
#include "list/ServerList.h"
#include "list/LastModifications.h"

#include "Thread_Server.h"
#include "Log_Manager.h"


#include <iostream>
#include <set>


ACE_HANDLE Local_Event_Handler::get_handle(void) const {
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::get_handle"));
	return remote_peer_.get_handle();
}

int Local_Event_Handler::open(){
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::open"));
	myclient = NULL ;
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ LocalEvent_Handler::open ]\n")));
	return (reactor()->register_handler(this,ACE_Event_Handler::READ_MASK));
}

int Local_Event_Handler::handle_close(ACE_HANDLE, ACE_Reactor_Mask){
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::handle_close"));
	remote_peer_.close();
	if (myclient != NULL)
		delete myclient;
	return 0;
}

int Local_Event_Handler::handle_input(ACE_HANDLE)
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::handle_input"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::handle_input ]\n")));	
	ACE_TString doc = receive_xml();

	if (doc != ACE_TEXT(""))
	{
		if (actions_switch(doc) != 0)
			return -1;
		else
			return 0;
	}
	else 
		return -1;
}

int Local_Event_Handler::get_serverlist()
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::get_serverlist"));
	ACE_TString xmlserverlist = create_xmlserverlistresponse();
	return send_xml(xmlserverlist.c_str());
}

int Local_Event_Handler::actions_switch(ACE_TString xml)
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::actions_switch"));
	//ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::actions_switch ]  \n \n %s \n \n "), xml.c_str()));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::actions_switch ]  \n")));	
	int type = 0;
	int target = 0;

	XML_Parser parser(xml);

	if (parser.getInfos(ACE_TEXT("type")) == ACE_TEXT(""))
		return -1; //Socket shutdown
	else
		type = ACE_OS::atoi( parser.getInfos(ACE_TEXT("type")).c_str() );

	if (!(parser.getInfos(ACE_TEXT("target")) == ACE_TEXT("")))
		target = ACE_OS::atoi( parser.getInfos(ACE_TEXT("target")).c_str() );

	switch (type) 
	{
	case REQUEST :
		switch (target) 
		{
		case FLOWLIST:    	    
			return flowlist_request();
			break;
		case GETFLOW:
			return getflow_request(xml);     
			break;  
		case REMOVEFLOW:
			return remflow_request(xml); 
			break;
		case GETLASTMODIFICATIONS:
			return getlastmodifications_request();     
			break;     
		case LAUNCH:
			return launch_request(xml);
			break;
		case STOP:
			return stop_request(xml);
			break;
		case SERVERLIST:
			return get_serverlist();
			break;
		case PAUSE:
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::actions_switch  ][PAUSE] \n")));
			pauseFlow(xml);
			break;
		case RESTART :
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::actions_switch  ][RESTART] \n")));
			restartFlow(xml);
			break;
		default:
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::actions_switch  ] Invalid Request \n")));
			ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler Invalid Request"));
			return send_xml(createxmlfailure.c_str()); 
			break;
		}
		break;
	case CONNECT :
		return connect(xml);
		break;
	case ANNOUNCE :
		switch (target) 
		{
		case LOG_FROM_CLIENT :
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::actions_switch  ][LOG] \n")));
			logAnnounce(xml);
			break;
		default:
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::actions_switch  ] Invalid Request \n")));
			ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler Invalid Request"));
			return send_xml(createxmlfailure.c_str()); 
			break;
		}
		break;
	default :
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::actions_switch  ] Invalid Command \n")));
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler Invalid Request"));
		return send_xml(createxmlfailure.c_str());   
		break;
	}
	return 0; 
}

int Local_Event_Handler::connect(ACE_TString xml)
{  
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::connect"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::connect ]\n")));

	GlobalList *gl = GLOBAL_LIST::instance(); 
	//SocketList *sl = SOCKET_LIST::instance();   
	ServerList *sl = SERVER_LIST::instance(); 
	LastModifications *lm = LAST_MODIFICATIONS::instance();   

	XML_Parser parser(xml);
	ACE_TString clientname = (parser.getInfos(ACE_TEXT("clientname"))).c_str() ; 
	ACE_TString clientgroup= (parser.getInfos(ACE_TEXT("clientgroup"))).c_str()  ;

	if (  clientname.length() == 0 )
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ LocalEvent_Handler::connect ] clientname.length==0\n")));	
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler Connect : Error XML Description"));
		send_xml(createxmlfailure.c_str());
		return -1;	
	}	
	if (  clientgroup.length() ==0 )
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ LocalEvent_Handler::connect ] clientgroup.length==0\n")));	
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler Connect : Error XML Description"));
		send_xml(createxmlfailure.c_str());
		return -1;	
	}	

	ACE_TString clientid = Client::getIDFromGroupAndName( clientgroup , clientname );  

	if (gl->findClient(clientid) == 0)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::connect ] Clientid already exists\n")));
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Client already exists"));
		send_xml(createxmlfailure.c_str());	
		return -1;     
	}	
	myclient = new Client(clientgroup, clientname); 
	if (myclient == NULL)
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Error Local_Handler Connect  : Creation"));
		send_xml(createxmlfailure.c_str());	
		return -1;	
	}		

	ACE_TString clientpath = (parser.getInfos(ACE_TEXT("clientpath"))).c_str() ;
	if ( myclient->setPath( clientpath ) != 0)
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Error Local_Handler Connect : setPath"));
		send_xml(createxmlfailure.c_str());
		return -1;
	}	

	ACE_TString flowinfo = (parser.getInfos(ACE_TEXT("flowinfo"))).c_str() ;
	if ( myclient->setFlowInfo( flowinfo ) != 0)
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Error Local_Handler Connect : setFlowInfo"));
		send_xml(createxmlfailure.c_str());
		return -1;
	}	

	ACE_TString clientserver = (parser.getInfos(ACE_TEXT("clientserver"))).c_str() ;
	if ( myclient->setServer( clientserver ) != 0)
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Error Local_Handler Connect : setServer"));
		send_xml(createxmlfailure.c_str());
		return -1;
	}	

	ACE_TString clientparam = (parser.getInfos(ACE_TEXT("clientparam"))).c_str() ;
	if ( myclient->setParams( clientparam ) != 0)
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Error Local_Handler Connect : setParam"));
		send_xml(createxmlfailure.c_str());
		return -1;
	}		

	for (SERVER_ITERATOR si = sl->first(); si != sl->last(); si++) {
		Peer_Event_Handler::send_xml_to_servers((*si).int_id_,  xml.c_str());
	}
	//for (SOCKET_ITERATOR si = sl->firstServer(); si != sl->lastServer(); si++)
	//{
	//	send_xml_to_servers(sl->getServer(SocketList::getIDFromIterator(si)) , xml.c_str());        
	//}	 	

	//Adding modifications
	ACE_TString modif ;
	modif = xml.c_str() ;
	lm->addModifications(modif) ;	

	if ( clientgroup == ACE_TEXT("admin") )
	{	
		lm->addClient(myclient);
	}		 	

	if (gl->addClient(myclient) == 0)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::connect ] Adding %s \n") , clientid.c_str() ));		
		//	sl->addClient(clientid, &remote_peer_);
		ACE_TString createxmlsuccess = create_xmlsuccess();
		send_xml(createxmlsuccess.c_str());	
		return 0;
	}
	else
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Error Local_Handler Connect : addClient"));
		send_xml(createxmlfailure.c_str());
		return -1;
	}
}


int Local_Event_Handler::flowlist_request()
{ 
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::flowlist_request"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::flowlist_request ]\n")));
	ACE_TString xmlflowlist = create_xmlflowlistresponse();
	if (xmlflowlist != ACE_TEXT(""))
		return send_xml(xmlflowlist.c_str());
	else
		return -1;
}

int Local_Event_Handler::getlastmodifications_request()
{ 
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::getlastmodifications_request"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::getlastmodifications_request ]\n")));
	if (myclient->getGroup() == ACE_TEXT("admin"))
	{
		ACE_TString xmlmodificationslist = create_xmllastmodifications();
		if (send_xml(xmlmodificationslist.c_str()) == 0)
		{
			myclient->clear();
			return 0;
		}
		else
			return -1;  		
	}
	else
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Error You can't request that"));
		send_xml(createxmlfailure.c_str());
		return 0;
	}
} 

int  Local_Event_Handler::pauseFlow(ACE_TString xml) {
    ACE_TRACE(ACE_TEXT("Local_Event_Handler::pauseFlow"));
	ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Local_Event_Handler::pause_flow ]\n %s"),xml.c_str()));
    XML_Parser parser(xml);

	ACE_TString clientName = (parser.getInfos(ACE_TEXT("clientname"))).c_str();
	ACE_TString clientGroup = (parser.getInfos(ACE_TEXT("clientgroup"))).c_str();
	ACE_TString clientID = clientGroup + ACE_TEXT(":") + clientName;

	ACE_TString flowType = (parser.getInfos(ACE_TEXT("flowtype"))).c_str();
	ACE_TString flowGroup = (parser.getInfos(ACE_TEXT("flowgroup"))).c_str();
	ACE_TString flowID = flowGroup + ACE_TEXT(":") + flowType;

	GlobalList *gl = GLOBAL_LIST::instance();
	//SocketList *sl = SOCKET_LIST::instance(); 
	ServerList *sl = SERVER_LIST::instance();
	DuplicatorList *dl = DUPLICATOR_LIST::instance();  

	//ACE_SOCK_Stream *dupli = NULL;
	Duplicator_Event_Handler *dupli = NULL;
	int result;
	//int buffer_size = dm.get_buffer_size();


	if ( gl->findClient(clientID) != 0 )
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Client not found !"));
		send_xml(createxmlfailure.c_str());
		return 0;
	}

	ACE_TCHAR namelocal[256];
	if (ACE_OS::hostname(namelocal,256) ==1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::pause_flow ] ACE_OS::hostname \n")));
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler_Launch::pause_flow : ACE_OS::hostname"));
		send_xml(createxmlfailure.c_str());
		return 0;
	}	

	ACE_TString localaddr = namelocal ;
	ACE_TString consumerserverrequest = gl->getClient(clientID)->getServer(); 

	// if local consummer then  list update +  notify pause client to all servers + send pause to the duplicator provider
	if (localaddr == consumerserverrequest)
	{

		//update
		// 1. get the Flow
		// 2. switched_OFF client for this flow
		// 3. if no consumer for this flow then flow_status = pending and producer_status = waiting 
		Flow *f;
		f = gl->getFlow(flowID);
		if (f == NULL) {
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::pauseFlow ] flow == NULL \n") ));
			ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Event_Handler::pauseFlow : flow == NULL"));
			send_xml(createxmlfailure.c_str());
			return -1;
		}

		// send 'pause' to the duplicator of this flow 

		dupli = dl->getDupEventHandler(f->getID());
		if ( dupli == NULL ) {
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[  Local_Event_Handler::pauseFlow ] ERROR: dupli==NULL\n")));
			return -1;
			//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[  Local_Event_Handler::pauseFlow ] ERROR: dupli==NULL\n")), -1);
		}
		//result = dm.send_message(dupli,xml.c_str());
		result = dupli->send_xml(xml.c_str());
		//if (result < buffer_size) {
		if (result == -1) {
			ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Event_Handler::pauseFlow : send to duplicator failed"));
			send_xml(createxmlfailure.c_str());
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::pauseFlow ] ERROR: send not successful\n")));
			return -1;
			//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ Local_Event_Handler::pauseFlow ] ERROR: send not successful\n")), -1);
		}

		// Inform all the others servers
		//ACE_TCHAR serverPauseAnnounceMessage[MAX_MESSAGE_SIZE]={0};
		int size_needed = sizeof(PAUSE_ANNOUNCE_MESSAGE) + clientName.length() + clientGroup.length() + flowType.length() + flowGroup.length() + consumerserverrequest.length() + 1;
		ACE_TCHAR *serverPauseAnnounceMessage = (ACE_TCHAR *) ACE_OS::malloc(size_needed); 
		ACE_OS::memset(serverPauseAnnounceMessage, 0, size_needed);
		ACE_OS::sprintf(serverPauseAnnounceMessage,PAUSE_ANNOUNCE_MESSAGE,clientName.c_str(),clientGroup.c_str(),flowType.c_str(),flowGroup.c_str(),consumerserverrequest.c_str());
		
		for (SERVER_ITERATOR si = sl->first(); si != sl->last(); si++) {
			Peer_Event_Handler::send_xml_to_servers((*si).int_id_, serverPauseAnnounceMessage);
		}
		//for (SOCKET_ITERATOR si = sl->firstServer(); si != sl->lastServer(); si++) 
		//	send_xml_to_servers(sl->getServer(SocketList::getIDFromIterator(si)) , serverPauseAnnounceMessage);

		// update flow status
		if (f->setConsumerStatus(clientID, CLIENT_STATUS_SWITCHED_OFF) == -1 ) 
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::pause_flow ] error set consumer status \n")));

		if (f->countSwitchedOnConsumers() == 0 ) {
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::pause_flow ] no more consumer,provider going to sleep ... \n")));
			if (f->setStatus(FLOW_STATUS_PENDING) == -1)
				ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::pause_flow ] error set flow status \n")));
			if (f->setProducerStatus(CLIENT_STATUS_WAITING) == -1 )
				ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::pause_flow ] error set producer status\n")));
		}

		// Update modifications
		LastModifications *lm = LAST_MODIFICATIONS::instance(); 
		lm->addModifications(serverPauseAnnounceMessage);
		
		ACE_OS::free(serverPauseAnnounceMessage);
	}
	else // remote consumer
	{
		// else notify pause client to the server of this consumer
		//ACE_TCHAR serverPauseRequestMessage[MAX_MESSAGE_SIZE]={0};
		int size_needed = sizeof(PAUSE_REQUEST_MESSAGE) + clientName.length() + clientGroup.length() + flowType.length() + flowGroup.length() + consumerserverrequest.length() + 1;
		ACE_TCHAR *serverPauseRequestMessage = (ACE_TCHAR *) ACE_OS::malloc(size_needed); 
		ACE_OS::memset(serverPauseRequestMessage, 0, size_needed);		
		ACE_OS::sprintf(serverPauseRequestMessage,PAUSE_REQUEST_MESSAGE,clientName.c_str(),clientGroup.c_str(),flowType.c_str(),flowGroup.c_str(),consumerserverrequest.c_str());
		
		ACE_TString serverrequest = gl->getClient(clientID)->getServer(); 
		//send_xml_to_servers(sl->getServer(serverrequest),serverPauseRequestMessage);
		Peer_Event_Handler::send_xml_to_servers(sl->get(serverrequest, ServerList::NAME), serverPauseRequestMessage);
		
		ACE_OS::free(serverPauseRequestMessage);
	}

 	ACE_TString createxmlsuccess = create_xmlsuccess();
	send_xml(createxmlsuccess.c_str());

	return 0;
}


int  Local_Event_Handler::restartFlow(ACE_TString xml) { 
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::restartFlow"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::restart_flow ]\n")));
	XML_Parser parser(xml);
 
	ACE_TString clientName = (parser.getInfos(ACE_TEXT("clientname"))).c_str();
	ACE_TString clientGroup = (parser.getInfos(ACE_TEXT("clientgroup"))).c_str();
	ACE_TString clientID = clientGroup + ACE_TEXT(":") + clientName;
	
	ACE_TString flowType = (parser.getInfos(ACE_TEXT("flowtype"))).c_str();
	ACE_TString flowGroup = (parser.getInfos(ACE_TEXT("flowgroup"))).c_str();
	ACE_TString flowID = flowGroup + ACE_TEXT(":") + flowType;
	
	GlobalList *gl = GLOBAL_LIST::instance();
	//SocketList *sl = SOCKET_LIST::instance();
	ServerList *sl = SERVER_LIST::instance(); 
	DuplicatorList *dl = DUPLICATOR_LIST::instance();  

	//ACE_SOCK_Stream *dupli = NULL;
	Duplicator_Event_Handler *dupli = NULL;
	int result;
	//int buffer_size = dm.get_buffer_size();


	if ( gl->findClient(clientID) != 0 )
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Client not found !"));
		send_xml(createxmlfailure.c_str());
		return 0;
	}

	ACE_TCHAR namelocal[256];
	if (ACE_OS::hostname(namelocal,256) ==1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::restart_flow ] ACE_OS::hostname \n")));
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler_Launch::restart_flow : ACE_OS::hostname"));
		send_xml(createxmlfailure.c_str());
		return 0;
	}	

	ACE_TString localaddr = namelocal ;
	ACE_TString consumerserverrequest = gl->getClient(clientID)->getServer(); 

	if (localaddr == consumerserverrequest) {
		Flow *f;
		f = gl->getFlow(flowID);
		if (f == NULL) {
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::restartFlow ] flow == NULL \n") ));
			ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Event_Handler::restartFlow : flow == NULL"));
			send_xml(createxmlfailure.c_str());
			return -1;
		}

		// send 'restart' to the duplicator of this flow 

		dupli = dl->getDupEventHandler(f->getID());
		if ( dupli == NULL ) {
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::restartFlow ] ERROR: dupli==NULL\n")));
			return -1;
			//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ Local_Event_Handler::restartFlow ] ERROR: dupli==NULL\n")), -1);
		}
		//result = dm.send_message(dupli,xml.c_str());
		result = dupli->send_xml(xml.c_str());
		//if (result < buffer_size) {
		if (result == -1) {
			ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Event_Handler::restartFlow : send to duplicator failed"));
			send_xml(createxmlfailure.c_str());
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::restartFlow ] ERROR: send not successful\n")));
			return -1;
			//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ Local_Event_Handler::restartFlow ] ERROR: send not successful\n")), -1);
		}

		// inform all the servers
		//ACE_TCHAR serverRestartAnnounceMessage[MAX_MESSAGE_SIZE]={0};
		int size_needed = sizeof(RESTART_ANNOUNCE_MESSAGE) + clientName.length() + clientGroup.length() + flowType.length() + flowGroup.length() + consumerserverrequest.length() + 1;
		ACE_TCHAR *serverRestartAnnounceMessage = (ACE_TCHAR *) ACE_OS::malloc(size_needed); 
		ACE_OS::memset(serverRestartAnnounceMessage, 0, size_needed);		
		ACE_OS::sprintf(serverRestartAnnounceMessage,RESTART_ANNOUNCE_MESSAGE,clientName.c_str(),clientGroup.c_str(),flowType.c_str(),flowGroup.c_str(),consumerserverrequest.c_str());
		
		for (SERVER_ITERATOR si = sl->first(); si != sl->last(); si++) {
			Peer_Event_Handler::send_xml_to_servers((*si).int_id_, serverRestartAnnounceMessage);
		}
		//for (SOCKET_ITERATOR si = sl->firstServer(); si != sl->lastServer(); si++) 
		//	send_xml_to_servers(sl->getServer(SocketList::getIDFromIterator(si)) , serverRestartAnnounceMessage);

		// update
		if (f->setConsumerStatus(clientID, CLIENT_STATUS_SWITCHED_ON) == -1 ) 
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::restart_flow ] error set consumer status \n")));
		if (f->setStatus(FLOW_STATUS_RUNNING) == -1)
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::restart_flow ] error set flow status \n")));
		if (f->setProducerStatus(CLIENT_STATUS_SWITCHED_ON) == -1 )
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::restart_flow ] error set producer status\n")));

		// Update modifications
		LastModifications *lm = LAST_MODIFICATIONS::instance(); 
		lm->addModifications(serverRestartAnnounceMessage);
		
		ACE_OS::free(serverRestartAnnounceMessage);
	} else {
		//ACE_TCHAR serverRestartRequestMessage[MAX_MESSAGE_SIZE]={0};
		int size_needed = sizeof(RESTART_REQUEST_MESSAGE) + clientName.length() + clientGroup.length() + flowType.length() + flowGroup.length() + consumerserverrequest.length() + 1;
		ACE_TCHAR *serverRestartRequestMessage = (ACE_TCHAR *) ACE_OS::malloc(size_needed);
		ACE_OS::memset(serverRestartRequestMessage, 0, size_needed);
		ACE_OS::sprintf(serverRestartRequestMessage,RESTART_REQUEST_MESSAGE,clientName.c_str(),clientGroup.c_str(),flowType.c_str(),flowGroup.c_str(),consumerserverrequest.c_str());
		
		ACE_TString serverrequest = gl->getClient(clientID)->getServer(); 
		Peer_Event_Handler::send_xml_to_servers(sl->get(serverrequest, ServerList::NAME), serverRestartRequestMessage);
		//send_xml_to_servers(sl->getServer(serverrequest),serverRestartRequestMessage);
		
		ACE_OS::free(serverRestartRequestMessage);
	}

 	ACE_TString createxmlsuccess = create_xmlsuccess();
	send_xml(createxmlsuccess.c_str());

	return 0; 
}


int Local_Event_Handler::stop_request( ACE_TString xml )
{ 
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::stop_request"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::stop_request ]\n")));
	XML_Parser parser(xml);
	ACE_TString clientname = (parser.getInfos(ACE_TEXT("clientname"))).c_str();
	ACE_TString clientgroup = (parser.getInfos(ACE_TEXT("clientgroup"))).c_str();
	ACE_TString clientID = Client::getIDFromGroupAndName( clientgroup , clientname ); 

	GlobalList *gl = GLOBAL_LIST::instance();

	if ( gl->findClient(clientID) != 0 )
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Client not found !"));
		send_xml(createxmlfailure.c_str());
		return 0;
	}

	ACE_TCHAR namelocal[256];
	if (ACE_OS::hostname(namelocal,256) ==1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::stop_request ] ACE_OS::hostname \n")));
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler_Launch::stop_request : ACE_OS::hostname"));
		send_xml(createxmlfailure.c_str());
		return 0;
	}	

	ACE_TString localaddr = namelocal ;
	ACE_TString serverrequest = gl->getClient(clientID)->getServer(); 

	if (localaddr == serverrequest)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::stop_request ] Stop of a local client requested \n")));
		//Find the socket of the client and send him a message
		SocketList *sl = SOCKET_LIST::instance();  
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::stop_request ] Client %s with Socket %x will be shutdown ! \n"), clientID.c_str(), sl->getClient(clientID) ));	 
		ACE_TCHAR size[512] = {0};
		ACE_OS::snprintf(size, sizeof(size), ACE_TEXT("%d"), STOP );
		if (sl->getClient(clientID) == 0 )
		{
			//can't happen ... in theory ;)
			ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Error socket Client not found  ! Internal Server Problem"));
			send_xml(createxmlfailure.c_str());
			return 0;
		}		
		sl->getClient(clientID)->send_n(size, sizeof(size));
		ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Local_Event_Handler::stop_request ] Client %s with Socket %x has been notified of the shutdown request ! \n"), clientID.c_str(), sl->getClient(clientID) ));		
		ACE_TString createxmlsuccess = create_xmlsuccess();
		return send_xml(createxmlsuccess.c_str());
	}
	else
	{
		//SocketList *sl = SOCKET_LIST::instance();  
		ServerList *sl = SERVER_LIST::instance();
		//if (sl->findServer(serverrequest) == 0)
		if (sl->find(serverrequest, ServerList::NAME) == 0)
		{      	        	
			//send_xml_to_servers(sl->getServer(serverrequest) , xml.c_str() ); 
			Peer_Event_Handler::send_xml_to_servers(sl->get(serverrequest, ServerList::NAME), xml.c_str());
			ACE_TString createxmlsuccess = create_xmlsuccess();
			return send_xml(createxmlsuccess.c_str() );	            	
		}	
		else 
		{
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::stop_request ] Problem Connexion with remote Server \n")));
			ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Problem with remote server"));
			return send_xml(createxmlfailure.c_str());	   	
		}
	}	
}

int Local_Event_Handler::launch_request( ACE_TString xml )
{  
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::launch_request"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::launch_request ]\n")));
	XML_Parser parser(xml);
	ACE_TString serverrequest = (parser.getInfos(ACE_TEXT("server"))).c_str();
	ACE_TString localaddr;

	if ( serverrequest == ACE_TEXT(""))
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler Launch_Request : Error XML Description"));
		send_xml(createxmlfailure.c_str());
		return 0;
	}

	ACE_TCHAR namelocal[256];
	if (ACE_OS::hostname(namelocal,256) ==1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::launch_request ] ACE_OS::hostname \n")));
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler Launch_Request : ACE_OS::hostname"));
		send_xml(createxmlfailure.c_str());
		return 0;
	}

	localaddr = namelocal;

	if (  parser.getInfos(ACE_TEXT("clientpath")) == ACE_TEXT(""))
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::launch_request ] Error XML Description : ClientPath\n")));
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler Launch_Request : Error XML Description"));
		send_xml(createxmlfailure.c_str());
		return 0;
	}

	if (  parser.getInfos(ACE_TEXT("args")) == ACE_TEXT(""))
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::launch_request ] Error XML Description : Args\n")));
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler Launch_Request : Error XML Description"));
		send_xml(createxmlfailure.c_str());
		return 0;
	}

	/*
	if (  parser.getInfos("clientparam") == "")
	{
	ACE_DEBUG ((LM_DEBUG,"Local_Handler Launch_Request : Error XML Description : ClientParam\n"));
	ACE_TString createxmlfailure = create_xmlfailure("Local_Handler Launch_Request : Error XML Description");
	send_xml(createxmlfailure.c_str());
	return 0;
	}	
	*/

	if (localaddr == serverrequest)
	{
		ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Local_Event_Handler::launch_request ] Launch of a local client requested \n")));
		ACE_TString tospawn = parser.getInfos(ACE_TEXT("clientpath")).c_str() ;
		tospawn += ACE_TEXT(" ");
		ACE_TString arguments = parser.getInfos(ACE_TEXT("args")).c_str() ;
		tospawn += arguments ;
		tospawn += ACE_TEXT(" ") ;
		tospawn += parser.getInfos(ACE_TEXT("clientparam")).c_str() ;    

		// We get the clientgroup and clientname from the arguments passed by the CC
		int indice1 = arguments.find(ACE_TEXT("--sf clientname="));
		indice1 += 16; // length of "--sf clientname="
		int indice2 = arguments.find(ACE_TEXT(" "), indice1);
		ACE_TString clientname = arguments.substr(indice1, indice2-indice1);

		indice1 = arguments.find(ACE_TEXT("--sf clientgroup="));
		indice1 += 17; // length of "--sf clientgroup="
		indice2 = arguments.find(ACE_TEXT(" "), indice1);
		ACE_TString clientgroup = arguments.substr(indice1, indice2-indice1);

		Thread_Args* args = new Thread_Args( tospawn, clientname, clientgroup );	  
		ACE_Thread_Manager::instance()->spawn( (ACE_THR_FUNC) Thread_Server::run_svc, ACE_static_cast(void*, args), THR_DETACHED | THR_SCOPE_SYSTEM );
		ACE_TString createxmlsuccess = create_xmlsuccess();
		return send_xml(createxmlsuccess.c_str());
	}
	else
	{
		ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Local_Event_Handler::launch_request ] Launch of a remote client requested\n"))); 
		//SocketList *sl = SOCKET_LIST::instance(); 
		ServerList *sl = SERVER_LIST::instance();  
		if (sl->find(serverrequest, ServerList::NAME) == 0)
		{     
			Peer_Event_Handler::send_xml_to_servers(sl->get(serverrequest, ServerList::NAME), xml.c_str());
			//send_xml_to_servers(sl->getServer(serverrequest) , xml.c_str() ); 
			ACE_TString createxmlsuccess = create_xmlsuccess();
			return send_xml(createxmlsuccess.c_str() );	            	
		}	
		else 
		{
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::launch_request ] Remote Server not found\n")));
			ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Remote Server not found"));
			return send_xml(createxmlfailure.c_str());	   	
		}
	}
}

int Local_Event_Handler::getflow_request(ACE_TString xml)
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::getflow_request"));
	//ACE_DEBUG((LM_DEBUG, "[ Local_Event_Handler::getflow_request ]\n"));
	if (myclient == NULL)
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Client not registered"));
		return send_xml(createxmlfailure.c_str());
	}
	
	//We give the XML message to the parser
	XML_Parser parser(xml);

	//We extract some information about the flow from the parser
	ACE_TString flowgroup = (parser.getInfos(ACE_TEXT("group"))).c_str();
	ACE_TString flowtype = (parser.getInfos(ACE_TEXT("typeflow"))).c_str();
	ACE_TString flowname = (parser.getInfos(ACE_TEXT("name"))).c_str();

	//Time to check we got all the information we need from the XML message

	//Is the flowgroup field present in the XML message?
	if (flowgroup.length() == 0)
	{
		//No, the flow has no group. It shouldn't happen.
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler Getflow : Error XML Description"));
		return send_xml(createxmlfailure.c_str());
	}
	
	//Is the flowgroup field present in the XML message?
	if (  flowtype.length() == 0 )
	{
		//No, the flow has no type. It shouldn't happen.
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler Getflow : Error XML Description"));
		return send_xml(createxmlfailure.c_str());
	}

	//Is the flowname field present in the XML message?
	if (  flowname.length() == 0 )
	{
		//No, the flow has no name. It shouldn't happen.
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler Getflow : Error XML Description"));
		return send_xml(createxmlfailure.c_str());
	}

	ACE_TString nbConsumers = (parser.getInfos(ACE_TEXT("synchronized"))).c_str();
	//Is the synchronized field (giving the number of expected consumers present in the XML message?
	if ( nbConsumers == ACE_TEXT(""))
	{
		// No, the flow has no value for synchronized. It shouldn't happen.
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error XML Description"));
		return send_xml(createxmlfailure.c_str());
	}

	//We generate an ID for the flow from its name and group.
	ACE_TString flowid = Flow::getIDFromGroupAndType( flowgroup, flowtype);
	GlobalList *gl = GLOBAL_LIST::instance();   
	DuplicatorList *dl = DUPLICATOR_LIST::instance();
	//SocketList *sl = SOCKET_LIST::instance();
	ServerList *sl = SERVER_LIST::instance();
	LastModifications *lm = LAST_MODIFICATIONS::instance(); 

	int check = 0 ;	

	//Is the field register_flow present in the XML message?
	if ( ACE_OS::strcmp((parser.getInfos(ACE_TEXT("register_flow"))).c_str(),ACE_TEXT("")) == 0 )
	{
		// No, the flow has no value for register_flow. It shouldn't happen. 
		check = -1 ;
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Register_Flow Flags missing"));
		return send_xml(createxmlfailure.c_str());
	}
	else
	{
		//We get the value from register_flow and store it in "check"
		check = ACE_OS::atoi((parser.getInfos(ACE_TEXT("register_flow"))).c_str()) ;
	}

	//If check == 0, that means we are first testing if the duplicator for the flow exists or not.
	if (check == 0)
	{
		// check if the flow already exists
		if (gl->findFlow(flowid) == 0) 
		{
			// the flow already exists
			Flow *f = gl->getFlow(flowid);

			// check if there is a producer
			if (f->getProducerStatus() == CLIENT_STATUS_UNKNOWN)
			{
				// there are consumers (because the flow already exists, but there is no producer for this flow yet
				// We test if a duplicator exists on the local machine for this flow
				// If yes, we test if it has connected to the server yet or not
				//ACE_SOCK_Stream *dupli = NULL;
				Duplicator_Event_Handler *dupli = NULL;
				if (dl->find(flowid) == 0) {
					dupli = dl->getDupEventHandler(flowid);
					ACE_INET_Addr test_addr;
					if ( ( dupli != NULL ) && dupli->peer().get_remote_addr(test_addr) == -1) {	
						//The duplicator hasn't connected yet.
						ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] Flag Unsuccess\n")));
						ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("The duplicator hasn't connected to the server yet"));
						return send_xml(createxmlfailure.c_str());	
					}
				}
			}
		}
	}

	//At this point, check is different than 0, that means we actually want to create a flow.

	//Is the field provider present in the XML message?
	if (parser.getInfos(ACE_TEXT("provider")) == ACE_TEXT(""))
	{
		// No, the flow has no value for provider. It shouldn't happen.
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error XML Description"));
		return send_xml(createxmlfailure.c_str());
	}

	//At this point, we get the value from the "provider" field in the XML message
	// Producer   
	if (ACE_OS::atoi((parser.getInfos(ACE_TEXT("provider"))).c_str()) == smartflow::FLOW_OUTPUT)
	{
		//If we are here, that means the creation of an output flow (provider) has been requested
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] Provider part\n")));

		unsigned int peer;

		// check if the flow already exists
		if (gl->findFlow(flowid) == 0) 
		{
			// the flow already exists
			Flow *f = gl->getFlow(flowid);

			// check if there is a producer
			if (f->getProducerStatus() == CLIENT_STATUS_UNKNOWN)
			{
				// there are consumers (because the flow already exists, but there is no producer for this flow yet

				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] No producer, Candidate: '%s'\n"), myclient->getID().c_str()));

				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] Status PENDING -> RUNNING | Producer connected\n")));

				bool local_consumer_exists = false;

				// We test if a duplicator exists on the local machine for this flow
				// If yes, it means that a local consumer is present on the machine
				// => We have to open the provider part of the duplicator
				//ACE_SOCK_Stream *dupli = NULL;
				Duplicator_Event_Handler *dupli = NULL;
				if (dl->find(flowid) == 0) {
					local_consumer_exists = true;
					dupli = dl->getDupEventHandler(flowid);
					dl->remove(flowid);
				}

				//Serverside Communication		
				if (dl->addProducer(flowid) == -1)
				{
					ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Error DuplicatorList : addProducer\n"));
					return send_xml(createxmlfailure.c_str());
				}   		   
				peer = dl->nextPortSupposedToBeAvailable();  

				if (local_consumer_exists) {
				
					//We open the provider part of the duplicator
					//ACE_TCHAR openProviderPartRequestMessage[MAX_MESSAGE_SIZE]={0};
					char tmp_str[11];
					int size_needed = sizeof(OPEN_PROVIDER_PART_REQUEST_MESSAGE) + ACE_OS::sprintf(tmp_str, "%d", peer)  + 1;
					ACE_TCHAR *openProviderPartRequestMessage = (ACE_TCHAR *) ACE_OS::malloc(size_needed);
					ACE_OS::memset(openProviderPartRequestMessage, 0, size_needed);
					ACE_OS::sprintf(openProviderPartRequestMessage, OPEN_PROVIDER_PART_REQUEST_MESSAGE, peer);
								   
					if ( dupli == NULL ) {
						ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] Duplicator for flow not connected to server yet! We'll notify it later\n")));
						dl->setMustOpenProviderPart(flowid, true);
					}
					else {
						int result = dupli->send_xml(openProviderPartRequestMessage);
						if (result == -1) {
							ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("[ Local_Event_Handler::getflow_request ] ERROR: dupli->send_xml not successful\n"));
							return send_xml(createxmlfailure.c_str());
						}
											
						// We update the duplicator list with the right socket to use when we want to contact the duplicator
						dl->setDupEventHandler(flowid, dupli);
					}
					ACE_OS::free(openProviderPartRequestMessage);
				}
				else
					ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] Local consumer doesn't exist\n")));

				if (peer == 0)
				{
					ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Error DuplicatorList : getClientsPortNumber\n" ));
					return send_xml(createxmlfailure.c_str());
				}       	
				
				//args parameter can be empty
				//if ( parser.getInfos(ACE_TEXT("args")) == ACE_TEXT("")  )
				//{
				//	ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error XML Description"));
				//	return send_xml(createxmlfailure.c_str());
				//}

				f->setName(flowname);
				f->setProducer(myclient->getID());
				f->setServerProducerID( myclient->getServer() );
				f->setStatus(FLOW_STATUS_RUNNING);
				f->setProducerStatus(CLIENT_STATUS_SWITCHED_ON);
				f->setNumberOfExpectedConsumers( ACE_OS::atoi(nbConsumers.c_str()) );

				ACE_TString modif = xml.c_str();
				lm->addModifications(modif) ;		        				
				
				// If we have just opened the provider part of the duplicator,
				// we don't need to create a new one so we just send XML success and we return;
				if (local_consumer_exists) {
					ACE_TString createxmlsuccess = create_xmlsuccess();				
					return send_xml(createxmlsuccess.c_str());
				}
			}//end if (f->getProducerStatus() == CLIENT_STATUS_UNKNOWN)
			else
			{
				// there is already a producer for this flow
				ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] Producer :  Flow already exists\n")));
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Flow already exists\n"));
				return send_xml(createxmlfailure.c_str());
			}
		} //end if the flow exists;
		
		else {
			//an output flow, which doesn't exist 
			//Serverside Communication		
			if (dl->addProducer(flowid) == -1)
			{
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Error DuplicatorList : addProducer\n"));
				return send_xml(createxmlfailure.c_str());
			}   		   
			peer = dl->nextPortSupposedToBeAvailable();          
			if (peer == 0)
			{
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Error DuplicatorList : getClientsPortNumber\n" ));
				return send_xml(createxmlfailure.c_str());
			}       	
			//args parameter can be empty
			//if ( parser.getInfos(ACE_TEXT("args")) == ACE_TEXT("")  )
			//{
			//	ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error XML Description"));
			//	return send_xml(createxmlfailure.c_str());
			//}
			if ( parser.getInfos(ACE_TEXT("blocksize")) == ACE_TEXT("") )
			{
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error XML Description - blocksize parameter is empty"));
				return send_xml(createxmlfailure.c_str());
			}
			if ( parser.getInfos(ACE_TEXT("blockcount")) == ACE_TEXT("") )
			{
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error XML Description - blockcount parameter is empty"));
				return send_xml(createxmlfailure.c_str());
			}

			//Flow Creation    
			Flow *flow1 = new Flow( flowgroup , flowtype );
			if (flow1 == NULL )
			{
				dl->remove(flowid);
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error Flow Registration"));
				return send_xml(createxmlfailure.c_str());
			}     
			if (flow1->setName(flowname) != 0)
			{
				dl->remove(flowid);
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error Flow setname"));
				return send_xml(createxmlfailure.c_str());
			}      
			if (flow1->setBlockSize( ACE_OS::atoi( parser.getInfos(ACE_TEXT("blocksize")).c_str() )) != 0)
			{
				dl->remove(flowid);
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error Flow setBlockSize"));
				return send_xml(createxmlfailure.c_str());
			}
			if (flow1->setBlockCount( ACE_OS::atoi( parser.getInfos(ACE_TEXT("blockcount")).c_str()) ) != 0)
			{
				dl->remove(flowid);
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error Flow setBlockCount"));
				return send_xml(createxmlfailure.c_str());
			}
			if (flow1->setProducer(myclient->getID()) != 0)
			{
				dl->remove(flowid);
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error Flow setProducer"));
				return send_xml(createxmlfailure.c_str());
			}
			if (flow1->setServerProducerID( myclient->getServer() ) != 0)
			{
				dl->remove(flowid);
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error Flow setServerProducerID"));
				return send_xml(createxmlfailure.c_str());
			}      
			if (flow1->setNumberOfExpectedConsumers( ACE_OS::atoi(nbConsumers.c_str() ) ) != 0)
			{
				dl->remove(flowid);
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error Flow setNumberOfExpectedConsumers"));
				return send_xml(createxmlfailure.c_str());
			}     
			if (gl->addFlow(flow1) != 0)  
			{
				dl->remove(flowid);
				delete flow1;
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request :GlobalList addFlow"));
				return send_xml(createxmlfailure.c_str());
			}         
		}//end block: an output flow, which doesn't exist


		// A new argument has to be given to the duplicator, to tell him its synchronization status
		//At this point, we need to know if a synchronized flow has been requested: this info is required by the duplicator.


		//Duplicator Creation              

		ACE_TString tospawn = ACE_TEXT("duplicator") ;
#if (defined (ACE_WIN32))
		tospawn += ACE_TEXT(".exe") ;
#endif
		tospawn += ACE_TEXT(" ") ;
		tospawn += flowid.c_str() ;
		tospawn += ACE_TEXT(" ") ;
		ACE_TCHAR dtype[4] = {0};
		ACE_OS::snprintf(dtype, sizeof(dtype), ACE_TEXT("%d"), PROVIDER );
		tospawn += dtype ;      
		tospawn += ACE_TEXT(" ") ;
		tospawn += parser.getInfos(ACE_TEXT("blocksize")).c_str();
		tospawn += ACE_TEXT(" ");
		tospawn +=  parser.getInfos(ACE_TEXT("blockcount")).c_str() ;
		tospawn += ACE_TEXT(" ") ;     
		ACE_TCHAR dpeer[6] = {0};
		ACE_OS::snprintf(dpeer, sizeof(dpeer), ACE_TEXT("%d"), peer );
		tospawn += dpeer ;
		tospawn += ACE_TEXT(" ") ;

		//we add the log severity defined when the server was launched
		//if none, we use the default one
		ACE_TString tmp_level = "loglevel";
		ACE_TString log_level = LOG_PREF::instance()->getPref(tmp_level);
		if ( log_level == "debug" ) {
			tospawn += log_level.c_str();
		}
		else if ( log_level == "warning" ) {
			tospawn += log_level.c_str();
		}
		else {
			ACE_TString tmp_error = "error";
			tospawn += tmp_error.c_str();
		}	
		tospawn += ACE_TEXT(" ") ;

		//we add the syslog capabilities if defined, default value otherwise
		ACE_TString tmp_syslog = "syslog";	
		ACE_TString hasSyslog = LOG_PREF::instance()->getPref(tmp_syslog);
		if ( hasSyslog == "1" ) {
			tospawn += hasSyslog.c_str();
		}
		else {
			tospawn += "0";
		}
		tospawn += ACE_TEXT(" ") ;

		//we enable the callback used to forward the log if defined, default value otherwise
		ACE_TString tmp_forward = "logforward";	
		ACE_TString isCallbackEnabled = LOG_PREF::instance()->getPref(tmp_forward);
		if ( isCallbackEnabled == "0" ) {
			tospawn += "0";
		}
		else {
			tospawn += "1";
		}
		tospawn += ACE_TEXT(" ") ;

		//we add the path in where the log file should be saved
		//if none, we gave the default value NULL to disable the capability
		ACE_TString tmp_file = "logfile";
		ACE_TString log_file = LOG_PREF::instance()->getPref(tmp_file);
		if ( log_file.length() != 0 ) {
			const ACE_TCHAR *dirName = ACE::dirname(log_file.c_str());
			tospawn += dirName;
			tospawn += ACE_DIRECTORY_SEPARATOR_CHAR;	
		}
		else {
			tospawn += "NULL";
		}

		Thread_Args* args = new Thread_Args( tospawn.c_str() );
		ACE_Thread_Manager::instance()->spawn( (ACE_THR_FUNC) Thread_Server::run_svc, ACE_static_cast(void*, args),  THR_DETACHED | THR_SCOPE_SYSTEM );

		// If we are here, it means that there was no problem, we notify the client of the success     

		ACE_TString modif = xml.c_str();
		lm->addModifications(modif) ;	

		ACE_TString createxmlsuccess = create_xmlsuccess();
		return send_xml(createxmlsuccess.c_str());
	}

	//Consumer
	else
	{                       
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] Consumer\n")));  	


			Flow *f = NULL;

			if (gl->findFlow(flowid) == 0)
				// the flow already exists
				f = gl->getFlow(flowid);
			else {
				//the flow doesn't exist
				f = new Flow( flowgroup , flowtype );
				if (f == NULL )
				{
					dl->remove(flowid);
					ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error Flow Registration"));
					return send_xml(createxmlfailure.c_str());
				}     
				if (f->setName(flowname) != 0)
				{
					dl->remove(flowid);
					ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error Flow setname"));
					return send_xml(createxmlfailure.c_str());
				}      
				if (f->setBlockSize( ACE_OS::atoi( parser.getInfos(ACE_TEXT("blocksize")).c_str() )) != 0)
				{
					dl->remove(flowid);
					ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error Flow setBlockSize"));
					return send_xml(createxmlfailure.c_str());
				}      
				if (f->setBlockCount( ACE_OS::atoi( parser.getInfos(ACE_TEXT("blockcount")).c_str()) ) != 0)
				{
					dl->remove(flowid);
					ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error Flow setBlockCount"));
					return send_xml(createxmlfailure.c_str());
				}   
				
				//We are dealing with a consumer => nbConsumers MUST be 0
				//If not, we correct that on the fly and notify there was a problem in the XML message.
				if ( ACE_OS::atoi(nbConsumers.c_str()) != 0 )
					ACE_DEBUG((SF2D_WARNING_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] synchronized value from XML incorrect. Value corrected." )));
				if ( f->setNumberOfExpectedConsumers( 0 ) != 0)
				{
					ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : Error Flow setNumberOfExpectedConsumers"));
					return send_xml(createxmlfailure.c_str());
				}

				if (gl->addFlow(f) != 0)  
				{
					dl->remove(flowid);
					delete f;
					ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request :GlobalList addFlow"));
					return send_xml(createxmlfailure.c_str());
				} 
			} // end the flow doesn't exist

			// By Imad
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] Count Switched off Consumers : %d \n"),f->countSwitchedOffConsumers() ));

			ACE_TCHAR namelocal[256];
			if (ACE_OS::hostname(namelocal,256) ==1)
			{
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] ACE_OS::hostname \n")));
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler getflow_request : ACE_OS::hostname"));
				return send_xml(createxmlfailure.c_str());
			}
			ACE_TString localaddr = namelocal ;

			ACE_TString providerserverrequest = "";
			Client *c = gl->getClient(f->getProducer());
			if (c != NULL) {
				providerserverrequest = c->getServer();

				if ( (f->countSwitchedOffConsumersOnServer(localaddr) > 0) && (f->countSwitchedOnConsumersOnServer(localaddr) == 0) ) { 

					if (localaddr != providerserverrequest) { // we don't need it when it 's a local consumer
						//ACE_TCHAR wakeUpDupliRequestMessage[MAX_MESSAGE_SIZE]={0};
						int size_needed = sizeof(WAKE_UP_REQUEST_MESSAGE) + flowid.length() + localaddr.length() + 1;
						ACE_TCHAR *wakeUpDupliRequestMessage = (ACE_TCHAR *) ACE_OS::malloc(size_needed);
						ACE_OS::memset(wakeUpDupliRequestMessage, 0, size_needed);
						
						ACE_OS::sprintf(wakeUpDupliRequestMessage,WAKE_UP_REQUEST_MESSAGE,flowid.c_str(),localaddr.c_str()); 
						Peer_Event_Handler::send_xml_to_servers(sl->get(providerserverrequest, ServerList::NAME), wakeUpDupliRequestMessage);
						//send_xml_to_servers(sl->getServer(providerserverrequest), wakeUpDupliRequestMessage);

						ACE_OS::free(wakeUpDupliRequestMessage);
						// update
					} //end if (localaddr != providerserverrequest)
				}
			}

			


			// *******************************************************************************************************************************
			if (dl->find(flowid) == 0)
			{
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] Duplicator already exists for that flow\n" )));			
			}
			else
			{
				//there is no duplicator for this flow yet, we need to spawn it
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::getflow_request ] No Duplicator found for this flow, creation in progress...\n" )));    				

				if (dl->addConsumer(flowid) != 0)
				{
					ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : DuplicatorList addConsumer"));
					return send_xml(createxmlfailure.c_str());
				}												

				ACE_TString peeraddr = "";
				if (c != NULL) {
					ACE_TString serverName = c->getServer();
					ServerList *serverList = SERVER_LIST::instance();
					peeraddr = serverList->get(serverName, ServerList::NAME)->getIP();
					if ( peeraddr.length() == 0)
					{
						ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : GlobalList getServer"));
						return send_xml(createxmlfailure.c_str());
					}						
					peeraddr += ACE_TEXT(":");
					ACE_TCHAR porttosend[6] ;  
					ACE_OS::snprintf(porttosend, sizeof(porttosend), ACE_TEXT("%d"), gl->getFlow(flowid)->getPortNumber() ); 
					peeraddr += porttosend;														
				}
				else
					peeraddr = "noprovider";

			//we need to give a new parameter to the dupli to handle the synchronization
			//This duplicator is created to handle a consumer, so the synchronized value will be 0


				ACE_TString tospawn = ACE_TEXT("duplicator") ;
#if (defined (ACE_WIN32))
				tospawn += ACE_TEXT(".exe") ;
#endif
				tospawn += ACE_TEXT(" ") ;
				tospawn += flowid.c_str() ;
				tospawn += ACE_TEXT(" ") ;
				ACE_TCHAR dtype[4] = {0};
				ACE_OS::snprintf(dtype, sizeof(dtype), ACE_TEXT("%d"), CONSUMER );
				tospawn += dtype ;      
				tospawn += ACE_TEXT(" ") ;
				tospawn += parser.getInfos(ACE_TEXT("blocksize")).c_str();
				tospawn += ACE_TEXT(" ");
				tospawn +=  parser.getInfos(ACE_TEXT("blockcount")).c_str() ;
				tospawn += ACE_TEXT(" ") ;    
				tospawn += peeraddr.c_str() ; 
				tospawn += ACE_TEXT(" ") ;

				//we add the log severity defined when the server was launched
				//if none, we use the default one
				ACE_TString tmp_level = "loglevel";
				ACE_TString log_level = LOG_PREF::instance()->getPref(tmp_level);
				if ( log_level == "info" ) {
					tospawn += log_level.c_str();
				}
				else if ( log_level == "warning" ) {
					tospawn += log_level.c_str();
				}
				else {
					ACE_TString tmp_error = "error";
					tospawn += tmp_error.c_str();
				}	
				tospawn += ACE_TEXT(" ") ;

				//we add the syslog capabilities if defined, default value otherwise
				ACE_TString tmp_syslog = "syslog";	
				ACE_TString hasSyslog = LOG_PREF::instance()->getPref(tmp_syslog);
				if ( hasSyslog == "1" ) {
					tospawn += hasSyslog.c_str();
				}
				else {
					tospawn += "0";
				}
				tospawn += ACE_TEXT(" ") ;

				//we enable the callback used to forward the log if defined, default value otherwise
				ACE_TString tmp_forward = "logforward";	
				ACE_TString isCallbackEnabled = LOG_PREF::instance()->getPref(tmp_forward);
				if ( isCallbackEnabled == "0" ) {
					tospawn += "0";
				}
				else {
					tospawn += "1";
				}
				tospawn += ACE_TEXT(" ") ;

				//we add the path in where the log file should be saved
				//if none, we gave the default value NULL to disable the capability
				ACE_TString tmp_file = "logfile";
				ACE_TString log_file = LOG_PREF::instance()->getPref(tmp_file);
				if ( log_file.length() != 0 ) {
					const ACE_TCHAR *dirName = ACE::dirname(log_file.c_str());
					tospawn += dirName;
					tospawn += ACE_DIRECTORY_SEPARATOR_CHAR;	
				}
				else {
					tospawn += "NULL";
				}

				Thread_Args* args = new Thread_Args( tospawn.c_str() );
				ACE_Thread_Manager::instance()->spawn( (ACE_THR_FUNC) Thread_Server::run_svc, ACE_static_cast(void*, args),  THR_DETACHED | THR_SCOPE_SYSTEM );	
			} //end of duplicator creation


			//ServerSide Communication	        	        	    
			ACE_TString add_flow_request_server = add_flow_xml(
					parser.getInfos(ACE_TEXT("name")).c_str() ,
					parser.getInfos(ACE_TEXT("typeflow")).c_str() ,
					parser.getInfos(ACE_TEXT("group")).c_str() ,
					parser.getInfos(ACE_TEXT("args")).c_str() ,
					parser.getInfos(ACE_TEXT("blocksize")).c_str() , 
					parser.getInfos(ACE_TEXT("blockcount")).c_str() ,
					myclient->getID().c_str(),
					myclient->getServer().c_str(),
					0,
					parser.getInfos(ACE_TEXT("provider")).c_str(),
					"0" //this should always be 0 for a consume. f->getNumberOfExpectedConsumers() returns 0 for a consumer
					);	
					
			for (SERVER_ITERATOR si = sl->first(); si != sl->last(); si++) {
				Peer_Event_Handler::send_xml_to_servers((*si).int_id_, add_flow_request_server.c_str());
			}
			
			//for (SOCKET_ITERATOR si = sl->firstServer(); si != sl->lastServer(); si++)
			//{
			//	send_xml_to_servers(sl->getServer(SocketList::getIDFromIterator(si)) , add_flow_request_server.c_str());        
			//}   		

			if (gl->getFlow(flowid)->addConsumer( myclient->getID(), parser.getInfos(ACE_TEXT("name")).c_str() ) )
			{
				ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("Local_Handler GetFlow_Request : GlobalList addConsumer"));
				return send_xml(createxmlfailure.c_str());
			}

			ACE_TString modif ;
			modif = add_flow_request_server.c_str();
			//modif = xml.c_str() ;
			lm->addModifications(modif) ;	

			// If we are here, it means that there were no problem, we notify the client of the success	    			   
			ACE_TString createxmlsuccess = create_xmlsuccess();
			return send_xml(createxmlsuccess.c_str());
	}    
	return 0;
} 

int Local_Event_Handler::remflow_request(ACE_TString xml) {
	
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::remflow_request"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::remflow_request ]\n")));
	
	if (myclient == NULL)
	{
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("[ Local_Event_Handler::remflow_request ] Client not registered ]"));
		send_xml(createxmlfailure.c_str());
		return 0;
	}
	
	//first we remove the flow from the table if necessary
	XML_Parser parser(xml);
	ACE_TString flowType = (parser.getInfos(ACE_TEXT("typeflow"))).c_str();
	ACE_TString flowName = (parser.getInfos(ACE_TEXT("name"))).c_str();
	ACE_TString flowGroup = (parser.getInfos(ACE_TEXT("group"))).c_str();

	//Is the typeflow field present in the XML message?
	if (flowGroup.length() == 0)
	{
		//No, the flow has no group. It shouldn't happen.
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("[ Local_Event_Handler::remflow_request : Error XML Description ]"));
		send_xml(createxmlfailure.c_str());
		return -1;
	}
	
	//Is the group field present in the XML message?
	if (  flowType.length() == 0 )
	{
		//No, the flow has no type. It shouldn't happen.
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("[ Local_Event_Handler::remflow_request : Error XML Description ]"));
		send_xml(createxmlfailure.c_str());
		return -1;
	}

	//Is the name field present in the XML message?
	if (  flowName.length() == 0 )
	{
		//No, the flow has no name. It shouldn't happen.
		ACE_TString createxmlfailure = create_xmlfailure(ACE_TEXT("[ Local_Event_Handler::remflow_request : Error XML Description ]"));
		send_xml(createxmlfailure.c_str());
		return -1;
	}

	//We generate an ID for the flow from its name and group.
	ACE_TString theFlowid = Flow::getIDFromGroupAndType( flowGroup, flowType);
	GlobalList *gl = GLOBAL_LIST::instance();

	int result = dm.notify_duplicator_to_die(theFlowid, myclient->getID());
	if (result != 0)
	{
			ACE_ERROR((LM_ERROR, ACE_TEXT("[ Local_Event_Handler::receive_xml ] ERROR: notify_duplicators_to_die not successfull\n")));
			return -1;
	}
	
	if (gl->findFlow(theFlowid) == 0) {
		// the flow already exists
		Flow *f = gl->getFlow(theFlowid);

		//we removed the client from the flow's client list. 
		if ( f->getProducer() == myclient->getID() ) {
			f->unsetProducer();		
		}
		else {
			if ( f->containsConsumer(myclient->getID()) == 0)
				f->removeConsumer(myclient->getID());
		}
		
		if ( ( f->countConsumers() == 0 ) && ( f->getProducer() == "" ) ) {
			//the flow doesn't have any producer, nor consumers..
			//we remove it from the table.
			gl->removeFlow(theFlowid);
			delete f;
			
		}
	}
	
	//then we broadcast the change	
	//SocketList *sl = SOCKET_LIST::instance();  
	ServerList *sl = SERVER_LIST::instance();         
	ACE_TString removeFlowMessage = remove_flowxml( myclient->getName().c_str(), myclient->getGroup().c_str(), flowType, flowGroup);
	
	for (SERVER_ITERATOR si = sl->first(); si != sl->last(); si++) {
		Peer_Event_Handler::send_xml_to_servers((*si).int_id_, removeFlowMessage.c_str());
	}
	//for (SOCKET_ITERATOR si = sl->firstServer(); si != sl->lastServer(); si++)
	//{	
	//	send_xml_to_servers(sl->getServer(SocketList::getIDFromIterator(si)) , removeFlowMessage.c_str());        
	//}   
	
	//If we are here, that means the flow has been properly removed from the tables.
	//We add the changes in the last modification list.
	LastModifications *lm = LAST_MODIFICATIONS::instance();
	ACE_TString modif ;
	modif = removeFlowMessage.c_str() ;
	lm->addModifications(modif) ;
		
	//the client node is notified that the flow has been successfully removed
	ACE_TString createxmlsuccess = create_xmlsuccess();
	return send_xml(createxmlsuccess.c_str());
}

int Local_Event_Handler::logAnnounce(ACE_TString xml) {
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::logAnnounce"));
	//we just get a log message from a client node.
	
	//1) Create a list of server handling at least one 'admin' client
	//2) Send the log to each server of this list

	//ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Local_Event_Handler::logAnnounce ] XML=%s\n"),xml.c_str()));

	XML_Parser parser(xml);
	
	//we need to reformat the message to include the name and group of the client node. 
	ACE_TString messages = "";
	ACE_TString mess = "";
	std::vector<ACE_TString> logs = parser.getLogMessages();
	for ( std::vector<ACE_TString>::iterator iter = logs.begin(); iter != logs.end(); iter++ ) {
	  mess = Log_Event_Handler::create_XML_message(*iter);
	  messages += mess;
	}

	int size_needed = sizeof(LOG_MESSAGE_CLIENT) + myclient->getName().length() + myclient->getGroup().length() + messages.length() + 1;
	ACE_TCHAR *reworkedMessage = (ACE_TCHAR *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(reworkedMessage, 0, size_needed);
	//char reworkedMessage[MAX_MESSAGE_SIZE]={0};
	ACE_OS::sprintf(reworkedMessage, LOG_MESSAGE_CLIENT, myclient->getName().c_str(), myclient->getGroup().c_str(), messages.c_str());

	std::set<ACE_TString> tmp_serverList;
	tmp_serverList.clear();
	
	//1) Create a list of server handling at least one 'admin' client
	GlobalList *gl = GLOBAL_LIST::instance();
	for (CLIENT_ITERATOR ci = gl->getFirstClient(); ci != gl->getLastClient(); ci++) {
		Client *client = gl->getClientFromIterator(ci);
		if ( client != NULL ) {
			ACE_TString clientGroup =  client->getGroup();
			if ( clientGroup == ACE_TEXT("admin") ) {
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::logAnnounce ] ADMIN FOUND, server=%s\n"),client->getServer().c_str()));
				tmp_serverList.insert(client->getServer());
			}
		}
	}

	std::set<ACE_TString>::iterator iter;
	//SocketList *sl = SOCKET_LIST::instance();  
	ServerList *sl = SERVER_LIST::instance();
	
	for (SERVER_ITERATOR si = sl->first(); si != sl->last(); si++) {
		iter = tmp_serverList.find((*si).int_id_->getName());
		if ( iter != tmp_serverList.end() ) {
			Peer_Event_Handler::send_xml_to_servers((*si).int_id_, reworkedMessage);
		}
	}
	
	//for (SOCKET_ITERATOR si = sl->firstServer(); si != sl->lastServer(); si++)
	//{	
	//	iter = tmp_serverList.find(SocketList::getIDFromIterator(si));
	//	if ( iter != tmp_serverList.end()) {
	//		send_xml_to_servers(sl->getServer(SocketList::getIDFromIterator(si)) , reworkedMessage);
	//	}    
	//}   

	//We add the changes in the last modification list.
	LastModifications *lm = LAST_MODIFICATIONS::instance();
	ACE_TString modif ;
	modif = reworkedMessage;
	lm->addModifications(modif);

	ACE_OS::free(reworkedMessage);

	ACE_TString createxmlsuccess = create_xmlsuccess();
	send_xml(createxmlsuccess.c_str());

	return 0;
}

int Local_Event_Handler::send_xml(ACE_TString xml)
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::send_xml"));
	char size[512] = {0};
	ACE_OS::snprintf(size, sizeof(size), "%d", xml.length());
	remote_peer_.send_n(size, sizeof(size));

	const char *xml_message = ACE_TEXT_ALWAYS_CHAR(xml.c_str());

	remote_peer_.send_n(xml_message, ACE_OS::strlen(xml_message));	
	
	return 0;
}

/*int Local_Event_Handler::send_xml_to_servers(ACE_SOCK_Stream *peer_server, ACE_TString xml)
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::send_xml_to_servers"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::send_xml_to_servers ]\n")));	
	char size[512] = {0};
	ACE_OS::snprintf(size, sizeof(size), "%d", xml.length());
	peer_server->send_n(size, sizeof(size));

    const char *xml_message = ACE_TEXT_ALWAYS_CHAR(xml.c_str());
    peer_server->send_n(xml_message, ACE_OS::strlen(xml_message));

	return 0;
}*/

/*int Local_Event_Handler::send_xml_to_servers(Server *peer_server, ACE_TString xml) {
	
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::send_xml_to_servers"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::send_xml_to_servers ]\n")));
	if ( (peer_server != NULL) && (peer_server->getHandler() != NULL) ) {
		ACE_SOCK_Stream *sock = &(peer_server->getHandler()->peer());
		if ( sock ) {
			//here we are trying to send, if it is not successful,
			//we test for EWOULDBLOCK, we queue the message to send it later.
			char size[512] = {0};
			ACE_OS::snprintf(size, sizeof(size), "%d", xml.length());
			const char *xml_message = ACE_TEXT_ALWAYS_CHAR(xml.c_str());
			ssize_t send_cnt = sock->send(size, sizeof(size));
			if ( send_cnt == sizeof(size) ) {
				//we successfully sent the size, we know send the message itself
				send_cnt = sock->send(xml_message, ACE_OS::strlen(xml_message));
				ssize_t messageLength = ACE_static_cast(size_t, ACE_OS::strlen(xml_message));
				if ( send_cnt == messageLength )
					//success, the message has been sent properly
					return 0;
				else if ( send_cnt == -1 && ACE_OS::last_error() != EWOULDBLOCK )
					//something really wrong is happening...
					return -1;
				
				//the socket is not closed but the message couldn't be send properly
				if ( send_cnt == -1 )
					//nothing has been sent so far... we adjust send_cnt to reflect the proper number of byte sent so far
					send_cnt = 0;
				
				//we know need to queue the message to send it later. 
				size_t remaining = ACE_static_cast(size_t, (ACE_OS::strlen(xml_message) - send_cnt));
				return peer_server->getHandler()->putMessageInQueue(xml_message, remaining, true);
			}
			else if ( send_cnt == -1 && ACE_OS::last_error() != EWOULDBLOCK )
				//something really wrong is happening...
				return -1;
			//the socket is not closed but the message couldn't be send properly
			if ( send_cnt == -1 )
				//nothing has been sent so far... we adjust send_cnt to reflect the proper number of byte sent so far
				send_cnt = 0;
			//we know need to queue the message to send it later. 
			size_t remaining = ACE_static_cast(size_t, (sizeof(size) - send_cnt));
			return peer_server->getHandler()->putMessageInQueue(xml_message, remaining, false);	
		}
	}
	return -1;
}*/


ACE_TString Local_Event_Handler::receive_xml()
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::receive_xml"));
	ACE_TString doc;
	int result;
	char size_[512] = {0};  
	if (remote_peer_.recv_n(size_,sizeof (size_) ) != sizeof (size_) )
	{     
		if ( myclient == NULL )
		{
			doc = ACE_TEXT("");
			return doc;	
		} 

		//Local List
		GlobalList *gl = GLOBAL_LIST::instance();                

		result = dm.notify_duplicators_to_die_local(myclient->getID());
		if (result != 0)
		{
			ACE_ERROR((LM_ERROR, ACE_TEXT("[ Local_Event_Handler::receive_xml ] ERROR: notify_duplicators_to_die not successfull\n")));
		}

		result = gl->removeClient(myclient->getID());
		if (result != 0)
		{
			ACE_ERROR((LM_ERROR, ACE_TEXT("[ Local_Event_Handler::receive_xml ] ERROR: removeClient not successfull\n")));
		}

		//Remote List 
		SocketList *sol = SOCKET_LIST::instance();  
		ServerList *sel = SERVER_LIST::instance();  
		ACE_TString remove_client_announce_server = remove_clientxml( myclient->getName().c_str(),myclient->getGroup().c_str());
		
		for (SERVER_ITERATOR si = sel->first(); si != sel->last(); si++) {
			Peer_Event_Handler::send_xml_to_servers((*si).int_id_, remove_client_announce_server.c_str());
		}

		LastModifications *lm = LAST_MODIFICATIONS::instance();
		ACE_TString modif = remove_client_announce_server.c_str();;
		lm->addModifications(modif) ;	

		if (myclient->getGroup() == ACE_TEXT("admin"))
		{
			lm->remove(myclient->getID());
		}
		
		ACE_SOCK_Stream *sock_client = sol->getClient(myclient->getID());
		sol->removeClient(myclient->getID()) ; 		
		if ( sock_client ) //  ACE_SOCK_STream was built in Local2_Acceptor
			delete sock_client;
			
		ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("Client '%s' removed\n"),myclient->getID().c_str()));  
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::receive_xml ] Client %s disconnected\n"),myclient->getName().c_str())); 
		delete myclient;
		myclient = NULL;
		doc = ACE_TEXT("");
		return doc;  
	}

	ACE_TString size = ACE_TEXT_CHAR_TO_TCHAR(size_);
	int length = ACE_OS::atoi(size.c_str()); 
	if (length == 0)
	{  	  
		doc = ACE_TEXT("");
		return doc;
	}

	char *xml = new char[length+1];
	ACE_OS::memset(xml,0,length + 1);

	if ( remote_peer_.recv_n(xml, length) != length)
	{
		if ( myclient == NULL )
		{
			doc = ACE_TEXT("");
			return doc;	
		} 

		//Local List
		GlobalList *gl = GLOBAL_LIST::instance(); 
		//Should we test if there are still some consumers connected to the duplicator before killing it?
		result = dm.notify_duplicators_to_die_local(myclient->getID());
		if (result != 0)
		{
			ACE_ERROR((LM_ERROR, ACE_TEXT("[ Local_Event_Handler::receive_xml ] ERROR: notify_duplicators_to_die not successfull\n")));
		}

		result = gl->removeClient(myclient->getID());
		if (result != 0)
		{
			ACE_ERROR((LM_ERROR, ACE_TEXT("[ Local_Event_Handler::receive_xml ] ERROR: removeClient not successfull\n")));
		}        
		//Remote List 
		SocketList *sol = SOCKET_LIST::instance(); 
		ServerList *sel = SERVER_LIST::instance();
		
		ACE_TString remove_client_announce_server = remove_clientxml( myclient->getName().c_str(),myclient->getGroup().c_str());
		
		for (SERVER_ITERATOR si = sel->first(); si != sel->last(); si++) {
			Peer_Event_Handler::send_xml_to_servers((*si).int_id_, remove_client_announce_server.c_str());
		}

		LastModifications *lm = LAST_MODIFICATIONS::instance();
		ACE_TString modif  = remove_client_announce_server.c_str();
		lm->addModifications(modif) ;	

		if (myclient->getGroup() == ACE_TEXT("admin"))
		{
			lm->remove(myclient->getID());
		}

		ACE_SOCK_Stream *sock_client = sol->getClient(myclient->getID());
		sol->removeClient(myclient->getID()) ; 		
		if ( sock_client ) //  ACE_SOCK_STream was built in Local2_Acceptor
			delete sock_client;
			
		ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Local_Event_Handler::receive_xml ] Client %s disconnected\n"),myclient->getName().c_str()));        
		delete myclient;
		myclient = NULL;
		doc = ACE_TEXT("");
		return doc;
	}

	doc = ACE_TEXT_CHAR_TO_TCHAR(xml); // copy of the data into the ACE_TString object

	delete xml;
	return doc;
}

// The following is the different kinds of XML messages we can send 
ACE_TString Local_Event_Handler::create_xmlsuccess()
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::create_xmlsuccess"));
	ACE_TString doc; 
	doc = ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n<type>2</type>\n<target/>\n<code>Successful</code>\n</smartspace>");
	return doc;
}

ACE_TString Local_Event_Handler::create_xmlfailure(ACE_TString des_)
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::create_xmlfailure"));
	ACE_TString doc ;
	doc += ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n<type>2</type>\n<target/>\n<code>Failure</code>\n<description>");
	doc += des_ ;
	doc += ACE_TEXT("</description>\n</smartspace>") ;
	return doc;

}

ACE_TString Local_Event_Handler::remove_clientxml(ACE_TString name_, ACE_TString group_ )
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::remove_clientxml"));
	ACE_TString doc ;
	doc += ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n<type>4</type>\n<target/>\n<clientname>");
	doc += name_ ;
	doc += ACE_TEXT("</clientname>\n<clientgroup>");
	doc += group_;
	doc += ACE_TEXT("</clientgroup>\n</smartspace>");
	return doc; 
}

ACE_TString Local_Event_Handler::remove_flowxml(ACE_TString clientname_, ACE_TString clientgroup_ ,ACE_TString flowtype_, ACE_TString flowgroup_ )
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::remove_flowxml"));
	ACE_TString doc ;
	doc += ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n<type>1</type>\n<target>12</target>\n<clientname>");
	doc += clientname_ ;
	doc += ACE_TEXT("</clientname>\n<clientgroup>");
	doc += clientgroup_;
	doc += ACE_TEXT("</clientgroup>\n<flowtype>");
	doc += flowtype_;
	doc += ACE_TEXT("</flowtype>\n<flowgroup>");
	doc += flowgroup_;
	doc += ACE_TEXT("</flowgroup>\n</smartspace>");
	return doc; 
}

/*ACE_TString Local_Event_Handler::remove_flowxml(ACE_TString flowID_, ACE_TString clientID_ )
{
	ACE_TString doc ;
	doc += ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n<type>1</type>\n<target>12</target>\n<flowID>");
	doc += flowID_ ;
	doc += ACE_TEXT("</flowID>\n<clientID>");
	doc += clientID_;
	doc += ACE_TEXT("</clientID>\n</smartspace>");
	return doc; 
}*/

ACE_TString Local_Event_Handler::create_xmllastmodifications()
{	
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::create_xmllastmodifications"));
	ACE_TString doc ;
	doc += ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n<type>2</type>\n<target>17</target>\n<ModificationsList>");
	ACE_Unbounded_Queue_Iterator<ACE_TString> iter(myclient->modifications);
	while (!iter.done ())
	{	
		doc += ACE_TEXT("\n<modif>\n");
		ACE_TString *tmp = NULL;
		iter.next(tmp);
		doc += tmp->c_str() ;
		doc += ACE_TEXT("\n</modif>\n");
		iter.advance();
	}
	doc += ACE_TEXT("</ModificationsList>\n</smartspace>");
	return doc;
}

ACE_TString Local_Event_Handler::create_xmlserverlistresponse()
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::create_xmlserverlistresponse"));
	ACE_TCHAR namelocal[256];
	if (ACE_OS::hostname(namelocal,256) == 1)
	{
		ACE_DEBUG((SF2D_WARNING_PREFIX ACE_TEXT("[ Local_Event_Handler::create_xmlserverlistresponse ] Problem : ACE_OS::hostname\n")));
		ACE_OS::strcpy(namelocal,ACE_TEXT("localhost"));
	} 
	ACE_TString doc ;
	doc += ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n<type>2</type>\n<target>14</target>\n<Serverlist>\n");

	doc += ACE_TEXT("<server>\n<name>");
	doc += namelocal;
	doc += ACE_TEXT("</name>\n<os>");
	
	#if (defined (ACE_WIN32))
		doc += ACE_TEXT("winxp");
	#elif (defined (ACE_CONFIG_MACOSX_TIGER_H))
		doc += ACE_TEXT("OSX Tiger");
	#elif (defined (ACE_CONFIG_MACOX_LEOPARD_H))
		doc += ACE_TEXT("OSX Leopard)";
	#else
		doc += ACE_TEXT("linux");
	#endif

	doc += ACE_TEXT("</os>\n<port>1714</port>\n</server>");	
	ServerList *sl = SERVER_LIST::instance(); 
	for (SERVER_ITERATOR si = sl->first(); si != sl->last(); si++)
	{
		doc += ACE_TEXT("\n<server>\n<name>");
		doc += (*si).int_id_->getName().c_str();
		doc += ACE_TEXT("</name>\n<os>");
		doc += (*si).int_id_->getOS().c_str();
		doc += ACE_TEXT("</os>\n<port>1714</port>\n</server>");
	}

	doc += ACE_TEXT("\n</Serverlist>\n</smartspace>");
	return doc;

}

ACE_TString Local_Event_Handler::create_xmlflowlistresponse()
{ 
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::create_xmlflowlistresponse"));
	GlobalList *gl = GLOBAL_LIST::instance();
	ACE_TString doc = ACE_TEXT("") ;
	doc += ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n<type>2</type>\n<target>10</target>\n");
	doc +=  gl->toXml().c_str() ;
	doc += ACE_TEXT("</smartspace>");
	return doc;
}

ACE_TString Local_Event_Handler::add_flow_xml(ACE_TString name_, ACE_TString typeflow_, ACE_TString group_, ACE_TString args_, ACE_TString blocksize_, ACE_TString blockcount_, ACE_TString clientid_, ACE_TString serverid_, int port_, ACE_TString provider_, ACE_TString nbConsumersExpected_)
{
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::add_flow_xml"));
	ACE_TString doc ;
	doc += ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n<type>1</type>\n<target>11</target>\n");

	doc += ACE_TEXT("<typeflow>");
	doc += typeflow_  ;
	doc += ACE_TEXT("</typeflow>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<name>");
	doc += name_  ;
	doc += ACE_TEXT("</name>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<group>");
	doc += group_  ;
	doc += ACE_TEXT("</group>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<args>");
	doc += args_  ;
	doc += ACE_TEXT("</args>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<clientid>");
	doc += clientid_  ;
	doc += ACE_TEXT("</clientid>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<serverid>");
	doc += serverid_  ;
	doc += ACE_TEXT("</serverid>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<provider>");
	doc += provider_  ;
	doc += ACE_TEXT("</provider>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<blocksize>");
	doc += blocksize_  ;
	doc += ACE_TEXT("</blocksize>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<blockcount>");
	doc += blockcount_  ;
	doc += ACE_TEXT("</blockcount>");

	doc += ACE_TEXT("\n") ;

	ACE_TCHAR porttosend[6] ;
	ACE_OS::snprintf(porttosend, sizeof(porttosend), ACE_TEXT("%d"),port_ );

	doc += ACE_TEXT("<port>");
	doc += porttosend ;
	doc += ACE_TEXT("</port>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<synchronized>");
	doc += nbConsumersExpected_ ;
	doc += ACE_TEXT("</synchronized>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("</smartspace>");
	return doc;
}

//Duplicator_Event_Handler implementation

ACE_HANDLE Duplicator_Event_Handler::get_handle(void) const {
	ACE_TRACE(ACE_TEXT("Duplicator_Event_Handler::get_handle"));
	return peer_.get_handle();
}

int Duplicator_Event_Handler::open(){
	ACE_TRACE(ACE_TEXT("Duplicator_Event_Handler::open"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::open ]\n")));
	return (reactor()->register_handler(this,ACE_Event_Handler::READ_MASK));
}

int Duplicator_Event_Handler::handle_close(ACE_HANDLE, ACE_Reactor_Mask){
	ACE_TRACE(ACE_TEXT("Duplicator_Event_Handler::handle_close"));
	peer_.close();
	return 0;
}

int Duplicator_Event_Handler::handle_input(ACE_HANDLE)
{
	ACE_TRACE(ACE_TEXT("Duplicator_Event_Handler::handle_input"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::handle_input ]\n")));	
	ACE_TString doc = receive_xml();

	if (doc != ACE_TEXT(""))
	{
		if (actions_switch(doc) != 0)
			return -1;
		else
			return 0;
	}
	else
		return -1;
}

int Duplicator_Event_Handler::send_xml(ACE_TString xml)
{
	ACE_TRACE(ACE_TEXT("Duplicator_Event_Handler::send_xml"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::send_xml ] \n")));
	char size[512] = {0};
	ACE_OS::snprintf(size, sizeof(size), "%d", xml.length());

	unsigned int result;

	//first we send the size of the message 
	result = peer_.send_n(size, sizeof(size));

	if ( result < sizeof(size) ) {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::send_xml_to_servers ] Error sending message size to server \n")));
		return -1;
	}

	const char *xml_message = ACE_TEXT_ALWAYS_CHAR(xml.c_str());
	
	//then we send the message itself 
	result = peer_.send_n(xml_message, ACE_OS::strlen(xml_message));	
	if ( result < ACE_OS::strlen(xml_message) ) {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::send_xml_to_servers ] Error sending message to server \n")));
		return -1;
	}

	return 0;
}

//now use unique static method instead
/*int Duplicator_Event_Handler::send_xml_to_servers(Server *peer_server, ACE_TString xml) {
		ACE_TRACE(ACE_TEXT("Local_Event_Handler::send_xml_to_servers"));
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Local_Event_Handler::send_xml_to_servers ]\n")));
		if ( (peer_server != NULL) && (peer_server->getHandler() != NULL) ) {
			ACE_SOCK_Stream *sock = &(peer_server->getHandler()->peer());
			if ( sock ) {
				//here we are trying to send, if it is not successful,
				//we test for EWOULDBLOCK, we queue the message to send it later.
				char size[512] = {0};
				ACE_OS::snprintf(size, sizeof(size), "%d", xml.length());
				const char *xml_message = ACE_TEXT_ALWAYS_CHAR(xml.c_str());
				ssize_t send_cnt = sock->send(size, sizeof(size));
				if ( send_cnt == sizeof(size) ) {
					//we successfully sent the size, we know send the message itself
					send_cnt = sock->send(xml_message, ACE_OS::strlen(xml_message));
					ssize_t messageLength = ACE_static_cast(size_t, ACE_OS::strlen(xml_message));
					if ( send_cnt == messageLength )
						//success, the message has been sent properly
						return 0;
					else if ( send_cnt == -1 && ACE_OS::last_error() != EWOULDBLOCK )
						//something really wrong is happening...
						return -1;

					//the socket is not closed but the message couldn't be send properly
					if ( send_cnt == -1 )
						//nothing has been sent so far... we adjust send_cnt to reflect the proper number of byte sent so far
						send_cnt = 0;

					//we know need to queue the message to send it later. 
					size_t remaining = ACE_static_cast(size_t, (ACE_OS::strlen(xml_message) - send_cnt));
					return peer_server->getHandler()->putMessageInQueue(xml_message, remaining, true);
			}
			else if ( send_cnt == -1 && ACE_OS::last_error() != EWOULDBLOCK )
				//something really wrong is happening...
				return -1;
			//the socket is not closed but the message couldn't be send properly
			if ( send_cnt == -1 )
				//nothing has been sent so far... we adjust send_cnt to reflect the proper number of byte sent so far
				send_cnt = 0;
			//we know need to queue the message to send it later. 
			size_t remaining = ACE_static_cast(size_t, (sizeof(size) - send_cnt));
			return peer_server->getHandler()->putMessageInQueue(xml_message, remaining, false);	
		}
	}
	return -1;
}*/
						
/*int Duplicator_Event_Handler::send_xml_to_servers(ACE_SOCK_Stream *peer_server, ACE_TString xml)
{
	ACE_TRACE(ACE_TEXT("Duplicator_Event_Handler::send_xml_to_servers"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[  Duplicator_Event_Handler::send_xml_to_servers ]\n")));	
	char size[512] = {0};
	ACE_OS::snprintf(size, sizeof(size), "%d", xml.length());

	unsigned int result;
	
	//first we send the size of the message 
	result = peer_server->send_n(size, sizeof(size));
	if ( result < sizeof(size) ) {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::send_xml_to_servers ] Error sending message size to server \n")));
		return -1;
	}

    const char *xml_message = ACE_TEXT_ALWAYS_CHAR(xml.c_str());
    
	//then we send the message itself 
	result = peer_server->send_n(xml_message, ACE_OS::strlen(xml_message));
	if ( result < ACE_OS::strlen(xml_message) ) {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::send_xml_to_servers ] Error sending message to server \n")));
		return -1;
	}

	return 0;
}*/

ACE_TString Duplicator_Event_Handler::receive_xml()
{
	ACE_TRACE(ACE_TEXT("Duplicator_Event_Handler::receive_xml"));
	//old way to receiver XML
	/*
#if (!(defined (ACE_WIN32)))
	if ( peer_.recv(flowID, 512 * sizeof(ACE_TCHAR)) <= 0 ) {
#else
	if ( peer_.recv_n(flowID, 512 * sizeof(ACE_TCHAR)) <= 0 ) {
#endif

	// check if no bytes were received or an error occurred
		ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ Duplicator_Acceptor_Event_Handler::handle_input ] ERROR: no bytes were received or an error occurred\n")), -1);
	}

	//ACE_DEBUG((LM_DEBUG, "[ Duplicator_Acceptor_Event_Handler::handle_input ] Duplicator connection successful for flow '%s'\n", flowID));

	ACE_DEBUG ((LM_DEBUG,"[ Duplicator_Acceptor_Event_Handler::handle_input ] END \n")); 
	return 0;
	*/

	ACE_TString doc;
	char size_[512] = {0};
	
	//first we get the message size
	if (peer_.recv_n(size_,sizeof (size_) ) != sizeof (size_) )
	{    
		//what should we do in case of communication error?
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::receive_xml() ] ERROR: no bytes were received or an error occurred\n")));
		doc = ACE_TEXT("");
		return doc;
	}

	//we check the message size is correct
	ACE_TString size = ACE_TEXT_CHAR_TO_TCHAR(size_);
	int length = ACE_OS::atoi(size.c_str()); 
	if (length == 0)
	{  	  
		doc = ACE_TEXT("");
		return doc;
	}

	//we allocate a buffer to contain the message
	char *xml = new char[length+1];
	ACE_OS::memset(xml,0,length + 1);

	//then we get the message itself
	if ( peer_.recv_n(xml, length) != length )
	{
		//what should we do in case of communication error?
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::receive_xml() ] ERROR: no bytes were received or an error occurred\n")));
		doc = ACE_TEXT("");
		return doc;
	}
	
	doc = ACE_TEXT_CHAR_TO_TCHAR(xml); // copy of the data into the ACE_TString object

	delete xml;
	return doc;
}

int Duplicator_Event_Handler::actions_switch(ACE_TString xml)
{
	ACE_TRACE(ACE_TEXT("Duplicator_Event_Handler::actions_switch"));
	//ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::actions_switch ] \n %s \n"), xml.c_str()));	
	//It is very risky to display the xml message here if the logforward option is enabled
	//As a result, the XML message containing logs entry (bounded with CDATA tags) could contain messages bounded with CDATA tags
	//So the whole xml message becomes invalid and cannot be parsed by the Control Center.
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::actions_switch ]\n")));
	int type = 0;
	int target = 0;

	XML_Parser parser(xml);

	if (parser.getInfos(ACE_TEXT("type")) == ACE_TEXT(""))
		return -1; //Socket shutdown
	else
		type = ACE_OS::atoi( parser.getInfos(ACE_TEXT("type")).c_str() );

	if (!(parser.getInfos(ACE_TEXT("target")) == ACE_TEXT("")))
		target = ACE_OS::atoi( parser.getInfos(ACE_TEXT("target")).c_str() ); 

	switch (type) 
	{
	case ANNOUNCE:
		  switch (target)
		  {
			case CONSUMER_CONNECTED:		  
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::actions_switch ] An new consumer client node is connected to the duplicator\n")));
				announceNewConsumerNodeConnectedToDuplicator(xml);
				break;
			case NB_DUPLICATOR_CONNECTED:
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::actions_switch ] A new remote duplicator is connected to the duplicator\n")));
				announceSuccessulConnectionToRemoteDuplicator(xml);
				break;
			case LOG_FROM_DUPLI :
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::actions_switch  ][LOG] \n")));
				logAnnounce(xml);
				break;
			case SOCKET_FOR_REMOTE_CONSUMERS_OPENED:
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::actions_switch ] A duplicator provider succesfully opened a port for remote consumers to connect to\n")));
				updateFlowPortNumber(xml);
				break;
			default:
				ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::actions_switch  ] Invalid Request \n")));
				return -1; 
				break;
		  }
		  break;
	case CONNECT:
		  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::actions_switch  ] A Local duplicator is connecting \n")));
		  return connect(xml);
		  break;
	default:
		  ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::actions_switch  ] Invalid Request \n")));
		  return -1;
		  break;
	}

	return 0; 
}

int Duplicator_Event_Handler::announceNewConsumerNodeConnectedToDuplicator(ACE_TString xml) {
	ACE_TRACE(ACE_TEXT("Duplicator_Event_Handler::announceNewConsumerNodeConnectedToDuplicator"));
	//the server need to store this information somewhere and broadcast it if necessary
	
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceNewConsumerNodeConnectedToDuplicator ]\n")));
    
	//first we get the infos from the XML file
	XML_Parser parser(xml);

	ACE_TString clientID = (parser.getInfos(ACE_TEXT("clientID"))).c_str();
	ACE_TString flowID = (parser.getInfos(ACE_TEXT("flowID"))).c_str();
	
	//then we check the info are correct
	if (  flowID.length() == 0 )
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceNewConsumerNodeConnectedToDuplicator ] flowID.length==0\n")));	
		return -1;	
	}	
	if (  clientID.length() == 0 )
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceNewConsumerNodeConnectedToDuplicator ] clientID.length==0\n")));	
		return -1;	
	}	

	//then we update the tables
	//First we find the flow
	GlobalList *gl = GLOBAL_LIST::instance();
	if ( !gl ) {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceNewConsumerNodeConnectedToDuplicator ] GLOBAL_LIST::instance()==NULL\n")));	
		return -1;	
	}
	
	if ( gl->findFlow(flowID) == -1 ) {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceNewConsumerNodeConnectedToDuplicator ] Flow Not Found\n")));	
		return -1;
	}
	
	Flow *tmp_flow = gl->getFlow(flowID);
	if ( tmp_flow ) {
		if ( tmp_flow->containsConsumer(clientID) == -1 ) {
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceNewConsumerNodeConnectedToDuplicator ] Consumer Not Found %s\n"), clientID.c_str()));	
			return -1;
		}
		else {
			//if we are here, the flow and the consumer exists in the list
			tmp_flow->setConsumerConnectionToDuplicatorVerified(clientID, 1);
			
			//the table of our local server is now up-to-date
			ServerList *sl = SERVER_LIST::instance();
			if ( sl == NULL ) {
				ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceNewConsumerNodeConnectedToDuplicator ] SERVER_LIST::instance()==NULL\n")));	
				return -1;
			}
			else {
				//we now send the changes to every other servers. We just forward the announce.
				for (SERVER_ITERATOR si = sl->first(); si != sl->last(); si++) 
				{
					Peer_Event_Handler::send_xml_to_servers((*si).int_id_, xml.c_str());
				}
			 
				//HERE WE NEED TO CHECK IF THE FLOW IS SYNCHRONIZED AND IF WE HAVE TO START THE TRANSFER
				if ( ( tmp_flow->getNumberOfExpectedConsumers() > 0 ) && ( tmp_flow->isSynchronizedFlowMustBeStarted() == 0 ) ) {			
					notifyDuplicatorToStartDataTransfer();
				}

				//the job is done :)
				return 0;
			}
		}
	}
	else {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceNewConsumerNodeConnectedToDuplicator ] flow==NULL\n")));
		return -1;
	}
}

int Duplicator_Event_Handler::announceSuccessulConnectionToRemoteDuplicator(ACE_TString xml) {
	ACE_TRACE(ACE_TEXT("Duplicator_Event_Handler::announceSuccessulConnectionToRemoteDuplicator"));
	//the server need to store this information somewhere
	//this info will never be broadcasted, because it is only useful for the server handling a duplicator provider

	//If we receive this message, that means the server handles a duplicator provider with a synchronized flow

	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceSuccessulConnectionToRemoteDuplicator ]\n %s"),xml.c_str()));

	//first we get the infos from the XML file
	XML_Parser parser(xml);

	ACE_TString flowID = (parser.getInfos(ACE_TEXT("flowID"))).c_str();
	ACE_TString nbRemoteConsumers = (parser.getInfos(ACE_TEXT("nbRemoteConsumers"))).c_str();
	
	//then we check the info are correct
	if (  flowID.length() == 0 )
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceSuccessulConnectionToRemoteDuplicator ] flowID.length==0\n")));	
		return -1;	
	}	
	if (  nbRemoteConsumers.length() == 0 )
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceSuccessulConnectionToRemoteDuplicator ] nbRemoteConsumers.length==0\n")));	
		return -1;	
	}

	//then we update the tables
	//First we find the flow
	GlobalList *gl = GLOBAL_LIST::instance();
	if ( !gl ) {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceNewConsumerNodeConnectedToDuplicator ] GLOBAL_LIST::instance()==NULL\n")));	
		return -1;	
	}
	
	if ( gl->findFlow(flowID) == -1 ) {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceNewConsumerNodeConnectedToDuplicator ] Flow Not Found\n")));	
		return -1;
	}
	
	Flow *tmp_flow = gl->getFlow(flowID);
	if ( tmp_flow ) {
		//here we must update the number of remote consumer (i.e duplicators) connected to the local duplicator.
		tmp_flow->setNumberOfRemoteConsumersConnectedToLocalDuplicator(ACE_OS::atoi(nbRemoteConsumers.c_str()));
		
		//Here we need to check if we have to start the flow.
		if ( ( tmp_flow->getNumberOfExpectedConsumers() > 0 ) && ( tmp_flow->isSynchronizedFlowMustBeStarted() == 0 ) )
			notifyDuplicatorToStartDataTransfer();
		ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceSuccessulConnectionToRemoteDuplicator ] Duplicator notified to start data transfer\n")));
		return 0;
	}
	else {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::announceSuccessulConnectionToRemoteDuplicator ] flow==NULL\n")));
		return -1;
	}
}

int Duplicator_Event_Handler::logAnnounce(ACE_TString xml) {
	ACE_TRACE(ACE_TEXT("Local_Event_Handler::logAnnounce"));
	//we just get a log message from a duplicator.
	//ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::log ] xml=\n %s"), xml.c_str()));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::logAnnounce ]\n")));
	//we need to send the message to the peer servers, which have an admin client.	
	
	//1) Create a list of server handling at least one 'admin' client
	//2) Send the log to each server of this list

	std::set<ACE_TString> tmp_serverList;
	tmp_serverList.clear();
	
	//1) Create a list of server handling at least one 'admin' client
	GlobalList *gl = GLOBAL_LIST::instance();
	for (CLIENT_ITERATOR ci = gl->getFirstClient(); ci != gl->getLastClient(); ci++) {
		Client *client = gl->getClientFromIterator(ci);
		if ( client != NULL ) {
			ACE_TString clientGroup =  client->getGroup();
			if ( clientGroup == ACE_TEXT("admin") ) {
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::logAnnounce ] ADMIN FOUND, server=%s\n"),client->getServer().c_str()));
				tmp_serverList.insert(client->getServer());
			}
			/*else
				ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::logAnnounce ] ADMIN NOT FOUND\n")));*/
		}
	}

	std::set<ACE_TString>::iterator iter;
	ServerList *sl = SERVER_LIST::instance();
	for (SERVER_ITERATOR si = sl->first(); si != sl->last(); si++) 
	{
		iter = tmp_serverList.find((*si).int_id_->getName());
		if ( iter != tmp_serverList.end()) {
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::logAnnounce ] Log sent to server %s\n"), (*si).int_id_->getName().c_str()));
			Peer_Event_Handler::send_xml_to_servers((*si).int_id_, xml.c_str());
		}  
	}

	//We add the changes in the last modification list.
	LastModifications *lm = LAST_MODIFICATIONS::instance();
	ACE_TString modif ;
	modif = xml.c_str() ;
	lm->addModifications(modif);

	return 0;
}

int Duplicator_Event_Handler::notifyDuplicatorToStartDataTransfer() {
	ACE_TRACE(ACE_TEXT("Duplicator_Event_Handler::notifyDuplicatorToStartDataTransfer"));
	// We send a request to the duplicator to start transferring data 

	//ACE_TCHAR message[MAX_MESSAGE_SIZE]={0};
	//ACE_OS::sprintf(message, START_DATA_TRANSFER_REQUEST);
	ACE_TString message(START_DATA_TRANSFER_REQUEST);
	return send_xml(message);
}

int Duplicator_Event_Handler::updateFlowPortNumber(ACE_TString xml) {
	ACE_DEBUG((LM_DEBUG, "[ Duplicator_Event_Handler::updateFlowPortNumber ]\n"));

	XML_Parser parser(xml);
	ACE_TString flowID = (parser.getInfos(ACE_TEXT("flowID"))).c_str() ; 

	if (  flowID.length() == 0 )
	{
		ACE_DEBUG((LM_DEBUG, ACE_TEXT("[ Duplicator_Event_Handler::connect ] flowID.length==0\n")));	
		return -1;	
	}

	GlobalList *gl = GLOBAL_LIST::instance();

	Flow *flow = gl->getFlow(flowID); 
	if (flow != 0) {
		ACE_TString portNumber = (parser.getInfos(ACE_TEXT("portnumber"))).c_str() ; 

		int port = ACE_OS::atoi(portNumber.c_str());

		flow->setPortNumber(port);

		ACE_TCHAR block_size[6] = {0};
		ACE_OS::snprintf(block_size, sizeof(block_size), ACE_TEXT("%d"), flow->getBlockSize() );

		ACE_TCHAR block_count[6] = {0};
		ACE_OS::snprintf(block_count, sizeof(block_count), ACE_TEXT("%d"), flow->getBlockCount() );

		ACE_TCHAR number_of_expected_consumers[6] = {0};
		ACE_OS::snprintf(number_of_expected_consumers, sizeof(number_of_expected_consumers), ACE_TEXT("%d"), flow->getNumberOfExpectedConsumers() );

		ACE_TString add_flow_request_server = add_flow_xml(
			flow->getName().c_str(),
			flow->getType().c_str(),
			flow->getGroup().c_str(),
			"",
			block_size,
			block_count,
			flow->getProducer().c_str(),
			flow->getServerProducerID().c_str(),
			port,
			"2",
			number_of_expected_consumers
			);

		ServerList *sl = SERVER_LIST::instance();				
		for (SERVER_ITERATOR si = sl->first(); si != sl->last(); si++) {
			Peer_Event_Handler::send_xml_to_servers((*si).int_id_, add_flow_request_server.c_str());
		}
						
	}
	return 0;
}

int Duplicator_Event_Handler::connect(ACE_TString xml) {
	ACE_TRACE(ACE_TEXT("Duplicator_Event_Handler::connect"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::connect ]\n")));

	XML_Parser parser(xml);
	ACE_TString flowID = (parser.getInfos(ACE_TEXT("flowID"))).c_str() ; 

	if (  flowID.length() == 0 )
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::connect ] flowID.length==0\n")));	
		return -1;	
	}	

	DuplicatorList *dl = DUPLICATOR_LIST::instance();

	if ( dl ) {
		dl->setDupEventHandler(flowID, this);

		if (dl->mustConnectToDistantDupli(flowID) == true) {
			GlobalList *gl = GLOBAL_LIST::instance();

			Flow *flow = gl->getFlow(flowID); 

			if (flow != 0) {
				int port = flow->getPortNumber();
				ACE_TString serverID = flow->getServerProducerID();
				if (port != 0 && serverID != "") {
					ServerList *serverList = SERVER_LIST::instance();
					ACE_TString serverIP = serverList->get(serverID, ServerList::NAME)->getIP();
					
					char tmp_str[8];
					int size_needed = sizeof(CONNECT_TO_DUPLI_REQUEST_MESSAGE) + serverIP.length() + ACE_OS::sprintf(tmp_str, "%d", port) + 1;
					ACE_TCHAR* connectToDupliRequestMessage = (ACE_TCHAR *) ACE_OS::malloc(size_needed);
					ACE_OS::memset(connectToDupliRequestMessage, 0, size_needed);
					
					//ACE_TCHAR connectToDupliRequestMessage[MAX_MESSAGE_SIZE]={0};
					ACE_OS::sprintf(connectToDupliRequestMessage, CONNECT_TO_DUPLI_REQUEST_MESSAGE, serverIP.c_str(), port);
					ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Duplicator_Event_Handler::connect ] We notify the duplicator that it must connect to the distant provider dupli:%s\n"), connectToDupliRequestMessage));	
					this->send_xml(connectToDupliRequestMessage);
					dl->setMustConnectToDistantDupli(flowID, false);
					
					ACE_OS::free(connectToDupliRequestMessage);
				}
			}
		}
		else if (dl->mustOpenProviderPart(flowID)) {
			//We open the provider part of the duplicator
			unsigned int peer = dl->nextPortSupposedToBeAvailable();
			
			char tmp_str[8];
			int size_needed = sizeof(OPEN_PROVIDER_PART_REQUEST_MESSAGE) + ACE_OS::sprintf(tmp_str, "%d", peer) + 1;
			ACE_TCHAR * openProviderPartRequestMessage = (ACE_TCHAR *) ACE_OS::malloc(size_needed);
			ACE_OS::memset(openProviderPartRequestMessage, 0, size_needed);
			
			//ACE_TCHAR openProviderPartRequestMessage[MAX_MESSAGE_SIZE]={0};
			ACE_OS::sprintf(openProviderPartRequestMessage, OPEN_PROVIDER_PART_REQUEST_MESSAGE, peer);
			this->send_xml(openProviderPartRequestMessage);
			dl->setMustOpenProviderPart(flowID, false);
			
			ACE_OS::free(openProviderPartRequestMessage);
		}
	} 
	else
		return -1;
		
	return 0;
}

ACE_TString Duplicator_Event_Handler::add_flow_xml(ACE_TString name_, ACE_TString typeflow_, ACE_TString group_, ACE_TString args_, ACE_TString blocksize_, ACE_TString blockcount_, ACE_TString clientid_, ACE_TString serverid_, int port_, ACE_TString provider_, ACE_TString nbConsumersExpected_)
{
	ACE_TString doc ;
	doc += ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n<type>1</type>\n<target>11</target>\n");

	doc += ACE_TEXT("<typeflow>");
	doc += typeflow_  ;
	doc += ACE_TEXT("</typeflow>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<name>");
	doc += name_  ;
	doc += ACE_TEXT("</name>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<group>");
	doc += group_  ;
	doc += ACE_TEXT("</group>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<args>");
	doc += args_  ;
	doc += ACE_TEXT("</args>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<clientid>");
	doc += clientid_  ;
	doc += ACE_TEXT("</clientid>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<serverid>");
	doc += serverid_  ;
	doc += ACE_TEXT("</serverid>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<provider>");
	doc += provider_  ;
	doc += ACE_TEXT("</provider>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<blocksize>");
	doc += blocksize_  ;
	doc += ACE_TEXT("</blocksize>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<blockcount>");
	doc += blockcount_  ;
	doc += ACE_TEXT("</blockcount>");

	doc += ACE_TEXT("\n") ;

	ACE_TCHAR porttosend[6] ;
	ACE_OS::snprintf(porttosend, sizeof(porttosend), ACE_TEXT("%d"),port_ );

	doc += ACE_TEXT("<port>");
	doc += porttosend ;
	doc += ACE_TEXT("</port>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("<synchronized>");
	doc += nbConsumersExpected_ ;
	doc += ACE_TEXT("</synchronized>");

	doc += ACE_TEXT("\n") ;

	doc += ACE_TEXT("</smartspace>");
	return doc;
}

/**********************************************************************/

Log_Event_Handler::Log_Event_Handler(ACE_Reactor *reactor){
	ACE_TRACE(ACE_TEXT("Log_Event_Handler::Log_Event_Handler"));
	reactor->register_handler(this, ACE_Event_Handler::READ_MASK);
	this->reactor(reactor);
	ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Log_Event_Handler::Log_Event_Handler ] Log_Event_Handler registered\n")));
}

int Log_Event_Handler::handle_input(ACE_HANDLE) {
	ACE_TRACE(ACE_TEXT("Log_Event_Handler::handle_input"));
	//ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Log_Event_Handler::handle_input ] Notification received\n")));
	//here we dequeue as many message as possible to create a message to send to the server.
	//the log messages needs to be formatted in XML first

	ACE_TString messages;
	int result = 0;
	int stop = 0;
	unsigned int nbMessages = 0;
	ACE_Message_Block *mb;
	ACE_TString logMess = LOG_MESSAGE;
	size_t lengthMessage = logMess.length();

	char namelocal[256] = "";
	ACE_OS::hostname(namelocal,256);
	std::string serverName;
	serverName = namelocal;

	lengthMessage += serverName.length();
	
	ACE_Message_Queue<ACE_SYNCH> *messageQueue = LOG_MANAGER::instance()->getQueue();
	if ( messageQueue ) {
		nbMessages = messageQueue->message_count();
						
		std::set<ACE_TString> tmp_serverList;
		tmp_serverList.clear();
						
		//no need to do anything if we don't have any message
		if ( nbMessages != 0 ) {
			//Create a list of server handling at least one 'admin' client
			GlobalList *gl = GLOBAL_LIST::instance();
			for (CLIENT_ITERATOR ci = gl->getFirstClient(); ci != gl->getLastClient(); ci++) {
				Client *client = gl->getClientFromIterator(ci);
				if ( client != NULL ) {
					ACE_TString clientGroup =  client->getGroup();
					if ( clientGroup == ACE_TEXT("admin") ) {
						//ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("[ Log_Event_Handler::logAnnounce ] ADMIN FOUND, server=%s\n"),client->getServer().c_str()));
						tmp_serverList.insert(client->getServer());
					}
				}
			}
		}
		
		//no need to do anything if there is no server to send the log to.
		if ( tmp_serverList.size() == 0 ) {
			//no need to do anything if there is no server to send the log to.
			//we just flush the message queue
			messageQueue->flush();
			LOG_MANAGER::instance()->setNotificationSent(0);
		}
		else {
			//we have messages and at least one server to send them to.
			while ( (nbMessages != 0) && !stop ) {
				//std::cout << "------*******----->DUP_Log::handle_input size = " << messageQueue->message_count() << std::endl;
				//we dequeue the message...
				ACE_Time_Value timeout(ACE_OS::gettimeofday());
				result = messageQueue->dequeue_head(mb, &timeout);
				if ( result == -1 )
					stop = 1;
				else {
					//get its size
					int lengthMB = mb->length();

					if ( (lengthMessage + lengthMB + 1)  > MAX_MESSAGE_SIZE ) {
						stop = 1;
						//we put back the message in the queue
						ACE_Time_Value timeout (ACE_OS::time (0) + 1, 0);
						result = messageQueue->enqueue_head(mb, &timeout);
					}
					else {
						//set up its read pointer on the beginning of the message
						mb->rd_ptr(mb->base());
						//Allocate the buffer to copy the message
						ACE_TCHAR *dest = (ACE_TCHAR *) ACE_OS::malloc(lengthMB+1);
						ACE_OS::memset(dest, 0, lengthMB+1);
						//copy the message into dest
						ACE_OS::memcpy(dest, mb->rd_ptr(), lengthMB);
		
						ACE_TString messTString = dest;
						ACE_TString messInXML = create_XML_message(messTString);
						messages += messInXML;
						
						//we update the size
						lengthMessage += lengthMB;
						
						nbMessages--;

						//we don't need the message block anymore, we delete it
						delete mb;
						mb = NULL;
						//we free the dest variable
						ACE_OS::free(dest);
					}
				}
			}
						
			if ( messages.length() > 0 ) {
				//we have at least one log message int mess.	
				//char theMessage[MAX_MESSAGE_SIZE]={0};
				int size_needed = sizeof(LOG_MESSAGE) + serverName.length() + messages.length() + 1;
				ACE_TCHAR * theMessage = (ACE_TCHAR *) ACE_OS::malloc(size_needed);
				ACE_OS::memset(theMessage, 0, size_needed);
				ACE_OS::sprintf(theMessage,LOG_MESSAGE, serverName.c_str(), messages.c_str());
				
				//we iterate on the list of server we need to send the log to.
				std::set<ACE_TString>::iterator iter;
				ServerList *sl = SERVER_LIST::instance();             
				for (SERVER_ITERATOR si = sl->first(); si != sl->last(); si++) {
					iter = tmp_serverList.find((*si).int_id_->getName());
					if ( iter != tmp_serverList.end()) {
						Peer_Event_Handler::send_xml_to_servers((*si).int_id_, theMessage);
					}    
				}

				//We add the changes in the last modification list.
				LastModifications *lm = LAST_MODIFICATIONS::instance();
				lm->addModifications(theMessage);
				
				ACE_OS::free(theMessage);
			}
						
			if ( nbMessages != 0 ){
				//we didn't take care of everymessage, we will finish the job later
				//because we don't want to let the reactor blocked for too long..
				//so we notify the reactor to call again this method later
				ACE_Reactor::instance()->notify(LOG_MANAGER::instance()->getLogHandler(), ACE_Event_Handler::READ_MASK);
			}
			else {
				//here we notify the Log_Callback object that we received the notification
				//so it can notify the reactor to invoke this method later
				LOG_MANAGER::instance()->setNotificationSent(0);
			}
		}//end else -- we have messages and a server to send them to.
	} //end if ( messageQueue ) {
	return 0;
}

int Log_Event_Handler::handle_timeout(const ACE_Time_Value &currentTime, const void *) {
	ACE_UNUSED_ARG(currentTime);
	ACE_Message_Queue<ACE_SYNCH> *messageQueue = LOG_MANAGER::instance()->getQueue();
	if ( messageQueue ) {
		unsigned int nbMessages = messageQueue->message_count();
		if ( nbMessages != 0 && LOG_MANAGER::instance()->isNotificationSent() == 0 ) {
			reactor()->notify(LOG_MANAGER::instance()->getLogHandler(), ACE_Event_Handler::READ_MASK);
			LOG_MANAGER::instance()->setNotificationSent(1);
		}
	}
	return 0;
}

ACE_TString Log_Event_Handler::create_XML_message(ACE_TString message_){
	ACE_TRACE(ACE_TEXT("Log_Event_Handler::create_XML_message"));
	ACE_TString doc ;
	doc += ACE_TEXT("<message>");
	doc += ACE_TEXT("<![CDATA[");
	doc += message_;
	doc += ACE_TEXT("]]>");
	doc += ACE_TEXT("</message>\n");
	
	return doc;
}

void Log_Event_Handler::enableTimer() {
    if ( reactor() ) {
        ACE_Time_Value delay(1);
        ACE_Time_Value interval(1);
        reactor()->schedule_timer(this, 0, delay, interval);
    }
}

void Log_Event_Handler::disableTimer() {
	if ( reactor() )
			reactor()->cancel_timer(this);
}

/*int Log_Event_Handler::send_xml_to_servers(Server *peer_server, ACE_TString xml) {
	ACE_TRACE(ACE_TEXT("Log_Event_Handler::send_xml_to_servers"));
	//ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Log_Event_Handler::send_xml_to_servers ]\n")));
	if ( (peer_server != NULL) && (peer_server->getHandler() != NULL) ) {
		ACE_SOCK_Stream *sock = &(peer_server->getHandler()->peer());
		if ( sock ) {
			//here we are trying to send, if it is not successful,
			//we test for EWOULDBLOCK, we queue the message to send it later.
			char size[512] = {0};
			ACE_OS::snprintf(size, sizeof(size), "%d", xml.length());
			const char *xml_message = ACE_TEXT_ALWAYS_CHAR(xml.c_str());
			ssize_t send_cnt = sock->send(size, sizeof(size));
			if ( send_cnt == sizeof(size) ) {
				//we successfully sent the size, we know send the message itself
				send_cnt = sock->send(xml_message, ACE_OS::strlen(xml_message));
				ssize_t messageLength = ACE_static_cast(size_t, ACE_OS::strlen(xml_message));
				if ( send_cnt == messageLength )
					//success, the message has been sent properly
					return 0;
				else if ( send_cnt == -1 && ACE_OS::last_error() != EWOULDBLOCK )
					//something really wrong is happening...
					return -1;
				
				//the socket is not closed but the message couldn't be send properly
				if ( send_cnt == -1 )
					//nothing has been sent so far... we adjust send_cnt to reflect the proper number of byte sent so far
					send_cnt = 0;
				
				//we know need to queue the message to send it later. 
				size_t remaining = ACE_static_cast(size_t, (ACE_OS::strlen(xml_message) - send_cnt));
				return peer_server->getHandler()->putMessageInQueue(xml_message, remaining, true);
			}
			else if ( send_cnt == -1 && ACE_OS::last_error() != EWOULDBLOCK )
				//something really wrong is happening...
				return -1;
			//the socket is not closed but the message couldn't be send properly
			if ( send_cnt == -1 )
				//nothing has been sent so far... we adjust send_cnt to reflect the proper number of byte sent so far
				send_cnt = 0;
			//we know need to queue the message to send it later. 
			size_t remaining = ACE_static_cast(size_t, (sizeof(size) - send_cnt));
			return peer_server->getHandler()->putMessageInQueue(xml_message, remaining, false);	
		}
	}
	return -1;
}*/


/*int Log_Event_Handler::send_xml_to_servers(ACE_SOCK_Stream *peer_server, ACE_TString xml)
{
	ACE_TRACE(ACE_TEXT("Log_Event_Handler::send_xml_to_servers"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[  Log_Event_Handler::send_xml_to_servers ]\n")));	
	char size[512] = {0};
	ACE_OS::snprintf(size, sizeof(size), "%d", xml.length());

	unsigned int result;
	
	//first we send the size of the message 
	result = peer_server->send_n(size, sizeof(size));
	if ( result < sizeof(size) ) {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Log_Event_Handler::send_xml_to_servers ] Error sending message size to server \n")));
		return -1;
	}

    const char *xml_message = ACE_TEXT_ALWAYS_CHAR(xml.c_str());
    
	//then we send the message itself 
	result = peer_server->send_n(xml_message, ACE_OS::strlen(xml_message));
	if ( result < ACE_OS::strlen(xml_message) ) {
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ Log_Event_Handler::send_xml_to_servers ] Error sending message to server \n")));
		return -1;
	}

	return 0;
}*/
						
Server_Stopper::Server_Stopper(ACE_Reactor *reactor, int signum) {
	this->reactor(reactor);
	ACE_Reactor::instance()->register_handler(signum, this);
		
}
						
int Server_Stopper::handle_signal(int signum, siginfo_t*, ucontext_t *) {
	ACE_UNUSED_ARG(signum);
	reactor()->end_reactor_event_loop();
	return 0;
}

