#ifndef _LOG_MANAGER_H
#define _LOG_MANAGER_H

#define SF2D_DEBUG_PREFIX		LM_DEBUG,		ACE_TEXT ("%I[SF2D DEBUG   %T] ") 
#define SF2D_WARNING_PREFIX		LM_WARNING,		ACE_TEXT ("%I[SF2D WARNING %T] ")
#define SF2D_ERROR_PREFIX		LM_ERROR,		ACE_TEXT ("%I[SF2D ERROR   %T] ")
#define SF2D_USER_PREFIX		LM_ERROR,		ACE_TEXT ("%I[SF2D USER    %T] ")
#define SF2D_ANNOUNCE_PREFIX	LM_ERROR,		ACE_TEXT ("%I[SF2D ANNOUNCE%T] ")

#include "ace/OS.h"
#include "ace/streams.h"
#include "ace/Log_Msg.h"
#include "ace/Log_Msg_Callback.h"
#include "ace/Log_Record.h"
#include "ace/SString.h"
#include "ace/Message_Queue.h"
#include "ace/Thread_Manager.h"
#include "ace/Null_Mutex.h"
#include "ace/Singleton.h"

#include <map>

class Log_Event_Handler;
class Log_Callback;

/** \brief Class Log_Manager_ACE
 * This class is used to manage the log system.
 * It is used to define the desired severity.
 * It is also used to send the log info to the server, 
 * write them in a file if desired and print them on the stderr
 */
class Log_Manager {
   
	friend class ACE_Singleton<Log_Manager, ACE_Null_Mutex>;

   protected:
	   int init() {return 0;}
	   int fini() {return 0;}
   public:
    
	 /** \brief Class constructor. 
     * \warning This method does not initialise the object, the init() method
     * must be used for this purpose.
     * 
     */
    Log_Manager();

     /** \brief Class constructor. 
     * \warning This method does not initialise the object, the init() method
     * must be used for this purpose.
     * 
     */
    Log_Manager(Log_Event_Handler *logHandler);

	/** \brief Class destructor.
     * Take care of any cleanup that is necessary to free the object.
     */
	virtual ~Log_Manager();
    
	/** \brief Class initialiser. Must be called just after the object
     * construction. 
     * \param sf The Smartflow object in which this object will be be registered.
     * \return 0 if the object is correctly initialised, otherwise -1. If possible,
     * an error will be set via Smartflow::seterror() to explain the error.
     */
	virtual int init(const char* name);

	Log_Event_Handler * getLogHandler() {return log_handler; }

	void setLogHandler(Log_Event_Handler *lh) { log_handler = lh; }

	void setReactor(ACE_Reactor *reactor) { react = reactor; }

	ACE_Reactor *getReactor() { return react; }

	ACE_Message_Queue<ACE_SYNCH> *getQueue() { return &logQueue; }

	int isNotificationSent() { return notificationSent;}
	
	void setNotificationSent(int notif) { notificationSent = notif; }
	
	bool forwardLog() { return logForwardEnabled; }
    
private:
	Log_Callback *cb;
	Log_Event_Handler *log_handler;
	ACE_Reactor *react;
	ACE_OSTREAM_TYPE *file;
	ACE_Message_Queue<ACE_SYNCH> logQueue;
	int notificationSent;
	bool logForwardEnabled;
};

typedef ACE_Singleton<Log_Manager, ACE_Null_Mutex> LOG_MANAGER;

/** \brief Class Log_Callback .
 * This class is used to implement a callback method
 * invoked when a logging operation occured.
 * It is used to propagate the log to the server
 * \author Nist Smartspace Group
 * \date 04-19-2004
 */
class Log_Callback : public ACE_Log_Msg_Callback {
	public:
		Log_Callback(Log_Manager *manager);
		~Log_Callback();
		void log(ACE_Log_Record &log_record);

    private:
		int sendLogToServer();
		void putLogInQueue(ACE_Log_Record &log_record);
		Log_Manager *manager_;
};

class Log_Preferences {
	friend class ACE_Singleton<Log_Preferences, ACE_Null_Mutex>;

	public:
		Log_Preferences();
		~Log_Preferences();

		int initPreferences(int &argc, char** &argv);
		ACE_TString getPref(ACE_TString key);

		void displayPrefs();

	private:
		void addPreference(ACE_TString key, ACE_TString value);
		void remPreference(ACE_TString key);
		bool exists(ACE_TString &key);
		std::map<ACE_TString, ACE_TString> prefs;
};

typedef ACE_Singleton<Log_Preferences, ACE_Null_Mutex> LOG_PREF;

#endif
