#include "Multicast_Server_Events.h"

#include "XML_Parser.h"
#include "Log_Manager.h"

ACE_HANDLE MultiCast_Server_Events::get_handle (void) const
{
  ACE_TRACE(ACE_TEXT("MultiCast_Server_Events::get_handle"));
  return this->mcast_dgram_.get_handle ();
}

int MultiCast_Server_Events::open ()
{
	ACE_TRACE(ACE_TEXT("MultiCast_Server_Events::open"));
	//if (this->mcast_dgram_.subscribe (this->mcast_addr_) == -1)
	if (this->mcast_dgram_.join (this->mcast_addr_) == -1)
	{
    	ACE_ERROR ((LM_ERROR,"%p\n","subscribe")); 
    	return -1;
	}
    return 0;
}

MultiCast_Server_Events::MultiCast_Server_Events (u_short port, const char *mcast_addr) : mcast_addr_ (port, mcast_addr), local_addr_((u_short)0),dgram_(local_addr_){
	ACE_TRACE(ACE_TEXT("MultiCast_Server_Events::MultiCast_Server_Events"));
	ACE_OS::sprintf(application_name_, ACE_TEXT("default"));
}

MultiCast_Server_Events::MultiCast_Server_Events (u_short port, const char *mcast_addr, ACE_TCHAR *application_name) : mcast_addr_ (port, mcast_addr), local_addr_((u_short)0),dgram_(local_addr_){
	ACE_TRACE(ACE_TEXT("MultiCast_Server_Events::MultiCast_Server_Events"));
	ACE_OS::sprintf(application_name_, application_name);
}

MultiCast_Server_Events::~MultiCast_Server_Events (void)
{
	ACE_TRACE(ACE_TEXT("MultiCast_Server_Events::~MultiCast_Server_Events"));
	this->mcast_dgram_.close() ;
}

// This function is called when a multicast datagram
// is received by the server.
// This multicast datagram means that a new Smartflow 
// Server has just been launched
int MultiCast_Server_Events::handle_input (ACE_HANDLE)
{  
  ACE_TRACE(ACE_TEXT("MultiCast_Server_Events::handle_input"));
  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ MultiCast_Server_Events::handle_input ]\n")));
  // We receive the datagram
  if (mcast_dgram_.recv(&mcast_info,sizeof (mcast_info),remote_addr_) == -1)
      return -1;
  
  XML_Parser parser(ACE_TEXT_CHAR_TO_TCHAR(mcast_info));

  ACE_TString servername = parser.getInfos(ACE_TEXT("servername"));
  ACE_TString serveros = parser.getInfos(ACE_TEXT("serveros"));
  ACE_TString applicationname = parser.getInfos(ACE_TEXT("applicationname"));

  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ MultiCast_Server_Events::handle_input ] Application name: %s\n"), applicationname.c_str() ));

  ACE_TString application_name (application_name_);

  // We answer only if we manage the same application
  if (applicationname == application_name) {

	// We save the name of the current machine (the machine on which the Smartflow Server is running)
	ACE_TCHAR hostname[99+1];
	if (ACE_OS::hostname (hostname,99) == -1) ACE_ERROR ((LM_ERROR,ACE_TEXT("%p\n"),ACE_TEXT("hostname")));
    
	// We test if the datagram has been sent by the current server or by another
	if (ACE_OS::strcmp(servername.c_str(), hostname)) {
			// The datagram has been sent by another server
			// It means that the server has just been launched
			
			//There is a tricky bug we need to handle here
			//In case of crash or physical network disconnection, the broken connection is not detected by the reactor
			//The following code is written to handle this kind of situation:
			//1) Server A and B are connected
			//2) network cable of A is unplugged
			//3) Server A is killed
			//4) Network cable is replugged 
			//5) Server A is relaunched
			//		=> B thinks A still exists and receive a new connection from A
			//So if it happens, we clean everything about A from the lists before adding it again.
		
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ MultiCast_Server_Events::handle_input ] Another Server (%s) running %s has started, trying to connect...\n"), servername.c_str(), serveros.c_str() ));
			ACE_SOCK_Connector connector;
			ACE_INET_Addr peer_addr;
			// We connect a new socket with a new Peer_Event_Handler in order to communicate with the new Server
			peer_addr.set( SF2D_DEFAULT_REMOTE_PORT ,remote_addr_.get_host_addr() );
			Peer_Event_Handler *peer_handler =0;
			ACE_NEW_RETURN(peer_handler,Peer_Event_Handler(reactor()),-1);
			connector.connect(peer_handler->peer(),peer_addr);  	
			if (peer_handler->open() == -1)
			{
				ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ MultiCast_Server_Events::handle_input ] Connection to new server failed\n")));
		 		peer_handler->handle_close();
		  		return -1;
			}
			
			//Does the server still exist in the list?
			//SocketList *sl = SOCKET_LIST::instance();
			ServerList *serverList = SERVER_LIST::instance();
			
		    if ( serverList->find(servername, ServerList::NAME) == 0 ) {
				//we found the server, it shouldn't be in the list anymore, so we clean it up
				// A server has just quit. We will delete his entry in the socket list, server list and then delete the entries of his flow/clients in the GlobalList
				
				ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ MultiCast_Server_Events::handle_input ] Server '%s' still in list. Fixing problem...\n"), servername.c_str()));
				Server *server = serverList->get(servername, ServerList::NAME);
				if ( !server )
					ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ MultiCast_Server_Events::handle_input ] Server '%s' running %s not found in list\n"), servername.c_str(), serveros.c_str() ));
				else {
					//we need to flush the old handler of the server
					ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ MultiCast_Server_Events::handle_input ] Removing '%s' from lists...\n"), servername.c_str() ));
					
					//We remove the server from the list
					serverList->remove(servername, ServerList::NAME);
					
					Peer_Event_Handler *ph = server->getHandler();
					
					//We remove the old handler to avoid an exit signal from this zombie socket
					if ( ph != NULL ) {
						ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ MultiCast_Server_Events::handle_input ] Removing handler from the reactor...\n")));
						reactor()->remove_handler(ph, ACE_Event_Handler::READ_MASK);
						//do we need to delete the handler?
					}	

					GlobalList *gl = GLOBAL_LIST::instance();
					
					gl->removeAllClientsOnServer( server->getName() );	
					//Update modifications
					LastModifications *lm = LAST_MODIFICATIONS::instance(); 
					ACE_TString modif = remove_serverxml(server->getName());
					lm->addModifications(modif.c_str());
					
					//we delete the server object
					delete server;
				}
			}
			
			// We add the new Server to the Server list
			Server *server = new Server(servername, ACE_TEXT_CHAR_TO_TCHAR (remote_addr_.get_host_addr()), serveros);
			server->setHandler(peer_handler);
			serverList->add(server);

			//	GlobalList *gl = GLOBAL_LIST::instance();

			// We add the modification (the new server connection) in the modification list
			LastModifications *lm = LAST_MODIFICATIONS::instance();
			ACE_TString modif ;
			modif = add_server_message( servername, serveros );
			lm->addModifications(modif) ;	

			// Finally we send the local representation of the application (only the client running on the current server) to the new server
			peer_handler->construct_and_send_local_application_description();
	}
  }
  else
	  ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("Server handling a different application name: '%s' detected\n"), applicationname.c_str() ));

  return 0; 
}

int MultiCast_Server_Events::welcome_to_multicast_group(){
	ACE_TRACE(ACE_TEXT("MultiCast_Server_Events::welcome_to_multicast_group"));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ MultiCast_Server_Events::welcome_to_multicast_group ]\n")));
	ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("Joining application domain: %s\n"), ACE_TEXT_ALWAYS_CHAR(application_name_)));

	char hostname[99+1];
	if (ACE_OS::hostname (hostname,99) == -1) 
		ACE_ERROR ((LM_ERROR, ACE_TEXT("%p\n"), ACE_TEXT("hostname")));

	#if (defined (ACE_WIN32))
		char os[] = "winxp";
	#elif (defined (ACE_CONFIG_MACOSX_TIGER_H))
		char os[] = "OSX Tiger";
	#elif (defined (ACE_CONFIG_MACOX_LEOPARD_H))
		char os[] = "OSX Leopard";
	#else
		char os[] = "linux";
	#endif


	ACE_OS::sprintf(mcast_info, MULTICAST_MESSAGE, hostname, os, ACE_TEXT_ALWAYS_CHAR(application_name_));
   
    if (dgram_.send(&mcast_info, sizeof (mcast_info), mcast_addr_)==-1)
      return -1;
    return 0;
}
 
ACE_TString MultiCast_Server_Events::add_server_message(ACE_TString name_, ACE_TString os_)
{ 
	ACE_TRACE(ACE_TEXT("MultiCast_Server_Events::add_server_message"));
	ACE_TString doc = ACE_TEXT("");
	doc += ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n<type>3</type>\n<target/>\n<servername>");
	doc += name_ ;
	doc += ACE_TEXT("</servername>\n<serveros>");
	doc += os_ ;
	doc += ACE_TEXT("</serveros>\n</smartspace>");
	return doc;
  
}

ACE_TString MultiCast_Server_Events::remove_serverxml(ACE_TString name_)
{
	ACE_TRACE(ACE_TEXT("Peer_Event_Handler::remove_serverxml"));
	ACE_TString doc;
	doc = ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n<type>4</type>\n<target/>\n<servername>");
	doc += name_ ;
	doc += ACE_TEXT("</servername>\n</smartspace>");
	return doc;
}
