#ifndef _PEER_EVENT_HANDLER_H
#define _PEER_EVENT_HANDLER_H

#include "ace/Message_Block.h"
#include "ace/Message_Queue.h"
#include "ace/Synch.h"
#include "ace/OS.h"
#include "ace/SString.h"
#include "ace/Process.h"
#include "ace/Event_Handler.h"
#include "ace/INET_Addr.h"
#include "ace/Reactor.h"
#include "ace/SOCK_Acceptor.h"
#include "ace/SOCK_Stream.h"
#include "ace/IPC_SAP.h"

#include "list/Client.h"
#include "list/Server.h"

#include "sf2d.h"
#include "DuplicatorManager.h"

#define CONNECT_TO_DUPLI_REQUEST_MESSAGE    ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>3</target> <duplicatoraddress>%s</duplicatoraddress>\n <duplicatorport>%d</duplicatorport>\n </smartspace>")
#define OPEN_PROVIDER_PART_REQUEST_MESSAGE  ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>29</target> <PortForRemoteConsumers>%d</PortForRemoteConsumers>\n </smartspace>")
#define CLOSE_PROVIDER_PART_REQUEST_MESSAGE  ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>30</target>\n </smartspace>")
#define DIE_MESSAGE      ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>24</target></smartspace>") 

// By Imad

#define PAUSE_REQUEST_MESSAGE    ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>21</target> <clientname>%s</clientname>\n <clientgroup>%s</clientgroup>\n <flowtype>%s</flowtype>\n <flowgroup>%s</flowgroup>\n <hostname>%s</hostname>\n </smartspace>")
#define PAUSE_ANNOUNCE_MESSAGE   ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>5</type>\n <target>21</target> <clientname>%s</clientname>\n <clientgroup>%s</clientgroup>\n <flowtype>%s</flowtype>\n <flowgroup>%s</flowgroup>\n <hostname>%s</hostname>\n </smartspace>")

#define RESTART_REQUEST_MESSAGE    ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>22</target> <clientname>%s</clientname>\n <clientgroup>%s</clientgroup>\n <flowtype>%s</flowtype>\n <flowgroup>%s</flowgroup>\n <hostname>%s</hostname>\n </smartspace>")
#define RESTART_ANNOUNCE_MESSAGE   ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>5</type>\n <target>22</target> <clientname>%s</clientname>\n <clientgroup>%s</clientgroup>\n <flowtype>%s</flowtype>\n <flowgroup>%s</flowgroup>\n <hostname>%s</hostname>\n </smartspace>")
 
#define PAUSE_DUPLI_MESSAGE      ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>25</target> <hostname>%s</hostname>\n </smartspace>") 
#define WAKE_UP_REQUEST_MESSAGE  ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>26</target>\n <flowid>%s</flowid>\n <hostname>%s</hostname>\n </smartspace>")
#define WAKE_UP_DUPLI_MESSAGE    ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>27</target>\n <hostname>%s</hostname>\n </smartspace>")
#define MAX_MESSAGE_SIZE 131072

//Antoine
#define START_DATA_TRANSFER_REQUEST  ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>33</target> </smartspace>")
#define CONSUMER_CONNECTED_ANNOUNCE_MESSAGE ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>5</type>\n <target>31</target> <clientname>%s</clientname>\n <clientgroup>%s</clientgroup>\n <flowtype>%s</flowtype>\n <flowgroup>%s</flowgroup>\n <hostname>%s</hostname>\n </smartspace>")

#define LOG_MESSAGE ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>5</type>\n <target>37</target> <servername>%s</servername>\n <messages>%s</messages></smartspace>")
#define LOG_MESSAGE_CLIENT ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>5</type>\n <target>35</target>\n <clientname>%s</clientname>\n <clientgroup>%s</clientgroup>\n <messages>%s</messages>\n </smartspace>")

/// This class describes the Handler which is used to handle the connection with a Server of the NIST Data Flow System 2. She is registered to the reactor. She is instanced by the Peer_Acceptor_Event_Handler at each new "Accept" on the SF2D_DEFAULT_REMOTE_PORT.
class Peer_Event_Handler : public ACE_Event_Handler
{

	protected :

	/// Socket used to communicate with a smartflow2 server.

	ACE_SOCK_Stream remote_peer_;
 	ACE_INET_Addr remoteaddr;
	DuplicatorManager dm;
	
    ACE_Message_Queue<ACE_SYNCH> message_queue;
	
	/// Send the xml to the other server connected to the handler  
  
	/**
	 * @param xml XML message to send
	 * @return 0 for Success, -1 otherwise
	 */  

	int send_xml(ACE_TString xml);

 	/**
	 * @param xml XML message to send
	 * @param peer_server Socket to send the message
	 * @param xml Message to send
	 * @return 0 for Success, -1 otherwise
	 */  
	
	//int send_xml_to_servers(ACE_SOCK_Stream *peer_server, ACE_TString xml);
	
	/**
	 * @param xml XML message to send
	 * @param peer_server Server to send the message
	 * @param xml Message to send
	 * @return 0 for Success, -1 otherwise
	 */  
	
	//int send_xml_to_servers(Server *peer_server, ACE_TString xml);

	/// This method is used to read the message on the socket. It first reads the size of the message to allocate the right memory space, then the message himself.
    
	/**
	 * @return xml message read from the socket
	 */    
  
	ACE_TString receive_xml();
       
	//Actions available   

	/// This function is called at each new input event on the socket. She will decide which methods to call regarding the type and target tags of the xml message 
  
	/**
	 * @param xml XML message to analyse
	 * @return 0 for Success, -1 otherwise
	 */  
    
	int actions_switch(ACE_TString xml);


	/// The connect method takes care of the connection of a new smartflow2 client on another server.

	/**
	 * @param xml XML message about a connection request from a client running on another server.
	 * @return 0 for Success, -1 otherwise
	 */  

	int connect_request(ACE_TString xml);

	int disconnect_request(ACE_TString xml); 

	/// The "GetFlow Request" method handles the creation of a new flow (provider side) AND the subscription for a flow (consumer side) by a client running on another server.
  
	/**
	 * @param xml XML message about registering/subscribing a flow
	 * @return 0 for Success, -1 otherwise
	 */  

	int getflow_request(ACE_TString xml);
	
	/// The "RemFlow Request" method removes a flow from the tables.
  
	/**
	 * @param xml XML message about unregistering a flow
	 * @return 0 for Success, -1 otherwise
	 */  

	int remflow_request(ACE_TString xml);
	
	/// The "Launch Request" method handles the launch of a new smartflow2 client. If the path is not absolute, the client executable must be in the "PATH" environnement variable. A request of launching a client on this server has been requested.

	/**
	 * @param xml XML message the useful information to be able to spawn the client
	 * @return 0 for Success, -1 otherwise
	 */  

	int launch_request(ACE_TString xml);
 
 
 	/// The "Stop Request" method handles the "stop" of a new smartflow2 client. Only the Control Center is allowed to use that method.

	/**
	 * @param xml XML message with the ID of the client to be killed
	 * @return 0 for Success, -1 otherwise
	 */  

	int stop_request(ACE_TString xml);
	
	/// The logAnnounce is used to add the log messages from server, dupli and client nodes in the last modification list
	/**
	 * @param xml XML message containing a log message
	 * @return 0 for Success, -1 otherwise
	 */
	int logAnnounce(ACE_TString xml);

	/**
	 * @param xml XML message 
	 * @return 0 for Success, -1 otherwise
	 * Imad
	 */  
 
	int notifyPauseFlowToAllServers(ACE_TString xml);
	int notifyRestartFlowToAllServers(ACE_TString xml);
	int pauseFlow(ACE_TString xml);
	int restartFlow(ACE_TString xml);
	int notify_dupli_to_wakup_host(ACE_TString xml);

	//Response    

	ACE_TString create_xmlsuccess();
	ACE_TString create_xmlfailure(ACE_TString des_);
	ACE_TString remove_serverxml(ACE_TString name_);

	ACE_TString add_server_message(ACE_TString name_);
  
	//int notify_duplicators_to_die(ACE_TString clientID_);
  
  
	public :
	
        // When a new server starts, a synchronization is needed.
	// That's why each remote server will only send us the list
	// of clients which are locally running on its machine.
	// This method is in charge of managing all received descriptions
	// by filling out properly all the tables of the current server.

	int manage_remote_application_description(ACE_TString xml);

	int construct_and_send_local_application_description();
	
	/// Let the server know that a new consumer has connected to the duplicator  
	/**
	 * @param xml XML message containing information about the consumer client node connected to the duplicator
	 * @return 0 for Success, -1 otherwise
	 */  
	int announceNewConsumerNodeConnectedToDuplicator(ACE_TString xml);

	///Put the message in the message queue of the server for sending the message later.
	int putMessageInQueue(const char* message, ssize_t send_cnt, bool isSizeSent);

	// These functions are overrided functions from ACE_Event_Handler.

	Peer_Event_Handler(ACE_Reactor *reactor):ACE_Event_Handler(reactor) {}
	virtual ~Peer_Event_Handler() {};
	virtual int open();
	virtual int handle_input(ACE_HANDLE);
	virtual int handle_close(ACE_HANDLE = ACE_INVALID_HANDLE, ACE_Reactor_Mask = 0);
	virtual int handle_output(ACE_HANDLE);
	virtual ACE_HANDLE get_handle() const;
	ACE_SOCK_Stream &peer() {return remote_peer_;}
	
	static int send_xml_to_servers(Server *peer_server, ACE_TString xml);
};

/// This class describes the Handler which is used to handle the connection with a Client of the NIST Data Flow System 2. She is registered to the reactor. She is instanced by the Local_Acceptor_Event_Handler at each new "Accept" on the SF2D_DEFAULT_LOCAL_PORT.

class Local_Event_Handler : public ACE_Event_Handler
{
	protected :

	/// Socket used to communicate with a smartflow2 client.
	  
	ACE_SOCK_Stream remote_peer_;
	DuplicatorManager dm;

	/// This function is called at each new input event on the socket. She will decide which methods to call regarding the type and target tags of the xml message 

	/**
	 * @param xml XML message to analyse
	 * @return 0 for Success, -1 otherwise
	 */  

	int actions_switch(ACE_TString xml);
  
	int pauseFlow(ACE_TString xml);
	int restartFlow(ACE_TString xml);

	/// This method is used to read the message on the socket. It first reads the size of the message to allocate the right memory space, then the message himself.

 	/**
	 * @return xml message read from the socket
	 */    

	ACE_TString receive_xml();

	//int notify_duplicators_to_die(ACE_TString clientID_);

	/// Send the xml to the client connected to the handler  

	/**
	 * @param xml XML message to send
	 * @return 0 for Success, -1 otherwise
	 */  

	int send_xml(ACE_TString xml);

	/// Send the xml message to the specified socket ( which is a server socket )

	/**
	 * @param xml XML message to send
	 * @param peer_server Socket to send the message
	 * @return 0 for Success, -1 otherwise
	 */  

	//int send_xml_to_servers(ACE_SOCK_Stream *peer_server, ACE_TString xml);

	/**
	 * @param xml XML message to send
	 * @param peer_server Server to send the message
	 * @param xml Message to send
	 * @return 0 for Success, -1 otherwise
	 */  
	
	//int send_xml_to_servers(Server *peer_server, ACE_TString xml);
	
	/// The connect method takes care of the connection of a new smartflow2 client.

	/**
	 * @param xml XML message about a connection request from a client 
	 * @return 0 for Success, -1 otherwise
	 */  

	int connect(ACE_TString xml);

	/// The "GetFlow Request" method handles the creation of a new flow (provider side) AND the subscription for a flow (consumer side)

	/**
	 * @param xml XML message about registering/subscribing a flow
	 * @return 0 for Success, -1 otherwise
	 */  

	int getflow_request(ACE_TString xml);

	/// The "RemFlow Request" method removes a flow from the tables.
  
	/**
	 * @param xml XML message about unregistering a flow
	 * @return 0 for Success, -1 otherwise
	 */  

	int remflow_request(ACE_TString xml);


	/// The "Launch Request" method handles the launch of a new smartflow2 client. Only the Control Center is allowed to use that method. If the path is not absolute, the client executable must be in the "PATH" environnement variable.

	/**
	 * @param xml XML message the useful information to be able to spawn the client
	 * @return 0 for Success, -1 otherwise
	 */  

	int launch_request(ACE_TString xml);


	/// The "Stop Request" method handles the "stop" of a new smartflow2 client. Only the Control Center is allowed to use that method.

	/**
	 * @param xml XML message with the ID of the client to be killed
	 * @return 0 for Success, -1 otherwise
	 */  

	int stop_request(ACE_TString xml);

	/// The logAnnounce methods is used to propagate the log from a client on the NDFS II network.
	/**
	 * @param xml XML message containing a log message
	 * @return 0 for Success, -1 otherwise
	 */
	int logAnnounce(ACE_TString xml);

	/// Method which deals with an Application Description request. It creates the message and sends the response to the client.  

	int flowlist_request();

	/// Method which deals with a lastmodifications request. It creates the message and sends the response to the client. This method is used with a thread in the Control Center to have a dynamic view of the map

	/**
	 * @return 0 for Success, -1 otherwise
 	 */  

	int getlastmodifications_request();

	/// Method which deals with a serverlist request. It creates the message and sends the response to the client. 

	int get_serverlist();  

	/// Success message
  
	/**
	 * @return Success XML message
	 */ 
  
	ACE_TString create_xmlsuccess();

	/// Failure message, Description is the reason of the failure

	/**
	 * @param des_ Reason of the failure
	 * @return Failure XML message
	 */  

	ACE_TString create_xmlfailure(ACE_TString des_);

	/// Removal message of a client given its name and group
  
	/**
	 * @param name_ Name of the client to remove
	 * @param group_ Group of the client to remove
	 * @return Removal XML message
	 */  

	ACE_TString remove_clientxml(ACE_TString name_, ACE_TString group_);

	/// Removal message of a flow given its ID and clientID
  
	/**
	 * @param flowID_ ID of the flow to remove
	 * @param clientID_ ID of the client to remove
	 * @return Removal XML message
	 */  
	 
	//ACE_TString remove_flowxml(ACE_TString flowID_, ACE_TString clientID_ );
	ACE_TString remove_flowxml(ACE_TString clientname_, ACE_TString clientgroup_ ,ACE_TString flowtype_, ACE_TString flowgroup_ );

	/// Message which contains the description of all the application. Thanks to it, the control center can be launched whenever.  

	/**
	 * @return XML message which reprensents the description of the running application
	 */ 

	ACE_TString create_xmlflowlistresponse();

	/// Message which contains a list of all the Smartflow2 servers enabled.

	/**
	 * @return XML message which contains the list of all the enabled Smartflow2 server
	 */ 

	ACE_TString create_xmlserverlistresponse();

	/// Message which contains the last modifications of the system. It is used by the control center to get a realtime view of the system.

	/**
	 * @return XML message which contains the last modifications of the system
	 */ 

	ACE_TString create_xmllastmodifications();  

	///  Message sent to other servers when a local "getflow" is performed. This message provides more information than the getflow xml message, as the clientID.

	/**
	 * @param port Port which is listened by the running duplicator of that flow, 0 if it is a consumer message
	 * @param nameclient_ ID of the client which did that request
	 * @return XML message which contains the last modifications of the system
	 */ 

	ACE_TString add_flow_xml(ACE_TString name_, ACE_TString typeflow_, ACE_TString group_, ACE_TString args_, ACE_TString blocksize_, ACE_TString blockcount_, ACE_TString nameclient_, ACE_TString serverid_, int port, ACE_TString provider_, ACE_TString nbConsumersExpected_);

	/// Instance of the client connected to this handler.
	Client *myclient;
	
	public :

	// These functions are overrided functions from ACE_Event_Handler.

	Local_Event_Handler(ACE_Reactor *reactor):ACE_Event_Handler(reactor) {}
	virtual ~Local_Event_Handler() {};
	virtual int open();
	virtual int handle_input(ACE_HANDLE);
	virtual int handle_close(ACE_HANDLE = ACE_INVALID_HANDLE, ACE_Reactor_Mask = 0);
	virtual ACE_HANDLE get_handle() const;
	ACE_SOCK_Stream &peer() {return remote_peer_;}
};

/// This class describes the Handler which is used to handle the connection with a Duplicator of the NIST Data Flow System 2. She is registered to the reactor. She is instanced by the Local_Acceptor_Event_Handler at each new "Accept" on the SF2D_DEFAULT_LOCAL_PORT.

class Duplicator_Event_Handler : public ACE_Event_Handler
{
	protected :

	/// Socket used to communicate with a duplicator client.	  
	ACE_SOCK_Stream peer_;

	//may be useless
	DuplicatorManager dm;

	/// This function is called at each new input event on the socket. She will decide which methods to call regarding the type and target tags of the xml message 
	/**
	 * @param xml XML message to analyse
	 * @return 0 for Success, -1 otherwise
	 */  
	int actions_switch(ACE_TString xml);
  
	/// This method is used to read the message on the socket. It first reads the size of the message to allocate the right memory space, then the message himself.
 	/**
	 * @return xml message read from the socket
	 */    
	ACE_TString receive_xml();

	/// Let the server know that a new consumer has connected to the duplicator  
	/**
	 * @param xml XML message containing information about the consumer client node connected to the duplicator
	 * @return 0 for Success, -1 otherwise
	 */  
	int announceNewConsumerNodeConnectedToDuplicator(ACE_TString xml);
	
	/// Send the xml message to the specified socket ( which is a server socket )
	/**
	 * @param xml XML message to send
	 * @param peer_server Socket to send the message
	 * @return 0 for Success, -1 otherwise
	 */  
	//int send_xml_to_servers(ACE_SOCK_Stream *peer_server, ACE_TString xml);
	
	/**
	 * @param xml XML message to send
	 * @param peer_server Server to send the message
	 * @param xml Message to send
	 * @return 0 for Success, -1 otherwise
	 */  
	//int send_xml_to_servers(Server *peer_server, ACE_TString xml);
	
	/// Let the server know that a new remote duplicator has connected to the duplicator   
	/**
	 * @param xml XML message containing information about the new remote duplicator connected to the duplicator
	 * @return 0 for Success, -1 otherwise
	 */  
	int announceSuccessulConnectionToRemoteDuplicator(ACE_TString xml);

	/// The logAnnounce methods is used to propagate the log from a duplicator on the NDFS II network.
	/**
	 * @param xml XML message containing a log message
	 * @return 0 for Success, -1 otherwise
	 */
	int logAnnounce(ACE_TString xml);

	/**
	 * @param xml XML message about a connection request from a duplicator 
	 * @return 0 for Success, -1 otherwise
	 */  
	int connect(ACE_TString xml);

	/**
	 * @param xml XML message about the port number open by the duplicator provider for the remote consumers to connect to
	 * @return 0 for Success, -1 otherwise
	 */
	int updateFlowPortNumber(ACE_TString xml);

	ACE_TString add_flow_xml(ACE_TString name_, ACE_TString typeflow_, ACE_TString group_, ACE_TString args_, ACE_TString blocksize_, ACE_TString blockcount_, ACE_TString clientid_, ACE_TString serverid_, int port_, ACE_TString provider_, ACE_TString nbConsumersExpected_);

	public :

	// These functions are overrided functions from ACE_Event_Handler.

	Duplicator_Event_Handler(ACE_Reactor *reactor):ACE_Event_Handler(reactor) {}
	virtual ~Duplicator_Event_Handler() {};
	virtual int open();
	virtual int handle_input(ACE_HANDLE);
	virtual int handle_close(ACE_HANDLE = ACE_INVALID_HANDLE, ACE_Reactor_Mask = 0);
	virtual ACE_HANDLE get_handle() const;
	ACE_SOCK_Stream &peer() {return peer_;}
	
	/// Send the xml to the duplicator connected to the handler  
	/**
	 * @param xml XML message to send
	 * @return 0 for Success, -1 otherwise
	 */  
	int send_xml(ACE_TString xml);

	/// Send the data transfer request to the duplicator.
	/**
	 * @return 0 for Success, -1 otherwise
	 */  
	int notifyDuplicatorToStartDataTransfer();
};


class Log_Event_Handler : public ACE_Event_Handler {
  public:
	  Log_Event_Handler(ACE_Reactor *reactor);
	  virtual int handle_input(ACE_HANDLE handle_ = ACE_INVALID_HANDLE);
	  virtual int handle_timeout(const ACE_Time_Value &currentTime, const void * = 0 );
	  static ACE_TString create_XML_message(ACE_TString message_);
	  
	  void enableTimer();
	  void disableTimer();
  private:
	  //int send_xml_to_servers(ACE_SOCK_Stream *peer_server, ACE_TString xml);
	  //int send_xml_to_servers(Server *peer_server, ACE_TString xml);
};

class Server_Stopper : public ACE_Event_Handler {
  public:
	Server_Stopper(ACE_Reactor *reactor, int signum = SIGTERM);
	virtual int handle_signal(int signum, siginfo_t* = 0, ucontext_t * = 0);
};

#endif
