#include "ace/Process.h"

#include "Thread_Server.h"
#include "Peer_Event_Handler.h"
#include "Log_Manager.h"
#include "list/LastModifications.h"
#include "list/SocketList.h"
#include "list/ServerList.h"

#define LAUNCH_ERROR_MESSAGE    ACE_TEXT("<?xml version='1.0'?>\n<smartspace>\n <type>5</type>\n <target>28</target> <clientname>%s</clientname>\n <clientgroup>%s</clientgroup>\n </smartspace>")


void *Thread_Server::run_svc(void *arg)
{	
	ACE_TRACE(ACE_TEXT("Thread_Server::run_svc"));
	Thread_Args* args = (ACE_static_cast(Thread_Args*, arg));
	ACE_TString tospawn = args->spawn;

	ACE_TString clientname = args->clientname;
	ACE_TString clientgroup = args->clientgroup;
	ACE_Process_Options options(1, 20480);
	ACE_Process child;    
	options.command_line(ACE_TEXT("%s"), tospawn.c_str() );
	
	pid_t pid = child.spawn(options);	   		
	if (pid != -1) { // pid == -1 on windows when the path is wrong
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Thread_Server::run_svc ] Process Launched : %s --- PID: %d\n") , options.command_line_buf (), pid ));	
		child.wait() ; 
		int exit_code = child.exit_code();
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Thread_Server::run_svc ] Process %s regturns exit code: %d\n and return value: %d\n") , options.command_line_buf (), exit_code, child.return_value() ));
		if (exit_code != 512) { // it seems that 512 is the returned code on linux when the path is wrong
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Thread_Server::run_svc ] Process Killed : %s \nExit code : %d\n") , options.command_line_buf (), exit_code ));
			return NULL;
		}
	}
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ Thread_Server::run_svc ] Problem with the process : %s \n") , options.command_line_buf () ));	
	//Adding modifications for the client only, not for the duplicator
	if (clientname != ACE_TEXT("") && clientgroup != ACE_TEXT("")) {
		LastModifications *lm = LAST_MODIFICATIONS::instance(); 
		ACE_TCHAR modif[1024]={0};
		ACE_OS::sprintf(modif,LAUNCH_ERROR_MESSAGE, clientname.c_str(), clientgroup.c_str());
		lm->addModifications(modif) ;	
		
		//We inform the others servers of the modification	 
		ServerList *sl = SERVER_LIST::instance();
		ACE_TString modifString(modif);
		for (SERVER_ITERATOR si = sl->first(); si != sl->last(); si++) 
		{
			Peer_Event_Handler::send_xml_to_servers((*si).int_id_, modifString.c_str());
		}
		
		
		/*SocketList *sl = SOCKET_LIST::instance();
		for (SOCKET_ITERATOR si = sl->firstServer(); si != sl->lastServer(); si++)
		{
			ACE_TString modifString(modif);
			char size[512] = {0};
			ACE_OS::snprintf(size, sizeof(size), "%d", modifString.length());
			sl->getServer(SocketList::getIDFromIterator(si))->send_n(size, sizeof(size));
			
			const char *xml_message = ACE_TEXT_ALWAYS_CHAR(modifString.c_str());
			sl->getServer(SocketList::getIDFromIterator(si))->send_n(xml_message, ACE_OS::strlen(xml_message));
		} */ 
	}
	return NULL;
}
