#include "XML_Parser.h"
#include "ace/Version.h"
#include "Log_Manager.h"

XML_SaxHandler::XML_SaxHandler ()
{
  logMessages.clear();
}

XML_SaxHandler::~XML_SaxHandler (void)
{
  logMessages.clear();
}

void XML_SaxHandler::startDocument (ACEXML_ENV_SINGLE_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
{
}

void XML_SaxHandler::endDocument (ACEXML_ENV_SINGLE_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
{
}

void XML_SaxHandler::startElement( const ACEXML_Char *uri,const ACEXML_Char *name, const ACEXML_Char *qName, ACEXML_Attributes *alist ACEXML_ENV_ARG_DECL_NOT_USED)
  ACE_THROW_SPEC ((ACEXML_SAXException))
{
	ACE_TRACE(ACE_TEXT("XML_SaxHandler::startElement"));
	ACE_UNUSED_ARG(uri);
	ACE_UNUSED_ARG(qName);
	ACE_UNUSED_ARG(alist);
	state = name ;
}

void XML_SaxHandler::endElement (const ACEXML_Char * uri, const ACEXML_Char *name, const ACEXML_Char *qName ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
{
	ACE_TRACE(ACE_TEXT("XML_SaxHandler::endElement"));
	ACE_UNUSED_ARG(uri);
	ACE_UNUSED_ARG(name);
	ACE_UNUSED_ARG(qName);
	state = ACE_TEXT("");
}

//void XML_SaxHandler::characters (const ACEXML_Char *cdata, size_t start, size_t length ACEXML_ENV_ARG_DECL_NOT_USED ) ACE_THROW_SPEC ((ACEXML_SAXException))
 #if (ACE_MAJOR_VERSION < 5)
  void XML_SaxHandler::characters(const ACEXML_Char *cdata,int start,int length  ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
  #endif    
  
  #if (ACE_MAJOR_VERSION > 5)
  void XML_SaxHandler::characters(const ACEXML_Char *cdata,size_t start,size_t length  ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
  #endif
        
  #if (ACE_MAJOR_VERSION == 5)
    #if (ACE_MINOR_VERSION < 5)
        void XML_SaxHandler::characters(const ACEXML_Char *cdata,int start,int length  ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
    #endif
    #if (ACE_MINOR_VERSION > 5)
        void XML_SaxHandler::characters(const ACEXML_Char *cdata,size_t start,size_t length  ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
    #endif
    #if (ACE_MINOR_VERSION == 5)
        #if (ACE_BETA_VERSION >= 4)
            void XML_SaxHandler::characters(const ACEXML_Char *cdata,size_t start,size_t length  ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
       #else
            void XML_SaxHandler::characters(const ACEXML_Char *cdata,int start,int length  ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
       #endif
    #endif
  #endif
{
	ACE_TRACE(ACE_TEXT("XML_SaxHandler::characters"));
	ACE_UNUSED_ARG(start);
	ACE_UNUSED_ARG(length);
	ACE_TString infos = cdata ;
	if ( state != "message" )
	  Xml_infos.rebind( state , infos );
	else
	  logMessages.push_back(infos);
}

void XML_SaxHandler::fatalError (ACEXML_SAXParseException& ex ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException)) {
  ACE_TRACE(ACE_TEXT("XML_SaxHandler::fatalErro"));
  ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ XML_SaxHandler::FATAL ERROR ] \n")));
  ex.print();
}

void XML_SaxHandler::error (ACEXML_SAXParseException & ex ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException)) {
  ACE_TRACE(ACE_TEXT("XML_SaxHandler::error"));
  ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ XML_SaxHandler::ERROR ] \n")));
  ex.print();
}

void XML_SaxHandler::warning (ACEXML_SAXParseException & ex ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException)) {
  ACE_TRACE(ACE_TEXT("XML_SaxHandler::warning"));
  ACE_UNUSED_ARG(ex);
  ACE_DEBUG((SF2D_WARNING_PREFIX ACE_TEXT("[ XML_SaxHandler::WARNING ] \n")));
  //ex.print();
}

void XML_SaxHandler::ignorableWhitespace (const ACEXML_Char *, int, int  ACEXML_ENV_ARG_DECL_NOT_USED)  ACE_THROW_SPEC ((ACEXML_SAXException))
{
  ACE_TRACE(ACE_TEXT("XML_SaxHandler::ignorableWhitespace"));
  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ XML_SaxHandler::ignorableWhitespace ] *Event ignorableWhitespace () ***************\n")));
}

ACE_TString XML_SaxHandler::getInfos(ACE_TString s) {
  ACE_TRACE(ACE_TEXT("XML_SaxHandler::getInfos"));
  ACE_TString r;
  if (Xml_infos.find(s, r) == -1)
    {
      ACE_DEBUG((SF2D_WARNING_PREFIX ACE_TEXT("[ XML_SaxHandler::getInfos ] No value found for element \"%s\" \n"),s.c_str()));
      return ACE_TEXT("");
    }
  else
    {
      return r;
    }
}
