#include "ClientList.h"
#include "../Log_Manager.h"

ClientList::ClientList()
{
	ACE_TRACE(ACE_TEXT("ClientList::ClientList"));

	init();
}

ClientList::~ClientList()
{
	ACE_TRACE(ACE_TEXT("ClientList::~ClientList"));

	fini();
}

int ClientList::init()
{
	ACE_TRACE(ACE_TEXT("ClientList::init"));
	
	clients.open();
	
	return 0;
}
/*
ClientList::ClientList(ACE_TString xml_)
{
	init(xml_);
}

int ClientList::init(ACE_TString xml_)
{
	int pos, pos2;
	
	// <ClientList> tag
	
	pos = xml_.strstr(ACE_TEXT("<ClientList>"));
	if (pos == -1)
	{
		ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ClientList::fromXml ] ERROR: Parse error, <ClientList> tag not found\n")), -1);
	}
	else
	{
		xml_ = xml_.substr(pos + 12);
	}

	// Process all clients
	
	pos = xml_.strstr(ACE_TEXT("<Client ID=\""));
	while (pos != -1)
	{
		pos2 = xml_.strstr(ACE_TEXT("</Client>"));
		if (pos2 == -1)
		{
			ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ClientList::fromXml ] ERROR: Parse error, </Client> tag not found\n")), -1);
		}
		else
		{
			Client *c = new Client(xml_.substr(0, pos2 + 9));
			add(c);
			xml_ = xml_.substr(pos2 + 9);
		}
		pos = xml_.strstr(ACE_TEXT("<Client ID=\""));
	}
	
	// </ClientList> tag
	
	pos = xml_.strstr(ACE_TEXT("</ClientList>"));
	if (pos == -1)
	{
		ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ClientList::fromXml ] ERROR: Parse error, </ClientList> tag not found\n")), -1);
	}
	
	return 0;
}*/

int ClientList::fini()
{
	ACE_TRACE(ACE_TEXT("ClientList::fini"));
	
	clients.close();
	
	return 0;
}

int ClientList::count()
{
	ACE_TRACE(ACE_TEXT("ClientList::count"));
	
	return clients.current_size();
}

int ClientList::add(Client *client_)
{
	ACE_TRACE(ACE_TEXT("ClientList::add"));

	int result = clients.trybind(client_->getID(), client_);
		
	if (result == 1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ ClientList::add ] ERROR: Client '%s' already exists\n"), client_->getID().c_str()));
		return 1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ClientList::add ] ERROR: Client '%s' already exists\n"), client_->getID().c_str()), 1);
	}
	else if (result == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ ClientList::add ] ERROR: Client '%s' not added\n"), client_->getID().c_str()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ClientList::add ] ERROR: Client '%s' not added\n"), client_->getID().c_str()), -1);
	}
	else
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ClientList::add ] Client '%s' successfully added\n"), client_->getID().c_str()));
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ClientList::add ] List of clients in clientlist : \n")));
		
		for (CLIENT_ITERATOR ci = clients.begin(); ci != clients.end(); ci++)
		{
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("      %s \n"), (*ci).int_id_->getID().c_str()));
		}
		
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("\n")));
		return 0;
	}
}

int ClientList::remove(ACE_TString clientID_)
{
	ACE_TRACE(ACE_TEXT("ClientList::remove"));

	Client *c = 0;
	if (clients.find(clientID_, c) == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ ClientList::remove ] ERROR: Client '%s' not found\n"), clientID_.c_str()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ClientList::remove ] ERROR: Client '%s' not found\n"), clientID_.c_str()), -1);
	}
	else
	{
		// compare returns 0 if the two strings are equal
		ACE_ASSERT(clientID_.compare(c->getID()) == 0);
		
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ClientList::remove ] Client '%s' found : '%s'\n"), clientID_.c_str(), c->getID().c_str()));
		if (clients.unbind(clientID_) == -1)
		{
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ ClientList::remove ] ERROR: Client '%s' not removed\n"), clientID_.c_str()));
			return -1;
			//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ClientList::remove ] ERROR: Client '%s' not removed\n"), clientID_.c_str()), -1);
		}
		else
		{
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ClientList::remove ] Client '%s' removed\n"), clientID_.c_str()));
			return 0;
		}
	}
}

Client *ClientList::get(ACE_TString clientID_)
{
	ACE_TRACE(ACE_TEXT("ClientList::get"));

	Client *c = 0;
	if (clients.find(clientID_, c) == -1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ClientList::get ] Client '%s' not found\n"), clientID_.c_str()));
		return 0;
	}
	else
	{
		/* compare returns 0 if the two strings are equal */
		ACE_ASSERT(clientID_.compare(c->getID()) == 0);
		
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ClientList::get ] Client '%s' found : '%s'\n"), clientID_.c_str(), c->getID().c_str()));
		return c;
	}
}

CLIENT_ITERATOR ClientList::first()
{
	ACE_TRACE(ACE_TEXT("ClientList::first"));
	
	return clients.begin();
}

CLIENT_ITERATOR ClientList::last()
{
	ACE_TRACE(ACE_TEXT("ClientList::last"));

	return clients.end();
}

Client *ClientList::fromIterator(CLIENT_ITERATOR clientIterator_)
{
	ACE_TRACE(ACE_TEXT("ClientList::fromIterator"));
	
	return (*clientIterator_).int_id_;
}

int ClientList::find(ACE_TString clientID_)
{
	ACE_TRACE(ACE_TEXT("ClientList::find"));
	
	if (clients.find(clientID_) == -1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ClientList::find ] This clientlist does not contain client '%s'\n"), clientID_.c_str()));
		return -1;
	}
	else
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ClientList::find ] This clientlist contains client '%s'\n"), clientID_.c_str()));
		return 0;
	}
}

int ClientList::find(ACE_TString group_, ACE_TString name_)
{
	ACE_TRACE(ACE_TEXT("ClientList::find"));
	
	ACE_TString id = Client::getIDFromGroupAndName(group_, name_);

	int result = clients.find(id);
	
	
	if (result == -1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ClientList::find ] This client list does not contain client of group '%s' and name '%s'\n"), group_.c_str(), name_.c_str()));
	}
	else
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ClientList::find ] This client list contains client of group '%s' and name '%s'\n"), group_.c_str(), name_.c_str()));
	}
	
	return result;
}

ACE_TString ClientList::toXmlWithIndent(int n_)
{
	ACE_TRACE(ACE_TEXT("ClientList::toXmlWithIndent"));
	
	if (n_ < 0 || n_ > 255)
	{
		ACE_ERROR((LM_ERROR, ACE_TEXT("[ Client::toXmlWithIndent ] ERROR: indent lower than 0 or greater than 255\n")));
	}

	ACE_TString unit = ACE_TEXT("    ");
	ACE_TString indent = ACE_TEXT("");
	
	for (int i = 0; i < n_; i++)
	{
		indent += unit;
	}	

	ACE_TString xml = ACE_TEXT("");
	
	xml += indent;
	xml += ACE_TEXT("<ClientList>\n");
	
	for (CLIENT_ITERATOR ci = clients.begin(); ci != clients.end(); ci++)
	{
		xml += (*ci).int_id_->toXmlWithIndent(n_ + 1);
	}

	xml += indent;
	xml += ACE_TEXT("</ClientList>\n");

	return xml;
}

ACE_TString ClientList::toXml()
{
	ACE_TRACE(ACE_TEXT("ClientList::toXml"));
	
	return toXmlWithIndent(0);
}
