#ifndef _CLIENTLIST_H_
#define _CLIENTLIST_H_

#include "ace/ACE.h"
#include "ace/SString.h"
#include "ace/Log_Msg.h"
#include "ace/Null_Mutex.h"
#include "ace/Hash_Map_Manager_T.h"

#include "Client.h"

typedef ACE_Hash_Map_Manager_Ex<ACE_TString, Client *, ACE_Hash<ACE_TString>, ACE_Equal_To<ACE_TString>, ACE_Null_Mutex> CLIENT_MAP;
typedef ACE_Hash_Map_Iterator_Ex<ACE_TString, Client *, ACE_Hash<ACE_TString>, ACE_Equal_To<ACE_TString>, ACE_Null_Mutex> CLIENT_ITERATOR;

/// This class describes a client list object.

class ClientList
{
	protected:
		int init();
		//int init(ACE_TString xml_);
		int fini();
		
		/// Internal map used to store data
		CLIENT_MAP clients;
	
	public:
		/// Create an empty client list
		ClientList();
		
		/// Create a client list giving an Xml string
		//ClientList(ACE_TString xml_);

		/// Destroy the client list
		virtual ~ClientList();
		
		/// Add a client to the client list
		
		/**
		 * @param client_ pointer to the client to add
		 * @return 0 if the client was added succesfully, 1 if the client already
		 * exists in the list, and -1 if it was not added
		 */
		int add(Client *client_);

		/// Remove a client from the client list
		
		/**
		 * @param clientID_ ID of the client to remove
		 * @return 0 if the client was removed succesfully, and -1 if not
		 */
		int remove(ACE_TString clientID_);

		/// Return a pointer to a client giving its ID
		
		/**
		 * @param clientID_ ID of the client to get
		 * @return a pointer to the client, null pointer if client is not in the client list
		 */
		Client *get(ACE_TString clientID_);
		
		/// Check if a client is in the client list giving its ID
		
		/**
		 * @param clientID_ ID of the client to check
		 * @return 0 if the client is in the list, and -1 if not
		 */
		int find(ACE_TString clientID_);

		/// Check if a client is in the client list giving its group and name
		
		/**
		 * @param group_ group of the client to check
		 * @param name_ name of the client to check
		 * @return 0 if the client is in the list, and -1 if not
		 */
		int find(ACE_TString group_, ACE_TString name_);

		/// Count the number of clients in the client list
		
		/**
		 * @return the number of clients in the client list
		 */
		int count();

		/// Return an iterator on the first client of the client list
		
		/**
		 * @return iterator on the first client of the client list
		 */
		CLIENT_ITERATOR first();

		/// Return an iterator on the last client of the client list
		
		/**
		 * @return iterator on the last client of the client list
		 */
		CLIENT_ITERATOR last();

		/// Return a pointer to the client of the entry pointed by the iterator
		
		/**
		 * @param clientIterator_ the client iterator
		 * @return a pointer to the client
		 */	
		static Client *fromIterator(CLIENT_ITERATOR clientIterator_);

		/// Return the client list represented as an Xml string
		
		/**
		 * @return the client list represented as an Xml string
		 */
		ACE_TString toXml();

		/// Return the client list represented as an Xml string shifted at the specified offset
		
		/**
		 * @param n_ the offset of the shift
		 * @return the client list represented as an Xml string shifted at the specified offset
		 */
		ACE_TString toXmlWithIndent(int n_);
};

#endif /* _CLIENTLIST_H_ */
