#include "DuplicatorList.h"
#include "../Log_Manager.h"

DuplicatorInfo::DuplicatorInfo() {
	ACE_TRACE(ACE_TEXT("DuplicatorInfo::DuplicatorInfo"));
	dupli = 0;
	mustConnectToDistantDupli = false;
	mustOpenProviderPart = false;
}

DuplicatorList::DuplicatorList()//unsigned int minPortNumber_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::DuplicatorList"));
	init();//minPortNumber_);
}

DuplicatorList::~DuplicatorList()
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::~DuplicatorList"));
	fini();
}

int DuplicatorList::init()//unsigned int minPortNumber_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::init"));

	minPortNumber = 25000;//minPortNumber_;
	currentPortNumber = minPortNumber;

	duplicators.open();

	return 0;
}

int DuplicatorList::fini()
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::fini"));
	
	return 0;
}

unsigned int DuplicatorList::nextPortSupposedToBeAvailable()
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::nextPortSupposedToBeAvailable"));

	currentPortNumber++;
	
	return currentPortNumber;
}

void DuplicatorList::setNextAvailablePort(unsigned int portUsed) {
	if (portUsed > currentPortNumber)
		currentPortNumber = portUsed;
}

int DuplicatorList::count()
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::count"));
	
	return duplicators.current_size();
}

int DuplicatorList::add(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::add"));
	
	DuplicatorInfo *di;
	ACE_NEW_RETURN(di, DuplicatorInfo, 0);
	
	int result = duplicators.trybind(flowID_, di);
		
	if (result == 1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("Flow '%s' already exists in list\n"), flowID_.c_str()));
		return 1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("Flow '%s' already exists in list\n"), flowID_.c_str()), 1);
	}
	else if (result == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorList::add ] ERROR: Flow '%s' not added\n"), flowID_.c_str()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorList::add ] ERROR: Flow '%s' not added\n"), flowID_.c_str()), -1);
	}

	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorList::add ] Flow '%s' added\n"), flowID_.c_str()));
	
	return 0;
}

int DuplicatorList::addProducer(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::addProducer"));
	
	int result;
	
	// Found one available port, add the flow to the list
	result = add(flowID_);

	return result;
}

int DuplicatorList::addConsumer(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::addConsumer"));
	
	int result;
	
	// Add the flow to the list with 0 as port which means it is a consumer
	result = add(flowID_);

	return result;
}

int DuplicatorList::remove(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::remove"));
	
	duplicators.unbind(flowID_);
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorList::remove ] Flow '%s' successfully removed\n"), flowID_.c_str()));
		
	return 0;
}

Duplicator_Event_Handler* DuplicatorList::getDupEventHandler(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::getDupEventHandler"));
	
	DuplicatorInfo *di;
	ACE_NEW_RETURN(di, DuplicatorInfo, 0);
	
	if (duplicators.find(flowID_, di) == -1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorList::getDupEventHandler ] Flow '%s' does not exist\n"), flowID_.c_str()));
		return NULL;
	}

	return di->dupli;
}

int DuplicatorList::setDupEventHandler(ACE_TString flowID_, Duplicator_Event_Handler* handler)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::setDupEventHandler"));
	
	DuplicatorInfo *di;
	ACE_NEW_RETURN(di, DuplicatorInfo, 0);
	
	if (duplicators.find(flowID_, di) == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorList::setDupEventHandler ] Flow '%s' does not exist\n"), flowID_.c_str()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorList::setDupEventHandler ] Flow '%s' does not exist\n"), flowID_.c_str()), -1);
	}

	di->dupli = handler;

	return 0;
}

int DuplicatorList::setMustConnectToDistantDupli(ACE_TString flowID_, bool mustConnectToDistantDupli) {
	ACE_TRACE(ACE_TEXT("DuplicatorList::setMustConnectToDistantDupli"));

	DuplicatorInfo *di;
	ACE_NEW_RETURN(di, DuplicatorInfo, 0);
	
	if (duplicators.find(flowID_, di) == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorList::setMustConnectToDistantDupli ] Flow '%s' does not exist\n"), flowID_.c_str()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorList::setMustConnectToDistantDupli ] Flow '%s' does not exist\n"), flowID_.c_str()), -1);
	}

	di->mustConnectToDistantDupli = mustConnectToDistantDupli;

	return 0;
}

bool DuplicatorList::mustConnectToDistantDupli(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::mustConnectToDistantDupli"));
	
	DuplicatorInfo *di;
	ACE_NEW_RETURN(di, DuplicatorInfo, 0);
	
	if (duplicators.find(flowID_, di) == -1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorList::mustConnectToDistantDupli ] Flow '%s' does not exist\n"), flowID_.c_str()));
		return false;
	}

	return di->mustConnectToDistantDupli;
}

int DuplicatorList::setMustOpenProviderPart(ACE_TString flowID_, bool mustOpenProviderPart) {
	ACE_TRACE(ACE_TEXT("DuplicatorList::setMustOpenProviderPart"));

	DuplicatorInfo *di;
	ACE_NEW_RETURN(di, DuplicatorInfo, 0);
	
	if (duplicators.find(flowID_, di) == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ DuplicatorList::setMustOpenProviderPart ] Flow '%s' does not exist\n"), flowID_.c_str()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ DuplicatorList::setMustOpenProviderPart ] Flow '%s' does not exist\n"), flowID_.c_str()), -1);
	}

	di->mustOpenProviderPart = mustOpenProviderPart;

	return 0;
}

bool DuplicatorList::mustOpenProviderPart(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::mustOpenProviderPart"));
	
	DuplicatorInfo *di;
	ACE_NEW_RETURN(di, DuplicatorInfo, 0);
	
	if (duplicators.find(flowID_, di) == -1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorList::mustOpenProviderPart ] Flow '%s' does not exist\n"), flowID_.c_str()));
		return false;
	}

	return di->mustOpenProviderPart;
}

DUPLICATOR_ITERATOR DuplicatorList::first()
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::first"));

	return duplicators.begin();
}

DUPLICATOR_ITERATOR DuplicatorList::last()
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::last"));

	return duplicators.end();
}

int DuplicatorList::find(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::find"));
	
	if (duplicators.find(flowID_) == -1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorList::find ] This duplicator list does not contain flow '%s'\n"), flowID_.c_str()));
		return -1;
	}
	else
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ DuplicatorList::find ] This duplicator list contains flow '%s'\n"), flowID_.c_str()));
		return 0;
	}
}

ACE_TString DuplicatorList::toXml() {
	ACE_TRACE(ACE_TEXT("DuplicatorList::toXml"));
	
	return toXmlWithIndent(0);
}

ACE_TString DuplicatorList::toXmlWithIndent(int n_) {
	
	ACE_TRACE(ACE_TEXT("DuplicatorList::toXmlWithIndent"));
	
	if (n_ < 0 || n_ > 255)
	{
		ACE_ERROR((LM_ERROR, ACE_TEXT("[ DuplicatorList::toXmlWithIndent ] ERROR: indent lower than 0 or greater than 255\n")));
	}


	// temporary for testing purpose only
	//this class only handle the list of the LOCAL duplicators!!!
	char namelocal[256] = "";
	ACE_OS::hostname(namelocal,256);
	//std::string serverName;
	//serverName = namelocal;
	ACE_TString serverName(namelocal);

	ACE_TString unit = ACE_TEXT("    ");
	ACE_TString indent = ACE_TEXT("");
	
	for (int i = 0; i < n_; i++)
	{
		indent += unit;
	}	

	ACE_TString xml = ACE_TEXT("");
	
	xml += indent;
	xml += ACE_TEXT("<DuplicatorList>\n");
	
	for (DUPLICATOR_ITERATOR di = first(); di != last(); di++)
	{
		//xml += (*ci).int_id_->toXmlWithIndent(n_ + 1);
		ACE_TString theId = (*di).ext_id_;
		xml += dupInfoToXmlWithIndent(n_ + 1, theId, serverName);
		ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("Found one dup, key=%s, server=%s"), theId.c_str(), serverName.c_str()));
	}

	xml += indent;
	xml += ACE_TEXT("</DuplicatorList>\n");

	return xml;

}

ACE_TString DuplicatorList::dupInfoToXmlWithIndent(int n_, ACE_TString dup_id_, ACE_TString server_name_) {
	ACE_TRACE(ACE_TEXT("DuplicatorList::dupInfoToXmlWithIndent"));
	
	if (n_ < 0 || n_ > 255)
	{
		ACE_ERROR((LM_ERROR, ACE_TEXT("[ DuplicatorList::dupInfoToXmlWithIndent ] ERROR: indent lower than 0 or greater than 255\n")));
	}

	ACE_TString unit = ACE_TEXT("    ");
	ACE_TString indent = ACE_TEXT("");
	
	for (int i = 0; i < n_; i++)
	{
		indent += unit;
	}	

	ACE_TString xml = ACE_TEXT("");
	
	xml += indent;
	xml += ACE_TEXT("<Duplicator ID=\"");
	xml += dup_id_;
	xml += ACE_TEXT("\">\n");
	
	xml += indent;
	xml += unit;
	xml += ACE_TEXT("<Server>");
	xml += server_name_;
	xml += ACE_TEXT("</Server>\n");

	xml += indent;
	xml += ACE_TEXT("</Duplicator>\n");
	
	return xml;
}

Duplicator_Event_Handler* DuplicatorList::getDupEventHandlerFromIterator(DUPLICATOR_ITERATOR duplicatorIterator_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::getDupEventHandlerFromIterator"));
	
	return (*duplicatorIterator_).int_id_->dupli;
}

ACE_TString DuplicatorList::getIDFromIterator(DUPLICATOR_ITERATOR duplicatorIterator_)
{
	ACE_TRACE(ACE_TEXT("DuplicatorList::getIDFromIterator"));

	return (*duplicatorIterator_).ext_id_;
}
