#include "FlowList.h"
#include "../Log_Manager.h"

FlowList::FlowList()
{
	ACE_TRACE(ACE_TEXT("FlowList::FlowList"));
	init();
}

FlowList::~FlowList()
{
	ACE_TRACE(ACE_TEXT("FlowList::~FlowList"));
	fini();
}

int FlowList::init()
{
	ACE_TRACE(ACE_TEXT("FlowList::init"));
	
	flows.open();
	
	return 0;
}

/*FlowList::FlowList(ACE_TString xml_)
{
	init(xml_);
}

int FlowList::init(ACE_TString xml_)
{
	ACE_TRACE(ACE_TEXT("FlowList::init"));

	int pos, pos2;
	
	// <FlowList> tag
	
	pos = xml_.strstr(ACE_TEXT("<FlowList>"));
	if (pos == -1)
	{
		ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ FlowList::fromXml ] ERROR: Parse error, <FlowList> tag not found\n")), -1);
	}
	else
	{
		xml_ = xml_.substr(pos + 10);
	}

	// Process all flows
	
	pos = xml_.strstr(ACE_TEXT("<Flow ID=\""));
	while (pos != -1)
	{
		pos2 = xml_.strstr(ACE_TEXT("</Flow>"));
		if (pos2 == -1)
		{
			ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ FlowList::fromXml ] ERROR: Parse error, </Flow> tag not found\n")), -1);
		}
		else
		{
			Flow *c = new Flow(xml_.substr(0, pos2 + 7));
			add(c);
			xml_ = xml_.substr(pos2 + 7);
		}
		pos = xml_.strstr(ACE_TEXT("<Flow ID=\""));
	}
	
	// </FlowList> tag
	
	pos = xml_.strstr(ACE_TEXT("</FlowList>"));
	if (pos == -1)
	{
		ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ FlowList::fromXml ] ERROR: Parse error, </FlowList> tag not found\n")), -1);
	}
	
	return 0;
}*/

int FlowList::fini()
{
	ACE_TRACE(ACE_TEXT("FlowList::fini"));

	flows.close();

	return 0;
}

int FlowList::count()
{
	ACE_TRACE(ACE_TEXT("FlowList::count"));
	
	return flows.current_size();
}

int FlowList::add(Flow *flow_)
{
	ACE_TRACE(ACE_TEXT("FlowList::add"));
	
	int result = flows.trybind(flow_->getID(), flow_);
		
	if (result == 1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ FlowList::add ] Flow '%s' already exists\n"), flow_->getID().c_str()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ FlowList::add ] Flow '%s' already exists\n"), flow_->getID().c_str()), 1);
	}
	else if (result == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ FlowList::add ] ERROR: flow '%s' not added\n"), flow_->getID().c_str()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ FlowList::add ] ERROR: flow '%s' not added\n"), flow_->getID().c_str()), -1);
	}
	else
	{
		ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("Flow '%s' added\n"), flow_->getID().c_str()));
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::add ] Flow '%s' successfully added\n"), flow_->getID().c_str()));
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::add ] List of flows in flowlist : \n")));
		
		for (FLOW_ITERATOR fi = flows.begin(); fi != flows.end(); fi++)
		{
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("%s \n"), (*fi).int_id_->getID().c_str()));
		}
		
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("\n")));
		
		return 0;
	}
}

int FlowList::remove(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("FlowList::remove"));
	
	flows.unbind(flowID_);
	ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("Flow '%s' removed\n"), flowID_.c_str()));
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::remove ] Flow '%s' successfully removed\n"), flowID_.c_str()));

	return 0;
}

Flow *FlowList::get(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("FlowList::get"));
	
	Flow *f = 0;
	
	if (flows.find(flowID_, f) == -1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::get ] Flow '%s' does not exist\n"), flowID_.c_str()));
		return 0;
	}
	else
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::get ] Flow '%s' exists\n"), flowID_.c_str()));
		return f;
	}
}

FLOW_ITERATOR FlowList::first()
{
	ACE_TRACE(ACE_TEXT("FlowList::first"));

	return flows.begin();
}

FLOW_ITERATOR FlowList::last()
{
	ACE_TRACE(ACE_TEXT("FlowList::last"));

	return flows.end();
}

int FlowList::find(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("FlowList::find"));
	
	if (flows.find(flowID_) == -1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::find ] This flowlist does not contain flow '%s'\n"), flowID_.c_str()));
		return -1;
	}
	else
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::find ] This flowlist contains flow '%s'\n"), flowID_.c_str()));
		return 0;
	}
}

int FlowList::find(ACE_TString group_, ACE_TString type_)
{
	ACE_TRACE(ACE_TEXT("FlowList::find"));
	
	ACE_TString id = Flow::getIDFromGroupAndType(group_, type_);

	int result = flows.find(id);
	
	if (result == -1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::find ] This flow list does not contain client of group '%s' and type '%s'\n"), group_.c_str(), type_.c_str()));
	}
	else
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::find ] This flow list contains client of group '%s' and type '%s'\n"), group_.c_str(), type_.c_str()));
	}
	
	return result;
}

int FlowList::contains(ACE_TString flowXML_)
{
	ACE_TRACE(ACE_TEXT("FlowList::contains"));

	int pos;
	ACE_TString flowID_;

	//We parse the client parameter in order to find the flow ID
	
	pos = flowXML_.strstr(ACE_TEXT("<Flow ID=\""));
	if (pos == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ FlowList::contains ] ERROR: Parse error, <Flow> tag not found\n")));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ FlowList::contains ] ERROR: Parse error, <Flow> tag not found\n")), -1);
	}
	else
	{
		flowXML_ = flowXML_.substr(pos + 10);
	}

	pos = flowXML_.strstr(ACE_TEXT("\">"));
	if (pos == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ FlowList::contains ] ERROR: Parse error, ending of <Flow> tag not found\n")));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ FlowList::contains ] ERROR: Parse error, ending of <Flow> tag not found\n")), -1);
	}
	else
	{
		flowID_ = flowXML_.substr(0, pos);
	}

	// We search for the flow thanks to its flow ID

	if (flows.find(flowID_) == -1)
	{
	  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::contains ] This flowlist does not contain flow\n")));
		return -1;
	}
	else
	{
	  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::contains ] This flowlist contains flow\n")));
		return 0;
	}
}

int FlowList::updateConsumersOfFlow(ACE_TString flowXML_)
{
	ACE_TRACE(ACE_TEXT("FlowList::update"));

	int pos;
	ACE_TString flowID_;
	ACE_TString flowXML = flowXML_;
		
	//We parse the client parameter in order to find the flow ID
	
	pos = flowXML.strstr(ACE_TEXT("<Flow ID=\""));
	if (pos == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ FlowList::update ] ERROR: Parse error, <Flow> tag not found\n")));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ FlowList::update ] ERROR: Parse error, <Flow> tag not found\n")), -1);
	}
	else
	{
		flowXML = flowXML.substr(pos + 10);
	}

	pos = flowXML.strstr(ACE_TEXT("\">"));
	if (pos == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ FlowList::update ] ERROR: Parse error, ending of <Flow> tag not found\n")));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ FlowList::update ] ERROR: Parse error, ending of <Flow> tag not found\n")), -1);
	}
	else
	{
		flowID_ = flowXML.substr(0, pos);
	}

	// We try to get the flow thanks to its flow ID
	
	Flow *flow = get(flowID_);

	if (flow == NULL)
	{
	  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::update ] This flowlist does not contain flow\n")));
		return -1;
	}
	else
	{
	  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::update ] This flowlist contains flow, we will update it\n")));
	  return flow->updateConsumers(flowXML_);
	}
}

int FlowList::updateProducerOfFlow(ACE_TString flowXML_)
{
	ACE_TRACE(ACE_TEXT("FlowList::update"));

	int pos;
	ACE_TString flowID_;
	ACE_TString flowXML = flowXML_;
		
	//We parse the client parameter in order to find the flow ID
	
	pos = flowXML.strstr(ACE_TEXT("<Flow ID=\""));
	if (pos == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ FlowList::update ] ERROR: Parse error, <Flow> tag not found\n")));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ FlowList::update ] ERROR: Parse error, <Flow> tag not found\n")), -1);
	}
	else
	{
		flowXML = flowXML.substr(pos + 10);
	}

	pos = flowXML.strstr(ACE_TEXT("\">"));
	if (pos == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ FlowList::update ] ERROR: Parse error, ending of <Flow> tag not found\n")));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ FlowList::update ] ERROR: Parse error, ending of <Flow> tag not found\n")), -1);
	}
	else
	{
		flowID_ = flowXML.substr(0, pos);
	}

	// We try to get the flow thanks to its flow ID
	
	Flow *flow = get(flowID_);

	if (flow == NULL)
	{
	  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::update ] This flowlist does not contain flow\n")));
		return -1;
	}
	else
	{
	  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ FlowList::update ] This flowlist contains flow, we will update it\n")));
	  return flow->updateProducer(flowXML_);
	}
}

ACE_TString FlowList::toXmlWithIndent(int n_)
{
	ACE_TRACE(ACE_TEXT("FlowList::toXmlWithIndent"));
	
	if (n_ < 0 || n_ > 255)
	{
		ACE_ERROR((LM_ERROR, ACE_TEXT("[ Client::toXmlWithIndent ] ERROR: indent lower than 0 or greater than 255\n")));
	}

	ACE_TString unit = ACE_TEXT("    ");
	ACE_TString indent = ACE_TEXT("");
	
	for (int i = 0; i < n_; i++)
	{
		indent += unit;
	}	

	ACE_TString xml = ACE_TEXT("");
	
	xml += indent;
	xml += ACE_TEXT("<FlowList>\n");
	
	for (FLOW_ITERATOR fi = flows.begin(); fi != flows.end(); fi++)
	{
		xml += (*fi).int_id_->toXmlWithIndent(n_ + 1);
	}

	xml += indent;
	xml += ACE_TEXT("</FlowList>\n");

	return xml;
}

ACE_TString FlowList::toXml()
{
	ACE_TRACE(ACE_TEXT("FlowList::toXml"));
	
	return toXmlWithIndent(0);
}
