#include "GlobalList.h"
#include "../Log_Manager.h"

GlobalList::GlobalList()
{
	ACE_TRACE(ACE_TEXT("GlobalList::GlobalList"));
	init();
}

/*GlobalList::GlobalList(ACE_TString xml_)
{
	init(xml_);
}*/

GlobalList::~GlobalList()
{
	ACE_TRACE(ACE_TEXT("GlobalList::~GlobalList"));
	fini();
}

int GlobalList::init()
{
	ACE_TRACE(ACE_TEXT("GlobalList::init"));

	clientList = new ClientList();
	flowList = new FlowList();

	return 0;
}

int GlobalList::fini()
{
	ACE_TRACE(ACE_TEXT("GlobalList::fini"));
	
	if (clientList  != NULL )
		delete clientList;
	if (flowList  != NULL )
		delete flowList;
	
	return 0;
}

ClientList *GlobalList::getClientList()
{
	ACE_TRACE(ACE_TEXT("GlobalList::getClientList"));

	return clientList;
}

FlowList *GlobalList::getFlowList()
{
	ACE_TRACE(ACE_TEXT("GlobalList::getFlowList"));

	return flowList;
}

ACE_TString GlobalList::toXml()
{
	ACE_TRACE(ACE_TEXT("GlobalList::toXml"));
	
	ACE_TString unit = ACE_TEXT("    ");
	ACE_TString xml = ACE_TEXT("");
	
	xml += ACE_TEXT("<GlobalList>\n");
	xml += clientList->toXmlWithIndent(1);
	xml += flowList->toXmlWithIndent(1);
	xml += ACE_TEXT("</GlobalList>\n");
	
	return xml;
}

Flow *GlobalList::getFlow(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::getFlow"));
	
	return getFlowList()->get(flowID_);
}
	
int GlobalList::findFlow(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::findFlow"));
	
	return getFlowList()->find(flowID_);
}

int GlobalList::containsFlow(ACE_TString flowXML_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::containsFlow"));
	
	return getFlowList()->contains(flowXML_);
}

int GlobalList::updateProducerOfFlow(ACE_TString flowXML_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::updateFlow"));
	
	return getFlowList()->updateProducerOfFlow(flowXML_);
}

int GlobalList::updateConsumersOfFlow(ACE_TString flowXML_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::updateFlow"));
	
	return getFlowList()->updateConsumersOfFlow(flowXML_);
}

int GlobalList::addFlow(Flow *flow_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::addFlow"));
	
	return getFlowList()->add(flow_);
}
	
int GlobalList::removeFlow(ACE_TString flowID_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::removeFlow"));

	return getFlowList()->remove(flowID_);
}
	
FLOW_ITERATOR GlobalList::getFirstFlow()
{
	ACE_TRACE(ACE_TEXT("GlobalList::getFirstFlow"));
	
	return getFlowList()->first();
}
	
FLOW_ITERATOR GlobalList::getLastFlow()
{
	ACE_TRACE(ACE_TEXT("GlobalList::getLastFlow"));
	
	return getFlowList()->last();
}

Flow *GlobalList::getFlowFromIterator(FLOW_ITERATOR flowIterator_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::getFlowFromIterator"));
	
	return (*flowIterator_).int_id_;
}

Client *GlobalList::getClient(ACE_TString clientID_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::getClient"));
	
	return getClientList()->get(clientID_);
}
	
int GlobalList::findClient(ACE_TString clientID_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::findClient"));
	
	return getClientList()->find(clientID_);
}
	
int GlobalList::addClient(Client *client_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::addClient"));
	
	return getClientList()->add(client_);
}

int GlobalList::removeClient(ACE_TString clientID_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::removeClient"));
	
	Flow *f;
	int ret;
	StringVector flowsNames;
	
	// For each flow, check if clientID_ is a producer, a consumer, or nothing
	for (FLOW_ITERATOR fi = getFirstFlow(); fi != getLastFlow(); fi++)
	{
		f = GlobalList::getFlowFromIterator(fi);
		
		// Check if clientID_ is the producer of this flow
		if (clientID_ == f->getProducer())
		{
			// clientID_ is the producer of this flow
			// unset the producer of this flow
			f->unsetProducer();
			
			// Check if there are still consumers for this flow
			if (f->countConsumers() == 0)
			{
				// We want to remove the flow because there are no producer and no more
				// consumers left. We save it in the vector. We don't remove it immediately 
				// so the iterator is not in an unstable mode
				flowsNames.push_back(f->getID());
			}
		} // if
		else
		{
			// Check if clientID_ is a consumer of this flow
			if (f->containsConsumer(clientID_) == 0)
			{
				// clientID_ is a consumer of this flow
				ret = f->removeConsumer(clientID_);
	
				// Check if removeConsumer failed			
				if (ret != 0)
				{
					ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ GlobalList::removeClient ] ERROR: Removing the consumer failed\n")));
					return ret;
					//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ GlobalList::removeClient ] ERROR: Removing the consumer failed\n")), ret);
				}
	
				// Check if it was the last consumer of this flow
				// and there is no producer
				if ((f->countConsumers() == 0) && (f->getProducerStatus() == CLIENT_STATUS_UNKNOWN))
				{
					// We want to remove the flow because there are no producer and no more
					// consumers left. We save it in the vector. We don't remove it immediately 
					// so the iterator is not in an unstable mode
					flowsNames.push_back(f->getID());
				}
			} // if
		} // else
	} // for
	
	StringIterator si (flowsNames);
	ACE_TString *flowName;
	Flow *tmp_flow = NULL;
	while (!si.done ()) {
      si.next (flowName);
	  tmp_flow = getFlow(*flowName);
	  ret = removeFlow(*flowName);
	  if ( tmp_flow )
		  delete tmp_flow;
	  tmp_flow = NULL;
	  // Check if removeflow failed			
	  if (ret != 0) 
	  {
		  ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ GlobalList::removeClient ] ERROR: Removing the flow failed\n")));
		  return ret;
		  //ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ GlobalList::removeClient ] ERROR: Removing the flow failed\n")), ret);
	  }
      si.advance();
    }
	// Remove the client from the client list
	ret = getClientList()->remove(clientID_);
	
	return ret;
}

int GlobalList::removeAllClientsOnServer(ACE_TString serverID_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::removeAllClientsOnServer"));
	
	Client *c;
	int ret = 0;
	StringVector clientsNames;

	// For each client, check if clientID_ is connected to this server
	for (CLIENT_ITERATOR ci = getFirstClient(); ci != getLastClient(); ci++)
	{  ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ GlobalList::removeAllClientsOnServer ] Client found in the list\n")));	
		c = GlobalList::getClientFromIterator(ci);
		
		// Check if the client is connected to this server
		if (serverID_ == c->getServer())
		{
			// The client 'c' is connected to server 'serverID_', we save it in the vector
			// We don't remove it immediately so the iterator is not in an unstable mode
			clientsNames.push_back(c->getID());
		}
	}

	StringIterator si (clientsNames);
	ACE_TString *clientName;
	Client *tmp_client = NULL;
	while (!si.done ()) {
      si.next (clientName);
	  tmp_client = getClient(*clientName);
	  ret = removeClient(*clientName);
	  if ( tmp_client )
		  delete tmp_client;
	  tmp_client = NULL;
	  // Check if removeConsumer failed			
	  if (ret != 0) 
	  {
		  ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ GlobalList::removeAllClientsOnServer ] ERROR: Removing the client failed\n")));
		  return ret;
	  }
      si.advance();
    }	
		
	return ret;
}

CLIENT_ITERATOR GlobalList::getFirstClient()
{
	ACE_TRACE(ACE_TEXT("GlobalList::getFirstClient"));
	
	return getClientList()->first();
}

CLIENT_ITERATOR GlobalList::getLastClient()
{
	ACE_TRACE(ACE_TEXT("GlobalList::getLastClient"));
	
	return getClientList()->last();
}

Client *GlobalList::getClientFromIterator(CLIENT_ITERATOR clientIterator_)
{
	ACE_TRACE(ACE_TEXT("GlobalList::getClientFromIterator"));
	
	return (*clientIterator_).int_id_;
}
