#include "ServerList.h"
#include "../Log_Manager.h"

ServerList::ServerList()
{
	ACE_TRACE(ACE_TEXT("ServerList::ServerList"));
	init();
}

ServerList::~ServerList()
{
	ACE_TRACE(ACE_TEXT("ServerList::~ServerList"));
	fini();
}

int ServerList::init()
{
	ACE_TRACE(ACE_TEXT("ServerList::init"));
	
	serversWithIPsAsKeys.open();
	serversWithNamesAsKeys.open();
	
	return 0;
}

int ServerList::fini()
{
	ACE_TRACE(ACE_TEXT("ServerList::fini"));
	
	serversWithIPsAsKeys.close();
	serversWithNamesAsKeys.close();
	
	return 0;
}

int ServerList::count()
{
	ACE_TRACE(ACE_TEXT("ServerList::count"));
	
	return serversWithIPsAsKeys.current_size();
}

int ServerList::add(Server *server_)
{
	ACE_TRACE(ACE_TEXT("ServerList::add"));

	int result1 = serversWithNamesAsKeys.trybind(server_->getName(), server_);
	int result2 = serversWithIPsAsKeys.trybind(server_->getIP(), server_);
		
	if (result1 == 1 || result2 == 1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ ServerList::add ] ERROR: The name '%s' or the IP address '%s' already exists\n"), server_->getName().c_str(), server_->getIP().c_str()));
		return 1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ServerList::add ] ERROR: The name '%s' or the IP address '%s' already exists\n"), server_->getName().c_str(), server_->getIP().c_str()), 1);
	}
	else if (result1 == -1 || result2 == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ ServerList::add ] ERROR: Server '%s' not added\n"), server_->getName().c_str()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ServerList::add ] ERROR: Server '%s' not added\n"), server_->getName().c_str()), -1);
	}
	else
	{
		ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("Server '%s' added\n"), server_->getName().c_str()));
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::add ] Server '%s' added\n"), server_->getName().c_str()));
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::add ] List of servers in serverlist : \n")));
		
		for (SERVER_ITERATOR si = serversWithNamesAsKeys.begin(); si != serversWithNamesAsKeys.end(); si++)
		{
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("      %s:%s:%s \n"), (*si).int_id_->getName().c_str(), (*si).int_id_->getIP().c_str(), (*si).int_id_->getOS().c_str()));
		}
		
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("\n")));

		return 0;
	}
}

int ServerList::remove(ACE_TString serverNameOrIP, typeArg type)
{
	ACE_TRACE(ACE_TEXT("ServerList::remove"));
	
	if (type == NAME) {
		Server *s = NULL;
		if (serversWithNamesAsKeys.find(serverNameOrIP, s) == -1) 
		{
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ ServerList::remove ] ERROR: Server '%s' not found\n"), serverNameOrIP.c_str()));
			return -1;
			//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ServerList::remove ] ERROR: Server '%s' not found\n"), serverNameOrIP.c_str()), -1);
		}
		else {
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::remove ] Server '%s' found\n"), serverNameOrIP.c_str()));
			if (serversWithNamesAsKeys.unbind(serverNameOrIP) == -1 || serversWithIPsAsKeys.unbind(s->getIP()) == -1) 
			{
				ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ ServerList::remove ] ERROR: Server '%s' not removed\n"), serverNameOrIP.c_str()));
				return -1;
				//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ServerList::remove ] ERROR: Server '%s' not removed\n"), serverNameOrIP.c_str()), -1);
			}
			else {
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::remove ] Server '%s' removed\n"), serverNameOrIP.c_str()));
				return 0;
			}
		}
	}
	else {
		Server *s = NULL;
		if (serversWithIPsAsKeys.find(serverNameOrIP, s) == -1) 
		{
			ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ ServerList::remove ] ERROR: Server '%s' not found\n"), serverNameOrIP.c_str()));
			return -1;
			//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ServerList::remove ] ERROR: Server '%s' not found\n"), serverNameOrIP.c_str()), -1);
		}
		else {
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::remove ] Server '%s' found\n"), serverNameOrIP.c_str()));
			if (serversWithIPsAsKeys.unbind(serverNameOrIP) == -1 || serversWithNamesAsKeys.unbind(s->getName()) == -1) 
			{
				ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ ServerList::remove ] ERROR: Server '%s' not removed\n"), serverNameOrIP.c_str()));
				return -1;
				//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ ServerList::remove ] ERROR: Server '%s' not removed\n"), serverNameOrIP.c_str()), -1);
			}
			else {
				ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::remove ] Server '%s' removed\n"), serverNameOrIP.c_str()));
				return 0;
			}
		}
	}
}

Server *ServerList::get(ACE_TString serverNameOrIP, typeArg type)
{
	ACE_TRACE(ACE_TEXT("ServerList::get"));

	if (type == NAME) { 
		Server *s = NULL;
		if (serversWithNamesAsKeys.find(serverNameOrIP, s) == -1)
		{
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::get ] Server '%s' not found\n"), serverNameOrIP.c_str()));
			return 0;
		}
		else
		{
			/* compare returns 0 if the two strings are equal */
			ACE_ASSERT(serverNameOrIP.compare(s->getName()) == 0);
		
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::get ] Server '%s' found : '%s'\n"), serverNameOrIP.c_str(), s->getName().c_str()));
			return s;
		}
	}
	else {
		Server *s = NULL;
		if (serversWithIPsAsKeys.find(serverNameOrIP, s) == -1)
		{
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::get ] Server '%s' not found\n"), serverNameOrIP.c_str()));
			return 0;
		}
		else
		{
			/* compare returns 0 if the two strings are equal */
			ACE_ASSERT(serverNameOrIP.compare(s->getIP()) == 0);
		
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::get ] Server '%s' found : '%s'\n"), serverNameOrIP.c_str(), s->getIP().c_str()));
			return s;
		}

	}
}

int ServerList::find(ACE_TString serverNameOrIP, typeArg type)
{
	ACE_TRACE(ACE_TEXT("ServerList::find"));
	
	if (type == NAME) {
		if (serversWithNamesAsKeys.find(serverNameOrIP) == -1) {
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::find ] This serverlist does not contain server '%s'\n"), serverNameOrIP.c_str()));
			return -1;
		}
		else {
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::find ] This serverlist contains server '%s'\n"), serverNameOrIP.c_str()));
			return 0;
		}
	}
	else {
		if (serversWithIPsAsKeys.find(serverNameOrIP) == -1) {
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::find ] This serverlist does not contain server '%s'\n"), serverNameOrIP.c_str()));
			return -1;
		}
		else {
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ ServerList::find ] This serverlist contains server '%s'\n"), serverNameOrIP.c_str()));
			return 0;
		}	
	}
}

SERVER_ITERATOR ServerList::first()
{
	ACE_TRACE(ACE_TEXT("ServerList::first"));
	
	return serversWithIPsAsKeys.begin();
}

SERVER_ITERATOR ServerList::last()
{
	ACE_TRACE(ACE_TEXT("ServerList::last"));

	return serversWithIPsAsKeys.end();
}



