#ifndef _SERVERLIST_H_
#define _SERVERLIST_H_

#include "ace/ACE.h"
#include "ace/SString.h"
#include "ace/Log_Msg.h"
#include "ace/Singleton.h"
#include "ace/Null_Mutex.h"
#include "ace/Hash_Map_Manager_T.h"

#include "Server.h"

typedef ACE_Hash_Map_Manager_Ex<ACE_TString, Server *, ACE_Hash<ACE_TString>, ACE_Equal_To<ACE_TString>, ACE_Null_Mutex> SERVER_MAP;
typedef ACE_Hash_Map_Iterator_Ex<ACE_TString, Server *, ACE_Hash<ACE_TString>, ACE_Equal_To<ACE_TString>, ACE_Null_Mutex> SERVER_ITERATOR;

/// This class describes a server list object.

class ServerList
{
	friend class ACE_Singleton<ServerList, ACE_Null_Mutex>;

	protected:
		int init();
		int fini();

		/// Internal map used to store the servers name and IP. On this map, the key is the IP address 
		SERVER_MAP serversWithIPsAsKeys;
		/// Internal map used to store the servers name and IP. On this map, the key is the name
		SERVER_MAP serversWithNamesAsKeys;
	
	public:

		typedef enum {
			NAME,
			IP			
		} typeArg;


		/// Create an empty server list
		ServerList();
		
		/// Destroy the server list
		virtual ~ServerList();
		
		/// Add a server to the server list
		
		/**
		 * @param server_ pointer to the server to add
		 * @return 0 if the server was added succesfully, 1 if the server already
		 * exists in the list, and -1 if it was not added
		 */
		int add(Server *server_);

		/// Remove a server from the server list
		
		/**
		 * @param serverNameOrIP the name or the IP address of the server to remove
		 * @param type says if the first parameter is an ip or a name
		 * @return 0 if the server was removed succesfully, and -1 if not
		 */
		int remove(ACE_TString serverNameOrIP, typeArg type);

		/**
		 * @param serverNameOrIP the name or the IP address of the server to remove
		 * @param type says if the first parameter is an ip or a name
		 * @return a pointer to the server, null pointer if server is not in the server list
		 */
		Server *get(ACE_TString serverNameOrIP, typeArg type);

		/// Check if a server is in the server list giving its name or IP address
		
		/**
		 * @param serverNameOrIP the name or the IP of the server to check
		 * @return 0 if the server is in the list, and -1 if not
		 */
		int find(ACE_TString serverNameOrIP, typeArg type);

		/// Count the number of servers in the server list
		
		/**
		 * @return the number of servers in the server list
		 */
		int count();

		/// Return an iterator on the first server of the server list

		/**
		 * @return iterator on the first server of the server list
		 */
		SERVER_ITERATOR first();

		/// Return an iterator on the last server of the server list
		
		/**
		 * @return iterator on the last server of the server list
		 */
		SERVER_ITERATOR last();

};

typedef ACE_Singleton<ServerList, ACE_Null_Mutex> SERVER_LIST;

#endif /* _SERVERLIST_H_ */
