#include "SocketList.h"
#include "../Log_Manager.h"

SocketList::SocketList()
{
	ACE_TRACE(ACE_TEXT("SocketList::SocketList"));
	init();
}

SocketList::~SocketList()
{
	ACE_TRACE(ACE_TEXT("SocketList::SocketList"));
	fini();
}

int SocketList::init()
{
	ACE_TRACE(ACE_TEXT("SocketList::init"));

	socketsClient.open();

	return 0;
}

int SocketList::fini()
{
	ACE_TRACE(ACE_TEXT("SocketList::fini"));
	
	return 0;
}

int SocketList::countClient()
{
	ACE_TRACE(ACE_TEXT("SocketList::countClient"));
	
	return socketsClient.current_size();
}

int SocketList::addClient(ACE_TString clientID_, ACE_SOCK_Stream *socket_)
{
	ACE_TRACE(ACE_TEXT("SocketList::addClient"));
	
	int ret = socketsClient.trybind(clientID_, socket_);
		
	if (ret == 1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ SocketList::addClient ] Client '%s' already exists in list\n"), clientID_.c_str()));
		return 1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ SocketList::addClient ] Client '%s' already exists in list\n"), clientID_.c_str()), 1);
	}
	else if (ret == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ SocketList::addClient ] ERROR: Client '%s' not added\n"), clientID_.c_str()));
		return -1;
		//ACE_ERROR_RETURN((LM_ERROR, ACE_TEXT("[ SocketList::addClient ] ERROR: Client '%s' not added\n"), clientID_.c_str()), -1);
	}
	else
	{
		ACE_DEBUG((SF2D_ANNOUNCE_PREFIX ACE_TEXT("Client '%s' added\n"), clientID_.c_str()));
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ SocketList::addClient ] Client '%s' successfully added\n"), clientID_.c_str()));
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ SocketList::addClient ] List of clients in socket list : \n")));
		
		for (SOCKET_ITERATOR si = socketsClient.begin(); si != socketsClient.end(); si++)
		{
			ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("      SOCKET : '%x' - SERVER : '%s'\n"), SocketList::getSocketFromIterator(si), SocketList::getIDFromIterator(si).c_str()));
		}
		
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("\n")));

		return 0;
	}
}

int SocketList::removeClient(ACE_TString clientID_)
{
	ACE_TRACE(ACE_TEXT("SocketList::removeClient"));
	
	socketsClient.unbind(clientID_);
	ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ SocketList::removeClient ] Client '%s' successfully removed\n"), clientID_.c_str()));

	return 0;
}

ACE_SOCK_Stream *SocketList::getClient(ACE_TString clientID_)
{
	ACE_TRACE(ACE_TEXT("SocketList::getClient"));
	
	ACE_SOCK_Stream *socket;
	
	if (socketsClient.find(clientID_, socket) == -1)
	{
		ACE_DEBUG((SF2D_ERROR_PREFIX ACE_TEXT("[ SocketList::getClient ] Client '%s' does not exist\n"), clientID_.c_str()));
		return 0;
	}
	else
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ SocketList::getClient ] Client '%s' exists\n"), clientID_.c_str()));
		return socket;
	}
}

SOCKET_ITERATOR SocketList::firstClient()
{
	ACE_TRACE(ACE_TEXT("SocketList::firstClient"));

	return socketsClient.begin();
}

SOCKET_ITERATOR SocketList::lastClient()
{
	ACE_TRACE(ACE_TEXT("SocketList::lastClient"));

	return socketsClient.end();
}

int SocketList::findClient(ACE_TString clientID_)
{
	ACE_TRACE(ACE_TEXT("SocketList::findClient"));
	
	if (socketsClient.find(clientID_) == -1)
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ SocketList::findClient ] This socket list does not contain client '%s'\n"), clientID_.c_str()));
		return -1;
	}
	else
	{
		ACE_DEBUG((SF2D_DEBUG_PREFIX ACE_TEXT("[ SocketList::findClient ] This socket list contains client '%s'\n"), clientID_.c_str()));
		return 0;
	}
}

ACE_SOCK_Stream *SocketList::getSocketFromIterator(SOCKET_ITERATOR socketIterator_)
{
	ACE_TRACE(ACE_TEXT("SocketList::getSocketFromIterator"));
	
	return (*socketIterator_).int_id_;
	
}
ACE_TString SocketList::getIDFromIterator(SOCKET_ITERATOR socketIterator_)
{
	ACE_TRACE(ACE_TEXT("SocketList::getIDFromIterator"));

	return (*socketIterator_).ext_id_;
}
