#include "ace/OS_main.h"
#include "ace/INET_Addr.h"
#include "ace/Reactor.h"
#include "ace/TP_Reactor.h"
#include "ace/Get_Opt.h"

#include "Acceptor_Event_Handler.h"
#include "Multicast_Server_Events.h"
#include "sf2d.h"
#include "list/DuplicatorList.h"
#include "Log_Manager.h"                                                                                                                                                                                                                                                                                                                                                                                                                                                                                         
#include "sf2d_common_ACE.h"

#include <stdlib.h>

static const char MCAST_ADDR[] = ACE_DEFAULT_MULTICAST_ADDR ;
static const int UDP_PORT = ACE_DEFAULT_MULTICAST_PORT ;


void displayHelp() {
	ACE_ERROR ((LM_ERROR,	"Usage: sf2d [--sf option=value]... [application_name]\n"
							"\n"
							"Optional parameters above are specified in brackets.\n"
							"To assign an option, prefix it with '--sf' and specify it as option=value.\n"
							"\n"
							"The 'application_name' is optional, it allows to logically separate different\n"
							"application domains running on the same local area network.\n"
							"\n"
							"Here is a list of the options with the default values specified in brackets:\n"
							"\n"
							"     --sf loglevel=[error]   Specify the logging severity, the default value\n"
							"                             is 'error', then in log refinement order: 'warning'\n"
							"                             and 'debug'.\n"
							"     --sf syslog=[0]         Enable or disable the log redirection to the system\n"
							"                             logger. By default this option is disabled, use '1'\n"
							"                             to enable it.\n"
							"     --sf logforward=[0]     Enable the internal log forwarding capability of\n"
							"                             NDFS-II. This option is required to monitor log\n"
							"                             messages from the Control Center. By default this\n"
							"                             option is enabled, use '0' to disable it.\n"
							"     --sf logfile=[]         Save the log messages in the file specified with\n"
							"                             its absolute path. By default the path is empty,\n"
							"                             which disables to save the log in a file.\n"
							"\n"
							"For example if you want to record the log in a file and join the\n"
							"application domain 'myappli', launch the NDFS-II server as following:\n"
							"     sf2d --sf logfile=/tmp/myLogFile.log myappli\n"
							"\n"
	));
}	

int ACE_TMAIN (int argc, ACE_TCHAR **argv) {
  
	bool daemon = true;

	//test display argc, argv before parsing
	/*cout << "BEFORE" << endl;
	cout << "argc=" << argc << endl;
	for (int i = 0; i < argc; i++ )
		cout << "   argv[" << i << "]=" << argv[i] << endl;
	cout << endl;
	*/

	ACE_Get_Opt opt(argc, argv, ACE_LIB_TEXT(""), 1, 0, ACE_Get_Opt::RETURN_IN_ORDER, 1);
	opt.long_option(ACE_TEXT("help"), 'h');
	opt.long_option(ACE_TEXT("no_daemon"), 2);
	int args ;
	while ( (args = opt()) != EOF) {
		switch(args) {
			case 2:
				daemon = false;
				break;
			case 'h':
				displayHelp();
				return 0;
			default:
				break;
		}
		
	}

	//we parse and store the log preferences specified by the user
	//for further use.
	Log_Preferences *logPref = LOG_PREF::instance();
	logPref->initPreferences(argc, argv);
	//logPref->displayPrefs();

	//test display argc, argv before parsing
	/*cout << "argc=" << argc << endl;
	for (int i = 0; i < argc; i++ )
		cout << "   argv[" << i << "]=" << argv[i] << endl;
	cout << endl;*/

#if (!(defined (ACE_WIN32)))
	if (daemon == true) {
		//this code is used to make sf2d a daemon
		int i=fork();
		// fork error 
		if (i<0) exit(1);
		// parent exits
		if (i>0) exit(0); 
		//obtain a new process group 
		setsid(); 
		//Setting user mask in order to prevent unsecure file privileges that may occur on file creation
		umask(027);
	}
#endif /* ACE_WIN32 */

#if (defined (ACE_WIN32))
  ACE_TP_Reactor tp_reactor;
  ACE_Reactor reactor(&tp_reactor);
#else
  ACE_Reactor reactor;
#endif
  
  unsigned short remote_port_number = SF2D_DEFAULT_REMOTE_PORT;
  unsigned short local_port_number = SF2D_DEFAULT_LOCAL_PORT ;
  unsigned short local2_port_number = SF2D_DEFAULT_LOCAL2_PORT ;
  unsigned short dupli_port_number = SF2D_DEFAULT_DUPLI_PORT ;

  ACE_INET_Addr remote_server_addr;
  if (remote_server_addr.set(remote_port_number) == -1 ) return 1;
  
  ACE_INET_Addr local_server_addr;
  if (local_server_addr.set(local_port_number) == -1 ) return 1;
  
  ACE_INET_Addr local2_server_addr;
  if (local2_server_addr.set(local2_port_number) == -1 ) return 1;
  
  ACE_INET_Addr dupli_server_addr;
  if (dupli_server_addr.set(dupli_port_number) == -1 ) return 1;	
  
  //DuplicatorList *dl = DUPLICATOR_LIST::instance();

  //Server Acceptor
  Peer_Acceptor_Event_Handler peer_acceptor_events(&reactor);
  if (peer_acceptor_events.open(remote_server_addr) == -1)
    {
      ACE_ERROR ((LM_ERROR,"%p\n%a","Peer Acceptor : ",1));
      return -1;
    }
  
  //Client Acceptor RECV
  Local_Acceptor_Event_Handler local_acceptor_events(&reactor);
  if(local_acceptor_events.open(local_server_addr) == -1)
    {
      ACE_ERROR ((LM_ERROR,"%p\n%a","Local Acceptor : ",1));
      return -1;
    }
  
  //Client Acceptor SEND
  Local2_Acceptor_Event_Handler local2_acceptor_events(&reactor);
  if(local2_acceptor_events.open(local2_server_addr) == -1)
    {
      ACE_ERROR ((LM_ERROR,"%p\n%a","Local Acceptor : ",1));
      return -1;
    }
  
  
  //Duplicator Acceptor
  Duplicator_Acceptor_Event_Handler dupli_acceptor_events(&reactor);
  if(dupli_acceptor_events.open(dupli_server_addr) == -1)
    {
      ACE_ERROR ((LM_ERROR,"%p\n%a","Duplicator Acceptor : ",1));
      return -1;
    }
  
  ACE_TCHAR application_name[512]={0};
  if (argc > 1)
	  ACE_OS::sprintf(application_name, argv[1]);
  else
	  ACE_OS::sprintf(application_name, ACE_TEXT("default"));

  MultiCast_Server_Events server_events (UDP_PORT, MCAST_ADDR, application_name);
  server_events.welcome_to_multicast_group();
  server_events.open();
  
  if (reactor.register_handler (&server_events,
                                ACE_Event_Handler::READ_MASK) == -1)
    ACE_ERROR ((LM_ERROR,"%p\n%a","register_handler",1));
  

  Log_Event_Handler lg(&reactor);
  //we initiate the logManager
  Log_Manager *logManager = LOG_MANAGER::instance();
  logManager->setReactor(&reactor);
  logManager->setLogHandler(&lg);
  logManager->init("sf2d");

  //Server_Stopper ss(&reactor);
	
  if (reactor.run_reactor_event_loop() == -1)
    ACE_ERROR_RETURN((LM_ERROR, "%p\n","run_reactor_event_loop()"),1);
	
  //lg.disableTimer();
  return 0;
  
  ACE_NOTREACHED (return 0;)
}

