#include "Flow_Java.h"
#include "duplicator_common_ACE.h"

#include <sstream>
#include <iostream>
#include <string>

using namespace std;

smartflow::Flow * create_flow(void)
{
	return new smartflow::Flow_Java();
}


namespace smartflow {

string      		Flow_Java::default_flow_type="Flow_Java";
buffer_size_t		Flow_Java::default_buffer_size = 4096000;
long        		Flow_Java::default_history_size = 10;
string      		Flow_Java::default_flow_desc="Fixed-size data blocks Flow";
	
Flow_Java::Flow_Java()
     :super(),current_buffer_size(0),current_history_size(0)
{
}

bool Flow_Java::init(FlowManager *manager,const std::string &type,const std::string &name,const std::string &rgroup,const std::string &args,flow_direction_t mode, flow_policy_t policy, bool isSynchronized)
{
   bool err = super::init(manager,type,name,rgroup,args,mode,policy, isSynchronized);
   if ( !err ) return err; /*forward error */
   current_buffer_size = default_buffer_size;
   current_history_size = default_history_size;
   int current_block_count = DUP_DEFAULT_BLOCK_COUNT;
   
   string tmp = parseArgs("blocksize",args);
   if ( ! tmp.empty() ) {
       istringstream is(tmp);
       is >> current_buffer_size;
   }
   tmp = parseArgs("history",args);
   if ( ! tmp.empty() ) {
       istringstream is(tmp);
       is >> current_history_size;
   }
   tmp = parseArgs("blockcount",args);
   if ( ! tmp.empty() ) {
     istringstream is(tmp);
     is >> current_block_count;
   }
   if ( !setMaxBufferSize(current_buffer_size) ) return false;
   if ( !setHistorySize(current_history_size)  ) return false;
   if ( !setBlockCount(current_block_count) )   return false;
   return true;
}

Flow_Java::~Flow_Java()
{
	current_buffer_size = current_history_size = 0;
}


std::string & Flow_Java::getFlowType(void)
{
	return default_flow_type;
}
std::string & Flow_Java::getFlowDesc(void)
{
	return default_flow_desc;
}

}//namespace
