#include "ace/OS.h"

#include "Buffer_ACE.h"
#include "ErrorManager.h"
#include "Flow.h"
#include "Log_Manager_ACE.h"

// DEBUG
#include <iostream>

//#include "ace/os_include/os_time.h"
//#include <stdio.h>


namespace smartflow {

#define TS_SIZE 8

Buffer_ACE::Buffer_ACE()
	:super(),ACE_Message_Block()
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::Buffer_ACE"));
}

Buffer_ACE::~Buffer_ACE()
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::~Buffer_ACE"));
}

bool Buffer_ACE::init(MemoryManager *manager,Flow * flow,buffer_size_t max_size)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::init"));
	if ( ACE_Message_Block::init(max_size) != 0 ) {
		getSmartflow()->setError(BUFFER_ALLOCATION_FAILED);
		if ( flow != NULL )
			ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Buffer_ACE::init ] Memory allocation of buffer for the flow %s failed"), flow->get_id().c_str()));
		else
			ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Buffer_ACE::init ] Memory allocation of buffer failed")));
		return false;
	}
	return super::init(manager,flow);
}

void * Buffer_ACE::data(void)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::data"));
	return rd_ptr();
}

buffer_size_t Buffer_ACE::getMaxSize(void)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::getMaxSize"));
	return capacity();
}

buffer_size_t Buffer_ACE::getSize(void)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::getSize"));
	return length();
}

bool Buffer_ACE::setSize(buffer_size_t new_size)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::setSize"));
	if ( (new_size > (getMaxSize())) || (new_size == 0)) {
		getSmartflow()->setError(BUFFER_INVALID_SIZE);
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Buffer_ACE::setSize ] Buffer Invalid Size")));
		return false;
	}
	wr_ptr(rd_ptr());
	wr_ptr(new_size);
	return true;
}

bool Buffer_ACE::clear(void)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::clear"));
	wr_ptr((size_t) 0);
	rd_ptr((size_t) 0);
	return true;
}

bool Buffer_ACE::initBuf(void)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::initBuf"));
	wr_ptr(rd_ptr());
	return true;
}

bool Buffer_ACE::gotoNextBlock()
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::gotoNextBlock"));
	if (length() == 0)
		return false;
	else
	{
		buffer_size_t size = getNextDataSize();
		if ( getNextDataType() != TIMESTAMP) 
		{
			rd_ptr(sizeof(buffer_size_t) + sizeof(data_type_t) + size);
			return true ;
		} 
		else  
		{
			//rd_ptr(2 * sizeof(buffer_size_t) + 2 * sizeof(data_type_t) + sizeof(ACE_Time_Value) + size);
			rd_ptr(2 * sizeof(buffer_size_t) + 2 * sizeof(data_type_t) + TS_SIZE + size);
			//std::cout << "ACE_Time_Value Size : "<< sizeof(ACE_Time_Value) << std::endl;
			return true ;
		}	
	}
}

data_type_t Buffer_ACE::getNextDataType()
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::getNextDataType"));
	if (length() == 0)
		return ENDOFBUFFER;
	else
		return ntohl( *(data_type_t *) (rd_ptr()+sizeof(buffer_size_t))) ;	
}

buffer_size_t Buffer_ACE::getNextDataSize()
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::getNextDataSize"));
	if (length() == 0)
		return ENDOFBUFFER;
	else
		if ( getNextDataType() != TIMESTAMP)
				return ntohl( *(buffer_size_t *) rd_ptr() ) ;
			else {
				//return ntohl( *(buffer_size_t *) ( rd_ptr() + sizeof(buffer_size_t) + sizeof(data_type_t) + sizeof(ACE_Time_Value) ) ) ;	
				return ntohl( *(buffer_size_t *) ( rd_ptr() + sizeof(buffer_size_t) + sizeof(data_type_t) + TS_SIZE ) ) ;
				}
}

ACE_Time_Value Buffer_ACE::getNextTimeStamp()
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::getNextTimeStamp"));
	ACE_Time_Value timestamp;
	memset(&timestamp,0,sizeof(ACE_Time_Value));
	ACE_UINT32 vec[2];
	memset(vec, 0, TS_SIZE);
	if ( getNextDataType() == TIMESTAMP) {
				//ACE_OS::memcpy(&timestamp,rd_ptr()+sizeof(buffer_size_t) + sizeof(data_type_t),sizeof(ACE_Time_Value));
				ACE_OS::memcpy(vec,rd_ptr()+sizeof(buffer_size_t) + sizeof(data_type_t),TS_SIZE);
				ACE_UINT32 secondes = static_cast<ACE_UINT32>(ntohl(vec[0]));
				ACE_UINT32 usecondes = static_cast<ACE_UINT32>(ntohl(vec[1]));
				timestamp.set(secondes, usecondes);
	}
	else 
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Buffer_ACE::getNextTimeStamp ] Error: No TimeStamp found")));
	return timestamp;
}

bool Buffer_ACE::pushDataTS(void *src, buffer_size_t size, ACE_Time_Value *ts)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::pushDataTS"));
	const void *tmp_src = src;
	return pushDataTS(tmp_src, size, ts);
}

bool Buffer_ACE::pushDataTS(const void *src, buffer_size_t size, ACE_Time_Value *ts)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::pushDataTS"));
	bool res = false;
	if (ts == 0)
	{
		ACE_Time_Value timestamp;
		timestamp = ACE_OS::gettimeofday();
		
		//we build a vector which contains the secondes and micro secondes of the timestamp in network representation
		ACE_UINT32 vec[2];
		vec[0] = static_cast<ACE_UINT32>(htonl(timestamp.sec()));
		vec[1] = static_cast<ACE_UINT32>(htonl(timestamp.usec()));
		res = addData(vec, TS_SIZE, TIMESTAMP );
		//res = add_data(&timestamp, sizeof(ACE_Time_Value), TIMESTAMP );
		
	}
	else  {
		//we build a vector which contains the secondes and micro secondes of the timestamp in network representation
		ACE_UINT32 vec[2];
		vec[0] = static_cast<ACE_UINT32>(htonl(ts->sec()));
		vec[1] = static_cast<ACE_UINT32>(htonl(ts->usec()));
		res = addData(vec, TS_SIZE, TIMESTAMP );
		//res = add_data(ts, sizeof(ACE_Time_Value), TIMESTAMP );
	}
	if (res)
		res = addData(src, size, DATA );
	return res;
}
	
bool Buffer_ACE::pushData(void *src, buffer_size_t size)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::pushData"));
	const void *tmp_src = src;
	return pushData(tmp_src, size);
}

bool Buffer_ACE::pushData(const void *src, buffer_size_t size)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::pushData"));
	return addData(src, size, DATA );
}
	
bool Buffer_ACE::pushMetadata(void *src, buffer_size_t size, data_type_t type)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::pushMetadata"));
	const void *tmp_src = src;
	return pushMetadata(tmp_src, size, type);
}

bool Buffer_ACE::pushMetadata(const void *src, buffer_size_t size, data_type_t type)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::pushMetadata"));
	if (type == 0)
		return addData(src, size, METADATA );
	else 
		return addData(src, size, type );
}
	
bool Buffer_ACE::addData(const void *src, buffer_size_t size, data_type_t type)
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::addData"));
	if (!src)
	{
		getSmartflow()->setError(BUFFER_INVALID_SIZE); 
		return false;
	}

	if ((size+8) > space()) {
			getSmartflow()->setError(BUFFER_INVALID_SIZE);
			ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Buffer_ACE::addData ] Error: Buffer Invalid Size - Not enough space")));
			return false;
	}		
	
	//We first push the size
	* ( (buffer_size_t *) wr_ptr() ) = htonl(size);
	wr_ptr(sizeof(buffer_size_t));
	//Then the type of the data
	* ( (data_type_t *) wr_ptr() ) = htonl(type);
	wr_ptr(sizeof(data_type_t));
	//Finally the data himself
	ACE_OS::memcpy(wr_ptr(),src,size);
	wr_ptr(size);	
	return true;
}
	
bool Buffer_ACE::popData(void *src, buffer_size_t size )
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::popData"));
	bool res = false;
	if (length() == 0)
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Buffer_ACE::popData ] Buffer is empty")));
		return res;
	}
	if (getNextDataType() == DATA)
			res = removeData(src,size);	
	return res;
}

bool Buffer_ACE::popDataTS(void *src, buffer_size_t size, ACE_Time_Value *ts )
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::popDataTS"));
	bool res = false;
	if (length() == 0)
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Buffer_ACE::popDataTS ] Buffer is empty")));
		return res;
	}
	char * tmp_ptr = rd_ptr();
	if (getNextDataType() == TIMESTAMP)
	{
		ACE_UINT32 vec[2];
		res = removeData(&vec, TS_SIZE);
		if (res) {
			res = removeData(src,size);
			ACE_UINT32 secondes = static_cast<ACE_UINT32>(ntohl(vec[0]));
			ACE_UINT32 usecondes = static_cast<ACE_UINT32>(ntohl(vec[1]));
			ACE_Time_Value *tmp_ts = new ACE_Time_Value(secondes, usecondes);
			ts = tmp_ts;
		}
	}
	if (!res)
		rd_ptr(tmp_ptr);
	return res;
}

bool Buffer_ACE::popMetadata(void *src, buffer_size_t size )
{
	ACE_TRACE(ACE_TEXT("Buffer_ACE::popMetadata"));
	bool res = false;
	if (length() == 0)
		{
			ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Buffer_ACE::popMetadata ] Buffer is empty")));
			return res;
		}	
	if ((getNextDataType() == METADATA) || (getNextDataType() > 9) )
			res = removeData(src,size);
	return res;		
}

bool Buffer_ACE::removeData(void * dest, buffer_size_t dest_size)
{	
	ACE_TRACE(ACE_TEXT("Buffer_ACE::removeData"));
	if (!dest) {getSmartflow()->setError(BUFFER_INVALID_SIZE); return false;}		
	buffer_size_t tmp_size = ntohl( *(buffer_size_t *) rd_ptr()) ;
	if ( dest_size < tmp_size ) 
		{
			getSmartflow()->setError(BUFFER_INVALID_SIZE); 
			ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Buffer_ACE::removeData ] Buffer has an invalid size (too small)")));
			return false;
		}
	rd_ptr(sizeof(buffer_size_t));	
	rd_ptr(sizeof(data_type_t));
	ACE_OS::memcpy(dest,rd_ptr(),tmp_size);
	rd_ptr(tmp_size);
	return true;
}

} // namespace SMARTFLOW
