#include "Client_Handler.h"
#include "Thread_Client.h"
#include "ConnectionManager_ACE.h"
#include "Log_Manager_ACE.h"
#include "sf2d_common_ACE.h"

#include <ace/Synch.h>
#include <ace/Static_Object_Lock.h>

namespace smartflow {

ACE_HANDLE Client_Handler::get_handle(void) const
{
	ACE_TRACE(ACE_TEXT("Client_Handler::get_handle"));
	return remote_peer_.get_handle();
}

Client_Handler::Client_Handler(ACE_Reactor *reactor, Thread_Client *parent_) : ACE_Event_Handler(reactor) {
	parent = parent_;
	this->reactor(reactor);
}
	
Client_Handler::~Client_Handler() {
	ACE_TRACE(ACE_TEXT("Client_Handler::~Client_Handler"));
}
	
int Client_Handler::handle_close(ACE_HANDLE, ACE_Reactor_Mask)
{
	ACE_TRACE(ACE_TEXT("Client_Handler::handle_close"));
	
	//here we need to stop the reactor from the thread
	reactor()->end_reactor_event_loop();

	if ( this->remote_peer_.get_handle() != ACE_INVALID_HANDLE ) {
		remote_peer_.close();
	}

	delete this;
	return 0;
}

/*	This method will be called when the server needs to send a data to a client.
 *  For example, when the server wants the client to quit */
int Client_Handler::handle_input(ACE_HANDLE)
{
	ACE_TRACE(ACE_TEXT("Client_Handler::handle_input"));
	ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ Client_Handler::handle_input ]\n" )));
	ACE_TCHAR size[512] = {0}; 

	/* Default behaviour : when the server quits, all the clients quits */
	if (remote_peer_.recv_n(size,sizeof(size)) <= 0 )
	{
		parent->setNeedToInvokeExitMethod(true);
		//closing the stream used to send messages to the server.
		//if the connection is broken with the server (therefore breaking the socket stream), and a send method is called 
		//on the socket before closing the socket, it may result in a segfault.
		parent->close_input_stream();
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Client_Handler::handle_input ] Server disconnected! Invoking exit method if specified\n")));

		return -1;
	}
	
	// The server wants the client to quit 
	if ( ACE_OS::atoi(size) == STOP)
	{
		parent->setNeedToInvokeExitMethod(true);
		//here the server wants to disconnect... let's disconnect and kill the thread before invoking the method...
		//Here doesn't seem to be the right spot
		ACE_DEBUG((ANNOUNCE_PREFIX ACE_TEXT("[ Client_Handler::handle_input ] Server wants me to exit\n")));
		parent->close_input_stream();
		return -1;
	}
	else
	{
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Client_Handler::handle_input ] Server Invalid Request : %d\n"), ACE_OS::atoi(size) ));
		parent->close_input_stream();
		return -1;
	}	

}

}
