#include "ace/INET_Addr.h"
#include "ace/Static_Object_Lock.h"
#include "ace/Synch.h"
#include "ace/SOCK_Connector.h"
#include "ace/Thread_Manager.h"

#include "ConnectionManager_ACE.h"
#include "PreferenceManager.h"
#include "ErrorManager.h"

#include "sf2d_common_ACE.h"

#include <iostream>


/* MAX_MESSAGE_SIZE is only used to initialize the memory for default message like : CONNECT_MESSAGE */
#define MAX_MESSAGE_SIZE 131072
#define PROTOCOL_SIZEPARAM_SIZE 512
#define PROTOCOL_SUCCESS_STRING "Successful"
#define PROTOCOL_FAILURE_STRING "Failure"

#define CONNECT_MESSAGE "<?xml version='1.0'?>\n<smartspace>\n <type>3</type>\n <target/> <clientname>%s</clientname>\n <clientgroup>%s</clientgroup>\n<clientparam>%s</clientparam>\n<clientpath>%s</clientpath>\n<flowinfo>%s</flowinfo>\n<clientserver>%s</clientserver>\n</smartspace>"
#define GETFLOWMESSAGE "<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>11</target> \n <typeflow>%s</typeflow>\n <name>%s</name>\n <group>%s</group>\n <args>%s</args> <provider>%d</provider>\n <blocksize>%d</blocksize>\n <blockcount>%d</blockcount>\n <register_flow>%d</register_flow>\n <synchronized>%d</synchronized>\n </smartspace>"
#define REMOVEFLOWMESSAGE "<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>12</target> \n <typeflow>%s</typeflow>\n <name>%s</name>\n <group>%s</group>\n </smartspace>"
#define REQUEST_MESSAGE "<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>%d</target>\n</smartspace>"
#define START_MESSAGE "<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>15</target> <clientpath>%s</clientpath>\n <args>%s</args>\n <clientparam>%s</clientparam>\n <server>%s</server>\n </smartspace>"
#define STOP_MESSAGE "<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>16</target> <clientname>%s</clientname>\n <clientgroup>%s</clientgroup>\n </smartspace>"
#define LOG_MESSAGE "<?xml version='1.0'?>\n<smartspace>\n <type>5</type>\n <target>35</target>\n <messages>%s</messages>\n </smartspace>"

//By Imad
#define PAUSE_MESSAGE   "<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>19</target> \n <clientname>%s</clientname>\n <clientgroup>%s</clientgroup> \n <flowtype>%s</flowtype>\n <flowgroup>%s</flowgroup> \n</smartspace>\n"
#define RESTART_MESSAGE "<?xml version='1.0'?>\n<smartspace>\n <type>1</type>\n <target>20</target> \n <clientname>%s</clientname>\n <clientgroup>%s</clientgroup> \n <flowtype>%s</flowtype>\n <flowgroup>%s</flowgroup>\n  \n </smartspace>\n"

 
namespace smartflow {
ConnectionManager_ACE::ConnectionManager_ACE()
	:super()
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::ConnectionManager_ACE"));
	isConnected_ = false;
}
ConnectionManager_ACE::~ConnectionManager_ACE()
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::~ConnectionManager_ACE"));
	//delete the thread here if the disconnect metho
	disconnectFromApplicationServer();
}
	
bool ConnectionManager_ACE::init(Smartflow *sf)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::init"));
	bool res =super::init(sf);
	
	/* create connection object here eventually using parameters 
	 * from the sf object */
    return res;
}

bool ConnectionManager_ACE::connectToApplicationServer(const std::string &clientname, const std::string &clientgroup)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::connect_to_application_server"));
	int  port_number = getSmartflow()->getPreferenceManager()->getPreferenceIntValue("sf2d_local_port_number");
	if ( port_number == -1 ) port_number = SF2D_DEFAULT_LOCAL_PORT;
	
	//AF_INET to force IPv4
	ACE_INET_Addr srv_addr(port_number,"localhost", AF_INET);
	ACE_SOCK_Connector connector;
	
	if ( connector.connect(srv_stream,srv_addr) != 0 )
	{	getSmartflow()->setError(SERVER_CANNOT_CONNECT);
		return false;
	}

	isConnected_ = true;
	
	ACE_TString real_name;
	std::string tmp = "clientname";
	real_name = ACE_TEXT_CHAR_TO_TCHAR((getSmartflow()->getPreferenceManager()->getPreferenceValue(tmp)).c_str());

	if ( real_name.length() == 0 )
	      real_name = ACE_TEXT_CHAR_TO_TCHAR(clientname.c_str()); 

	ACE_TString real_group;
	tmp = "clientgroup";
	real_group = ACE_TEXT_CHAR_TO_TCHAR((getSmartflow()->getPreferenceManager()->getPreferenceValue(tmp)).c_str());

	if ( real_group.length() == 0 ) 
	      real_group = ACE_TEXT_CHAR_TO_TCHAR(clientgroup.c_str()); 	

	char namelocal[256];
	if (ACE_OS::hostname(namelocal,256) ==1)
	{
		getSmartflow()->setError(SERVER_ERROR_RESPONSE);
		isConnected_ = false;
		return false;
	}

	std::string clientserver;
	clientserver = namelocal ;
	
	ACE_TString clientpath;	
	tmp = "clientpath";
	clientpath = ACE_TEXT_CHAR_TO_TCHAR((getSmartflow()->getPreferenceManager()->getPreferenceValue(tmp)).c_str());

	ACE_TString clientparam;	
	tmp = "clientparam";
	clientparam = ACE_TEXT_CHAR_TO_TCHAR((getSmartflow()->getPreferenceManager()->getPreferenceValue(tmp)).c_str());

	ACE_TString flowinfo;	
	tmp = "flowinfo";
	flowinfo = ACE_TEXT_CHAR_TO_TCHAR((getSmartflow()->getPreferenceManager()->getPreferenceValue(tmp)).c_str());

	//char welcome_message[MAX_MESSAGE_SIZE]={0};
	size_t size_needed = sizeof(CONNECT_MESSAGE) + real_name.length() + real_group.length() + clientparam.length() + clientpath.length() + flowinfo.length() + clientserver.length() + 1; 
	char *welcome_message = (char *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(welcome_message, 0, size_needed);
	
	ACE_OS::sprintf(welcome_message, CONNECT_MESSAGE, real_name.c_str(), real_group.c_str(), clientparam.c_str(), clientpath.c_str() , flowinfo.c_str(), clientserver.c_str());	
		
	bool result = exchange_message(welcome_message) ;

	ACE_OS::free(welcome_message);
	
	/* We are creating here the thread which will handle the connection server->client */
	if ( result )
	{
		thread_client.setNameAndGroup(real_name, real_group);
		thread_client.setConnectionManger(this);
		thread_client.activate();
	}

	return result;
		
}
bool ConnectionManager_ACE::disconnectFromApplicationServer(void)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::disconnect_from_application_server"));
	thread_client.stop_thread();
	thread_client.wait();
	//close the socket
	close_socket();
	return true;
}

bool ConnectionManager_ACE::getFlow(const std::string &type, const std::string &name, const std::string &group, const std::string &arg, flow_direction_t direction, int blockcount, int blocksize, int check, unsigned int numberOfConsumersExpected)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::getFlow"));
	//char getflow_message[MAX_MESSAGE_SIZE]={0};
	
	char tmp_str[11];
	size_t size_needed = sizeof(GETFLOWMESSAGE) + type.length() + name.length() + group.length() + arg.length() + ACE_OS::sprintf(tmp_str, "%d", direction)  + ACE_OS::sprintf(tmp_str, "%d", blocksize) + ACE_OS::sprintf(tmp_str, "%d", blockcount) + ACE_OS::sprintf(tmp_str, "%d", check) + ACE_OS::sprintf(tmp_str, "%d", numberOfConsumersExpected) + 1; 
	char *getflow_message = (char *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(getflow_message, 0, size_needed);
	ACE_OS::sprintf(getflow_message, GETFLOWMESSAGE, type.c_str(), name.c_str(), group.c_str(), arg.c_str(), direction , blocksize , blockcount , check, numberOfConsumersExpected);
	bool res = exchange_message(getflow_message);
		
	ACE_OS::free(getflow_message);
	return res;
}

bool ConnectionManager_ACE::removeFlow(const std::string &type, const std::string &name,const std::string &group)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::removeFlow"));
	//char remflow_message[MAX_MESSAGE_SIZE]={0};
	size_t size_needed = sizeof(REMOVEFLOWMESSAGE) + type.length() + name.length() + group.length() + 1; 
	char *remflow_message = (char *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(remflow_message, 0, size_needed);
	ACE_OS::sprintf(remflow_message, REMOVEFLOWMESSAGE, type.c_str(), name.c_str(), group.c_str());
	bool res = exchange_message(remflow_message);
	
	ACE_OS::free(remflow_message);
	return res;
}

bool ConnectionManager_ACE::checkFlow(const std::string &type,const std::string &name,const std::string &group)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::checkFlow"));
	//char getflow_message[MAX_MESSAGE_SIZE]={0};
	size_t size_needed = sizeof(GETFLOWMESSAGE) + type.length() + name.length() + group.length() + 5 + 1; 
	char *getflow_message = (char *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(getflow_message, 0, size_needed);
	ACE_OS::sprintf(getflow_message, GETFLOWMESSAGE, type.c_str(), name.c_str(), group.c_str(), "", 0 , 0 , 0 , 0, 0);
	bool res = exchange_message(getflow_message);
	
	ACE_OS::free(getflow_message);
	return res;
}

bool ConnectionManager_ACE::startClient(const std::string &clientpath, const std::string &server,const std::string &arguments,const std::string &parameters)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::startClient"));
	//char startclient_message[MAX_MESSAGE_SIZE]={0};
	size_t size_needed = sizeof(START_MESSAGE) + clientpath.length() + arguments.length() + parameters.length() + server.length() + 1; 
	char *startclient_message = (char *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(startclient_message, 0, size_needed);
	ACE_OS::sprintf(startclient_message, START_MESSAGE, clientpath.c_str(), arguments.c_str(), parameters.c_str(), server.c_str());
	bool res = exchange_message(startclient_message);
	
	ACE_OS::free(startclient_message);
	return res;
}

bool ConnectionManager_ACE::stopClient(const std::string &clientname,const std::string &clientgroup)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::stopClient"));
	//char stopclient_message[MAX_MESSAGE_SIZE]={0};
	size_t size_needed = sizeof(STOP_MESSAGE) + clientname.length() + clientgroup.length() + 1; 
	char *stopclient_message = (char *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(stopclient_message, 0, size_needed);
	ACE_OS::sprintf(stopclient_message, STOP_MESSAGE, clientname.c_str(), clientgroup.c_str());
	bool res = exchange_message(stopclient_message);	

	ACE_OS::free(stopclient_message);
	return res;
}

bool ConnectionManager_ACE::getApplicationDescription(std::string &flowlist)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::getApplicationDescription"));
	//char flowlist_message[MAX_MESSAGE_SIZE]={0};
	char tmp_str[11];
	size_t size_needed = sizeof(REQUEST_MESSAGE) + ACE_OS::sprintf(tmp_str, "%d", FLOWLIST) + 1; 
	char *flowlist_message = (char *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(flowlist_message, 0, size_needed);
	
	//char answer_message[MAX_MESSAGE_SIZE]={0};
	char *answer_message = NULL;
	
	bool result;
	ACE_OS::sprintf(flowlist_message, REQUEST_MESSAGE, FLOWLIST);
	result = exchange_message(flowlist_message, answer_message);	
	if ( result )
		flowlist.append(answer_message);	
		
	ACE_OS::free(flowlist_message);
	ACE_OS::free(answer_message);
	return result;	
}

bool ConnectionManager_ACE::getServerList(std::string &serverlist)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::getServerList"));
	//char serverlist_message[MAX_MESSAGE_SIZE]={0};
	char tmp_str[11];
	size_t size_needed = sizeof(REQUEST_MESSAGE) + ACE_OS::sprintf(tmp_str, "%d", SERVERLIST) + 1; 
	char *serverlist_message = (char *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(serverlist_message, 0, size_needed);
	
	//char answer_message[MAX_MESSAGE_SIZE]={0};
	char *answer_message = NULL;
	
	bool result;
	ACE_OS::sprintf(serverlist_message, REQUEST_MESSAGE, SERVERLIST);	
	result = exchange_message(serverlist_message, answer_message);
	if ( result )
		serverlist.append(answer_message);	
	
	ACE_OS::free(serverlist_message);
	ACE_OS::free(answer_message);
	return result;	
}

bool ConnectionManager_ACE::getLastModifications(std::string &modifications)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::getLastModifications"));
	//char lastmodifications_message[MAX_MESSAGE_SIZE]={0};
	char tmp_str[11];
	size_t size_needed = sizeof(REQUEST_MESSAGE) + ACE_OS::sprintf(tmp_str, "%d", GETLASTMODIFICATIONS) + 1; 
	char *lastmodifications_message = (char *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(lastmodifications_message, 0, size_needed);
	
	//char answer_message[MAX_MESSAGE_SIZE]={0};
	char *answer_message = NULL;
	
	bool result;
	ACE_OS::sprintf(lastmodifications_message, REQUEST_MESSAGE, GETLASTMODIFICATIONS);	
	result = exchange_message(lastmodifications_message, answer_message);
	if ( result )
		modifications.append(answer_message);	
	
	ACE_OS::free(lastmodifications_message);
	ACE_OS::free(answer_message);
	return result;	
}

/* Pause flow */
bool ConnectionManager_ACE::pauseFlow(const std::string &clientname,const std::string &clientgroup,const std::string &flowtype,const std::string &flowgroup)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::pauseFlow"));
    //char serverPauseMessage[MAX_MESSAGE_SIZE]={0};
	size_t size_needed = sizeof(PAUSE_MESSAGE) + clientname.length() + clientgroup.length() + flowtype.length() + flowgroup.length() + 1; 
	char *serverPauseMessage = (char *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(serverPauseMessage, 0, size_needed);
    ACE_OS::sprintf(serverPauseMessage, PAUSE_MESSAGE, clientname.c_str(), clientgroup.c_str(), flowtype.c_str(), flowgroup.c_str());
    bool res = exchange_message(serverPauseMessage);
	
	ACE_OS::free(serverPauseMessage);
	return res;
}

/* Restart flow */
bool ConnectionManager_ACE::restartFlow(const std::string &clientname,const std::string &clientgroup,const std::string &flowtype,const std::string &flowgroup)
{
    ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::restartFlow"));
    //char serverRestartMessage[MAX_MESSAGE_SIZE]={0};
	size_t size_needed = sizeof(RESTART_MESSAGE) + clientname.length() + clientgroup.length() + flowtype.length() + flowgroup.length() + 1; 
	char *serverRestartMessage = (char *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(serverRestartMessage, 0, size_needed);
    ACE_OS::sprintf(serverRestartMessage, RESTART_MESSAGE, clientname.c_str(), clientgroup.c_str(), flowtype.c_str(), flowgroup.c_str());
    bool res =  exchange_message(serverRestartMessage);
	
	ACE_OS::free(serverRestartMessage);
    return res;
}

bool ConnectionManager_ACE::sendLog(const std::string &logMessage) {
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::sendLog"));
	//char theLogMessage[MAX_MESSAGE_SIZE] = {0};
	size_t size_needed = sizeof(LOG_MESSAGE) + logMessage.length() + 1; 
	char *theLogMessage = (char *) ACE_OS::malloc(size_needed);
	ACE_OS::memset(theLogMessage, 0, size_needed);
	ACE_OS::sprintf(theLogMessage, LOG_MESSAGE, logMessage.c_str());
	
	//int res = send_message(theLogMessage);
	bool res =  exchange_message(theLogMessage);
	
	ACE_OS::free(theLogMessage);
	return res;
}
//need to change that in server to not respond 
/*int ConnectionManager_ACE::send_message(char * message) {
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::send_message"));
	char tmp[PROTOCOL_SIZEPARAM_SIZE] = {0};
	int res = 0;
	
	// Just to be sure that another function won't call the exchange_message in a threaded environnement 
	ACE_GUARD_RETURN(ACE_Recursive_Thread_Mutex, guard, *ACE_Static_Object_Lock::instance() , -1);
 
	// Send the size first, on 512 bytes ascii 
	ACE_OS::sprintf(tmp,"%d",message_size);
	err = srv_stream.send_n(tmp,PROTOCOL_SIZEPARAM_SIZE);
	if ( err != PROTOCOL_SIZEPARAM_SIZE) {
		getSmartflow()->setError(SERVER_INVALID_PROTOCOL);return -1;
	}	

	// Send the message then, on the size previously given 
	err  = srv_stream.send_n(message,message_size);
	if ( err != message_size) {
		getSmartflow()->setError(SERVER_INVALID_PROTOCOL);return -1;
	}
	
	return 0;
}*/

bool ConnectionManager_ACE::exchange_message(char * message, char *&answer)
{
	if ( isConnected_ ) {
		char tmp[PROTOCOL_SIZEPARAM_SIZE] = {0};
		char* tmp2;
		
		long message_size = ACE_OS::strlen(message);
		long err;
		if ( message_size == 0 ) {
			getSmartflow()->setError(SERVER_INVALID_PARAMETER);
			return false;
		}
		
		/* Just to be sure that another function won't call the exchange_message in a threaded environnement */
		//ACE_GUARD_RETURN(ACE_Recursive_Thread_Mutex, guard, *ACE_Static_Object_Lock::instance() , -1);
		ACE_Guard<ACE_Thread_Mutex> guard(this->theMutex);
	 
		/* Send the size first, on 512 bytes ascii */
		ACE_OS::sprintf(tmp,"%d",message_size);
		err = srv_stream.send_n(tmp,PROTOCOL_SIZEPARAM_SIZE);
		if ( err != PROTOCOL_SIZEPARAM_SIZE) {
			getSmartflow()->setError(SERVER_INVALID_PROTOCOL);
			return false;
		}	

		/* Send the message then, on the size previously given */
		err  = srv_stream.send_n(message,message_size);
		if ( err != message_size) {
			getSmartflow()->setError(SERVER_INVALID_PROTOCOL);
			return false;
		}

		/* Now, wait for the answer : first the size */
		err = srv_stream.recv_n(tmp,PROTOCOL_SIZEPARAM_SIZE);
		if ( err != PROTOCOL_SIZEPARAM_SIZE) {
			getSmartflow()->setError(SERVER_INVALID_PROTOCOL);
			return false;
		}

		message_size = ACE_OS::atoi(tmp);
		if ( message_size <=0 ) {
			getSmartflow()->setError(SERVER_INVALID_PROTOCOL);
			return false;
		}
		/* Allocate the memory of the buffer for the answer */
		tmp2 = (char *) ACE_OS::malloc(message_size+1);
		/* Clear the tmp2 variable, just to be sure */
		ACE_OS::memset(tmp2,0,message_size+1);

		err = srv_stream.recv_n(tmp2,message_size);	
		if ( err != message_size ) {
			getSmartflow()->setError(SERVER_INVALID_PROTOCOL);
			return false;
		}
		
		//if ( answer != NULL )
			//ACE_OS::strncpy(answer,tmp2,message_size);
			answer = tmp2;
		//else
		//	res = is_answer_successfull(tmp2);
		
		//ACE_OS::free(tmp2);
		return true;	
		}
	else {
		getSmartflow()->setError(SERVER_IS_NOT_CONNECTED);
		return false;
	}
}

bool ConnectionManager_ACE::exchange_message(char * message)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::exchange_message"));
	
	if ( isConnected_ ) {
		char tmp[PROTOCOL_SIZEPARAM_SIZE] = {0};
		char* tmp2;
		bool theRes = false;
		
		long message_size = ACE_OS::strlen(message);
		long err;
		if ( message_size == 0 ) {
			getSmartflow()->setError(SERVER_INVALID_PARAMETER);
			return false;
		}
		
		/* Just to be sure that another function won't call the exchange_message in a threaded environnement */
		//ACE_GUARD_RETURN(ACE_Recursive_Thread_Mutex, guard, *ACE_Static_Object_Lock::instance() , -1);
		ACE_Guard<ACE_Thread_Mutex> guard(this->theMutex);
		
		/* Send the size first, on 512 bytes ascii */
		ACE_OS::sprintf(tmp,"%d",message_size);
		err = srv_stream.send_n(tmp,PROTOCOL_SIZEPARAM_SIZE);
		if ( err != PROTOCOL_SIZEPARAM_SIZE) {
			getSmartflow()->setError(SERVER_INVALID_PROTOCOL);return false;
		}	

		/* Send the message then, on the size previously given */
		err  = srv_stream.send_n(message,message_size);
		if ( err != message_size) {
			getSmartflow()->setError(SERVER_INVALID_PROTOCOL);return false;
		}

		/* Now, wait for the answer : first the size */
		err = srv_stream.recv_n(tmp,PROTOCOL_SIZEPARAM_SIZE);
		if ( err != PROTOCOL_SIZEPARAM_SIZE) {
			getSmartflow()->setError(SERVER_INVALID_PROTOCOL);return false;
		}

		message_size = ACE_OS::atoi(tmp);
		if ( message_size <=0 ) {
			getSmartflow()->setError(SERVER_INVALID_PROTOCOL);return false;
		}
		/* Allocate the memory of the buffer for the answer */
		tmp2 = (char *) ACE_OS::malloc(message_size);
		/* Clear the tmp2 variable, just to be sure */
		ACE_OS::memset(tmp2,0,message_size);

		err = srv_stream.recv_n(tmp2,message_size);	
		if ( err != message_size ) {
			getSmartflow()->setError(SERVER_INVALID_PROTOCOL);return false;
		}	
		
		theRes = is_answer_successfull(tmp2);
		
		ACE_OS::free(tmp2);
		return theRes;	
	}
	else {
		getSmartflow()->setError(SERVER_IS_NOT_CONNECTED);
		return false;
	}
}

bool ConnectionManager_ACE::is_answer_successfull(char * answer)
{
	ACE_TRACE(ACE_TEXT("ConnectionManager_ACE::is_answer_successfull"));
	char * needle = ACE_OS::strnstr(answer,PROTOCOL_SUCCESS_STRING,ACE_OS::strlen(PROTOCOL_SUCCESS_STRING));
	if ( needle == NULL ) 
	{ 
			char *token; 
			token = ACE_OS::strtok(answer,"<>");				
			while(token)
			{				
				if (ACE_OS::strcmp(token,"description") == 0 )
				{				
					token = ACE_OS::strtok(NULL, "<>");
					std::cout << "Error : " <<  token << std::endl;
				}
				token = ACE_OS::strtok(NULL, "<>");
			}
			
			getSmartflow()->setError(SERVER_ERROR_RESPONSE); 
			return false;
	}
	return true;
}
	
void ConnectionManager_ACE::close_socket() {
	ACE_Guard<ACE_Thread_Mutex> guard(this->theMutex);
	isConnected_ = false;
	srv_stream.close();
}

} // namespace SMARTFLOW
