#ifndef _ConnectionManager_ACE_H
#define _ConnectionManager_ACE_H

#include "ace/svc_export.h" // for exported symbols (windows)

#include "ace/SOCK_Stream.h"

#include "Smartflow.h"
#include "ConnectionManager.h"
#include "Thread_Client.h"

namespace smartflow {

//forward declaration	
class Thread_Client;	

/** \brief Class ConnectionManager_ACE .
 * This class is the implementation using the ACE library of the Connection 
 * Manager. It is used to communicate with the Smartflow Application Server.
 * The class implement the protocol with the server, and has an instance of a 
 * Connection class to do the actual low-level exchange.
 * \author Nist Smartspace Group
 * \date 04-19-2004
*/
class ACE_Svc_Export ConnectionManager_ACE:public ConnectionManager {
typedef ConnectionManager super;
	friend class Thread_Client;
	friend class ExitHandler;
public:
	/** Class constructor. 
         * \warning This method does not initialise the object, the init() method
         * must be used for this purpose.
    */
	ConnectionManager_ACE();
    
    /** Class destructor.
         * Take care of any cleanup that is necessary to free the object.
    */
    virtual ~ConnectionManager_ACE();
    
    /** Class initialiser, Must be called right after the object construction. 
         * \param sf The Smartflow object in which this object will be registered
         * \return 0 if the object is correctly initialised, otherwise -1. If possible,
         * an error will be set via Smartflow::seterror() to explain the error.
    */
    virtual bool init(Smartflow *sf);
    virtual bool connectToApplicationServer(const std::string &name, const std::string &group);
	virtual bool disconnectFromApplicationServer(void);
	virtual bool getFlow(const std::string &type,const std::string &name,const std::string &group , const std::string &arg, flow_direction_t direction, int blockcount, int blocksize, int check, unsigned int numberOfConsumersExpected); 	
	virtual bool removeFlow(const std::string &type, const std::string &name, const std::string &group);
	virtual bool checkFlow(const std::string &type,const std::string &name,const std::string &group);
	
	/* the following methods are used int the control center */
	virtual bool startClient(const std::string &clientpath, const std::string &server,const std::string &arguments,const std::string &parameters);
	virtual bool stopClient(const std::string &clientname,const std::string &clientgroup);
	virtual bool getApplicationDescription(std::string &flowlist);
	virtual bool getServerList(std::string &serverlist);
	virtual bool getLastModifications(std::string &modifications);
	virtual bool sendLog(const std::string &logMessage);

	/* Pause flow */
	virtual bool pauseFlow(const std::string &clientname,const std::string &clientgroup,const std::string &flowtype,const std::string &flowgroup);
	/* Restart flow */
	virtual bool restartFlow(const std::string &clientname,const std::string &clientgroup,const std::string &flowtype,const std::string &flowgroup);
	
	bool isConnected() { return isConnected_; }
	
private:
	ACE_SOCK_Stream    srv_stream;
	ACE_Thread_Mutex theMutex;
	Thread_Client	thread_client;
	bool isConnected_;
	bool exchange_message(char * message, char *&answer);
	bool exchange_message(char * message);
	bool is_answer_successfull(char * answer);
	void close_socket();
}; // class ConnectionManager_ACE
}  // namespace SMARTFLOW
#endif
