#include "ace/ACE.h"
#include "ace/SString.h"

#include "PreferenceManager.h"
#include "Flow.h"
#include "FlowImpl.h"
#include "FlowImpl_ACE.h"
#include "ErrorManager.h"
#include "Log_Manager_ACE.h"
#include "duplicator_common_ACE.h"

#include <string>
#include <sstream>
#include <iostream>
#include <list>

namespace smartflow {

Flow::Flow()
{
	ACE_TRACE(ACE_TEXT("Flow::Flow"));
}

Flow::~Flow()
{
	ACE_TRACE(ACE_TEXT("Flow::~Flow"));
	delete flowimpl ; flowimpl = 0;
	args_.clear();
	id_.clear();
	type_.clear();
	mode_ = FLOW_INVALID;
	manager_ = 0;
	isSynchronized_ = false;
}
bool Flow::init(FlowManager *manager,const std::string &type, const std::string &name, const std::string &rgroup,const std::string &args,flow_direction_t mode, flow_policy_t policy, bool isSynchronized)
{
	ACE_TRACE(ACE_TEXT("Flow::init"));
  /* Setup the manager first */
  if (manager == NULL) {
    return false;
  }
  manager_ = manager;
        	
  /* Check that the requested type is the one of this instance */
  if ( this->getFlowType().compare(type) != 0 ) {
    get_smartflow()->setError(FLOW_DLL_INSTANCE_NOT_VALID);
    return false;
  }
  if ( (mode != FLOW_INPUT) && (mode != FLOW_OUTPUT) ) {
    get_smartflow()->setError(FLOW_INVALID_FLOW_STATE);
    return false;
  }
  if ( (policy != DO_NOT_DROP_BUFFER) && (policy != DROP_MOST_RECENT_BUFFER) && (policy != DROP_OLDEST_BUFFER) ) {
	get_smartflow()->setError(FLOW_QUEUE_POLICY_INVALID);
    return false;
  }

  mode_ = mode;
  policy_ = policy;
  type_ = type;
  rgroup_ = rgroup;
  
  id_ = rgroup;
  id_.append(":");
  id_.append(type);
 
  args_ = args;
  name_ = name;

  isSynchronized_ = isSynchronized;
  
  flowimpl = new FlowImpl_ACE();

  if (! flowimpl) {
    get_smartflow()->setError(FLOW_CANNOT_CREATE_IMPL);
    return false;
  }

  if ( !flowimpl->init(this) ){
    get_smartflow()->setError(FLOW_CANNOT_CREATE_IMPL);
    return false;
  }      

  return true;
}
	
Smartflow * Flow::getSmartflow(void)
{
	ACE_TRACE(ACE_TEXT("Flow::getSmartflow"));
	return manager_->get_smartflow();
}
	
Smartflow * Flow::get_smartflow(void)
{
	ACE_TRACE(ACE_TEXT("Flow::get_smartflow"));
	return this->getSmartflow();
}

FlowManager * Flow::getManager(void)
{
	ACE_TRACE(ACE_TEXT("Flow::getManager"));
	return manager_ ;
}
	
FlowManager * Flow::get_manager(void)
{
	ACE_TRACE(ACE_TEXT("Flow::get_manager"));
	return this->getManager();
}

flow_direction_t Flow::getMode(void) {
	ACE_TRACE(ACE_TEXT("Flow::getMode"));
	return mode_; 
}
	
flow_direction_t Flow::get_mode(void) {
	ACE_TRACE(ACE_TEXT("Flow::get_mode"));
	return this->getMode(); 
}
	
flow_policy_t Flow::getFlowQueuePolicy(void) {
	ACE_TRACE(ACE_TEXT("Flow::getFlowQueuePolicy"));
	return policy_;
}
	
flow_policy_t Flow::get_flow_queue_policy(void) {
	ACE_TRACE(ACE_TEXT("Flow::get_flow_queue_policy"));
	return this->getFlowQueuePolicy();
}   
	
std::string & Flow::getId(void) {
	ACE_TRACE(ACE_TEXT("Flow::getId"));
	return id_;
}

std::string & Flow::get_id(void) {
	ACE_TRACE(ACE_TEXT("Flow::get_id"));
	return this->getId();
}
    
std::string & Flow::getName(void) {
	ACE_TRACE(ACE_TEXT("Flow::getName"));
	return name_;
}

std::string & Flow::get_name(void) {
	ACE_TRACE(ACE_TEXT("Flow::get_name"));
	return this->getName();
}
    
std::string & Flow::getType(void) {
	ACE_TRACE(ACE_TEXT("Flow::getType"));
	return type_;
}

std::string & Flow::get_type(void) {
	ACE_TRACE(ACE_TEXT("Flow::get_type"));
	return this->getType();
}
    
std::string & Flow::getArgs(void) {
	ACE_TRACE(ACE_TEXT("Flow::getArgs"));
	return args_;
}

std::string & Flow::get_args(void) {
	ACE_TRACE(ACE_TEXT("Flow::get_args"));
	return this->getArgs();
}
    
std::string & Flow::getGroup(void) {
	ACE_TRACE(ACE_TEXT("Flow::getGroup"));
	return rgroup_;
}

std::string & Flow::get_group(void) {
	ACE_TRACE(ACE_TEXT("Flow::get_group"));
	return this->getGroup();
}
	
std::string & Flow::get_flow_type(void) {
	return this->getFlowType();
}

std::string & Flow::get_flow_desc(void) {
	return this->getFlowDesc();
}
	
bool  Flow::is_synchronized() {
	return this->isSynchronized();
}

long  Flow::getHistorySize(void)
{
	ACE_TRACE(ACE_TEXT("Flow::getHistorySize"));
	return flowimpl->getHistorySize();
}
	
long  Flow::get_history_size(void)
{
	ACE_TRACE(ACE_TEXT("Flow::get_history_size"));
	return this->getHistorySize();
}

bool  Flow::setHistorySize(long size)
{
	ACE_TRACE(ACE_TEXT("Flow::setHistorySize"));
	return flowimpl->setHistorySize(size);
}
	
bool  Flow::set_history_size(long size)
{
	ACE_TRACE(ACE_TEXT("Flow::set_history_size"));
	return this->setHistorySize(size);
}
	
bool Flow::setMaxBufferSize(buffer_size_t size)
{
	ACE_TRACE(ACE_TEXT("Flow::setMaxBufferSize"));
	return flowimpl->setMaxBufferSize(size);
}

bool Flow::set_max_buffer_size(buffer_size_t size)
{
	ACE_TRACE(ACE_TEXT("Flow::set_max_buffer_size"));
	return this->setMaxBufferSize(size);
}
	
buffer_size_t Flow::getMaxBufferSize(void)
{
	ACE_TRACE(ACE_TEXT("Flow::getMaxBufferSize"));
	return flowimpl->getMaxBufferSize();
}
	
buffer_size_t Flow::get_max_buffer_size(void)
{
	ACE_TRACE(ACE_TEXT("Flow::get_max_buffer_size"));
	return this->getMaxBufferSize();
}
	
bool Flow::setWaterMark(double water_mark)
{
	ACE_TRACE(ACE_TEXT("Flow::setWaterMark"));
	return flowimpl->setWaterMark(water_mark);
}
	
bool Flow::set_water_mark(double water_mark)
{
	ACE_TRACE(ACE_TEXT("Flow::set_water_mark"));
	return this->setWaterMark(water_mark);
}
	
double Flow::getWaterMark(void) 
{
	ACE_TRACE(ACE_TEXT("Flow::getWaterMark"));
	return flowimpl->getWaterMark();
}
	
double Flow::get_water_mark(void) 
{
	ACE_TRACE(ACE_TEXT("Flow::get_water_mark"));
	return this->getWaterMark();
}
	
double Flow::getFillingLevelOfQueue()
{
	ACE_TRACE(ACE_TEXT("Flow::getFillingLevelOfQueue"));
	return flowimpl->getFillingLevelOfQueue();
}
	
double Flow::get_filling_level_of_queue()
{
	ACE_TRACE(ACE_TEXT("Flow::get_filling_level_of_queue"));
	return this->getFillingLevelOfQueue();
}	
	
bool Flow::isBufferAvailable() 
{
	ACE_TRACE(ACE_TEXT("Flow::isBufferAvailable"));
	return flowimpl->isBufferAvailable();
}
	
bool Flow::is_buffer_available() 
{
	ACE_TRACE(ACE_TEXT("Flow::is_buffer_available"));
	return this->isBufferAvailable();
}
	
bool Flow::setBlockCount(int size)
{
	ACE_TRACE(ACE_TEXT("Flow::setBlockCount"));
	return flowimpl->setBlockCount(size);
}
	
bool Flow::set_block_count(int size)
{
	ACE_TRACE(ACE_TEXT("Flow::set_block_count"));
	return this->setBlockCount(size);
}
    
int Flow::getBlockCount(void) 
{
	ACE_TRACE(ACE_TEXT("Flow::getBlockCount"));
	return flowimpl->getBlockCount();
}
	
int Flow::get_block_count(void) 
{
	ACE_TRACE(ACE_TEXT("Flow::get_block_count"));
	return this->getBlockCount();
}

Buffer * Flow::getBuffer(void)
{
	ACE_TRACE(ACE_TEXT("Flow::getBuffer"));
	return flowimpl->getBuffer();
}
	
Buffer * Flow::get_buffer(void)
{
	ACE_TRACE(ACE_TEXT("Flow::get_buffer"));
	return this->getBuffer();
}

bool  Flow::releaseBuffer(Buffer *buf)
{
	ACE_TRACE(ACE_TEXT("Flow::releaseBuffer"));
	return flowimpl->releaseBuffer(buf);
}
	
bool  Flow::release_buffer(Buffer *buf)
{
	ACE_TRACE(ACE_TEXT("Flow::release_buffer"));
	return this->releaseBuffer(buf);
}

bool Flow::cancelBuffer(Buffer * buf)
{
	ACE_TRACE(ACE_TEXT("Flow::cancelBuffer"));
     return flowimpl->cancelBuffer(buf);
}

bool Flow::cancel_buffer(Buffer * buf)
{
	ACE_TRACE(ACE_TEXT("Flow::cancel_buffer"));
	return this->cancelBuffer(buf);
}
	
bool Flow::start(void)
{
	ACE_TRACE(ACE_TEXT("Flow::start"));
	bool result = flowimpl->start();
	if ( result == true)
		ACE_DEBUG((ANNOUNCE_PREFIX ACE_TEXT("[ Flow::start ] Flow: %s started\n"), get_id().c_str()));
	else
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Flow::start ] Error starting flow: %s\n"), get_id().c_str()));
	return result;
}

bool Flow::stop(bool waitForEmptyHistory)
{
	ACE_TRACE(ACE_TEXT("Flow::stop"));
	int result = flowimpl->stop(waitForEmptyHistory);
	if ( result == false)
		ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ Flow::stop ] Error stopping flow: %s\n"), get_id().c_str()));
	return result;
}

/* protected helper method */
std::string Flow::parseArgs(std::string keyword,std::string arg)
{
	ACE_TRACE(ACE_TEXT("Flow::parseArgs"));
    std::string result;
    std::string::size_type idx2;
    std::string::size_type idx=arg.find(keyword);
    if ( idx == std::string::npos ) {
       return result; /* Empty string */
    }
    idx = arg.find('=',idx);
    if ( idx == std::string::npos ) {
       return result; /* Empty string */
    }
    idx++;
    idx = arg.find_first_not_of(" \t\n\r",idx);
    if ( idx == std::string::npos ) {
       return result; /* Empty string */
    }
    idx2= arg.find_first_of(" \t\n\r\0",idx);
    result = arg.substr(idx,idx2-idx);
    return result;
}

//By Imad

bool Flow::pause() {
	ACE_TRACE(ACE_TEXT(" Flow::pause"));
  if (  get_mode() == FLOW_OUTPUT ) {
    get_smartflow()->setError(FLOW_PROVIDER_PAUSE_ERROR);
    return false;
  }
  else {
    ACE_TString client_name;
    std::string tmp = "clientname";
    client_name = ACE_TEXT_CHAR_TO_TCHAR((get_smartflow()->getPreferenceManager()->getPreferenceValue(tmp)).c_str());
    if ( client_name.length() == 0 )
      client_name = ACE_TEXT_CHAR_TO_TCHAR(get_smartflow()->getClientName().c_str()); 
    
    ACE_TString client_group;
    tmp = "clientgroup";
    client_group = ACE_TEXT_CHAR_TO_TCHAR((get_smartflow()->getPreferenceManager()->getPreferenceValue(tmp)).c_str());
    if ( client_group.length() == 0 )
      client_group = ACE_TEXT_CHAR_TO_TCHAR(get_smartflow()->getGroupName().c_str());
    
    ACE_TString flow_type;
    flow_type = ACE_TEXT_CHAR_TO_TCHAR(this->get_type().c_str());
    
    ACE_TString flow_rgroup;
    flow_rgroup = ACE_TEXT_CHAR_TO_TCHAR(this->get_group().c_str());
    
    ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ Flow::pause (%s)(%s)  ||  (%s)(%s)] \n"),flow_rgroup.c_str(),flow_type.c_str(),client_group.c_str(),client_name.c_str()));
	
	return get_smartflow()->pauseFlow(ACE_TEXT_ALWAYS_CHAR(client_name.c_str()), ACE_TEXT_ALWAYS_CHAR(client_group.c_str()),ACE_TEXT_ALWAYS_CHAR(flow_type.c_str()),ACE_TEXT_ALWAYS_CHAR(flow_rgroup.c_str()));
  }
}

bool Flow::restart(){
	ACE_TRACE(ACE_TEXT("Flow::restart"));
  if (  get_mode() == FLOW_OUTPUT ) {
    get_smartflow()->setError(FLOW_PROVIDER_PAUSE_ERROR);    
    return false;
  }
  else {
    ACE_TString client_name;
    std::string tmp = "clientname";
    client_name = ACE_TEXT_CHAR_TO_TCHAR((get_smartflow()->getPreferenceManager()->getPreferenceValue(tmp)).c_str());
    if ( client_name.length() == 0 )
      client_name = ACE_TEXT_CHAR_TO_TCHAR(get_smartflow()->getClientName().c_str()); 
  
    ACE_TString client_group;
    tmp = "clientgroup";
    client_group = ACE_TEXT_CHAR_TO_TCHAR((get_smartflow()->getPreferenceManager()->getPreferenceValue(tmp)).c_str());
    if ( client_group.length() == 0 )
      client_group = ACE_TEXT_CHAR_TO_TCHAR(get_smartflow()->getGroupName().c_str());
    
       
    ACE_TString flow_rgroup;
    flow_rgroup = ACE_TEXT_CHAR_TO_TCHAR(this->get_group().c_str());
    
    
    ACE_TString flow_type;
    flow_type = ACE_TEXT_CHAR_TO_TCHAR(this->get_type().c_str());
    
    ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ Flow::restart (%s)(%s)  ||  (%s)(%s)] \n"),flow_rgroup.c_str(),flow_type.c_str(),client_group.c_str(),client_name.c_str()));

	  return get_smartflow()->restartFlow(ACE_TEXT_ALWAYS_CHAR(client_name.c_str()), ACE_TEXT_ALWAYS_CHAR(client_group.c_str()),ACE_TEXT_ALWAYS_CHAR(flow_type.c_str()),ACE_TEXT_ALWAYS_CHAR(flow_rgroup.c_str()));
  }
}

} //namespace SMARTFLOW
