#ifndef _MemoryManager_ACE_H
#define _MemoryManager_ACE_H

#include "ace/svc_export.h" // for exported symbols (windows)

#include "ace/Malloc_Allocator.h"

#include "Smartflow.h"
#include "MemoryManager.h"
#include "Buffer_ACE.h"

namespace smartflow {

/** \brief Class MemoryManager_ACE .
 * This class is used to create, delete and manage the buffers for the flows,
 * and taking care of the size of each buffers and the pool.
 * The ACE version uses the ACE library to manage the memory pool and maintain
 * multi-plateform compatibility.
 * This is the *internal* manager only, it does not deals with the shared 
 * memory not the duplicator. (See the FlowImpl classes for that)
 * \author Nist Smartspace Group
 * \date 04-19-2004
 * \internal
*/
class ACE_Svc_Export MemoryManager_ACE:public MemoryManager,ACE_New_Allocator {
/** Redefine MemoryManager (which is the ancestor of MemoryManager_ACE) as
    \b super . Facility taken from the Java language.
*/
typedef MemoryManager super;
public:
	/** \brief Class constructor. 
         * \warning This method does not initialise the object, the init() method
         * must be used for this purpose.
        */
	MemoryManager_ACE();
        /** \brief Class destructor.
        */
    virtual ~MemoryManager_ACE();
        /** \brief Class initialiser. Must be called just after the object
         * construction. 
         * \param sf The Smartflow object in which this object will be registered
         * \return 0 if the object is correctly initialised, otherwise -1. If possible,
         * an error will be set via Smartflow::seterror() to explain the error.
        */
    virtual bool init(Smartflow *sf);
    
    /** Allocate a buffer for the given flow.
     * The flow object has the default size for the buffer, and the buffer 
     * will be at least that big (unless the size of the buffer is specified). 
    */
    virtual Buffer * mallocBuffer(Flow *flow, buffer_size_t sizeOfBuffer = 0);
    /** Return a buffer to the system. */
    virtual void     freeBuffer(Buffer *buf);
    
private:
}; // class MemoryManager_ACE
}  // namespace SMARTFLOW
#endif
