#include "PreferenceManager_STL.h"
#include "Preference_Args.h"
#include "Preference_XMLFile.h"

#include <iterator>
#include <iostream>


namespace smartflow {
PreferenceManager_STL::PreferenceManager_STL()
	:super()
{
	ACE_TRACE(ACE_TEXT("PreferenceManager_STL::PreferenceManager_STL"));
}
PreferenceManager_STL::~PreferenceManager_STL()
{
	ACE_TRACE(ACE_TEXT("PreferenceManager_STL::~PreferenceManager_STL"));
}
bool PreferenceManager_STL::init(Smartflow *sf)
{
	ACE_TRACE(ACE_TEXT("PreferenceManager_STL::init"));
	return super::init(sf);
}

void PreferenceManager_STL::addPref(string &key, string &value, string &comment) {
  ACE_TRACE(ACE_TEXT("PreferenceManager_STL::addPref"));
  thePrefs[key] = value;
  theComments[key] = comment;
}

void PreferenceManager_STL::addPref(string &key, string &value) { 
  ACE_TRACE(ACE_TEXT("PreferenceManager_STL::addPref"));
  thePrefs[key] = value;
}

void PreferenceManager_STL::remPref(string &key) {
	ACE_TRACE(ACE_TEXT("PreferenceManager_STL::remPref"));
    if ( exists(key) ) 
      thePrefs.erase(key);
    if ( existsComment(key) )
      theComments.erase(key); 
}

string PreferenceManager_STL::getPrefValue(string &key) {
  ACE_TRACE(ACE_TEXT("PreferenceManager_STL::getPrefValue"));
  if ( exists(key) )
    return thePrefs[key];
  else
    return "";
}

int PreferenceManager_STL::getPrefIntValue(string &key) {
  ACE_TRACE(ACE_TEXT("PreferenceManager_STL::getPrefIntValue"));
  if ( exists(key) )
    return atoi(thePrefs[key].c_str());
  else
    return -1;
}

double PreferenceManager_STL::getPrefDoubleValue(string &key) {
  ACE_TRACE(ACE_TEXT("PreferenceManager_STL::getPrefDoubleValue"));
  if ( exists(key) )
    return atof(thePrefs[key].c_str());
  else
    return -1.0;
}

string  PreferenceManager_STL::getPrefComment(string &key) {
  ACE_TRACE(ACE_TEXT("PreferenceManager_STL::getPrefComment"));
  if ( existsComment(key) )
    return theComments[key];
  else
    return "";
}

bool PreferenceManager_STL::exists(string &key) {
  ACE_TRACE(ACE_TEXT("PreferenceManager_STL::exists"));
  map<string,string>::iterator iter;
  iter = thePrefs.find(key);
  if (iter != thePrefs.end()) 
    return true;
  else
    return false;
}

bool PreferenceManager_STL::existsComment(string &key) {
  ACE_TRACE(ACE_TEXT("PreferenceManager_STL::existsComment"));
  map<string,string>::iterator iter;
  iter = theComments.find(key);
  if (iter != theComments.end()) 
    return true;
  else
    return false;
}

void  PreferenceManager_STL::viewPrefs() {
  ACE_TRACE(ACE_TEXT("PreferenceManager_STL::viewPrefs"));
  map<string,string>::iterator iter;
  cout << "Preference list :" << endl;
  for(iter = thePrefs.begin(); iter != thePrefs.end(); ++iter)
    cout << "Key: " << iter->first << "\t" << "Value: " << iter->second << endl;
}
  
string  PreferenceManager_STL::getPrefKey(int num) {
  ACE_TRACE(ACE_TEXT("PreferenceManager_STL::getPrefKey"));
  map<string,string>::iterator iter;
  iter = thePrefs.begin();
  int j;
  for(j=1 ; j < num; j++) {
    if (iter != thePrefs.end() ) 
      iter++;
  }
  if (  iter != thePrefs.end() )
    return iter->first;
  else
    return "";
}

int PreferenceManager_STL::count() {
  ACE_TRACE(ACE_TEXT("PreferenceManager_STL::count"));
  return thePrefs.size();
}

bool PreferenceManager_STL::initArgs(int &argc, char ** &argv, bool keepSFOptions) {
  ACE_TRACE(ACE_TEXT("PreferenceManager_STL::init_args"));
  Preference_Args    arg_prefs;

  if ( arg_prefs.init(this) == false) 
    { return false; }
  if ( arg_prefs.initPreferences(argc,argv, keepSFOptions) == false )
    { return false; }

  //We are looking if the user wants to load a XML preference file. 
  string prefString = "prefFile";
  string filename = getPrefValue(prefString);
  if ( filename != "" ) {
    Preference_XMLFile xml_prefs;
    if ( xml_prefs.init(this) == false )
      return false;
    if ( xml_prefs.readPreferences(filename) == false )
      return false;
  }
  return true;
}

} // namespace SMARTFLOW
