#ifndef _Preference_Args_H
#define _Preference_Args_H

#include "ace/svc_export.h" // for exported symbols (windows)

#include "Smartflow.h"
#include "Preference.h"

#include <string>

using namespace std;

namespace smartflow {

/** \brief Class Preference_Args .
 * Used to parse preferences from command-line.
 * The trigger for smartflow-specific arguments is --sf.
 * See the InitPreferences method documentation for details.
 * The list of preferences is not defined within the code : 
 * preferences are stored as-is in the PreferenceManager.
 * \author Nist Smartspace Group
 * \date 04-19-2004
 * \internal
*/
class ACE_Svc_Export Preference_Args:public Preference {
/** Redefine Preference (which is the ancestor of Preference_Args) as
    \b super . Facility taken from the Java language.
*/
typedef Preference super;
public:
	/** \brief Class constructor. 
         * \warning This method does not initialise the object, the init() method
         * must be used for this purpose.
        */
		Preference_Args();
        /** \brief Class destructor.t.
        */
        virtual ~Preference_Args();
        /** \brief Class initialiser. Must be called just after the object
         * construction. 
         * \param manager The manager in which this object will be registered
         * \return 0 if the object is correctly initialised, otherwise -1. If possible,
         * an error will be set via Smartflow::seterror() to explain the error.
        */
        virtual bool init(PreferenceManager *manager);
        
        /** This method allows to parse the arguments from the command line.
        * The arguments argc and argv come from the main function of the program.
        * The syntaxe is "--sf key=value".
        * @param argc the number of argument passed at the execution of the main class. 
        * @param argv a reference to the arguments passed at the execution of the main class.
        * @return -1 if an error occured, 0 otherwise.
        */
        bool initPreferences(int &argc, char** &argv, bool keepSFOptions=true);
private:
}; // class Preference_Args
}  // namespace SMARTFLOW
#endif
