#ifndef _Preference_XMLFile_H
#define _Preference_XMLFile_H

#include "ace/svc_export.h" // for exported symbols (windows)

#include "Smartflow.h"
#include "Preference.h"
#include <string>

using namespace std;
namespace smartflow {

/** \brief Class Preference_XMLFile .
 * Implements the necessary methods to parse an XML configuration file.
 * The preferences are parsed as a list of key=value, not typed.
 * \author Nist Smartspace Group
 * \date 04-19-2004
 * \internal
*/
class ACE_Svc_Export Preference_XMLFile:public Preference {
/** Redefine Preference (which is the ancestor of Preference_XMLFile) as
    \b super . Facility taken from the Java language.
*/
typedef Preference super;
public:
	/** \brief Class constructor. 
         * \warning This method does not initialise the object, the init() method
         * must be used for this purpose.
        */
	Preference_XMLFile();
        /** \brief Class destructor.
        */
        virtual ~Preference_XMLFile();
        /** \brief Class initialiser. Must be called just after the object
         * construction. 
         * \param manager The manager in which this object will be registered
         * \return 0 if the object is correctly initialised, otherwise -1. If possible,
         * an error will be set via Smartflow::seterror() to explain the error.
        */
        virtual bool init(PreferenceManager *manager);
        
        /** This method allows to read an XML file, in which preferences are stored.
        * The XML file must respect the DTD and its type must be SfPreferences.
   	* The name of the file is passed as an argument. The preferences, which are read 
   	* from the file, are stored in the preferenceManager.
        * @param filename The name of the XML file to read.
        * @return 0 if the file has been correctly read, otherwise -1.
   	*/
  	bool readPreferences(string &filename);
  
  	/** This method writes the preferences of the Smart Data Flow in an XML file.
        * The XML file respects the DTD. 
        * The name of the XML file is passed as an argument.
   	* @param filename The name of the XML file to write.
        * @return 0 if the file has been correctly written, otherwise -1.
   	*/
  	bool writePreferencesToXMLFile(string & filename);
        
	//private:
	//void writePreferences(XmlWriter *xw, QString *key,QString *value, QString *comment); 

}; // class Preference_XMLFile
}  // namespace SMARTFLOW
#endif
