#include "XML_File_Handler.h"

#include <iterator>
#include <iostream> 
#include "Smartflow.h"
#include "PreferenceManager.h"
#include "ErrorManager.h"
#include "Log_Manager_ACE.h"

#include "XML_SaxHandler.h"

using namespace std;

using namespace smartflow;

XML_File_Handler::XML_File_Handler(Preference_XMLFile *pref_)
{
  ACE_TRACE(ACE_TEXT("XML_File_Handler::XML_File_Handler"));
  pref = pref_;
  handler = 0;
}

XML_File_Handler::~XML_File_Handler()
{
	ACE_TRACE(ACE_TEXT("XML_File_Handler::~XML_File_Handler"));
  //delete handler;
}

bool XML_File_Handler::parse(const ACEXML_Char* fname)
{
  ACE_TRACE(ACE_TEXT("XML_File_Handler::parse"));
  bool status = false;
  
  ACEXML_FileCharStream *fstm = 0;

  ACE_NEW_RETURN (fstm, ACEXML_FileCharStream (), false);
  if (fstm->open (fname) != 0) {
    if ( pref ) {
      pref->get_smartflow()->setError(INVALID_PREFERENCE_FILE_XML);
    }
	  return false;
  }

  ACE_NEW_RETURN (handler, XML_SaxHandler(pref), false);
  auto_ptr<ACEXML_DefaultHandler> cleanup_handler (handler);
  

  ACEXML_Parser parser;
  ACEXML_InputSource input(fstm); 
  
  parser.setContentHandler(handler);
  parser.setDTDHandler(handler);
  parser.setErrorHandler(handler);
  parser.setEntityResolver(handler);
  ACEXML_DECLARE_NEW_ENV;

  ACEXML_TRY_EX(FIRST) {
    parser.parse(&input ACEXML_ENV_ARG_PARAMETER);
    ACEXML_TRY_CHECK_EX(FIRST);
  }
  ACEXML_CATCH(ACEXML_Exception, ex) {
    pref->get_smartflow()->setError(INVALID_PREFERENCE_FILE_XML);
    ex.print();
    ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ XML_File_Handler::parse ] Exception occurred. Exiting...\n")));
    status = true;
  }
  ACEXML_ENDTRY;

  return status;
  }

bool XML_File_Handler::write(const ACEXML_Char* fname) {

  ACE_TRACE(ACE_TEXT("XML_File_Handler::write"));
  string head = "<?xml version=\"1.0\"?>\n<!--This file describe the preferences of the SmartFlow-->\n<Preferences>\n";
  string tail = "</Preferences>\n";
  string prefOpenTag = "      <Preference>\n";
  string prefCloseTag = "      </Preference>\n";
  string indent = "                ";
  
  ACE_HANDLE fd = ACE_OS::open(fname,O_CREAT|O_TRUNC|O_WRONLY );
  if ( fd == 0 ) {
    pref->get_smartflow()->setError(INVALID_PREFERENCE_FILE_XML_NOWRITE);
    return false;
  }
  else {
    if ( ACE_OS::write (fd, (void *)((head).c_str()),head.length() ) == -1 ) {
      pref->get_smartflow()->setError(INVALID_PREFERENCE_FILE_XML_NOWRITE);
		return false;
    }
    
    string strKey, strValue, strComment;

    PreferenceManager *pm = pref->get_manager();
    if ( pm != NULL ) {
      for ( int ii=1; ii <= pm->count(); ii++ ) {
	strKey = pm->getPreferenceKey(ii);
	strValue = pm->getPreferenceValue(strKey);
	strComment = pm->getPreferenceComment(strKey);
	
	if ( strKey != "prefFile" ) {
	  if ( ACE_OS::write (fd, (void *)(prefOpenTag.c_str()),prefOpenTag.length() ) == -1 ) {
	    pref->get_smartflow()->setError(INVALID_PREFERENCE_FILE_XML_NOWRITE);
	    return false;
	  }
	  
	  string kk = indent + "<Key>" + strKey + "</Key>\n";
	  string vv = indent + "<Value>" + strValue + "</Value>\n";
	  string cc = indent + "<Comment>" + strComment + "</Comment>\n";
	  
	  if ( ACE_OS::write (fd, (void *)(kk.c_str()),kk.length() ) == -1 ) {
	    pref->get_smartflow()->setError(INVALID_PREFERENCE_FILE_XML_NOWRITE);
	    return false;
	  }
	
	  if ( ACE_OS::write (fd, (void *)(vv.c_str()), vv.length()) == -1) {
	    pref->get_smartflow()->setError(INVALID_PREFERENCE_FILE_XML_NOWRITE);
	    return false;
	  }
	  
	  if ( ACE_OS::write (fd, (void *)(cc.c_str()), cc.length()) == -1) {
	    pref->get_smartflow()->setError(INVALID_PREFERENCE_FILE_XML_NOWRITE);
	    return false;
	  }
	
	  if ( ACE_OS::write (fd, (void *)(prefCloseTag.c_str()),prefCloseTag.length() ) == -1 ) {
	    pref->get_smartflow()->setError(INVALID_PREFERENCE_FILE_XML_NOWRITE);
	    return false;
	  }
	}
      }//end for
    }
    else {
      return false;
    }
    if ( ACE_OS::write (fd, (void *)((tail).c_str()),tail.length() ) == -1 ) {
      pref->get_smartflow()->setError(INVALID_PREFERENCE_FILE_XML_NOWRITE);
      return false;
    }
    
    if ( ACE_OS::close(fd) == -1 ) {
      pref->get_smartflow()->setError(INVALID_PREFERENCE_FILE_XML_NOWRITE);
      return false;
    }
    
    return true;
  }
}

