#include "XML_SaxHandler.h"

#include "ace/Version.h"

#include "Log_Manager_ACE.h"

XML_SaxHandler::XML_SaxHandler(Preference_XMLFile *manager) : manager_(manager) {
	ACE_TRACE(ACE_TEXT("XML_SaxHandler::XML_SaxHandler"));
	key = value = comment = "";
}


XML_SaxHandler::~XML_SaxHandler (void)
{
	ACE_TRACE(ACE_TEXT("XML_SaxHandler::~XML_SaxHandler"));
}

void XML_SaxHandler::startDocument (ACEXML_ENV_SINGLE_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
{
}

void XML_SaxHandler::endDocument (ACEXML_ENV_SINGLE_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
{
}

void XML_SaxHandler::startElement( const ACEXML_Char *uri,const ACEXML_Char *name, const ACEXML_Char *qName, ACEXML_Attributes *alist ACEXML_ENV_ARG_DECL_NOT_USED)
  ACE_THROW_SPEC ((ACEXML_SAXException))
{
  ACE_TRACE(ACE_TEXT("XML_SaxHandler::startElement"));
  ACE_UNUSED_ARG(uri);
  ACE_UNUSED_ARG(qName);
  ACE_UNUSED_ARG(alist);
  state = name ;
 }

void XML_SaxHandler::endElement (const ACEXML_Char * uri, const ACEXML_Char *name, const ACEXML_Char *qName ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
{
  ACE_TRACE(ACE_TEXT("XML_SaxHandler::endElement"));
  state = ACE_TEXT("");
  ACE_UNUSED_ARG(uri);
  ACE_UNUSED_ARG(qName);
  endState = name;
 
  if ( endState == "Preference" ) {
    if ( key != "" && value != "" && comment != "" ) {
      if ( manager_ ) 
	manager_->addPreference(key,value,comment);
      key = value = comment = "";
    }
    else if ( key != "" && value != "" ) {
      if ( manager_ )
	manager_->addPreference(key,value);
      key = value = comment = "";
    }
    else {
      key = value = comment = "";
    }
  }
}

//void XML_SaxHandler::characters (const ACEXML_Char *cdata, size_t start, size_t length ACEXML_ENV_ARG_DECL_NOT_USED ) ACE_THROW_SPEC ((ACEXML_SAXException))
 #if (ACE_MAJOR_VERSION < 5)
  void XML_SaxHandler::characters(const ACEXML_Char *cdata,int start,int length  ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
  #endif    
  
  #if (ACE_MAJOR_VERSION > 5)
  void XML_SaxHandler::characters(const ACEXML_Char *cdata,size_t start,size_t length  ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
  #endif
        
  #if (ACE_MAJOR_VERSION == 5)
    #if (ACE_MINOR_VERSION < 5)
        void XML_SaxHandler::characters(const ACEXML_Char *cdata,int start,int length  ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
    #endif
    #if (ACE_MINOR_VERSION > 5)
        void XML_SaxHandler::characters(const ACEXML_Char *cdata,size_t start,size_t length  ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
    #endif
    #if (ACE_MINOR_VERSION == 5)
        #if (ACE_BETA_VERSION >= 4)
            void XML_SaxHandler::characters(const ACEXML_Char *cdata,size_t start,size_t length  ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
       #else
            void XML_SaxHandler::characters(const ACEXML_Char *cdata,int start,int length  ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException))
       #endif
    #endif
  #endif
{
  ACE_TRACE(ACE_TEXT("XML_SaxHandler::characters"));
  ACE_UNUSED_ARG(start);
  ACE_UNUSED_ARG(length);
  ACE_TString infos = cdata ;
  
  if ( state == "Key" ) 
    key = infos.c_str();
  else if ( state == "Value" ) 
    value = infos.c_str();
  else if ( state == "Comment" )
    comment = infos.c_str();

}

void XML_SaxHandler::fatalError (ACEXML_SAXParseException& ex ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException)) {
  ACE_TRACE(ACE_TEXT("XML_SaxHandler::fatalError"));
  ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ XML_SaxHandler::fatalError ]\n")));
  ex.print();
}

void XML_SaxHandler::error (ACEXML_SAXParseException & ex ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException)) {
  ACE_TRACE(ACE_TEXT("XML_SaxHandler::error"));
  ACE_DEBUG((ERROR_PREFIX ACE_TEXT("[ XML_SaxHandler::error ]\n")));
  ex.print();
}

void XML_SaxHandler::warning (ACEXML_SAXParseException & ex ACEXML_ENV_ARG_DECL_NOT_USED) ACE_THROW_SPEC ((ACEXML_SAXException)) {
  ACE_TRACE(ACE_TEXT("XML_SaxHandler::warning"));
  ACE_DEBUG((WARNING_PREFIX ACE_TEXT("[ XML_SaxHandler::warning ]\n")));
  ex.print();
}

void XML_SaxHandler::ignorableWhitespace (const ACEXML_Char *, int, int  ACEXML_ENV_ARG_DECL_NOT_USED)  ACE_THROW_SPEC ((ACEXML_SAXException))
{
  ACE_TRACE(ACE_TEXT("XML_SaxHandler::ignorableWhitespace"));
  ACE_DEBUG((DEBUG_PREFIX ACE_TEXT("[ XML_SaxHandler::ignorableWhitespace ] Event ignorableWhitespace\n")));
}
