#ifndef SF2D_COMMON_ACE_H
#define SF2D_COMMON_ACE_H

/** This file contains the common definitions between the SF2D Application server 
 * and the Smartflow Library.
 */

// Server defaults
#define SF2D_DEFAULT_LOCAL_PORT 1714
#define SF2D_DEFAULT_LOCAL2_PORT 1716
#define SF2D_DEFAULT_REMOTE_PORT 1713
#define SF2D_DEFAULT_DUPLI_PORT 1715

// XML TYPE
#define REQUEST 1
#define ANSWER 2
#define CONNECT 3
#define DISCONNECT 4
#define ANNOUNCE 5

// XML TARGET 
#define FLOWLIST 10
#define GETFLOW 11
#define REMOVEFLOW 12
#define FLOWARGS 13 
#define SERVERLIST 14
#define LAUNCH 15
#define STOP 16
#define GETLASTMODIFICATIONS 17
#define REMOTEAPPLICATION 18
#define PAUSE 19        // used in Local_Event_Handler, used by the client  
#define RESTART 20      // used in Local_Event_Handler, used by the client  
#define PAUSEFLOW 21   // used in Peer_Event_Handler
#define RESTARTFLOW 22 // used in Peer_Event_Handler
#define DIE 24
#define PAUSE_DUPLI 25 // dupli stop send flow to host 
#define WAKE_UP_FOR_SERVER 26
#define WAKE_UP_FOR_DUPLI  27
#define LAUNCH_ERROR  28
#define OPEN_PROVIDER_PART  29
#define CLOSE_PROVIDER_PART  30
#define CONSUMER_CONNECTED   31
#define NB_DUPLICATOR_CONNECTED 32
#define START_DATA_TRANSFER 33
#define SOCKET_FOR_REMOTE_CONSUMERS_OPENED 34
#define LOG_FROM_CLIENT 35
#define LOG_FROM_DUPLI 36
#define LOG_FROM_SERVER 37

enum duplicator_type_t {
	 CONSUMER = 0,
	 PROVIDER = 1
};	 

enum ClientStatus
{
	// when the client has just been created and has still no defined status
	CLIENT_STATUS_UNKNOWN		= 0,
	
	// when 
	CLIENT_STATUS_WAITING		= 1,
	
	// when the client is connected to the duplicator and willing to receive
	// data from the flow
	CLIENT_STATUS_SWITCHED_ON 	= 2,
	
	// when the client is connected to the duplicator and but not willing to receive
	// data from the flow
	CLIENT_STATUS_SWITCHED_OFF 	= 3
	
};

enum FlowStatus
{
	// when the flow has just been created and has still no defined status
	// or when it has no more consumers and no producer (just before detroying it)
	FLOW_STATUS_UNKNOWN = 0,
	
	// when the producer has left, but there are still consummers
	// or when there is a producer but no consumers (or all consumers are in pause)
	FLOW_STATUS_PENDING = 1,
	
	// when the flow has a producer and at least one consumer not in pause
	FLOW_STATUS_RUNNING	= 2
};

#endif
