#include "ClientDescGen.h"
#include "xmlwriter/xmlwriter.h"
#include "FlowDescriptionDialog.h"

#include <QtCore/QDir>
#include <QtGui/QMessageBox>
#include <QtGui/QFileDialog>
#include <QtXml/QDomDocument>

ClientDescGen::ClientDescGen(QWidget* parent, const char* name, Qt::WFlags fl ) : MainWindowClientDescGen(parent, name, fl) {
	init();
}

ClientDescGen::~ClientDescGen() {}

void ClientDescGen::fileNew()
{
	if ( needSave ) {
		if ( clientFileName.isEmpty() ) {
			int ret = QMessageBox::warning(this, tr("Attention"), 
										   tr("The Client Description File is not saved.\n"
											  "Do you want to save it?"),	
										   QMessageBox::Yes | QMessageBox::Default,
										   QMessageBox::No,
										   QMessageBox::Cancel | QMessageBox::Escape);
			if ( ret == QMessageBox::Yes )
				fileSave();
			else if ( ret == QMessageBox::Cancel )
				return;
		}	
		else {	
			int ret = QMessageBox::warning(this, tr("Attention"), 
										   tr("The Client Description File %1 is not saved.\n"
											  "Do you want to save it?")
										   .arg(QDir::convertSeparators(clientFileName)),
										   QMessageBox::Yes | QMessageBox::Default,
										   QMessageBox::No, 
										   QMessageBox::Cancel | QMessageBox::Escape);
			if (ret == QMessageBox::Yes)
				fileSave();
			else if (ret == QMessageBox::Cancel )
				return;
		}
    }
    clearAllFields();
}


void ClientDescGen::fileOpen()
{
    if ( needSave ) 
		fileSave();
    clearAllFields();
	
	QFileDialog dialog(this, "Open Client Description File", "", "Client files (*.client);;All (*)");
    if ( currentDir != "" )
		dialog.setDirectory( currentDir );
	dialog.setFileMode( QFileDialog::AnyFile );
	
    QString filename;
    if ( dialog.exec() == QDialog::Accepted ) {
		filename = dialog.selectedFile();
		currentDir = dialog.directory().absolutePath();
		if ( !filename.isEmpty() ) {
			if ( !readClientFile(filename))
				QMessageBox:: critical(this, tr("Attention"), 
									   tr("Error while reading the flow description file."),	
									   tr("Ok"));
			else {
				clientFileName = filename;
				needSave = false;
			}
		}
    }
}


void ClientDescGen::fileSave()
{
	if ( clientFileName.isEmpty() )
		fileSaveAs();
    else {	
		if ( !writeClientFile(clientFileName) ) {
			QMessageBox:: critical(this, tr("Attention"), 
								   tr("Error while saving the flow description file."),	
								   tr("Ok"));
		}
		else
			needSave = false;
	}	
}


void ClientDescGen::fileSaveAs()
{
	QFileDialog dialog(this, "Save Client Description File", "", "Client files (*.client);;All (*)");
    if ( currentDir != "" )
		dialog.setDirectory( currentDir );
	dialog.setFileMode( QFileDialog::AnyFile );
	dialog.setAcceptMode ( QFileDialog::AcceptSave );
	QString filename;
    if ( dialog.exec() == QDialog::Accepted ) {
		filename = dialog.selectedFile();
		currentDir = dialog.directory().absolutePath();
		if ( !filename.isEmpty() ) {
			QRegExp rxp( "\\.client", true, false );
			bool nb = filename.contains( rxp );
			if ( nb == false )
				filename = filename.append(".client");
			clientFileName = filename;
			if ( !writeClientFile(clientFileName) ) {
				QMessageBox:: critical(this, tr("Attention"), 
									   tr("Error while saving the flow description file."),	
									   tr("Ok"));
			}
			else
				needSave = false;
		}
    }
}

void ClientDescGen::fileExit()
{
    if ( needSave ) {
		if ( clientFileName.isEmpty() ) {
			int ret = QMessageBox::warning(this, tr("Attention"), 
										   tr("The Client Description File is not saved.\n"
											  "Do you want to save it?"),	
										   QMessageBox::Yes | QMessageBox::Default,
										   QMessageBox::No,
										   QMessageBox::Cancel | QMessageBox::Escape);
			if ( ret == QMessageBox::Yes )
				fileSave();
			else if ( ret == QMessageBox::Cancel )
				return;
		}	
		else {	
			int ret = QMessageBox::warning(this, tr("Attention"), 
										   tr("The Client Description File %1 is not saved.\n"
											  "Do you want to save it?")
										   .arg(QDir::convertSeparators(clientFileName)),
										   QMessageBox::Yes | QMessageBox::Default,
										   QMessageBox::No, 
										   QMessageBox::Cancel | QMessageBox::Escape);
			if (ret == QMessageBox::Yes)
				fileSaveAs();
			else if (ret == QMessageBox::Cancel )
				return;
		}
    }
    hide();
}


void ClientDescGen::browsePath()
{
	QFileDialog dialog(this, "Select Client Executable File", "", "Client Executable file (*)");
    if ( dialog.exec() == QDialog::Accepted ) {
		clientPath = dialog.selectedFile();
		if ( !clientPath.isEmpty() ) {   
			ClientPathLineEdit->setText(clientPath);
		}
    }
}


void ClientDescGen::checkClientName()
{
    needSave = true;
    if ( (ClientNameLineEdit->text()).isEmpty() ) {
		clientName = "";
		clientNameOK = false;
    }	
    else {
		clientName = ClientNameLineEdit->text();
		clientNameOK = true;
    }
    enableSave();
}


void ClientDescGen::init()
{
    clientPath = "";
    clientName = "";
    clientNameOK = false;
    clientPathOK = false;
    clientFileName = "";
    needSave = false;
    currentDir = "";
    
    QRegExp rxPathAndClientName( "\\w+$" );
    QValidator* validator = new QRegExpValidator( rxPathAndClientName, this );
    ClientNameLineEdit->setValidator( validator );
    
    enableSave();
    
    
    DeleteFlowButton->setEnabled(false);
    EditFlowButton->setEnabled(false);
    
}


void ClientDescGen::checkClientPath()
{
    needSave = true;
    if ( (ClientPathLineEdit->text()).isEmpty() ) {
		clientPath = "";
		clientPathOK = false;
    }	
    else {
		clientPath = ClientPathLineEdit->text();
		clientPathOK = true;
    }
    enableSave();
}


void ClientDescGen::enableSave()
{
    if ( clientPathOK && clientNameOK ) {
		fileSaveAction->setEnabled(true);
		fileSaveAsAction->setEnabled(true);
    }
    else {
		fileSaveAction->setEnabled(false);
		fileSaveAsAction->setEnabled(false);
    }
}

void ClientDescGen::closeEvent(QCloseEvent *e) {
    Q_UNUSED(e);
	fileExit();
}



bool ClientDescGen::readClientFile(QString fileName)
{
    if ( fileName.isEmpty() ) {
		return false;
    }	
    QFile f(fileName);
    QDomDocument myDoc;
    if ( !f.open( QIODevice::ReadOnly ) ) { 
		return false;
    }	
    if ( !myDoc.setContent( &f ) ) {
		f.close(); 	
		return false;
    }
    f.close();
    QString type = myDoc.doctype().name();
    if ( type != "clientDescription" ) {
		return false;
    }
    else {
		QString tagname = "";
		QDomElement root = myDoc.documentElement(); 
		QDomElement child = root.firstChild().toElement();
		tagname = child.tagName();
		if ( tagname == "client" ) {
			QDomElement smallchild = child.firstChild().toElement();
			QString tagname3 = "";
			int i = 0;
			QString theName = "";
			QString thePath = "";
			//how are we gonna store the flows?
			
			while( !smallchild.isNull() ) {
				tagname3 = smallchild.tagName();
				if ( tagname3 == "name" ) {
					theName = smallchild.text();
					i++;
				}
				if ( tagname3 == "path" ) {
					thePath = smallchild.text();
					i++;
				}
				if ( tagname3 == "flows" ) {
					i++;
					QDomElement smallsmallchild = smallchild.firstChild().toElement();
					QString tagname4 = "";
					while( !smallsmallchild.isNull() ) {
						tagname4 = smallsmallchild.tagName();
						if ( tagname4 == "input" ) {
							QDomElement smallsmallsmallchild = smallsmallchild.firstChild().toElement();
							QString tagname5 = "";
							int j = 0;
							QString flowName = "";
							QString flowType = "";
							while( !smallsmallsmallchild.isNull() ) {
								tagname5 = smallsmallsmallchild.tagName();
								if ( tagname5 == "name" ) {
									flowName = smallsmallsmallchild.text();
									j++;
								}
								if ( tagname5 == "type" ) {
									flowType = smallsmallsmallchild.text();
									j++;
								}
								if ( !flowName.isEmpty() && !flowType.isEmpty() && (j == 2) ) {
									j = 0;
									//we need to store the flow somewhere or use the listview
									addFlowInListView(flowName, flowType, 0);
									//cout << "We have an INPUT flow" << endl;
									flowName = "";
									flowType = "";
								}
								smallsmallsmallchild = smallsmallsmallchild.nextSibling().toElement();
							}
						}
						if ( tagname4 == "output" ) {
							QDomElement smallsmallsmallchild = smallsmallchild.firstChild().toElement();
							QString tagname5 = "";
							int j = 0;
							QString flowName = "";
							QString flowType = "";
							while( !smallsmallsmallchild.isNull() ) {
								tagname5 = smallsmallsmallchild.tagName();
								if ( tagname5 == "name" ) {
									flowName = smallsmallsmallchild.text();
									j++;
								}
								if ( tagname5 == "type" ) {
									flowType = smallsmallsmallchild.text();
									j++;
								}
								if ( !flowName.isEmpty() && !flowType.isEmpty() && (j == 2) ) {
									j = 0;
									//we need to store the flow somewhere or use the listview
									addFlowInListView(flowName, flowType, 1);
									//cout << "We have an INPUT flow" << endl;
									flowName = "";
									flowType = "";
								}
								smallsmallsmallchild = smallsmallsmallchild.nextSibling().toElement();
							}
						}
						smallsmallchild = smallsmallchild.nextSibling().toElement(); 
					}
				}// if ( tagname3 == "flows" ) {
				if ( !theName.isEmpty() && !thePath.isEmpty() && ( i > 2 ) ) {
					//we connect the flows to the client and add the client to the configuration
					i = 0;
					//we are good at this point the description is read from the file
					ClientNameLineEdit->setText(theName);
					ClientPathLineEdit->setText(thePath);
					//cout << "CLIENT READ" << endl;
					return true;
		    	    
				}
				smallchild = smallchild.nextSibling().toElement();
				}//while( !smallchild.isNull() ) {
				}
			}
    return false;
		}





bool ClientDescGen::writeClientFile( QString &fileName )
{
    QFile file(fileName);
    if ( !file.open( QIODevice::WriteOnly ) ) {
		return false;
    }		
    else {
		XmlWriter *xw;
		xw = new  XmlWriter(&file);
		xw->setAutoNewLine(true);
		xw->writeRaw("<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>");
		xw->newLine();
		xw->writeRaw("<!DOCTYPE clientDescription SYSTEM \"clientDesc.dtd\">");
		xw->newLine();
		xw->writeRaw("<!--This file describe a NSDFS client-->");
		xw->newLine();
		xw->writeOpenTag("smartspace");
		xw->writeOpenTag("client");
		xw->writeTaggedString("name", ClientNameLineEdit->text());
		xw->writeTaggedString("path", ClientPathLineEdit->text());
		xw->writeOpenTag("flows");
		QTreeWidgetItemIterator iter(flowListView);
		while (*iter) {
			xw->writeOpenTag((*iter)->text(1));
			xw->writeTaggedString("name", (*iter)->text(0));
			xw->writeTaggedString("type", (*iter)->text(2));	
			xw->writeCloseTag((*iter)->text(1));
			++iter;
		}
		xw->writeCloseTag("flows");
		xw->writeCloseTag("client");
		xw->writeCloseTag("smartspace");
		file.close();
		delete xw;
    }	
    return true;	
}


void ClientDescGen::clearAllFields()
{	
    ClientNameLineEdit->clear();
    ClientPathLineEdit->clear();
    flowListView->clear();
    needSave = false;
    clientFileName = "";
}


void ClientDescGen::addFlow()
{
    FlowDescriptionDialog *dialog = new FlowDescriptionDialog(this);
    if ( dialog->exec() == QDialog::Accepted ) {
		//cout << "dialog accepted" << endl;
		int direction  = -1;
		QString flowName = "";
		QString flowType = "";
		flowName =  dialog->FlowNameLineEdit->text();
		flowType = dialog->FlowTypeComboBox->currentText ();
		if ( dialog->InputRadioButton->isChecked() )
			direction = 0;
		else 
			direction = 1;
		//cout << "Result : FlowName = " <<  flowName << " FlowType=" << flowType << " Dir = " << direction << endl;
		//we gather the different information from the dialog and add a line in the listView
		addFlowInListView(flowName, flowType, direction);
		needSave = true;
    }
    delete dialog;
}


void ClientDescGen::deleteFlow()
{
    if ( flowListView->currentItem() != NULL )
		flowListView->takeTopLevelItem(flowListView->indexOfTopLevelItem(flowListView->currentItem()));
    if ( flowListView->topLevelItemCount() == 0 )
		disableDeleteAndEditButtons();
}


void ClientDescGen::editFlow()
{
    QTreeWidgetItem *theItem = flowListView->currentItem();
    if ( theItem != NULL  ) { 
		FlowDescriptionDialog *dialog = new FlowDescriptionDialog(this);
		dialog->FlowNameLineEdit->setText(theItem->text(0));
		dialog->setBaseName(theItem->text(0));
		if ( theItem->text(1) == "input" ) {
			(dialog->InputRadioButton)->setChecked(true);
			(dialog->OutputRadioButton)->setChecked(false);
		}
		else {
			(dialog->InputRadioButton)->setChecked(false);
			(dialog->OutputRadioButton)->setChecked(true);
		}
		dialog->FlowTypeComboBox->setCurrentText(theItem->text(2));  
		if ( dialog->exec() == QDialog::Accepted ) {
			theItem->setText(0, dialog->FlowNameLineEdit->text());
			if ( dialog->InputRadioButton->isChecked() )
				theItem->setText(1, "input");
			else 
				theItem->setText(1, "output");
			theItem->setText(2, dialog->FlowTypeComboBox->currentText() );
		}
    }	
}


void ClientDescGen::addFlowInListView(QString flowName, QString flowType, int direction)
{
    QString dir;
    if ( direction == 0 )
		dir = "input";
    else
		dir = "output";
	QStringList sList;
	sList << flowName << dir << flowType;
	QTreeWidgetItem *item = new QTreeWidgetItem(flowListView, sList);
	if ( direction == 0 )
		item->setIcon(0, QIcon(":/images/FlowIn.xpm"));
	else
		item->setIcon(0, QIcon(":/images/FlowOut.xpm"));
}


bool ClientDescGen::flowNameExists(QString name)
{
	QTreeWidgetItemIterator iter(flowListView);
	while (*iter) {
		if ( (*iter)->text(0) == name )	
			return true;
		++iter;
	}
	return false;
}


void ClientDescGen::enableDeleteAndEditButtons()
{
    DeleteFlowButton->setEnabled(true);
    EditFlowButton->setEnabled(true);
}


void ClientDescGen::disableDeleteAndEditButtons()
{
    DeleteFlowButton->setEnabled(false);
    EditFlowButton->setEnabled(false);
}


void ClientDescGen::helpAbout()
{
    QMessageBox::about (this, QMessageBox::tr("About the Client Description Generator"), QMessageBox::tr("NSDFS Client Description Generator.\nSmart Space Group.\nInformation Access Division.\nNIST.") );
}


void ClientDescGen::helpOverview()
{
    QString message = "The Client Description Generator has been designed to help using Smartflow client nodes from the Control Center.\nEach client has a name, a path and flow(s). Each one of these flows has a name, a type and a direction (input or output).\nThe NSDFS Control Center needs a complete description of client nodes in order to connect them together, to launch them or to stop them.\nThis description is described in an XML file, which is read by the NSDFS Control Center.\nThe goal of the Client Description Generator is to generate these XML description files.";
    QMessageBox::about (this, QMessageBox::tr("Overview of the Client Description Generator"), QMessageBox::tr(message) );
}
