#include "AddClientTool.h"
#include "mainwindow.h"

#include <QtCore/QVariant>
#include <QtGui/QVBoxLayout>
#include <QtGui/QHBoxLayout>
#include <QtGui/QGridLayout>
#include <QtGui/QSpacerItem>
#include <QtGui/QWidget>
#include <QtGui/QToolButton>
#include <QtGui/QPushButton>
#include <QtGui/QToolBox>
#include <QtGui/QToolTip>
#include <QtGui/QWhatsThis>
#include <QtGui/QButtonGroup>
#include <QtGui/QCursor>
#include <QtGui/QPixmap>

#include <iostream>

using namespace std;

/*
 *  Constructs a AddClientDialogBase as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
AddClientTool::AddClientTool( QWidget *parent, const char* name, bool modal, Qt::WFlags fl )
: AddClientToolBase( parent, name, modal, fl )
{
	current = 0;
	buttonList.clear();
	connect( this, SIGNAL(deleteSignal()), parentWidget(), SLOT(deleteAddClientTool() ) );
}

/*
 *  Destroys the object and frees any allocated resources
 */
AddClientTool::~AddClientTool()
{
	//here we changed the shape of the mouse
	// no need to delete child widgets, Qt does it all for us
	if ( parentWidget() != NULL ) {
		parentWidget()->setCursor(QCursor(Qt::ArrowCursor));
	}
}


void AddClientTool::unToogleButtons()
{
	std::set<ClientToolButton *>::iterator iter;
	for ( iter = buttonList.begin(); iter != buttonList.end(); iter++ ) {
		if ( (*iter) != current )
			(*iter)->setOn(false);
	}
}

void  AddClientTool::initToolButton() {
	deleteButtonList();
	std::map<unsigned int, ClientConf *>* clientList = MainWindow::instance()->global->getClientConfList()->getClientList();
	std::map<unsigned int, ClientConf *>::iterator iter;
	for ( iter = clientList->begin(); iter != clientList->end(); iter++ ) {
		ClientToolButton *c = new ClientToolButton(iter->second, clientListTab, iter->second->getName());
		connect( c, SIGNAL( clicked(ClientToolButton *) ), this, SLOT(setCurrentClient(ClientToolButton *) ) );
		connect( c, SIGNAL( clicked() ), c, SLOT(emitTheClick()) );
		//buttonLayout->addWidget( c );
		//vboxLayout3->addWidget( c );
		vboxLayout2->addWidget( c );
		buttonList.insert(c);
	}
	//newly added
	adjustSize();
}

void AddClientTool::setCurrentClient(ClientToolButton *c) {
	if ( c && ( c != current ) ) {
		current = c;
		unToogleButtons();
		//here we changed the shape of the mouse
		parentWidget()->setCursor(QCursor(Qt::CrossCursor)); 
	}
	else {
		current = NULL;
		//here we changed the shape of the mouse
		parentWidget()->setCursor(QCursor(Qt::ArrowCursor)); 
	}
}

void AddClientTool::deleteButtonList() {
	std::set<ClientToolButton *>::iterator iter;
	for ( iter = buttonList.begin(); iter != buttonList.end(); iter++ ) {
		// vboxLayout3->remove(*iter);
		vboxLayout2->remove(*iter);
		//buttonLayout->remove(*iter);
		delete (*iter);
	}
	buttonList.clear();
}

void AddClientTool::reject() {
	QDialog::reject();
	emit deleteSignal();
}


ClientToolButton::ClientToolButton(ClientConf *cc, QWidget * parent, const char * name) : QToolButton(parent, name) {
	model = cc; 
	setPaletteBackgroundColor( QColor( 255, 255, 255 ) );
	setToggleButton( TRUE );
	setIconSet( QIcon( ":/images/RegClient.xpm"));
	setUsesTextLabel( TRUE );
	setAutoRaise( TRUE );
	setTextPosition( QToolButton::BesideIcon );
	setContentsMargins(0,0,0,0);
	setTextLabel( model->getName() );
}

ClientConf* ClientToolButton::getModel() {
	return model;
}

void ClientToolButton::emitTheClick() {
	emit clicked(this);
}


