#include "ApplicationMonitor.h"
#include "mainwindow.h"
#include "model/GlobalCC.h"
#include "model/Server.h"
#include "model/Client.h"
#include "model/Flow.h"

#include <QtCore/QString>
#include <QtCore/QList>
#include <QtGui/qtabwidget.h>
#include <QtGui/QPixmap>

#include <map>

ApplicationMonitor::ApplicationMonitor(QWidget* parent, const char* name, Qt::WFlags fl) : ApplicationMonitorBase(parent, name, fl) {
    clearAllView();
    serverListView->setRootIsDecorated ( true );
    clientListView->setRootIsDecorated ( true );
    flowListView->setRootIsDecorated ( false );
    clearAllView();
}

ApplicationMonitor::~ApplicationMonitor() {}

void ApplicationMonitor::refreshServerView() {
    clearServerView();
    std::map<unsigned int, ClientApp *>*clients = MainWindow::instance()->global->getApplication()->getClientList();
    std::map<unsigned int, ClientApp *>::iterator iter;
    std::multimap<Server *, ClientApp *>::iterator iter_sercli;
    std::multimap<Server *, ClientApp *> sercli;
    sercli.clear();
	
	QList<QTreeWidgetItem *> items;
	
    for ( iter = clients->begin(); iter != clients->end(); iter++) {
		Server *s = iter->second->getServer();
		if ( s->getAddress() != "No Server" )
			sercli.insert(std::make_pair(s, iter->second));
    }
    
    std::multimap<Server *, ClientApp *>::iterator iter_sercli2;
    Server *oldServer = 0;
    for ( iter_sercli = sercli.begin(); iter_sercli != sercli.end(); iter_sercli++) {
		Server *s = iter_sercli->first;
		if ( s != oldServer ) { 
			QTreeWidgetItem *item = new QTreeWidgetItem(serverListView, QStringList(iter_sercli->first->getName()));
			item->setIcon(0, QIcon(":/images/RegServer.xpm"));
			for ( iter_sercli2 = sercli.lower_bound(s); iter_sercli2 != sercli.upper_bound(s); iter_sercli2++ ) {
				QTreeWidgetItem *tmp_item = new QTreeWidgetItem(item, QStringList(iter_sercli2->second->getName()));
				tmp_item->setIcon(0, QIcon(":/images/RegClient.xpm"));
				item->addChild(tmp_item);
			}
			items.append(item);
		}
		oldServer = s;
    }
	serverListView->insertTopLevelItems(0, items);
}

void ApplicationMonitor::refreshClientView() {
    clearClientView();
    std::map<unsigned int, ClientApp *>*clients = MainWindow::instance()->global->getApplication()->getClientList();
    std::map<unsigned int, ClientApp *>::iterator iter;
	
	QList<QTreeWidgetItem *> items;
	
    for ( iter = clients->begin(); iter != clients->end(); iter ++) {
		QTreeWidgetItem *item = new QTreeWidgetItem(clientListView, QStringList(iter->second->getName()));
		item->setIcon(0, QIcon(":/images/RegClient.xpm"));
		
		std::map<unsigned int, FlowOutletIn*>* foin = iter->second->getInputFlowList();
		std::map<unsigned int, FlowOutletOut*>* foout = iter->second->getOutputFlowList(); 
		std::map<unsigned int, FlowOutletIn*>::iterator iter_flow_in;
		std::map<unsigned int, FlowOutletOut*>::iterator iter_flow_out;
		for ( iter_flow_in = foin->begin(); iter_flow_in != foin->end(); iter_flow_in++) {
			QTreeWidgetItem *tmp = new QTreeWidgetItem(item, QStringList(iter_flow_in->second->getName()));
			tmp->setIcon(0, QIcon(":/images/FlowIn.xpm"));
			item->addChild(tmp);
		}
		for ( iter_flow_out = foout->begin(); iter_flow_out != foout->end(); iter_flow_out++) {
			QTreeWidgetItem *tmp = new QTreeWidgetItem(item, QStringList(iter_flow_out->second->getName()));
			tmp->setIcon(0, QIcon(":/images/FlowOut.xpm"));
			item->addChild(tmp);
		}
		
		items.append(item);
    }
	clientListView->insertTopLevelItems(0, items);
}

void ApplicationMonitor::refreshFlowView() {
    clearFlowView();
	std::map<unsigned int, FlowOutlet *>*flows = MainWindow::instance()->global->getApplication()->getFlowList();
	std::map<unsigned int, FlowOutlet *>::iterator iter;
	QList<QTreeWidgetItem *> items;
	for ( iter = flows->begin(); iter != flows->end(); iter++) {
		QStringList sList;
		sList << iter->second->getName() << iter->second->getDataType() << iter->second->getGroup();
		QTreeWidgetItem *item = new QTreeWidgetItem(flowListView, sList);
		item->setIcon(0, QIcon(":/images/RegClient.xpm"));
		
		if ( item ) {
			if ( iter->second->getType() == FlowOutlet::FLOW_IN )
				item->setIcon(0, QIcon(":/images/FlowIn.xpm"));
			else
				item->setIcon(0, QIcon(":/images/FlowOut.xpm"));
			items.append(item);
		}
    }
	flowListView->insertTopLevelItems(0, items);	
}

void ApplicationMonitor::refreshAllView() {
    refreshServerView();
    refreshClientView();
    refreshFlowView();
}

void ApplicationMonitor::clearAllView() {
    clearFlowView();
    clearServerView();
    clearClientView();
}

void ApplicationMonitor::clearFlowView() {
    flowListView->clear();
}

void ApplicationMonitor::clearServerView() {
    serverListView->clear();
}

void ApplicationMonitor::clearClientView() {
	clientListView->clear(); 
}
