#include "ClientEditor.h"
#include "model/Flow.h"
#include "model/GlobalCC.h"
#include "mainwindow.h"

#include <QtGui/QLineEdit>
#include <QtGui/QPushButton>
#include <QtGui/QIcon>
#include <QtGui/QFileDialog>

#include <map>

using namespace std;

ClientEditor::ClientEditor( QWidget* parent, const char* name, bool modal, Qt::WFlags fl) : ClientEditorBase(parent, name, modal, fl) {
    clientApproved = false;
}

ClientEditor::~ClientEditor() {}

void ClientEditor::accept() {
	
    QDialog::accept();
}

void ClientEditor::browseClient() { 
	QFileDialog dialog(this, "Open Client Description file", "", "Client Description file (*.client);;All (*)");
    QString clientname;
    if ( dialog.exec() == QDialog::Accepted ) {
		clientname = dialog.selectedFile();
		if ( !clientname.isEmpty() ) {   
			ClientPathLabel->setText(clientname);
			checkClient();
			
		}
    }
}

void ClientEditor::enableOKButton(){
    if ( clientApproved )
		ClientOkButton->setEnabled(true);
    else
		ClientOkButton->setEnabled(false);
}

void ClientEditor::checkClient() {
    QString theFileName = ClientPathLabel->text();
    tmp_client = MainWindow::instance()->global->getClientConfList()->readClientFromXMLFile(theFileName);
    ClientNameLabel->clear();
    ClientEmitsFlowListView->clear();
    ClientReceivesFlowListView->clear();
    //we need to check if we have a valid client, if yes clientApproved is set to TRUE
    if ( tmp_client != NULL ) {
		ClientNameLabel->setText(tmp_client->getName());
		//the we iter on the flows
		std::map<unsigned int, FlowOutletIn*>* inputFlowList =  tmp_client->getInputFlowList();
		std::map<unsigned int, FlowOutletIn*>::iterator iter_flowIn;
		QList<QTreeWidgetItem *> items_in;
		QList<QTreeWidgetItem *> items_out;
		for ( iter_flowIn = inputFlowList->begin(); iter_flowIn != inputFlowList->end(); iter_flowIn++ ) {
			FlowOutletIn *flowIn = (FlowOutletIn *) iter_flowIn->second;
			QStringList sList;
			sList << flowIn->getName() << flowIn->getDataType();
			QTreeWidgetItem *item = new QTreeWidgetItem(ClientReceivesFlowListView, sList);
			item->setIcon(0, QIcon(":/images/FlowIn.xpm"));
			items_in.append(item);
		}
		ClientReceivesFlowListView->insertTopLevelItems(0, items_in);
		
		std::map<unsigned int, FlowOutletOut*>* outputFlowList =  tmp_client->getOutputFlowList();
		std::map<unsigned int, FlowOutletOut*>::iterator iter_flowOut;
		for ( iter_flowOut = outputFlowList->begin(); iter_flowOut != outputFlowList->end(); iter_flowOut++ ) {
			FlowOutletOut *flowOut = (FlowOutletOut *) iter_flowOut->second;
			QStringList sList;
			sList << flowOut->getName() << flowOut->getDataType();
			QTreeWidgetItem *item = new QTreeWidgetItem(ClientEmitsFlowListView, sList);
			item->setIcon(0, QIcon(":/images/FlowOut.xpm"));
			items_out.append(item);
		}
		ClientEmitsFlowListView->insertTopLevelItems(0, items_out);
		clientApproved = true;
		
    }
    else {
	  clientApproved = false;
	  ClientPathLabel->clear();
	  QMessageBox::warning (this, QMessageBox::tr("Attention"), QMessageBox::tr("The client description file is not valid.\n") );
    }
    enableOKButton();
}

void ClientEditor::notApproved() {
    clientApproved = false;
    enableOKButton();
}
