#include "ConfigurationClientList.h"
#include "ClientEditor.h"
#include "ClientPathEditor.h"
#include "model/Client.h"
#include "mainwindow.h"

#include <QtCore/QVariant>
#include <QtCore/QString>
#include <QtCore/QMimeData>
#include <QtGui/QToolTip>
#include <QtGui/QWhatsThis>
#include <QtGui/QPixmap>
#include <QtGui/QMenu>
#include <QtGui/QPushButton>
#include <QtGui/QLineEdit>
#include <QtGui/QDrag>
#include <QtGui/QMessageBox>
#include <QtGui/QHBoxLayout>
#include <QtGui/QMouseEvent>
#include <QtGui/QVBoxLayout>

#include <map>
#include <set>
#include <iostream>

using namespace std;


/*
 *  Constructs a ConfigurationClientList as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 */
ConfigurationClientList::ConfigurationClientList(QWidget* parent, const char* name, Qt::WFlags fl) : QWidget( parent, name, fl ) {
	if ( !name )
		setName( tr("ConfigurationClientListBase") );
	setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)5, (QSizePolicy::SizeType)5, 0, 0, sizePolicy().hasHeightForWidth() ) );
	ConfigurationClientListBaseLayout = new QHBoxLayout( this, 0, 6, "ConfigurationClientListBaseLayout"); 
	
	layout29 = new QVBoxLayout( 0, 0, 6, "layout29"); 
	
	ClientListView = new ClientConfListView( this);
	layout29->addWidget( ClientListView );
	
	layout28 = new QHBoxLayout( 0, 0, 6, "layout28"); 
	
	AddClientButton = new QPushButton( this, "AddClientButton" );
	layout28->addWidget( AddClientButton );
	spacer8 = new QSpacerItem( 31, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	layout28->addItem( spacer8 );
	
	DeleteClientButton = new QPushButton( this, "DeleteClientButton" );
	DeleteClientButton->setEnabled( FALSE );
	layout28->addWidget( DeleteClientButton );
	layout29->addLayout( layout28 );
	ConfigurationClientListBaseLayout->addLayout( layout29 );
	languageChange();
	resize( QSize(334, 362).expandedTo(minimumSizeHint()) );
	//clearWState( WState_Polished );
	
	// signals and slots connections
	//customContextMenuRequested(const QPoint&)	
	ClientListView->setContextMenuPolicy(Qt::CustomContextMenu);
	connect( ClientListView, SIGNAL( customContextMenuRequested(const QPoint&) ), this, SLOT( contMenu(const QPoint&) ) );
	connect( AddClientButton, SIGNAL( clicked() ), this, SLOT( addClient() ) );
	connect( DeleteClientButton, SIGNAL( clicked() ), this, SLOT( deleteClient() ) );
	connect( ClientListView, SIGNAL(itemClicked (QTreeWidgetItem *, int)), this, SLOT( enableButtons(QTreeWidgetItem*) ) );
	
	// tab order
	setTabOrder( ClientListView, AddClientButton );
	setTabOrder( AddClientButton, DeleteClientButton );
}


/*
 *  Destroys the object and frees any allocated resources
 */
ConfigurationClientList::~ConfigurationClientList(){}





void ConfigurationClientList::disableButtons() {
    DeleteClientButton->setEnabled(true);
}

void ConfigurationClientList::refreshClientConfList() {
    ClientListView->clear();
    std::map<unsigned int, ClientConf *>* clientList = MainWindow::instance()->global->getClientConfList()->getClientList();
    std::map<unsigned int, ClientConf *>::iterator iter;
	QList<QTreeWidgetItem *> items;
    for ( iter = clientList->begin(); iter != clientList->end(); iter++ ) {
		iter->second->initTreeWidgetItem(ClientListView);
		items.append(iter->second->getClientConfViewItem());
    }
	if ( items.count() != 0 ) 
		ClientListView->insertTopLevelItems(0, items);
	else {
		ClientListView->takeTopLevelItem(0);
		DeleteClientButton->setEnabled(false);
	}
}

void ConfigurationClientList::contMenu(const QPoint & pos ) {
    QMenu *menu = new QMenu();
    menu->popup(QCursor::pos());
    menu->insertItem(tr("&Add Client..."), this, SLOT(addClient()));
    ClientConfItem *cci = dynamic_cast<ClientConfItem *>(ClientListView->itemAt(pos));
    if ( cci != NULL ) {
		menu->insertItem(tr("&Delete Client"), this, SLOT(deleteClient()));
		menu->insertSeparator();
		menu->insertItem(tr("&Edit Client..."), this, SLOT(editClient()));
    }    
}

void ConfigurationClientList::addClient(){
    ClientEditor *editor = new ClientEditor(this);
    if ( editor->exec()== QDialog::Accepted ) {
		ClientConf *cc = editor->tmp_client;
		if ( cc != 0  && !MainWindow::instance()->global->getClientConfList()->clientPathExists(cc->getPath()) ) {
			cc->initTreeWidgetItem(ClientListView);
			MainWindow::instance()->global->getClientConfList()->addClient(cc);
			MainWindow::instance()->configNeedSave = true;
		}
    }
    delete editor;
}	

void ConfigurationClientList::deleteClient(){
    ClientConfItem *cci = dynamic_cast<ClientConfItem *>(ClientListView->currentItem());
    if ( cci ) {
		ClientConf *cc = cci->getModel();
		if ( cc ) {
			if ( clientExistsInApplication(cc) )
				QMessageBox::about (this, QMessageBox::tr("Attention"), QMessageBox::tr("You cannot delete a client present in the Map.") );
			else {
				MainWindow::instance()->global->getClientConfList()->removeClient(cc);
				MainWindow::instance()->configNeedSave = true;
				delete cci;
				refreshClientConfList();
				if ( ClientListView->topLevelItemCount ()  == 0 )
					DeleteClientButton->setEnabled(false);
			}
		}	
    }	
}	

void ConfigurationClientList::editClient(){
	ClientConfItem *cci = dynamic_cast<ClientConfItem *>(ClientListView->currentItem());
	if ( cci ) {
		ClientConf *cc = cci->getModel();
		if ( cc ) {
			if ( !clientRunningInApplication(cc) ) {
				ClientPathEditor *editor = new ClientPathEditor(this, cc);
				if ( editor->exec() == QDialog::Accepted ) {
					QString tmp_newPath = editor->getNewPath();
					if ( cc->getPath() != tmp_newPath ) {
						changeClientPathInApplication(cc, tmp_newPath);
						cc->setPath(tmp_newPath);
						MainWindow::instance()->configNeedSave = true;
						MainWindow::instance()->appliNeedSave = true;
					}
				}
				delete editor;
			}
			else {
				QMessageBox::about (this, QMessageBox::tr("Attention"), QMessageBox::tr("You cannot edit a running client present in the Map.") );
			}
		}
	}
}

void ConfigurationClientList::enableButtons(QTreeWidgetItem *item) {
    if ( item ) {
		ClientConfItem *cci = dynamic_cast<ClientConfItem *> (item);
		if ( cci )
			DeleteClientButton->setEnabled(true);
		else
			DeleteClientButton->setEnabled(false);
    }
    else 
		DeleteClientButton->setEnabled(false);
}

void ConfigurationClientList::languageChange()
{
    setCaption( tr( "Client List" ) );
    AddClientButton->setText( tr( "Add..." ) );
    DeleteClientButton->setText( tr( "Delete" ) );
}

bool ConfigurationClientList::clientExistsInApplication(ClientConf *cc) {
    std::map<unsigned int, ClientApp *> *clientList = MainWindow::instance()->global->getApplication()->getClientList();
    std::map<unsigned int, ClientApp *>::iterator iter;
    iter = clientList->begin();
    bool found = false;
    while ( iter != clientList->end() && !found  ) {
		if ( iter->second->getPath() == cc->getPath() )
			found = true;
		iter++;
    }
    return found;
}

bool ConfigurationClientList::clientRunningInApplication(ClientConf *cc) {
	std::map<unsigned int, ClientApp *> *clientList = MainWindow::instance()->global->getApplication()->getClientList();
	std::map<unsigned int, ClientApp *>::iterator iter;
	iter = clientList->begin();
	bool found = false;
	while ( iter != clientList->end() && !found  ) {
		if ( iter->second->getPath() == cc->getPath() )
			if ( iter->second->isRunning() ||  iter->second->isRunning() )
				found = true;
		iter++;
	}
	return found;
}

void ConfigurationClientList::changeClientPathInApplication(ClientConf *cc, QString &newPath) {
	std::map<unsigned int, ClientApp *> *clientList = MainWindow::instance()->global->getApplication()->getClientList();
	std::map<unsigned int, ClientApp *>::iterator iter;
	iter = clientList->begin();
	for ( std::map<unsigned int, ClientApp *>::iterator iter = clientList->begin(); iter != clientList->end(); iter++ ) {
		if ( iter->second->getPath() == cc->getPath() )
			iter->second->setPath(newPath);
	}
}
/**********************************************************************/

ClientConfListView::ClientConfListView(QWidget *parent) : QTreeWidget(parent) {
	QStringList theLabel;
	theLabel << "Name";
	setHeaderLabels(theLabel);
	header()->setMovable(false);
	header()->setResizeMode(0, QHeaderView::Interactive);
    setRootIsDecorated ( true );
}

ClientConfListView::~ClientConfListView() {}

void ClientConfListView::mousePressEvent ( QMouseEvent * event ) {
	if ( event->button() == Qt::LeftButton )
		dragPos = event->pos();	
	QTreeWidget::mousePressEvent(event);
}

void ClientConfListView::mouseMoveEvent(QMouseEvent *event) {
	if (event->state() & Qt::LeftButton) {
		int distance = (event->pos() - dragPos).manhattanLength();
		if (distance > QApplication::startDragDistance())
			startDrag();
    }
}

void ClientConfListView::startDrag() {
    ClientConfItem *cci = dynamic_cast<ClientConfItem *> (currentItem());
    if ( cci ) {
		ClientConf *cc = cci->getModel();
		if ( cc ) {
			QString idCli = QString::number(cc->getId());
			QMimeData *mimeData = new QMimeData;
			QDrag *drag = new QDrag(this);
			
			mimeData->setText(idCli);
			drag->setMimeData(mimeData);
			
			drag->exec();
		}
    }
}

/**********************************************************************/

ClientConfItem::ClientConfItem(ClientConf *cl, QTreeWidget *parent, QStringList sList) : QTreeWidgetItem(parent, sList) {
    model = cl;
    setIcon (0 , QIcon(":/images/RegClient.xpm"));
    //setDragEnabled ( true ); 
    //we need to create something for the flows....
    //first input, then output....
    std::map<unsigned int, FlowOutletIn*>* foin = model->getInputFlowList();
    std::map<unsigned int, FlowOutletOut*>* foout = model->getOutputFlowList(); 
    std::map<unsigned int, FlowOutletIn*>::iterator iter_in;
    std::map<unsigned int, FlowOutletOut*>::iterator iter_out;
    for ( iter_in = foin->begin(); iter_in != foin->end(); iter_in++) {
		QTreeWidgetItem *tmp = new QTreeWidgetItem(this, QStringList(iter_in->second->getName()));
		tmp->setIcon (0 , QIcon(":/images/FlowIn.xpm"));
		addChild(tmp);//insertItem ( tmp );
    }
    for ( iter_out = foout->begin(); iter_out != foout->end(); iter_out++) {
		QTreeWidgetItem *tmp = new QTreeWidgetItem(this, QStringList(iter_out->second->getName()));
		tmp->setIcon (0 , QIcon(":/images/FlowOut.xpm"));
		addChild(tmp); //insertItem ( tmp );
    }
}

ClientConfItem::~ClientConfItem() {}

ClientConf* ClientConfItem::getModel() { return model; }
