#include "LogTab.h"
#include "LogView.h"
#include "../model/Client.h"
#include "../model/Server.h"
#include "../model/Duplicator.h"
#include "../mainwindow.h"

#include <QtCore/QTextStream>
#include <QtCore/QFile>
#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>
#include <QtGui/QStatusBar>
#include <QtGui/QTextDocument>

LogTab::LogTab(LogView* parent) : QWidget(parent) {
	logView = parent;
	
	tabLayout = new QHBoxLayout( this, 0, 6, "tabLayout");
	tabLayout2 = new QVBoxLayout();
	tabLayout2->setMargin(0);
	tabLayout2->setSpacing(6);
	tabLayout2->setObjectName(QString::fromUtf8("tabLayout2"));
	
	textEditLog = new QTextEdit( this, "textEditLog" );
	textEditLog->setReadOnly(true);
	textEditLog->setLineWidth(1);
	QFont myFont("Helvetica", 12, QFont::Normal);
	textEditLog->setCurrentFont(myFont);
	textEditLog->document()->setMaximumBlockCount(MAXLOGSIZE);
	tabLayout2->addWidget(textEditLog);
	
	buttonLayout = new QHBoxLayout( 0, 0, 6, "buttonLayout"); 
	
	leftButtonSpacer = new QSpacerItem( 31, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	buttonLayout->addItem( leftButtonSpacer );
	
	clearLogButton = new QPushButton( this, "clearLogButton" );
	clearLogButton->setText( tr( "Clear Log" ) );
	buttonLayout->addWidget( clearLogButton );
	
	middleButtonSpacer = new QSpacerItem( 31, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	buttonLayout->addItem( middleButtonSpacer );
	
	saveLogButton = new QPushButton( this, "saveLogButton" );
	saveLogButton->setText( tr( "Save..." ) );
	buttonLayout->addWidget( saveLogButton );
	
	leftButtonSpacer = new QSpacerItem( 31, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	buttonLayout->addItem( leftButtonSpacer );
	
	tabLayout2->addLayout( buttonLayout );
	tabLayout->addLayout( tabLayout2 );
	
	connect( saveLogButton, SIGNAL( clicked() ), this, SLOT( saveLog() ) );
	connect( clearLogButton, SIGNAL( clicked() ), this, SLOT( clearLog() ) );

	sliderPosition = 0;
	
	textEditLog->show();
	show();
}

LogTab::~LogTab(){
}

bool LogTab::saveLogFile(QString &filename, QString &log) {
	QFile file(filename);
	if ( !file.open( QIODevice::WriteOnly ) ) {
		return false;
	}
	else {
		QTextStream out;
		out.setDevice( &file );
		out.setEncoding( QTextStream::UnicodeUTF8 );
		out << log;
		out << endl;
		file.close();
		return true;
	}
}

ClientLogTab::ClientLogTab(ClientApp *cl, LogView* parent) : LogTab(parent) {
  client = cl;
}

ClientLogTab::~ClientLogTab(){
  client = NULL;
}

void ClientLogTab::addLogLine(QString &l) {
  textEditLog->append(l); 
}

void ClientLogTab::clearLog() {
  textEditLog->clear();
}

void ClientLogTab::saveLog() {
	QFileDialog dialog(this, "Save Client Log File", "", "Log files (*.log);;All (*)");
    QString tmp_dir = getLogView()->getLogDir();
	if ( tmp_dir != "" )
		dialog.setDirectory( tmp_dir );
	dialog.setFileMode( QFileDialog::AnyFile );
	dialog.setAcceptMode ( QFileDialog::AcceptSave );
    QString filename;
    if ( dialog.exec() == QDialog::Accepted ) {
		filename = dialog.selectedFile();
		getLogView()->setLogDir(dialog.directory().absolutePath());
		if ( !filename.isEmpty() ) {
			QRegExp rxp( "\\.log", true, false );
			bool nb = filename.contains( rxp );
			if ( nb == false )
				filename = filename.append(".log");
			if (QFile::exists(filename)) {
				int ret = QMessageBox::warning(this, tr("Attention"), 
							tr("File %1 already exists.\n"
							"Do you want to overwrite it?")
							.arg(QDir::convertSeparators(filename)),
							QMessageBox::Yes | QMessageBox::Default,
							QMessageBox::No | QMessageBox::Escape);
				if (ret == QMessageBox::Yes) {
				        QString logToSave = textEditLog->toPlainText();
					if (saveLogFile(filename, logToSave) ) {
						MainWindow::instance()->statusBar()->message(tr("Log file %1 saved").arg(QDir::convertSeparators(filename)));
					}
					else {
						MainWindow::instance()->statusBar()->message(tr("Log cannot be saved"));
					}
				}
			}
			else {
			        QString logToSave = textEditLog->toPlainText();
				if ( saveLogFile(filename, logToSave) ) {
					MainWindow::instance()->statusBar()->message(tr("Log file %1 saved").arg(QDir::convertSeparators(filename)));
				}
				else
					MainWindow::instance()->statusBar()->message(tr("Log cannot be saved"));
			}	
		}
	}

}

ServerLogTab::ServerLogTab(Server *s, LogView* parent) : LogTab(parent) {
	server = s;
	if ( s )
		serverName = s->getName();
}

ServerLogTab::~ServerLogTab(){
	server = NULL;
	serverName = "";
}

void ServerLogTab::addLogLine(QString &l) {
       textEditLog->append(l);
}

void ServerLogTab::setServer(Server *s) {
	if ( s ) {
		server = s;
		serverName = s->getName();
	}
}

QString& ServerLogTab::getServerName() {
	return serverName;
}

void ServerLogTab::clearLog() {
	textEditLog->clear();
}

void  ServerLogTab::saveLog() {
	QFileDialog dialog(this, "Save Server Log File", "", "Log files (*.log);;All (*)");
    QString tmp_dir = getLogView()->getLogDir();
	if ( tmp_dir != "" )
		dialog.setDirectory( tmp_dir );
	dialog.setFileMode( QFileDialog::AnyFile );
	dialog.setAcceptMode ( QFileDialog::AcceptSave );
	QString filename;
    if ( dialog.exec() == QDialog::Accepted ) {
		filename = dialog.selectedFile();
		getLogView()->setLogDir(dialog.directory().absolutePath());
		if ( !filename.isEmpty() ) {
			QRegExp rxp( "\\.log", true, false );
			bool nb = filename.contains( rxp );
			if ( nb == false )
				filename = filename.append(".log");
			if (QFile::exists(filename)) {
				int ret = QMessageBox::warning(this, tr("Attention"), 
											   tr("File %1 already exists.\n"
												  "Do you want to overwrite it?")
											   .arg(QDir::convertSeparators(filename)),
											   QMessageBox::Yes | QMessageBox::Default,
											   QMessageBox::No | QMessageBox::Escape);
				if (ret == QMessageBox::Yes) {
				        QString logToSave = textEditLog->toPlainText();
				        if ( saveLogFile(filename, logToSave) ) {
						MainWindow::instance()->statusBar()->message(tr("Log file %1 saved").arg(QDir::convertSeparators(filename)));
					}
					else {
						MainWindow::instance()->statusBar()->message(tr("Log cannot be saved"));
					}
				}
			}
			else {
			        QString logToSave = textEditLog->toPlainText();
				if ( saveLogFile(filename, logToSave) ) {
					MainWindow::instance()->statusBar()->message(tr("Log file %1 saved").arg(QDir::convertSeparators(filename)));
				}
				else
					MainWindow::instance()->statusBar()->message(tr("Log cannot be saved"));
			}	
		}
	}
	
}

DuplicatorLogTab::DuplicatorLogTab(Duplicator *dup, LogView* parent) : LogTab(parent) {
	duplicator = dup;
}

DuplicatorLogTab::~DuplicatorLogTab(){
	duplicator = NULL;
}

void DuplicatorLogTab::addLogLine(QString &l) {
       textEditLog->append(l); 
}

void DuplicatorLogTab::clearLog() {
	textEditLog->clear();
}


void DuplicatorLogTab::saveLog() {
	QFileDialog dialog(this, "Save Duplicator Log File", "", "Log files (*.log);;All (*)");
    QString tmp_dir = getLogView()->getLogDir();
	if ( tmp_dir != "" )
		dialog.setDirectory( tmp_dir );
	dialog.setFileMode( QFileDialog::AnyFile );
	dialog.setAcceptMode ( QFileDialog::AcceptSave );
	QString filename;
    if ( dialog.exec() == QDialog::Accepted ) {
		filename = dialog.selectedFile();
		getLogView()->setLogDir(dialog.directory().absolutePath());
		if ( !filename.isEmpty() ) {
			QRegExp rxp( "\\.log", true, false );
			bool nb = filename.contains( rxp );
			if ( nb == false )
				filename = filename.append(".log");
			if (QFile::exists(filename)) {
				int ret = QMessageBox::warning(this, tr("Attention"), 
											   tr("File %1 already exists.\n"
												  "Do you want to overwrite it?")
											   .arg(QDir::convertSeparators(filename)),
											   QMessageBox::Yes | QMessageBox::Default,
											   QMessageBox::No | QMessageBox::Escape);
				if (ret == QMessageBox::Yes) {
				        QString logToSave = textEditLog->toPlainText();
					if ( saveLogFile(filename, logToSave) ) {
						MainWindow::instance()->statusBar()->message(tr("Log file %1 saved").arg(QDir::convertSeparators(filename)));
					}
					else {
						MainWindow::instance()->statusBar()->message(tr("Log cannot be saved"));
					}
				}
			}
			else {
			        QString logToSave = textEditLog->toPlainText();
				if ( saveLogFile(filename, logToSave) ) {
					MainWindow::instance()->statusBar()->message(tr("Log file %1 saved").arg(QDir::convertSeparators(filename)));
				}
				else
					MainWindow::instance()->statusBar()->message(tr("Log cannot be saved"));
			}	
		}
	}
	
}

