#ifndef _LOGTAB_H
#define _LOGTAB_H

#include <QtCore/QString>
#include <QtGui/QWidget>
#include <QtGui/QTextEdit>
#include <QtGui/QTabWidget>
#include <QtGui/QLayout>
#include <QtGui/QHBoxLayout>
#include <QtGui/QPushButton>
#include <QtGui/QSpacerItem>

class LogView;

class LogTab : public QWidget {
	Q_OBJECT
public:
	LogTab(LogView* parent);
	~LogTab();

	virtual void addLogLine(QString &l) = 0;
	
	LogView* getLogView() { return logView; }
	
public slots:
	virtual void clearLog() = 0;
	virtual void saveLog() = 0;
	
protected:
	bool saveLogFile(QString &filename, QString &log);
	
	LogView *logView; 
	
	QTextEdit *textEditLog;
	QPushButton* clearLogButton;
	QPushButton* saveLogButton;
	QHBoxLayout* tabLayout;
	QVBoxLayout* tabLayout2;
	QHBoxLayout* buttonLayout;
	QSpacerItem* leftButtonSpacer;
	QSpacerItem* middleButtonSpacer;
	QSpacerItem* rightButtonSpacer;
	int sliderPosition;
};

class ClientApp;

class ClientLogTab : public LogTab {
   Q_OBJECT
 public:
  ClientLogTab(ClientApp *cl, LogView* parent);
  ~ClientLogTab();

  void addLogLine(QString &l);

 public slots:
   void clearLog();
   void saveLog();
   
 private:
   ClientApp* client;
};

class Server;

class ServerLogTab : public LogTab {
	Q_OBJECT
public:
	ServerLogTab(Server *s, LogView* parent);
	~ServerLogTab();
	
	void addLogLine(QString &l);
	void setServer(Server *s);
	
	QString& getServerName();
	
public slots:
	void clearLog();
	void saveLog();
	
private:
	Server *server;
	QString serverName;
};

class Duplicator;

class DuplicatorLogTab : public LogTab {
	Q_OBJECT
public:
	DuplicatorLogTab(Duplicator *dup, LogView* parent);
	~DuplicatorLogTab();
	
	void addLogLine(QString &l);
	
public slots:
	void clearLog();
	void saveLog();
	
private:
		Duplicator *duplicator;
};

#endif
