#include "LogView.h"

#include "mainwindow.h"
#include "../model/Client.h"
#include "../model/Server.h"
#include "../model/Duplicator.h"
#include "LogTab.h"

#include <QtCore/QVariant>
#include <QtGui/QTabWidget>
#include <QtGui/QToolTip>
#include <QtGui/QColor>
#include <QtGui/QPixmap>
#include <QtGui/QVBoxLayout>
#include <QtGui/QGridLayout>
#include <QtGui/QHBoxLayout>

/*
 *  Constructs a LogView as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 */
LogView::LogView( QWidget* parent, const char* name, Qt::WFlags fl ) : LogViewBase( parent, name, fl )
//  : QWidget( parent, name, fl )
{
    client_tabs.clear();
    server_tabs.clear();
	duplicator_tabs.clear();
	
    logDir = "";
    //clearWState( WState_Polished );
}

/*
 *  Destroys the object and frees any allocated resources
 */
LogView::~LogView()
{
    // no need to delete child widgets, Qt does it all for us
    removeAllClientTab();
}

/*
 *  Sets the strings of the subwidgets using the current
 *  language.
 */
void LogView::languageChange()
{
    setCaption( tr( "Logs" ) );
}

bool LogView::addClientTab(unsigned int clientID) {
	if ( MainWindow::instance()->global->getApplication()->clientExists(clientID) ) {
		ClientApp *tmp_cl = MainWindow::instance()->global->getApplication()->getClient(clientID);
		ClientLogTab *clLogTab = new ClientLogTab(tmp_cl, this);
		if ( clLogTab ) {
			QString tmp_name = tmp_cl->getName();
			tabWidget_clients->insertTab( clLogTab, tmp_name);
			client_tabs[clientID] = clLogTab;
		}
		else 
			return false;
		return true; 
	}
	else
		return false;
}

bool LogView::removeClientTab(unsigned int clientID) {
	ClientLogTab *tmp_tab = NULL;
	tmp_tab = client_tabs[clientID];
	if ( tmp_tab ) {
		tabWidget_clients->removePage(tmp_tab);
		client_tabs.erase(clientID);
		delete tmp_tab;
		return true;
	}
	return false;
}

void LogView::setClientTab(unsigned int clientID) {
	tabWidget_out->setCurrentPage(TAB_CLIENT);
	std::map<unsigned int, ClientLogTab *>::iterator iter;
	iter = client_tabs.find(clientID);
	if ( iter != client_tabs.end() ) {
		tabWidget_clients->setCurrentWidget(client_tabs[clientID]);
	}	
}

void LogView::addLogForClient(unsigned int clientID, QString &l) {
	std::map<unsigned int, ClientLogTab *>::iterator iter;
	iter = client_tabs.find(clientID);
	if ( iter != client_tabs.end() ) {
		client_tabs[clientID]->addLogLine(l);
	}
}

void LogView::clearClientLog(unsigned int clientID) {
	std::map<unsigned int, ClientLogTab *>::iterator iter;
	iter = client_tabs.find(clientID);
	if ( iter != client_tabs.end() ) {
		client_tabs[clientID]->clearLog();
	}
}

void LogView::removeAllClientTab() {
    std::map<unsigned int, ClientLogTab *>::iterator iter;
    for ( iter = client_tabs.begin(); iter != client_tabs.end(); iter++ ) {
		delete iter->second;
    }
    client_tabs.clear();
}

bool LogView::addServerTab(ServerList *sl, unsigned int serverID) {
	if ( sl->serverExists(serverID) ) {
		Server *tmp_s = sl->getServer(serverID);
		QString tmp_name = tmp_s->getName();
		if ( tmp_name != "No Server") {
			int servID = 0;
			ServerLogTab *sLogTab = findServerTab(tmp_name, servID);
			if ( sLogTab == NULL ) {
				sLogTab = new ServerLogTab(tmp_s, this);
				if ( sLogTab ) {
					tabWidget_servers->insertTab( sLogTab, tmp_name);
					server_tabs[serverID] = sLogTab;
					//cout << "SERVERTAB added name=" << tmp_name.toStdString() << " ID=" << serverID << endl;
				}
			}
			else {
				//remove the old id
				sLogTab->setServer(tmp_s);
				server_tabs.erase(servID);
				
				server_tabs[serverID] = sLogTab;
				//cout << "SERVERTAB already exists name=" << tmp_name.toStdString() << "Old ID=" << servID << " ID=" << serverID << endl;
				
			}
		}
		else 
			return false;
		return true; 
	}
	else
		return false;
}

bool LogView::removeServerTab(unsigned int serverID) {
	ServerLogTab *tmp_tab = NULL;
	tmp_tab = server_tabs[serverID];
	if ( tmp_tab ) {
		tabWidget_servers->removePage(tmp_tab);
		server_tabs.erase(serverID);
		delete tmp_tab;
		return true;
	}
	return false;
}

void LogView::setServerTab(unsigned int serverID) {
	tabWidget_out->setCurrentPage(TAB_SERVER);
	std::map<unsigned int, ServerLogTab *>::iterator iter;
	iter = server_tabs.find(serverID);
	if ( iter != server_tabs.end() ) {
		tabWidget_servers->setCurrentWidget(server_tabs[serverID]);
	}
}

void LogView::addLogForServer(unsigned int serverID, QString &l) {
	std::map<unsigned int, ServerLogTab *>::iterator iter;
	iter = server_tabs.find(serverID);
	if ( iter != server_tabs.end() ) {
		server_tabs[serverID]->addLogLine(l);
	}
}

void LogView::clearServerLog(unsigned int serverID) {
	std::map<unsigned int, ServerLogTab *>::iterator iter;
	iter = server_tabs.find(serverID);
	if ( iter != server_tabs.end() ) {
		server_tabs[serverID]->clearLog();
	}
}

void LogView::removeAllServerTab() {
    std::map<unsigned int, ServerLogTab *>::iterator iter;
    for ( iter = server_tabs.begin(); iter != server_tabs.end(); iter++ ) {
		delete iter->second;
    }
    server_tabs.clear();
}

void LogView::removeAllTab() {
	removeAllClientTab();
	removeAllServerTab();
	removeAllDuplicatorTab();
}

ServerLogTab* LogView::findServerTab(QString &sname, int &sid) {
	std::map<unsigned int, ServerLogTab *>::iterator iter;
	for ( iter = server_tabs.begin(); iter != server_tabs.end(); iter++ ) {
		if ( iter->second->getServerName() == sname ) {
			sid = iter->first;
			return iter->second;
		}
	}
	return NULL;
}	

bool LogView::addDuplicatorTab(unsigned int dupID) {
	if ( MainWindow::instance()->global->getApplication()->getDuplicatorList()->duplicatorExists(dupID) ) {
		Duplicator *tmp_dup = MainWindow::instance()->global->getApplication()->getDuplicatorList()->getDuplicator(dupID);
		DuplicatorLogTab *dupLogTab = new DuplicatorLogTab(tmp_dup, this);
		if ( dupLogTab ) {
			QString tmp_name = tmp_dup->getName();
			tabWidget_duplicators->insertTab( dupLogTab, tmp_name);
			duplicator_tabs[dupID] = dupLogTab;
		}
		else 
			return false;
		return true; 
	}
	else
		return false;
}

bool LogView::removeDuplicatorTab(unsigned int dupID) {
	DuplicatorLogTab *tmp_tab = NULL;
	tmp_tab = duplicator_tabs[dupID];
	if ( tmp_tab ) {
		tabWidget_duplicators->removePage(tmp_tab);
		duplicator_tabs.erase(dupID);
		delete tmp_tab;
		return true;
	}
	return false;
}

void LogView::setDuplicatorTab(unsigned int dupID) {
    tabWidget_out->setCurrentPage(TAB_DUPLI);
	std::map<unsigned int, DuplicatorLogTab *>::iterator iter;
	iter = duplicator_tabs.find(dupID);
	if ( iter != duplicator_tabs.end() ) {
		tabWidget_duplicators->setCurrentWidget(duplicator_tabs[dupID]);
	}
}

void LogView::addLogForDuplicator(unsigned int dupID, QString &l) {
	std::map<unsigned int, DuplicatorLogTab *>::iterator iter;
	iter = duplicator_tabs.find(dupID);
	if ( iter != duplicator_tabs.end() ) {
		duplicator_tabs[dupID]->addLogLine(l);
	}
}

void LogView::clearDuplicatorLog(unsigned int dupID) {
	std::map<unsigned int, DuplicatorLogTab *>::iterator iter;
	iter = duplicator_tabs.find(dupID);
	if ( iter != duplicator_tabs.end() ) {
		duplicator_tabs[dupID]->clearLog();
	}
}

void LogView::removeAllDuplicatorTab() {
	std::map<unsigned int, DuplicatorLogTab *>::iterator iter;
    for ( iter = duplicator_tabs.begin(); iter != duplicator_tabs.end(); iter++ ) {
		delete iter->second;
    }
    duplicator_tabs.clear();
}

QString LogView::getLogDir() { return logDir; }

void LogView::setLogDir(QString dir) {
	logDir = dir;
}

