#include "FlowTypeModel.h"
#include "Generate.h"

#include <QtCore/QFile>
#include <QtXml/QDomDocument>
#include <QtCore/QString>
#include <QtCore/QRegExp>
#include <iostream>

using namespace std;

FlowTypeModel::FlowTypeModel() {}

FlowTypeModel::~FlowTypeModel() {}

void FlowTypeModel::setDescription(QString _desc) {
	description = _desc;
}

void FlowTypeModel::setClassName(QString _class) {
	className = _class;
}

void FlowTypeModel::setBufferSize(QString _bsize) {
	bufferSize = _bsize;
}

void FlowTypeModel::setHistorySize(QString _hsize) {
	historySize = _hsize;
}

void FlowTypeModel::setSpecificMethods(QString _methods) {
    _methods = _methods.replace ( QChar('\n'), "|", true );
    _methods = _methods.simplifyWhiteSpace();
    _methods = _methods.replace ( QChar('|'), "\n", true );  
    QStringList tmp_specificMethods = QStringList::split( QRegExp("\n"), _methods);
    specificMethods.clear();
    for ( QStringList::Iterator it = tmp_specificMethods.begin(); it != tmp_specificMethods.end(); ++it ) {
		(*it).simplifyWhiteSpace();
		(*it).stripWhiteSpace();
		if ( !(*it).isEmpty() )
			specificMethods.append((*it));
    }
}

void FlowTypeModel::setIncludes(QString _includes) {
    _includes = _includes.replace ( QChar('\n'), "|", true );
    _includes = _includes.simplifyWhiteSpace();
    _includes = _includes.replace ( QChar('|'), "\n", true );  
    QStringList tmp_includes = QStringList::split( QRegExp("\n"), _includes);
    includes.clear();
    for ( QStringList::Iterator it = tmp_includes.begin(); it != tmp_includes.end(); ++it ) {
		(*it).simplifyWhiteSpace();
		(*it).stripWhiteSpace();
		if ( !(*it).isEmpty() )
			includes.append((*it));
    }	
}

void FlowTypeModel::setBaseName(QString &_baseName) {
    //we need to remove the ".flow" if present in the QString
    baseName = _baseName;
    QRegExp rxp( "\\.flow$", true, false );
    bool nb = baseName.contains( rxp );
    if ( nb == true ) {
		//the name contains the .flow extension
		baseName.replace(rxp, "");
    }
}

QString& FlowTypeModel::getDescription() { return description; }

QString& FlowTypeModel::getClassName() { return className; } 

QString& FlowTypeModel::getBufferSize() { return bufferSize; } 

QString& FlowTypeModel::getHistorySize() { return historySize; }

QStringList& FlowTypeModel::getSpecificMethods() { return specificMethods; }

QStringList& FlowTypeModel::getIncludes() { return includes; }

QString& FlowTypeModel::getBaseName() { return baseName; }

QStringList FlowTypeModel::splitMethod(QString &method) {
	QString tmp_method(method.latin1());
	tmp_method = tmp_method.simplifyWhiteSpace();  
	QStringList tmp_list = QStringList::split( QRegExp(" "), tmp_method);
	return tmp_list;
}


bool FlowTypeModel::readFlowFile(QString &filename) { 
    if ( filename.isEmpty() ) {
		return false;
    }	
    QFile f(filename);
    QDomDocument myDoc;
    if ( !f.open( QIODevice::ReadOnly ) ) { 
		return false;
    }	
    if ( !myDoc.setContent( &f ) ) {
		f.close(); 	
		return false;
    }
    f.close();
    QString type = myDoc.doctype().name();
    if ( type != "FlowGen" ) {
		return false;
    }
    else {		
		//everything is Okay, we can begin to parse the file
		QString tagname = "";
		QString name = "";
		QString desc = "";
		QString bsize = "";
		QString hsize = "";
		// Time to parse the tree 
		QDomElement root = myDoc.documentElement(); 
		QDomElement child = root.firstChild().toElement();
		while( !child.isNull() ) {
			tagname = child.tagName();
			if ( tagname == "Type" )
				name = child.text();
			if ( tagname == "Description" )
				desc = child.text();
			if ( tagname == "BufferSize" )
				bsize = child.text();
			if ( tagname == "HistorySize" )
				hsize = child.text();
			if ( tagname == "Includes" ) {
				QDomElement smallchild = child.firstChild().toElement();
				QString inc = "";
				while( !smallchild.isNull() ) {
					if ( smallchild.tagName() == "Include" ) {
						inc = inc.append(smallchild.text());
						inc.append('\n');
					}
					smallchild = smallchild.nextSibling().toElement();
				}
				setIncludes(inc);
			}
			else if ( tagname == "Methods" ) {
				QDomElement smallchild = child.firstChild().toElement();
				QString inc = "";
				while( !smallchild.isNull() ) {
					if ( smallchild.tagName() == "Method" ) {
						inc = inc.append(smallchild.text());
						inc.append('\n');
					}
					smallchild = smallchild.nextSibling().toElement();
				}
				setSpecificMethods(inc);
				return true;
			}
			
			if ( !name.isEmpty() && !desc.isEmpty() && !bsize.isEmpty()  && !hsize.isEmpty()) {
				setClassName(name);
				setDescription(desc);
				setBufferSize(bsize);
				setHistorySize(hsize);
			}
			child = child.nextSibling().toElement();
		}
		return false;
    }
}

bool FlowTypeModel::writeFlowFile(QString &filename) {
    QFile file(filename);
    if ( !file.open( QIODevice::WriteOnly ) ) {
		return false;
    }	
    else {
		XmlWriter *xw;
		xw = new  XmlWriter(&file);
		xw->setAutoNewLine(true);
		xw->writeRaw("<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>");
		xw->newLine();
		xw->writeRaw("<!DOCTYPE FlowGen SYSTEM \"flowgen.dtd\">");
		xw->newLine();
		xw->writeRaw("<!--This file describe a flow type of the SmartFlow-->");
		xw->newLine();
		xw->writeOpenTag("Flow");
		writeFlow(xw, getClassName(), getDescription(), getBufferSize(), getHistorySize());
		writeIncludes(xw);
		writeMethods(xw);
		xw->writeCloseTag("Flow");
		file.close();
		delete xw;
    }
    return true;	
}

bool FlowTypeModel::generateCode() {
	Generate_H *codeGenH = new Generate_H(this, getBaseName());
	codeGenH->generate_code_H();
	delete codeGenH;
	
	Generate_CPP *codeGenCPP = new Generate_CPP(this, getBaseName());
	codeGenCPP->generate_code_CPP();
	delete codeGenCPP;
	
	QString basename_tmp = getBaseName();
	basename_tmp.append("_functions");
	Generate_functions_CPP *codeGenFunctionsCPP = new Generate_functions_CPP(this, basename_tmp);
	codeGenFunctionsCPP->generate_code_functions_CPP();
	delete codeGenFunctionsCPP;
	
	Generate_PRO *codeGenPRO = new Generate_PRO(this, getBaseName());
	codeGenPRO->generate_code_PRO();
	delete codeGenPRO;
	
	return true;
}

void FlowTypeModel::printModel() {
	cout << endl;
	cout << "Flow:" << endl;
	cout << "   Description : " << getDescription().data() << endl;
	cout << "   Class Name  : " << getClassName().data() << endl;
	cout << "   Buffer Size : " << getBufferSize().data() << endl;
	cout << "   History Size: " << getHistorySize().data() << endl;
	cout << "   Basename    : " << getBaseName().data() << endl;
	cout << endl;
}

void FlowTypeModel::writeFlow(XmlWriter *xw, QString &type, QString &description, QString &bufferSize, QString &historySize) {
    xw->writeTaggedString("Type", type);
    xw->writeTaggedString("Description", description);
    xw->writeTaggedString("BufferSize", bufferSize);
    xw->writeTaggedString("HistorySize", historySize);
}

void FlowTypeModel::writeIncludes(XmlWriter *xw) {
	xw->writeOpenTag("Includes");
	for ( QStringList::Iterator it = includes.begin(); it != includes.end(); ++it ) {	
		if ( !(*it).isEmpty() )
			xw->writeTaggedString("Include", (*it));
	}
	xw->writeCloseTag("Includes");	 
}

void FlowTypeModel::writeMethods(XmlWriter *xw) {
	xw->writeOpenTag("Methods");
	for ( QStringList::Iterator it = specificMethods.begin(); it != specificMethods.end(); ++it ) {
		if ( !(*it).isEmpty() )
			xw->writeTaggedString("Method", (*it));
	}
	xw->writeCloseTag("Methods");
}
