//========================= Official Notice ===============================
//
// "This software was developed at the National Institute of Standards
// and Technology by employees of the Federal Government in the course of
// their official duties. Pursuant to Title 17 Section 105 of the United
// States Code this software is not subject to copyright protection and
// is in the public domain.
//
// The NIST Data Flow System (NDFS) is an experimental system and is
// offered AS IS. NIST assumes no responsibility whatsoever for its use
// by other parties, and makes no guarantees and NO WARRANTIES, EXPRESS
// OR IMPLIED, about its quality, reliability, fitness for any purpose,
// or any other characteristic.
//
// We would appreciate acknowledgement if the software is used.
//
// This software can be redistributed and/or modified freely provided
// that any derivative works bear some notice that they are derived from
// it, and any modified versions bear some notice that they have been
// modified from the original."
//
//=========================================================================



#include "Generate.h"
#include "FlowTypeModel.h"

#include <QtCore/QStringList>
#include <QtCore/QString>
#include <QtCore/QFile>
#include <iostream>

using namespace std;

Generate::Generate(FlowTypeModel *_model, QString basename, const char *ext, bool overwriteFile)
{
	model = _model;
	QString name(basename.latin1());
	name.append(".");
	name.append(ext);
	if ( overwriteFile ) {
		theFile = new QFile(name);
		if ( !theFile->open( QIODevice::WriteOnly ) ) {
			return;
		}
		stream = new QTextStream();
		stream->setDevice(theFile);
		stream->setEncoding( QTextStream::UnicodeUTF8 );
		needGeneration = true;
	}
	else { 
		if ( !QFile::exists(name) ) {
			theFile = new QFile(name);
			if ( !theFile->open( QIODevice::WriteOnly ) ) {
				return;
			}
			stream = new QTextStream();
			stream->setDevice(theFile);
			stream->setEncoding( QTextStream::UnicodeUTF8 );
			needGeneration = true;
		}
		else
			needGeneration = false; 
	}
}

Generate::~Generate()
{
	if ( needGeneration ) {
		stream->unsetDevice();
		theFile->close();
		if ( stream ) {
			delete stream;
			stream = NULL;
		}
		if ( theFile ) {
			delete theFile;
			theFile = NULL;
		}
		theFile = NULL;
	}
	model = NULL;
}

void Generate::write(const char* str) {
	QString s(str);
	(*stream) << s;
}

void Generate::writeLine(const char *str) {
	QString s(str);
	(*stream) << s << endl;
}

void Generate::write(const char* str, QString &arg) { 
	QString s(str);
	s = s.arg(arg);
	(*stream) << s;
}

void Generate::writeLine(const char* str, QString &arg) {
	QString s(str);
	s = s.arg(arg);
	(*stream) << s << endl;
}

void Generate::write(const char* str, QString &arg1, QString &arg2) {
	QString s(str);
	s = s.arg(arg1, arg2);
	(*stream) << s;
}

void Generate::writeLine(const char* str, QString &arg1, QString &arg2) {
	QString s(str);
	s = s.arg(arg1, arg2);
	(*stream) << s << endl;
}

void Generate::write(const char* str, QString &arg1, QString &arg2, QString &arg3) {
	QString s(str);
	s = s.arg(arg1, arg2, arg3);
	(*stream) << s;
}

void Generate::writeLine(const char* str, QString &arg1, QString &arg2, QString &arg3) {
	QString s(str);
	s = s.arg(arg1, arg2, arg3);
	(*stream) << s << endl;
}

void Generate::write(const char* str, QString &arg1, QString &arg2, QString &arg3, QString &arg4) {
	QString s(str);
	s = s.arg(arg1, arg2, arg3, arg4);
	(*stream) << s;
}

void Generate::writeLine(const char* str, QString &arg1, QString &arg2, QString &arg3, QString &arg4) {
	QString s(str);
	s = s.arg(arg1, arg2, arg3, arg4);
	(*stream) << s << endl;
}

void Generate::newLine()
{
	(*stream) << endl;
}

/****************************************************************************************************/

Generate_H::Generate_H(FlowTypeModel *_model, const QString basename) : Generate(_model, basename, "h", true)
{
	includeName = model->getClassName();
	includeName = includeName.upper();
	includeName.prepend('_');
	includeName.append("_H");
}

Generate_H::~Generate_H()
{
}

bool Generate_H::generate_code_H()
{
	if ( needGeneration ) {
		writeLine("/* This file has been automatically generated by NSDFS Flow Generator");
		 writeLine(" * Do not edit");
		 writeLine(" */");
		 newLine();
		 writeLine("#ifndef %1", includeName);
		 writeLine("#define %1", includeName);
		 newLine();
		 writeLine("#include \"ace/svc_export.h\"");
		 newLine();
		 writeLine("#include \"Smartflow.h\"");
		 writeLine("#include \"Flow.h\"");
		 writeLine("#include <string.h>");
		 newLine();
		 QStringList includeList = model->getIncludes();
		 for ( QStringList::Iterator it = includeList.begin(); it != includeList.end(); ++it )
			 writeLine("%1", (*it));
		 newLine();
		 writeLine("namespace smartflow {");
		 newLine();
		 writeLine("class ACE_Svc_Export %1 : public Flow {", model->getClassName());
		 writeLine("  typedef Flow super;");
		 writeLine("  public:");
		 writeLine("    %1();", model->getClassName());
		 writeLine("    virtual ~%1();", model->getClassName());
		 writeLine("    virtual bool init(FlowManager *manager,const std::string &type, const std::string &name, const std::string &rgroup, const std::string &args, flow_direction_t mode, flow_policy_t policy, bool isSynchronized);");
		 writeLine("    virtual long getDefaultHistorySize(void);");
		 writeLine("    virtual buffer_size_t getDefaultBufferSize();");
		 writeLine("    virtual std::string & getFlowType(void);");
		 writeLine("    virtual std::string & getFlowDesc(void);");
		 newLine();
		 writeLine("  private:");
		 writeLine("    static std::string     default_flow_type;");
		 writeLine("    static buffer_size_t   default_buffer_size;");
		 writeLine("    static long            default_history_size;");
		 writeLine("    static std::string     default_flow_desc;");
		 newLine();
		 writeLine("  public:");
		 writeLine("    bool  user_init(const std::string &args);");
		 writeLine("    void user_close();");
		 QStringList methodList = model->getSpecificMethods();
		 for ( QStringList::Iterator it = methodList.begin(); it != methodList.end(); ++it ) 
			 writeLine("    %1", (*it));
		 newLine();
		 writeLine("}; //class");
		 newLine();
		 writeLine("}  //namespace smartflow");
		 newLine();
		 writeLine("extern \"C\" Smartflow_Export smartflow::Flow * create_flow(void);");
		 newLine();
		 writeLine("#endif");
		 return true;
	}
	return false;
}

/****************************************************************************************************/

Generate_CPP::Generate_CPP(FlowTypeModel *_model, const QString basename) : Generate(_model, basename, "cpp", true)
{
	includeName = model->getClassName();
	includeName.append(".h");
}

Generate_CPP::~Generate_CPP()
{
}

bool Generate_CPP::generate_code_CPP() {
	if ( needGeneration ) {
		writeLine("/* This file has been automatically generated by NSDFS Flow Generator");
		 writeLine(" * Do not edit");
		 writeLine(" */");
		 newLine();
		 writeLine("#include <%1>", includeName);
		 writeLine("#include \"duplicator_common_ACE.h\"");
		 writeLine("#include <sstream>");
		 writeLine("#include <iostream>");
		 writeLine("#include <string>");
		 newLine();
		 writeLine("using namespace std;");
		 newLine();
		 writeLine("smartflow::Flow * create_flow(void)");
		 writeLine("{");
		 writeLine("    return new smartflow::%1();", model->getClassName());
		 writeLine("}");
		 newLine();
		 newLine();
		 writeLine("namespace smartflow {");
		 newLine();
		 writeLine("string          %1::default_flow_type    = \"%2\";", model->getClassName(), model->getClassName());
		 writeLine("buffer_size_t   %1::default_buffer_size  = %2;", model->getClassName(), model->getBufferSize());
		 writeLine("long            %1::default_history_size = %2;", model->getClassName(), model->getHistorySize());
		 writeLine("string          %1::default_flow_desc    = \"%2\";", model->getClassName(), model->getDescription());
		 newLine();
		 writeLine("long %1::getDefaultHistorySize(void)", model->getClassName());
		 writeLine("{");
		 writeLine("    return default_history_size;");
		 writeLine("}");
		 newLine();
		 writeLine("buffer_size_t %1::getDefaultBufferSize(void)", model->getClassName());
		 writeLine("{");
		 writeLine("    return default_buffer_size;");
		 writeLine("}");
		 newLine();
		 writeLine("std::string & %1::getFlowType(void)", model->getClassName());
		 writeLine("{");
		 writeLine("    return default_flow_type;");
		 writeLine("}");
		 newLine();
		 writeLine("std::string & %1::getFlowDesc(void)", model->getClassName());
		 writeLine("{");
		 writeLine("    return default_flow_desc;");
		 writeLine("}");
		 newLine();
		 writeLine("%1::%2() : super()", model->getClassName(), model->getClassName());
		 writeLine("{");
		 writeLine("    /* User must not do anything here - Use the init() method below instead */");
		 writeLine("}");
		 newLine();
		 writeLine("bool %1::init(FlowManager *manager,const std::string &type, const std::string &name, const std::string &rgroup, const std::string &args, flow_direction_t mode, flow_policy_t policy, bool isSynchronized)", model->getClassName());
		 writeLine("{");
		 writeLine("    bool err = super::init(manager, type, name, rgroup, args, mode, policy, isSynchronized);");
		 writeLine("    if ( !err ) return err; /* forward error */");
		 newLine();
		 writeLine("    buffer_size_t current_buffer_size = default_buffer_size;");
		 writeLine("    long current_history_size = default_history_size;");
		 writeLine("    int current_block_count = DUP_DEFAULT_BLOCK_COUNT;");
		 newLine();
		 writeLine("    string tmp = parseArgs(\"blocksize\",args);");
		 writeLine("    if ( ! tmp.empty() ) {");
		 writeLine("       istringstream is(tmp);");
		 writeLine("       is >> current_buffer_size;");
		 writeLine("    }");
		 newLine();
		 writeLine("    tmp = parseArgs(\"history\",args);");
		 writeLine("    if ( ! tmp.empty() ) {");
		 writeLine("       istringstream is(tmp);");
		 writeLine("       is >> current_history_size;");
		 writeLine("    }");
		 newLine();
		 writeLine("    tmp = parseArgs(\"blockcount\",args);");
		 writeLine("    if ( ! tmp.empty() ) {");
		 writeLine("       istringstream is(tmp);");
		 writeLine("       is >> current_block_count;");
		 writeLine("    }");
		 newLine();
		 writeLine("    if ( setMaxBufferSize(current_buffer_size) != 0 ) return -1;");
		 writeLine("    if ( setHistorySize(current_history_size) !=0  ) return -1;");
		 writeLine("    if ( setBlockCount(current_block_count) !=0  ) return -1;");
		 newLine();
		 writeLine("    if ( !user_init(args) ) return false;");
		 newLine();
		 writeLine("    return true;");
		 writeLine("}");
		 newLine();
		 writeLine("%1::~%2()", model->getClassName(), model->getClassName());
		 writeLine("{");
		 writeLine("    user_close();");
		 writeLine("}");
		 newLine();
		 writeLine("}  //namespace smartflow");
		 newLine();
		 return true;
	}
	return false;
}

/****************************************************************************************************/

Generate_functions_CPP::Generate_functions_CPP(FlowTypeModel *_model, QString basename) : Generate(_model, basename, "cpp", false) {
	includeName = model->getClassName();
	includeName.append(".h");
}

Generate_functions_CPP::~Generate_functions_CPP(){}

bool Generate_functions_CPP::generate_code_functions_CPP(){
	if ( needGeneration ) {
		writeLine("/* This file has been automatically generated by NSDFS Flow Generator");
		 writeLine(" * It will NOT be generated again if you don't remove it");
		 writeLine(" * You can implements methods in this file if needed");
		 writeLine(" */");
		 newLine();
		 writeLine("#include \"%1\"", includeName);
		 newLine();
		 writeLine("namespace smartflow {");
		 newLine();
		 writeLine("bool %1::user_init(const std::string &args)", model->getClassName());
		 writeLine("{");
		 writeLine("    /* User implementation here if needed - you can parse customs arguments here or initialize variables... */");
		 newLine();
		 writeLine("    return false;");
		 writeLine("}");
		 newLine();
		 writeLine("void %1::user_close()", model->getClassName());
		 writeLine("{");
		 writeLine("    /* User implementation here if needed - this method is called in the destructor */");
		 newLine();
		 writeLine("}");
		 newLine();
		 QStringList methodList = model->getSpecificMethods();
		 for ( QStringList::Iterator it = methodList.begin(); it != methodList.end(); ++it ) {
			 QStringList tmp_method = model->splitMethod((*it));
			 QStringList::Iterator it2 = tmp_method.begin();
			 
			 QString returnType = (*it2);
			 ++it2;
			 QString methodName = "";
			 for ( ; it2 != tmp_method.end(); ++it2 ) {
				 methodName.append((*it2));
				 methodName.append(" ");
			 }
			 methodName.remove(';');
			 writeLine("%1 %2::%3",returnType, model->getClassName(), methodName);
			 writeLine("{");
			 writeLine("    /* User implementation here */");
			 newLine();
			 writeLine("}");
			 newLine(); 
		 }
		 writeLine("}  //namespace smartflow");
		 newLine();
		 return true;
	}
	return false;
}

/****************************************************************************************************/

Generate_PRO::Generate_PRO(class FlowTypeModel *_model, QString basename) : Generate(_model, basename, "pro", false) {}

Generate_PRO::~Generate_PRO() {}

bool Generate_PRO::generate_code_PRO() {
	if ( needGeneration ) {
		writeLine("# This file has been automatically generated by NSDFS Flow Generator");
		writeLine("# It will NOT be generated again if don't remove it");
		writeLine("# You can modify this file if needed");
		writeLine("# In order to be used as it, this file and the sources files need to");
		writeLine("# be saved in the sub-directory src/flows/%1 of the Smartflow directory", model->getClassName());
		newLine();
		writeLine("include(../../../sf2.pro.inc)");
		writeLine("TEMPLATE    = lib");
		writeLine("INCLUDEPATH += . $$SF2/src/smartflow2");
		writeLine("DESTDIR = $$SF2/lib/flows");
		writeLine("LIBS     += $$SF2LIB -lsmartflow2");
		writeLine("QMAKE_CLEAN += $$SF2/lib/flows/lib%1*", model->getClassName());
		QString includeName = model->getClassName();
		includeName.append(".h");
		QString sourceName = model->getClassName();
		sourceName.append(".cpp");
		QString sourceFunctionsName = model->getClassName();
		sourceFunctionsName.append("_functions.cpp");
		writeLine("SOURCES  += %1 %2", sourceName, sourceFunctionsName);
		writeLine("HEADERS  += %1",includeName);
		newLine();
		return true;
	}
	return false;
}

