#include "MapView.h"
#include "PathFinding.h"
#include "model/Client.h"
#include "model/Duplicator.h"
#include "model/Server.h"
#include "model/GlobalCC.h"
#include "model/Flow.h"
#include "model/SfControl.h"
#include "dialog/ClientAppEditor.h"
#include "mainwindow.h"
#include "dialog/FlowInformation.h"
#include "dialog/AddClientTool.h"
#include "dialog/LogView.h"

#include <QtGui/QMatrix>
//#include <q3dragobject.h>
#include <QtGui/QPixmap>
#include <QtCore/QEvent>
#include <QtGui/QStatusBar>
#include <QtCore/QTimer>
#include <QtGui/QMessageBox>
#include <QtGui/QFont>

#include <iostream>
#include <iterator>
#include <set>
#include <deque>
using namespace std;

bool MapView::event(QEvent *event) {
	if (event->type() == QEvent::ToolTip) {
        QHelpEvent *helpEvent = static_cast<QHelpEvent *>(event);
		QPoint p = inverseWorldMatrix().map(helpEvent->pos());
		QString toolTipText = tip(p);
		if ( toolTipText != "" )
			QToolTip::showText(helpEvent->globalPos(), toolTipText);
		else
			QToolTip::hideText();
	}
	return QWidget::event(event);
}

void MapView::contentsMouseMoveEvent ( QMouseEvent * e ) {
	//cout << "ZOOM LEVEL=" << zoom << endl;
	//std::cout << "CONTENTS w=" << contentsWidth() << " h=" << contentsHeight() << std::endl;
	//std::cout << "VISIBLE w2=" << width() << " h2=" << height() << std::endl;
    MainWindow::instance()->statusBar()->message(getInfo(e->pos()));
	if(isMoving){
		QPoint p = inverseWorldMatrix().map(e->pos());	
		int x, y;
		if ( (p.x() -d_x + widthItem + TILE_SX) > (contentsWidth()/zoom) )
			//if ( (p.x() -d_x + widthItem + TILE_SX) > (contentsWidth()) )
			x = (int) (contentsWidth()/zoom - widthItem + d_x - TILE_SX);
		//x = (int) (contentsWidth() - widthItem + d_x - TILE_SX);
		else if ( (p.x()-d_x-TILE_SX) < 0 )
			x = d_x+TILE_SX;
		else
			x = p.x();
		if ( (p.y() -d_y + heightItem + TILE_SY) > (contentsHeight()/zoom) )
			y = (int) (contentsHeight()/zoom - heightItem + d_y - TILE_SY);
		//if ( (p.y() -d_y + heightItem + TILE_SY) > (contentsHeight()) )
		// y = (int) (contentsHeight() - heightItem + d_y - TILE_SY);
		else if ( (p.y()-d_y-TILE_SY) < 0 )
			y = d_y+TILE_SY;
		else
			y = p.y();
		
		int new_x = (int) ((x - d_x)/TILE_SX);
		int new_y = (int) ((y - d_y)/TILE_SY);
		if ( ((x - d_x)%TILE_SX) > 5 )
			new_x++;
		if ( ((y - d_y)%TILE_SY) > 5 )
			new_y++;
		new_x *=TILE_SX;
		new_y *=TILE_SY;
		
		if ( (mo->x() != new_x) || (mo->y() != new_y) ) {
			int old_x = (int) mo->x();
			int old_y = (int) mo->y();
			mo->moveRequest(new_x, new_y);
			Q3CanvasItemList il = mo->collisions (true);
			Q3CanvasItemList::iterator it; 
			bool collides = false;
			for ( it = il.begin(); it != il.end(); ++it )
				if ( (*it)->rtti() == ClientRTTI ) {
					collides = true;
					ClientGraphicObject *cl = (ClientGraphicObject*)(*it);
					clientsCol.insert(cl->getId());
				}
					if ( collides ) {
						mo->moveRequest(old_x, old_y);
					}
					else {
						mo->move();
						update();
					}
		}
	}
		else {
			QPoint pt = inverseWorldMatrix().map(e->pos());
			if(pt != moving_start) {
				int x1, x2, y1, y2;
				//int x, y;
				if(pt.x() > moving_start.x()) {
					x1 = moving_start.x();
					x2 = pt.x();
				} 
				else {
					x1 = pt.x();
					x2 = moving_start.x();
				}
				
				if(pt.y() > moving_start.y()) {
					y1 = moving_start.y();
					y2 = pt.y();
				}
				else {
					y1 = pt.y();
					y2 = moving_start.y();
				}
				if(isBoxing) {
					static_cast<MapCanvas *>(canvas())->setZoneRect(QRect(x1, y1, x2-x1+1, y2-y1+1));
					
					x1 /= TILE_SX;
					y1 /= TILE_SY;
					x2 = (x2 + TILE_SX-1)/TILE_SX;
					y2 = (y2 + TILE_SY-1)/TILE_SY;
					
					position.set(x1, x2, y1, y2);
					update();
				}
				else if ( connecting ) {
					QPoint pt_tmp = inverseWorldMatrix().map(e->pos());
					if (pt_tmp != moving_start) {
						static_cast<MapCanvas *>(canvas())->drawTheLine(moving_start, pt_tmp ,QRect(x1, y1, x2-x1+1, y2-y1+1));
						ClientGraphicObject *co = 0;
						FlowGraphicObject *fo = 0;
						FlowGraphicOutlet *oo = 0;
						findObject(pt_tmp, co, fo, oo);
						QString consoIDNotUsedHere = "";
						if ( oo ) {
						        if ( flo->getModel()->connectable(oo->getModel(), false, consoIDNotUsedHere) ) {
								removeAllSelected();
								addSelected(oo);
							}
						}
						else if ( fo ) {
						        if ( flo->getModel()->connectable(fo->getModel(), false, consoIDNotUsedHere) ) {
								removeAllSelected();
								addSelected(fo);
							}
						}
						else
							removeAllSelected();
					}
					update();
				}
			}
		}
}




void MapView::contentsMouseReleaseEvent ( QMouseEvent * e ){
	if (isMoving) {
		isMoving = false;
		
		mo = 0;
		set<int>::iterator iter;
		ClientGraphicObject *cl;
		cl = 0;
		for(iter = clientsCol.begin(); iter != clientsCol.end(); iter++ ) {
			cl = getClient(*iter);
			if ( cl ) {
				cl->move();
			}
			cl =  NULL;
		}
		if (!flowsCol.empty()) {
			FlowGraphicObject *tmp_f;
			tmp_f = 0;
			//erase every flows, which has an intersection with the client
			for(iter = flowsCol.begin(); iter != flowsCol.end(); iter++ ) {
				flowsCol.insert(*iter);
				tmp_f = getFlow(*iter);
				if ( tmp_f )
					tmp_f->erasePath();
				tmp_f = 0;
			}
			//draw every flows, which had an intersection with the client
			for(iter = flowsCol.begin(); iter != flowsCol.end(); iter++ ) {
				tmp_f = getFlow(*iter);
				if ( tmp_f )
					tmp_f->drawPathInMap();
				tmp_f = 0;
			}
		}
		update();
	}// end if(isMoving)
	else if(isBoxing) {
		static_cast<MapCanvas *>(canvas())->setZoneRect(QRect());
		
		int x, y;
		QPoint pt;
		//If the selection area is under the map, we correct it
		position.test();
		ClientGraphicObject *co = 0;
		FlowGraphicObject *fo = 0;
		FlowGraphicOutlet *oo = 0;
		AbstractGraphicObject *obj = 0;
		removeAllSelected();
		
		for(y=position.y1; y<position.y2; y++) {
			pt.setY(y*TILE_SY);
			for(x=position.x1; x<position.x2; x++) {
				pt.setX(x*TILE_SX);
				if ( findObject(pt, co, fo, oo) ) 
					obj = co ? static_cast<AbstractGraphicObject *>(co) : ( fo ? static_cast<AbstractGraphicObject *>(fo) : static_cast<AbstractGraphicObject *>(oo) );
				if ( obj ) {
					obj->setSelected(true);
					addSelected(obj);
					obj = 0;
				}
			}
		}
		position.clear();
		
		update();
		isBoxing = false;
	}
	else if ( connecting ) {
		static_cast<MapCanvas *>(canvas())->drawTheLine(QPoint(), QPoint(), QRect());
		ClientGraphicObject *co = 0;
		FlowGraphicObject *fo = 0;
		FlowGraphicOutlet *oo = 0;
		QPoint pt_tmp = inverseWorldMatrix().map(e->pos());
		findObject(pt_tmp, co, fo, oo);
		if ( oo ) {
			flo->getModel()->connect(oo->getModel(), false);
			removeAllSelected();
		}
		else if ( fo ) {
			flo->getModel()->connect(fo->getModel(), false);
			removeAllSelected();
		}
		update();
		connecting = false;
	}
}

void MapView::contentsMousePressEvent ( QMouseEvent * e ){
	QPoint p = inverseWorldMatrix().map(e->pos());
	isMoving = false;
	isBoxing = false;
	flowsCol.clear();
	clientsCol.clear();
	
	ClientGraphicObject *co = 0;
	FlowGraphicObject *fo = 0;
	FlowGraphicOutlet *oo = 0;
	AbstractGraphicObject *obj = 0;
	if ( findObject(p, co, fo, oo) )
		obj = co ? static_cast<AbstractGraphicObject *>(co) : ( fo ? static_cast<AbstractGraphicObject *>(fo) : static_cast<AbstractGraphicObject *>(oo) );
	
	switch (e->button()) {
		case Qt::LeftButton: {
			if ( co ) {
				d_x = p.x() - (int) co->x();
				d_y = p.y() - (int) co->y();
				widthItem = 0;
				heightItem = 0;
				widthItem = co->width();
				heightItem =  co->height();
				moving_start = p;
				isMoving = true;
				connecting = false;
				mo = co;
				flo = 0;
				MainWindow::instance()->logView->setClientTab(co->getModel()->getId());
			}
			else if ( oo ) {
				isMoving = false;
				connecting = true;
				moving_start = p;
				mo = 0;
				flo = oo;
				oo->switchToLogTab();
			}
			
			bool toggle = e->state() & Qt::ControlButton;
			if(!toggle && (!co || !co->isSelected()) && (!fo || !fo->isSelected()) && (!oo || !oo->isSelected()))
				removeAllSelected();
			
			position.clear();
			
			if ( !obj ) {
				if ( hasAddClientDialog ) {
					if ( addClientTool->getCurrentButton() != 0 ) {
						removeAllSelected();
						QPoint p = mapFrom(this,e->pos());
						p = inverseWorldMatrix().map(p);
						
						if ( (p.x() + 100 + TILE_SX) > contentsWidth()/zoom )
							p.setX((int)(contentsWidth()/zoom - 100 - TILE_SX));
						else if ( (p.x()-TILE_SX) < 0 )
							p.setX(TILE_SX);
						if ( (p.y() + 100 + TILE_SY) > contentsHeight()/zoom )
							p.setY((int)(contentsHeight()/zoom - 100 - TILE_SY));
						else if ( (p.y()-TILE_SY) < 0 )
							p.setY(TILE_SY); 
						
						createClientFromAddClientTool(p);
					}
				}
				else {
					removeAllSelected();
					moving_start = p;
					isBoxing = true;
					createClient();
				}
			}
			else {
				if ( obj->isSelected() ) {
					if ( toggle ) {
						removeSelected(obj);
						isMoving = false;
					}
					else {
						removeAllSelected();
						addSelected(obj);
					}
				}
				else {
					if ( toggle ) {
						addSelected(obj);
						isMoving = false;
					}
					else {
						removeAllSelected();
						addSelected(obj);
					}
				}
			}
			break;
		}
		case Qt::RightButton: {
			removeAllSelected();
			if(co) {
				addSelected(obj);
				co->showPopupMenu(e->globalPos());
			}
			else if(fo) {
				addSelected(obj);
				
				QPoint tmp_pt = mapFrom(this, e->pos());
				Q3CanvasItemList lst = canvas()->collisions(tmp_pt);
				int i = 0;
				Q3CanvasItemList::iterator it;
				SubFlowGraphicObject *tmp_subFlow = NULL;
				for ( it = lst.begin(); it != lst.end(); it++ ) {
					if ( (*it)->rtti() == SubFlowRTTI ) {
						i++;
						tmp_subFlow = dynamic_cast<SubFlowGraphicObject *>(*it); 
					}
				}
				if ( (i > 1) || (fo->countSubFlows() == 1) ) {
					fo->unsetSubFlowToBreak();
					fo->unsetSubFlowToPause();
				}
				else {
					fo->setSubFlowToBreak(tmp_subFlow);
					fo->setSubFlowToPause(tmp_subFlow);
				}
				
				fo->showPopupMenu(e->globalPos());
			}  
			else if(oo) {
				addSelected(obj);
				oo->showPopupMenu(e->globalPos());
			}  
			else
				showPopupMenu(e->globalPos());
			break;
		}  
		default:
			;
	}
}

void MapView::contentsMouseDoubleClickEvent ( QMouseEvent * e ) {
	QPoint p = inverseWorldMatrix().map(e->pos());
	ClientGraphicObject *co = 0;
	FlowGraphicObject *fo = 0;
	FlowGraphicOutlet *oo = 0;
	findObject(p, co, fo, oo);
	if ( co ) {
		co->getModel()->printInfo();
	}
	else if ( fo ) {
		fo->getModel()->printInfo();
	}
	else if ( oo ) {
		cout << "FlowGraphicOutlet, id=" << oo->getId() << endl;
	}
	else {
		pf->printIdMap();
		pf->printMap();
		cout << "NB Clients=" << clientObjects.size() << endl;
		cout << "NB Outlets=" << flowOutlets.size() << endl;
		cout << "NB Flows=" << flowObjects.size() << endl;
	}
    
	/*else {
		std::map<unsigned int, FlowGraphicObject *>::iterator iter;
    for ( iter = flowObjects.begin(); iter != flowObjects.end(); iter ++ )
		cout << "FlowObject ID=" << iter->first << endl;
	}*/
	
}

void MapView::dragEnterEvent( QDragEnterEvent *event ) {
    event->accept(event->provides("text/plain"));    
}

void MapView::dropEvent( QDropEvent *event ) {
    QString drop_result = event->mimeData()->text();
	ClientConf *cc =  MainWindow::instance()->global->getClientConfList()->getClient(drop_result.toUInt());	
	
	QPoint p = mapFrom(this,event->pos());
	int tmp_x = 0;
	int tmp_y = 0;
	viewportToContents (  p.x(), p.y(), tmp_x, tmp_y);
	p.setX(tmp_x); p.setY(tmp_y);
	p = inverseWorldMatrix().map(p);
	
	if ( (p.x() + 100 + TILE_SX) > (contentsWidth()/zoom) )
		p.setX((int) (contentsWidth()/zoom - 100 - TILE_SX));
	else if ( (p.x()-TILE_SX) < 0 )
		p.setX(TILE_SX);
	if ( (p.y() + 100 + TILE_SY) > (contentsHeight()/zoom) )
		p.setY((int) (contentsHeight()/zoom - 100 - TILE_SY));
	else if ( (p.y()-TILE_SY) < 0 )
		p.setY(TILE_SY);
	
	dropPoint = p;
	clientDrop = cc;
	QTimer::singleShot( 10, this, SLOT(createClient()) );
}

void MapView::keyReleaseEvent(QKeyEvent * e) {
	//std::cout << "We catched a keyboard event" << std::endl;
	if ( e->key() == Qt::Key_Delete ) {
		//if there are some objects, which are selected, we remove them from the map
		//we don't remove outlets.
		//first we remove the flows, breaking the link, then the selected clients. 
		if ( !selected_objects.empty() ) {
			std::set<AbstractGraphicObject *>::iterator iter;
			iter = selected_objects.begin();
			while ( iter != selected_objects.end() ) {
				AbstractGraphicObject *obj = (*iter);
				if ( obj ) {
					unsigned int temp_id = obj->getId();
					if ( flowExists(temp_id) ) {
						FlowGraphicObject *fgo = dynamic_cast<FlowGraphicObject *>(obj);
						if ( fgo && !fgo->getModel()->isRunning() ) {
							removeSelected(obj);
							iter = selected_objects.begin();
							fgo->unsetSubFlowToBreak();
							fgo->breakLinkAction();
						}
						else
							iter++;
					}
					else
						iter++;
				}
			}
			iter = selected_objects.begin();
			while ( iter != selected_objects.end() ) {
				//for ( iter = selected_objects.begin(); iter != selected_objects.end(); iter++ ) {
				AbstractGraphicObject *obj = (*iter);
				if ( obj ) {
					unsigned int temp_id = obj->getId();
					if ( clientExists(temp_id) ) {
						ClientGraphicObject *cgo = dynamic_cast<ClientGraphicObject *>(obj);
						if ( cgo && (!cgo->getModel()->isRunning() && !cgo->getModel()->isStarting() ) ) {
							removeSelected(obj);
							iter = selected_objects.begin();
							cgo->removeClientAction();
						}
						else
							iter++;
					}
					else 
						iter++;
				}
				}
			}
		}
	else if ( e->key() == Qt::Key_Minus ) {
		zoomOut();
	}
	else if ( e->key() == Qt::Key_Plus ) {
		zoomIn();
	}
	}

void MapView::createClient() {
	if ( clientDrop ) {
		ClientAppEditor *editor = new ClientAppEditor(clientDrop, this); //source of event
		editor->setModal(true);
		editor->setCaption(tr("Add Client"));
		if ( editor->exec() == QDialog::Accepted ) {
			//everything's ok we add the clien
			Server *s = editor->getServer();
			if ( s ) {
				QString param = editor->getParam();
				MainWindow::instance()->global->getApplication()->createClientApp(clientDrop, s, dropPoint, param);
			}	
		}
		clientDrop = NULL;
	}
}

void MapView::createClientFromAddClientTool(QPoint &position) {
	if ( hasAddClientDialog ) {
		ClientToolButton *clientButton = addClientTool->getCurrentButton();
		if ( clientButton ) {
			ClientConf *cc = clientButton->getModel();
			if ( cc ) {
				ClientAppEditor *editor = new ClientAppEditor(cc, this);
				editor->setModal(true);
				editor->setCaption(tr("Add Client"));
				if ( editor->exec() == QDialog::Accepted ) {
					//everything's ok we add the clien
					Server *s = editor->getServer();
					if ( s ) {
						QString param = editor->getParam();
						MainWindow::instance()->global->getApplication()->createClientApp(cc, s, position, param);
					}	
				}
			}
		}
	}
}


MapView::~MapView(){
	//removeAllClients();
	if ( mo ) {
		delete mo;
		mo = NULL;
	}
	if ( pf ) {
		delete pf;
		pf = NULL;
	}
	/*  if ( t ) {
	delete t;
	t = NULL;
	}*/
	if ( trashBin ) {
		delete trashBin;
		trashBin = NULL;
	}
	//deletePixmap();
}

MapView::MapView (QWidget* parent, const char* name, Qt::WFlags f) : Q3CanvasView (0, parent, name , f){
	setCanvas(new MapCanvas);
	
	//setHScrollBarMode(Q3ScrollView::AlwaysOff);
	//setVScrollBarMode(Q3ScrollView::AlwaysOff);
	resize(canvas()->size());
	
	zoom = 1;
	isMoving = false;
	isBoxing = false;
	connecting = false;
	flo = 0;
	mo = 0;
	newId = 0;
	memset(pixs, 0, 60*sizeof(void*));
	//initPixmap();
	pf = new PathFinding();
	pf->setMapView(this);
	position.clear();
	selected_objects.clear();
	flowOutlets.clear();
	flowObjects.clear();
	clientObjects.clear();
	setAcceptDrops(true);
	setCaption("Map");
	//t = new DynamicTip( this );
	viewport()->setMouseTracking(true);
	clientDrop = NULL;
	addClientTool = NULL;
	hasAddClientDialog = false;
	trashBin = new Q3Canvas();
}

void MapView::addSelected(AbstractGraphicObject *obj)
{
	if(selected_objects.find(obj) == selected_objects.end()) {
		selected_objects.insert(obj);
		obj->setSelected(true);
	}
}

void MapView::removeSelected(AbstractGraphicObject *obj)
{
	set<AbstractGraphicObject *>::iterator i = selected_objects.find(obj);
	if(i != selected_objects.end()) {
		selected_objects.erase(i);
		obj->setSelected(false);
	}
	ClientGraphicObject *cgo = dynamic_cast<ClientGraphicObject *>(obj);
	if ( cgo ) {
		FlowOutletOut *fo = 0;
		std::map<unsigned int, FlowOutletOut*> *fout = cgo->getModel()->getOutputFlowList();
		std::map<unsigned int, FlowOutletOut *>::iterator iter_out;
		for ( iter_out = fout->begin(); iter_out != fout->end(); iter_out++ ) {
			fo = iter_out->second;
			if(fo) {
				set<AbstractGraphicObject *>::iterator it_out = selected_objects.find(fo->getFlowOutletGraphic());
				if(it_out != selected_objects.end()) {
					selected_objects.erase(it_out);
					fo->getFlowOutletGraphic()->setSelected(false);
				}
			}
			fo = 0; 
		}
		FlowOutletIn *fi = 0;
		std::map<unsigned int, FlowOutletIn*> *fin = cgo->getModel()->getInputFlowList();
		std::map<unsigned int, FlowOutletIn*>::iterator iter_in;
		for ( iter_in = fin->begin(); iter_in != fin->end(); iter_in++ ) {
			fi = iter_in->second;
			if(fi) {
				set<AbstractGraphicObject *>::iterator it_in = selected_objects.find(fi->getFlowOutletGraphic());
				if(it_in != selected_objects.end()) {
					selected_objects.erase(it_in);
					fi->getFlowOutletGraphic()->setSelected(false);
				}
			}
			fi = 0; 
		}
	} 
}

void MapView::removeAllSelected()
{
	for(std::set<AbstractGraphicObject *>::iterator i = selected_objects.begin(); i!=selected_objects.end(); i++)
		(*i)->setSelected(false);
	selected_objects.clear();
}

Q3CanvasPixmapArray* MapView::getPixmap(int i, unsigned int size) {
	Q3CanvasPixmapArray *result = NULL;
	result = new Q3CanvasPixmapArray();
	int thepix = i+size*20;
	switch(thepix) {
		case ON:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegreen50.png")));
			break;
		case ON_S:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegreen50s.png")));
			break;
		case ON_LOG:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegreen50l.png")));
			break;
		case ON_LOG_S:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegreen50ls.png")));
			break;	
		case IDLE:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubeyellow50.png")));
			break;
		case IDLE_S:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubeyellow50s.png")));
			break;
		case IDLE_LOG:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubeyellow50l.png")));
			break;
		case IDLE_LOG_S:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubeyellow50ls.png")));
			break;
		case START:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im500000.png")));
			result->setImage(1, new Q3CanvasPixmap(QPixmap(":/images/im500001.png")));
			break;
		case START_S:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im50s0000.png")));
			result->setImage(1, new Q3CanvasPixmap(QPixmap(":/images/im50s0001.png")));
			break;
		case START_LOG:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im50l0000.png")));
			result->setImage(1, new Q3CanvasPixmap(QPixmap(":/images/im50l0001.png")));
			break;
		case START_LOG_S:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im50ls0000.png")));
			result->setImage(1, new Q3CanvasPixmap(QPixmap(":/images/im50ls0001.png")));
			break;
		case DISABLE:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegrey50.png")));
			break;
		case DISABLE_S:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegrey50s.png")));
			break;
		case DISABLE_LOG:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegrey50l.png")));
			break;
		case DISABLE_LOG_S:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegrey50ls.png")));
			break;
		case ERR:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubered50.png")));
			break;
		case ERR_S:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubered50s.png")));
			break;
		case ERR_LOG:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubered50l.png")));
			break;
		case ERR_LOG_S:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubered50ls.png")));
			break;
			
		case ON+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegreen70.png")));
			break;
		case ON_S+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegreen70s.png")));
			break;
		case ON_LOG+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegreen70l.png")));
			break;
		case ON_LOG_S+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegreen70ls.png")));
			break;	
		case IDLE+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubeyellow70.png")));
			break;
		case IDLE_S+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubeyellow70s.png")));
			break;
		case IDLE_LOG+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubeyellow70l.png")));
			break;
		case IDLE_LOG_S+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubeyellow70ls.png")));
			break;
		case START+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im700000.png")));
			result->setImage(1, new Q3CanvasPixmap(QPixmap(":/images/im700001.png")));
			break;
		case START_S+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im70s0000.png")));
			result->setImage(1, new Q3CanvasPixmap(QPixmap(":/images/im70s0001.png")));
			break;
		case START_LOG+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im70l0000.png")));
			result->setImage(1, new Q3CanvasPixmap(QPixmap(":/images/im70l0001.png")));
			break;
		case START_LOG_S+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im70ls0000.png")));
			result->setImage(1, new Q3CanvasPixmap(QPixmap(":/images/im70ls0001.png")));
			break;
		case DISABLE+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegrey70.png")));
			break;
		case DISABLE_S+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegrey70s.png")));
			break;
		case DISABLE_LOG+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegrey70l.png")));
			break;
		case DISABLE_LOG_S+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegrey70ls.png")));
			break;
		case ERR+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubered70.png")));
			break;
		case ERR_S+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubered70s.png")));
			break;
		case ERR_LOG+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubered70l.png")));
			break;
		case ERR_LOG_S+20:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubered70ls.png")));
			break;
			
		case ON+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegreen90.png")));
			break;
		case ON_S+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegreen90s.png")));
			break;
		case ON_LOG+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegreen90l.png")));
			break;
		case ON_LOG_S+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegreen90ls.png")));
			break;	
		case IDLE+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubeyellow90.png")));
			break;
		case IDLE_S+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubeyellow90s.png")));
			break;
		case IDLE_LOG+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubeyellow90l.png")));
			break;
		case IDLE_LOG_S+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubeyellow90ls.png")));
			break;
		case START+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im900000.png")));
			result->setImage(1, new Q3CanvasPixmap(QPixmap(":/images/im900001.png")));
			break;
		case START_S+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im90s0000.png")));
			result->setImage(1, new Q3CanvasPixmap(QPixmap(":/images/im90s0001.png")));
			break;
		case START_LOG+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im90l0000.png")));
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im90l0001.png")));
			break;
		case START_LOG_S+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/im90ls0000.png")));
			result->setImage(1, new Q3CanvasPixmap(QPixmap(":/images/im90ls0001.png")));
			break;
		case DISABLE+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegrey90.png")));
			break;
		case DISABLE_S+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegrey90s.png")));
			break;
		case DISABLE_LOG+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegrey90l.png")));
			break;
		case DISABLE_LOG_S+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubegrey90ls.png")));
			break;
		case ERR+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubered90.png")));
			break;
		case ERR_S+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubered90s.png")));
			break;
		case ERR_LOG+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubered90l.png")));
			break;
		case ERR_LOG_S+40:
			result->setImage(0, new Q3CanvasPixmap(QPixmap(":/images/cubered90ls.png")));
			break;
		default:
			delete result;
			result = NULL;
	}
	return result;
}

bool MapView::flowExists(unsigned int id) {
	map<unsigned int, FlowGraphicObject *>::iterator iter;
	iter = flowObjects.find(id);
	if (iter != flowObjects.end()) 
		return true;
	else
		return false;
}

bool MapView::clientExists(unsigned int id) {
	map<unsigned int, ClientGraphicObject *>::iterator iter;
	iter = clientObjects.find(id);
	if (iter != clientObjects.end()) 
		return true;
	else
		return false;
}

bool MapView::outletExists(unsigned int id) {
	map<unsigned int, FlowGraphicOutlet *>::iterator iter;
	iter = flowOutlets.find(id);
	if (iter != flowOutlets.end()) 
		return true;
	else
		return false;
}

void MapView::addClient(unsigned int id, ClientGraphicObject *obj) {
	if ( !clientExists(id) )
		clientObjects[id] = obj;
}

void MapView::addFlow(unsigned int id, FlowGraphicObject *obj) {
	if ( !flowExists(id) )
		flowObjects[id] = obj;
}

void MapView::addOutlet(unsigned int id, FlowGraphicOutlet *obj) {
	if ( !outletExists(id) )
		flowOutlets[id] = obj;
}

void MapView::remClient(unsigned int id) {
	if ( clientExists(id) ) {
		clientObjects[id]->hide();
		clientObjects.erase(id);
	} 
}

void MapView::removeAllClients() {
	std::map<unsigned int, ClientGraphicObject *>::iterator iter; 
	for ( iter = clientObjects.begin(); iter != clientObjects.end(); iter++ ) {
		iter->second->removeClientAction();
	}
	clientObjects.clear();
}

void MapView::remFlow(unsigned int id) {
	if ( flowExists(id) ) 
		flowObjects.erase(id);
}

void MapView::remOutlet(unsigned int id) {
	if ( outletExists(id) ) 
		flowOutlets.erase(id);
} 

/*Q3CanvasPixmapArray* MapView::getPixmap(int i, unsigned int size) {
Q3CanvasPixmapArray *tmpPix = new Q3CanvasPixmapArray(*(pixs[i+size*20]));
return pixs[i+size*20];
}*/

void MapView::update(){
	canvas()->update();
}

void MapView::clear(){
	//for ( int y=0; y < SIZEMAPY; y++)
	// for ( int x=0; x < SIZEMAPX; x++) 
	//   canvas()->setTile(x,y,0);
	//do we need to remove all objects?
	pf->clearMap();
	
}

bool MapView::findObject(QPoint pos, ClientGraphicObject *&co, FlowGraphicObject *&fo, FlowGraphicOutlet *&oo)
{
	int tx = pos.x()/TILE_SX;
	int ty = pos.y()/TILE_SY;
	int id = pf->getMapCellId(tx, ty);
	co = 0;
	fo = 0;
	oo = 0;
	if ( id != 0 ) {
		if( flowExists(id) ) {
			//very nasty....
			//When we look for an object, we looked into the map handled by the Pathfinding object.
			//This object doesn't make any difference between flow and flowOutlet.
			//It is therefore required to look directly on the Graphical map to check if we are located
			//above an outlet of the flow
			QPoint tmp_pt = mapFrom(this, pos);
			Q3CanvasItemList lst = canvas()->collisions(tmp_pt);
			int i = 0;
			Q3CanvasItemList::iterator it;
			FlowGraphicOutlet *tmp_outlet = NULL;
			for ( it = lst.begin(); it != lst.end(); it++ ) {
				if ( (*it)->rtti() ==  FlowGraphicOutletRTTI ) {
					i++;
					tmp_outlet = dynamic_cast<FlowGraphicOutlet *>(*it);
					oo = tmp_outlet;
					return true;
					//cout << "Outlet detected" << endl;
				}
			}
			fo = flowObjects[id];
			return true;
		}
		else if ( clientExists(id) ) {
			co = clientObjects[id];
			return true;
		}
		else if ( outletExists(id) ) {
			oo = flowOutlets[id];
			return true;
		}
	}
	return false;
}

FlowGraphicObject* MapView::getFlow(unsigned int id) {
	if ( flowExists(id) )
		return flowObjects[id];
	else 
		return NULL;
}

ClientGraphicObject* MapView::getClient(unsigned int id) {
	if ( clientExists(id) )
		return clientObjects[id];
	else 
		return NULL;
}

void MapView::showPopupMenu(const QPoint &p)
{
	popupMenu.clear();
	popupMenu.insertItem(tr("&Add Client..."), this, SLOT(addClientAction()));
	popupMenu.insertSeparator();
	SfControlThread *sfct = MainWindow::instance()->global->getSfControlThread();
	if ( sfct ) {
		SfControl *sfc = sfct->getSfControl();
		if ( sfc ) {
			if ( sfc->isConnectedToServer() ) {
				popupMenu.insertItem(tr("S&top All Clients"), this, SLOT(stopAllClientsAction()));
				popupMenu.insertItem(tr("&Start All Clients"), this, SLOT(startAllClientsAction()));
				popupMenu.insertSeparator();
			}
		}
	}
	popupMenu.insertItem(tr("Set Clients &Position"), this, SLOT(setClientsPositionAction()));
	popupMenu.popup(p);
} 

void MapView::trashGraphicObject(Q3CanvasItem *item) {
	//this method is tricky... We do not remove the flows, which are composed of SubFlowGraphicObject, from the canvas because if we do, we generate a segfault. Deleting an object, which is a subclass of a QCanvasPolygon or QCanvasPolygonItem, produce a segfault. This is a Qt bug. So instead of deleting the object, we associate it with an other canvas. 
	item->setCanvas(trashBin);
}

QString& MapView::getInfo(QPoint p) {
	QString *result = new QString("");
	ClientGraphicObject *co = 0;
	FlowGraphicObject *fo = 0;
	FlowGraphicOutlet *oo = 0;
	findObject(p, co, fo, oo);
	if ( co ) {
		result->append("Name: ");
		result->append(co->getModel()->getName());
	}
	else if ( fo ) {
		result->append("Name: ");
		result->append(fo->getModel()->getOutletOut()->getName());
		result->append(" -- Type: ");
		result->append(fo->getModel()->getDataType());
		result->append(" -- Connection ID: ");
		result->append(fo->getModel()->getOutletOut()->getGroup());
	}
	else if ( oo ) {
		result->append("Name: ");
		result->append(oo->getModel()->getName());
		result->append(" -- Type: ");
		result->append(oo->getModel()->getDataType());
		result->append(" -- Connection ID: ");
		result->append(oo->getModel()->getGroup());
	}
	return *result;
}

void MapView::refreshFlows() {
	std::map<unsigned int, FlowGraphicObject *>::iterator iter_f; 
	for ( iter_f = flowObjects.begin(); iter_f != flowObjects.end(); iter_f++ ) {
		iter_f->second->drawPathInMap();
		iter_f->second->refresh();
	}
}

void MapView::refreshClientsWithoutFlows() {
	std::map<unsigned int, ClientGraphicObject *>::iterator iter; 
	for ( iter = clientObjects.begin(); iter != clientObjects.end(); iter++ ) {
		iter->second->drawClientWithOutlets();
	}
}

void MapView::startAllClientsAction() {
	SfControlThread *sfct = MainWindow::instance()->global->getSfControlThread();
	if ( sfct ) {
		SfControl *sfc = sfct->getSfControl();
		if ( sfc ) 
			//sfc->startAllClients();
			sfc->emitStartAllClientsSignal();
	}
}

void MapView::stopAllClientsAction() {
	SfControlThread *sfct = MainWindow::instance()->global->getSfControlThread();
	if ( sfct ) {
		SfControl *sfc = sfct->getSfControl();
		if ( sfc )
			//sfc->stopAllClients();
			sfc->emitStopAllClientsSignal();
	}
}

void MapView::setClientsPositionAction() {
	setClientsPosition();
}

void MapView::addClientAction() {
	if ( !addClientTool ) {
		addClientTool = new AddClientTool(this);
		addClientTool->initToolButton();
		hasAddClientDialog = true;
	}
	addClientTool->show();
}

void MapView::deleteTheFlow(Flow *f) {
	if (f)
		delete f;
}

void MapView::deleteTheClient(ClientApp *c) {
	if ( c ) {
		delete c;
	}
}

void MapView::deleteAddClientTool() {
	if ( addClientTool ) {
		delete addClientTool;
		addClientTool = NULL;
		hasAddClientDialog = false;
	}
}

void MapView::zoomIn() {
	QMatrix m = worldMatrix();
	QSize s = sizeHint();
	//cout << "---------------> BEFORE SIZEHINT h=" << s.height() << " w=" << s.width() << endl;
	//cout << "ZoomIn Before scale m11=" << m.m11() << " m12=" << m.m12() << " m21=" << m.m21() << " m22=" << m.m22() << " det=" << m.det() << " dx=" << m.dx() << " dy=" << m.dy() << endl; 
	m.scale( 2.0, 2.0 );
	//cout << "ZoomIn After  scale m11=" << m.m11() << " m12=" << m.m12() << " m21=" << m.m21() << " m22=" << m.m22() << " det=" << m.det() << " dx=" << m.dx() << " dy=" << m.dy() << endl;
	setWorldMatrix( m );
	zoom *= 2;
	// s = sizeHint();
	//cout << "---------------> AFTER SIZEHINT h=" << s.height() << " w=" << s.width() << endl;
	//resizeContents((int)(TILE_SX*SIZEMAPX*zoom),(int)(TILE_SY*SIZEMAPY*zoom));
	//resizeContents((int)(contentsHeight()),(int)(contentsWidth())); 
	//resize((int)(contentsHeight()),(int)(contentsWidth())); 
	//cout << "ZOOM In CONTENT h=" << contentsHeight() << " w=" << contentsWidth() << endl;
	//cout << "ZOOM In VISIBLE h=" << height() << " w=" << width() << endl;
	//cout << "chunk size=" << canvas()->chunkSize() << endl; 
	//cout << "LEAVING ZOOM IN " << endl; 
}

void MapView::zoomOut() {
	QMatrix m = worldMatrix();
	//cout << "ZoomOut Before scale m11=" << m.m11() << " m12=" << m.m12() << " m21=" << m.m21() << " m22=" << m.m22() << " det=" << m.det() << " dx=" << m.dx() << " dy=" << m.dy() << endl; 
	m.scale( 0.5, 0.5 );
	//cout << "ZoomOut After  scalem 11=" << m.m11() << " m12=" << m.m12() << " m21=" << m.m21() << " m22=" << m.m22() << " det=" << m.det() << " dx=" << m.dx() << " dy=" << m.dy() << endl; 
	setWorldMatrix( m );
	zoom /= 2;
	//resizeContents((int)(TILE_SX*SIZEMAPX*zoom),(int)(TILE_SY*SIZEMAPY*zoom));
	//resizeContents((int)(contentsHeight()),(int)(contentsWidth())); 
	//resize((int)(contentsHeight()),(int)(contentsWidth()));  
	//cout << "ZOOM Out CONTENT h=" << contentsHeight() << " w=" << contentsWidth() << endl;
	//cout << "ZOOM Out VISIBLE h=" << height() << " w=" << width() << endl;
	//cout << "chunk size=" << canvas()->chunkSize() << endl;
	// cout << "LEAVING ZOOM OUT " << endl; 
}

void MapView::setClientsPosition() {
	unsigned int nbClients = 0;
	nbClients = clientObjects.size();
	if ( nbClients > 1 ) {
		int **m;
		m = new int*[nbClients];
		std::fill_n(m, nbClients, static_cast<int*>(0));
		for ( unsigned int i = 0; i < nbClients; ++i) {
			m[i] = new int[nbClients];
			std::fill_n(m[i], nbClients, static_cast<int>(0));
		}  
		
		map<unsigned int, unsigned int> id;
		map<unsigned int, unsigned int> id_inv;
		vector<int> sizeClient(nbClients);
		map<unsigned int, ClientGraphicObject *>::iterator iter;
		unsigned int i = 0;
		int tmp_size = 0;
		for ( iter = clientObjects.begin(); iter != clientObjects.end(); iter++ ) {
			id[i] = iter->first;
			id_inv[iter->first] = i;
			switch(iter->second->width()) {
				case 50:
					tmp_size = 0;
					break;
				case 70:
					tmp_size = 2;
					break;
				case 90:
					tmp_size = 4;
					break;
				default:
					break;
			}
			sizeClient[i] = tmp_size;
			i++;
		}
		std::map<unsigned int, FlowGraphicObject *>::const_iterator iter_f;
		for ( iter_f = flowObjects.begin(); iter_f != flowObjects.end(); iter_f++ ) {
			unsigned int idIn = (iter_f->second)->getModel()->getClientInId();
			std::vector<unsigned int> id_in_cli = (iter_f->second)->getModel()->getClientOutId();
			std::vector<unsigned int>::iterator iter_l;
			if ( idIn != 0 )
				for ( iter_l = id_in_cli.begin(); iter_l != id_in_cli.end(); iter_l++ ) {
					m[id_inv[idIn]][id_inv[*iter_l]] = 1;
				}      
		}
			
			pf->clearMap();
			MapRoute *mr = new MapRoute(m, nbClients, id, this, sizeClient);
			clear();
			mr->organize(); 
			refreshClientsWithoutFlows();
			refreshFlows();
			canvas()->setAllChanged();
			update();
			delete mr;
	}
		else if ( nbClients == 1 ) {
			map<unsigned int, ClientGraphicObject *>::iterator iter;
			iter = clientObjects.begin();
			iter->second->setPos(5, 5);
			iter->second->move();
		}
}

QString& MapView::tip( const QPoint & p ) {
    QString *result = new QString("");
    ClientGraphicObject *co = 0;
    FlowGraphicObject *fo = 0;
    FlowGraphicOutlet *oo = 0;
    findObject(p, co, fo, oo);
    if ( oo )
		return getInfo(p);
    else
		return *result;
}

int MapView::getNewId() { 
	return ++newId; 
}


/*********************************************************************************/


AbstractGraphicObject::AbstractGraphicObject(MapView *_mapView) {
	mapView = _mapView;
	popupMenu.clear();
	selected = false;
	//carefull about the id;
	//setId(mapView->getNewId());
}

AbstractGraphicObject::~AbstractGraphicObject() {}

void AbstractGraphicObject::showPopupMenu(const QPoint &pos) {
    popupMenu.popup(pos);
}

void AbstractGraphicObject::setSelected(bool s) {
	selected = s;
}



/************************************************************************************/

FlowGraphicOutlet::FlowGraphicOutlet(FlowOutlet *fo, MapView *mv) : AbstractGraphicObject(mv), Q3CanvasRectangle ( mv->canvas() ) {
	setX(0);
	setY(0);
	setZ(3);
	if ( fo )
		flowOutlet = fo;
	else {
		cout << "Abort :  FlowGraphicOutlet::FlowGraphicOutlet(FlowOutlet *fo, MapView *mv)" << endl;
		abort();
	}
}


FlowGraphicOutlet::~FlowGraphicOutlet() {
	
}

//newq
void FlowGraphicOutlet::drawShape ( QPainter & p ) {
	QPen thePen;
	if ( mapView->getZoomLevel() > 1 )
		thePen.setWidth(1);
	if ( isSelected() )
		thePen.setColor(Qt::red);
	else
		thePen.setColor(Qt::darkBlue);
	p.setPen(thePen);
	
	if ( getModel()->isRunning() && !(getModel()->isPaused()) )
		p.setBrush(QBrush(Qt::green));
	else if ( getModel()->isRunning() && (getModel()->isPaused()) )
		p.setBrush(QBrush(Qt::gray));
	else if ( getModel()->isDisable() )
		p.setBrush(QBrush(Qt::gray));
	else if ( getModel()->isIdle() )
		p.setBrush(QBrush(Qt::yellow));
	else if ( getModel()->isError() )
		p.setBrush(QBrush(Qt::red));
	
	p.drawRoundRect(((int)x())+2,((int)y())+2,6,6);
}

//new
void FlowGraphicOutlet::setPosition(int x, int y) {
	setX(x*TILE_SX);
	setY(y*TILE_SY);
}

//new
Q3PointArray  FlowGraphicOutlet::areaPoints () const {
	Q3PointArray tmp(4);
	tmp[0] = QPoint((int)x(),(int)y());
	tmp[1] = tmp[0] + QPoint(TILE_SX,0);
	tmp[2] = tmp[1] + QPoint(0,TILE_SY);
	tmp[3] = tmp[0] + QPoint(0,TILE_SY);
	return tmp;
}


void FlowGraphicOutlet::drawInMap(){
    mapView->getPf()->setMapCell((int)((int)x()/TILE_SX), (int)((int)y()/TILE_SY), PathFinding::BEF);
    mapView->getPf()->setMapCellId((int)((int)x()/TILE_SX), (int)((int)y()/TILE_SY), getId());
}

void FlowGraphicOutlet::drawFlowInMap() {
	if ( getModel()->hasFlow()) {
		getModel()->getFlow()->getFlowGraphic()->drawPathInMap();
	}
}

void FlowGraphicOutlet::drawOutletInMap() {
	if ( getModel()->hasFlow() )
		getModel()->getFlow()->getFlowGraphic()->drawOutletsInMap();
	else 
		drawInMap();
}

void FlowGraphicOutlet::drawFlowAndOutlet() {
	if ( getModel()->hasClient() ) 
		dynamic_cast<ClientApp *> (getModel()->getClient())->getClientGraphic()->move();
}

void FlowGraphicOutlet::erase() {
	//need to be changed... we know the coordinates
	mapView->getPf()->removePath(getId());
	//we need to erase it from the canvas
	//hide();
}

void FlowGraphicOutlet::eraseFlow() {
	if ( getModel()->hasFlow() )
		getModel()->getFlow()->getFlowGraphic()->erasePath();
}

void FlowGraphicOutlet::eraseFlowAndOutlet() {
	eraseFlow();
	erase();
}

unsigned int FlowGraphicOutlet::getId() { 
	if ( getModel()->hasFlow() ) 
		return getModel()->getFlow()->getId();
	else
		return flowOutlet->getId(); 
}

unsigned int FlowGraphicOutlet::getPrivateId() {
	return flowOutlet->getId();
}

//old
/*void FlowGraphicOutlet::setPosition(int x, int y) {
_x = x;
_y = y;
}*/

void FlowGraphicOutlet::refresh() {
	update();
}

void FlowGraphicOutlet::setSelected(bool s)
{
	if ( isSelected() != s ) {
		AbstractGraphicObject::setSelected(s);
		refresh();
	}
}

void FlowGraphicOutlet::showPopupMenu(const QPoint &p) {
	popupMenu.clear();
    popupMenu.insertItem(tr("&Info..."), this, SLOT(infoFlowAction()));
	if ( hasDupLog() ) {
		popupMenu.insertItem(tr("&Show Log..."), this, SLOT(showLogAction()));
	}
    //popupMenu.insertItem(tr("&View..."), this, SLOT(viewFlowAction()));
	popupMenu.popup(p);
}

void FlowGraphicOutlet::infoFlowAction() {
	FlowInformation *dialog = new FlowInformation(getModel());
	dialog->exec();
	delete dialog;
}

void FlowGraphicOutlet::showLogAction() {
	if ( hasDupLog() ) {
	//	MainWindow::instance()->viewLog();
		QString tmp_id = getModel()->getGroup();
		tmp_id.append(":");
		tmp_id.append(getModel()->getDataType());
		QString s_name = "";
		ClientApp* cl_app = dynamic_cast<ClientApp *>(getModel()->getClient());
		if ( cl_app ) {
			s_name = cl_app->getServer()->getName();
			Duplicator *dup = MainWindow::instance()->global->getApplication()->getDuplicatorList()->getDuplicator(tmp_id, s_name);
			if ( dup ) {
				MainWindow::instance()->viewLog();
				MainWindow::instance()->logView->setDuplicatorTab(dup->getId());
			}
		}
	}
}

void FlowGraphicOutlet::switchToLogTab() {
	if ( hasDupLog() ) {
		QString tmp_id = getModel()->getGroup();
		tmp_id.append(":");
		tmp_id.append(getModel()->getDataType());
		QString s_name = "";
		ClientApp* cl_app = dynamic_cast<ClientApp *>(getModel()->getClient());
		if ( cl_app ) {
			s_name = cl_app->getServer()->getName();
			Duplicator *dup = MainWindow::instance()->global->getApplication()->getDuplicatorList()->getDuplicator(tmp_id, s_name);
			if ( dup ) {
				MainWindow::instance()->logView->setDuplicatorTab(dup->getId());
			}
		}
	}
}

void FlowGraphicOutlet::viewFlowAction() {
	cout << "viewFlowAction() : not yet implemented" << endl;
}

bool FlowGraphicOutlet::hasDupLog() {
	QString tmp_id = getModel()->getGroup();
	tmp_id.append(":");
	tmp_id.append(getModel()->getDataType());
	QString server_name = "";
	ClientApp* cl_app = dynamic_cast<ClientApp *>(getModel()->getClient());
	if ( cl_app ) {
		server_name = cl_app->getServer()->getName();
		Duplicator *dup = MainWindow::instance()->global->getApplication()->getDuplicatorList()->getDuplicator(tmp_id, server_name);
		if ( dup )
			return true;
		else
			return false;
	}
	return false;
}

/************************************************************************************/

FlowGraphicOutletIn::FlowGraphicOutletIn(FlowOutlet *f, MapView *mv) : FlowGraphicOutlet(f,mv) {}

FlowGraphicOutletIn::~FlowGraphicOutletIn() {}


/************************************************************************************/

FlowGraphicOutletOut::FlowGraphicOutletOut(FlowOutlet *f, MapView *mv) : FlowGraphicOutlet(f,mv) {}

FlowGraphicOutletOut::~FlowGraphicOutletOut() {}


/************************************************************************************/
FlowGraphicObject::FlowGraphicObject(MapView *_mapView, Flow *f) : AbstractGraphicObject(_mapView), flow(f) {
	connect( this, SIGNAL(deleteTheFlow(Flow *)), mapView, SLOT(deleteTheFlow(Flow*)));
	subFlowToBreak = NULL;
	subFlowToPause = NULL;
}

FlowGraphicObject::~FlowGraphicObject() {
	//need to erase the subflow list?
	hide();
	std::map<unsigned int, SubFlowGraphicObject *>::iterator iter;
	for ( iter = subflowList.begin(); iter != subflowList.end(); iter++ ) {
		//iter->second->hide();
		mapView->trashGraphicObject(iter->second);
	}
}


bool FlowGraphicObject::drawPathInMap() {
	bool success = true;
	std::set<FlowOutletIn *> *ioutlets = flow->getOutletsIn();
	FlowOutletOut *ooutlet = flow->getOutletOut();
	if ( (ooutlet != 0)  && dynamic_cast<ClientApp *>(ooutlet->getClient())->getClientGraphic()->isShown() ) {
		set<FlowOutletIn *>::iterator iter;
		for ( iter = ioutlets->begin(); iter != ioutlets->end(); ++iter ) {
			if ( dynamic_cast<ClientApp *>((*iter)->getClient())->getClientGraphic()->isShown() ) {
				//here we calculate the path and draw it in the canvas
				Q3ValueList<QPoint> lst = mapView->getPf()->getPath(dynamic_cast<FlowOutletOut *>(ooutlet)->getFlowOutletGraphic()->getX(), dynamic_cast<FlowOutletOut *>(ooutlet)->getFlowOutletGraphic()->getY(), dynamic_cast<FlowOutletIn *>(*iter)->getFlowOutletGraphic()->getX(),  dynamic_cast<FlowOutletIn *>(*iter)->getFlowOutletGraphic()->getY(), getId());
				FlowOutletIn *foin = dynamic_cast<FlowOutletIn *>(*iter);
				if ( foin ) {
					SubFlowGraphicObject *tmp_subFlow = getSubFlow(foin->getId());
					if ( tmp_subFlow != NULL ) {
						if ( lst.count() == 0 ) {
							tmp_subFlow->setPointList(lst);
							success = false;
						}
						else {
							tmp_subFlow->setPointList(lst);
						}
					}
				}
			}
		}
	}
	show();
	return success;
} 

void FlowGraphicObject::drawOutletsInMap()
{
	std::set<FlowOutletIn *> *ioutlets = flow->getOutletsIn();
	std::set<FlowOutletIn *>::iterator iter;
	for ( iter = ioutlets->begin(); iter != ioutlets->end(); ++iter ) {
		if ( dynamic_cast<ClientApp *>((*iter)->getClient())->getClientGraphic()->isShown() ) {
			dynamic_cast<FlowOutletIn *>(*iter)->getFlowOutletGraphic()->drawInMap();
		}
	}
	FlowOutletOut *ooutlet = flow->getOutletOut();
	if ( (ooutlet != 0)  && dynamic_cast<ClientApp *>(ooutlet->getClient())->getClientGraphic()->isShown() ) {
		dynamic_cast<FlowOutletOut *>(ooutlet)->getFlowOutletGraphic()->drawInMap();
	}
}

void FlowGraphicObject::erasePath() {
	mapView->getPf()->removePath(this->getId());
	hide();
	//remove it from the canvas here?
}

void FlowGraphicObject::erasePathWithoutOutlet() {
	mapView->getPf()->removePath(this->getId());
	hide();
	//remove it from the canvas here?
	drawOutletsInMap();
}

//new
void FlowGraphicObject::draw(QPainter &p) {
	std::set<SubFlowGraphicObject *> regularFlowsList;
	std::set<SubFlowGraphicObject *> pausedFlowsList;
	regularFlowsList.clear();
	pausedFlowsList.clear();
	
	
	
	std::map<unsigned int, SubFlowGraphicObject *>::iterator iter;
	for ( iter = subflowList.begin(); iter != subflowList.end(); iter++ ) {
		SubFlowGraphicObject *tmp_subflow = dynamic_cast<SubFlowGraphicObject *> (iter->second);
		tmp_subflow->drawSubFlow1(p);
	}
	
	//we sort the subflows in two list, the ones, which are paused, and the ones, which are not paused.
	for ( iter = subflowList.begin(); iter != subflowList.end(); iter++ ) {
		SubFlowGraphicObject *tmp_subflow = dynamic_cast<SubFlowGraphicObject *> (iter->second);
		std::set<FlowOutletIn *> *flowOutletList = getModel()->getOutletsIn();
		std::set<FlowOutletIn *>::iterator iter_in = flowOutletList->begin();
		bool found = false;
		while ( iter_in != flowOutletList->end() && !found ) {
			if ( (*iter_in)->getId() == tmp_subflow->getSubId() ) {
				found = true;
				FlowOutletIn *tmp_outlet_in = (*iter_in);
				if ( tmp_outlet_in->isPaused() ) {
					pausedFlowsList.insert(tmp_subflow);
					//tmp_subflow->drawSubFlow2(p,true);
				}
				else {
					regularFlowsList.insert(tmp_subflow);
					//  tmp_subflow->drawSubFlow2(p,false);
				}
			}
			else
				iter_in++;
		} 
	}
	//now we got our two list ready
	
	//time to draw the flows.
	std::set<SubFlowGraphicObject *>::iterator iter_subflows;
	//first we draw the paused flows
	for ( iter_subflows = pausedFlowsList.begin();  iter_subflows !=  pausedFlowsList.end(); iter_subflows++ ) {
		(*iter_subflows)->drawSubFlow2(p,true);
	}
	//then we draw the unpaused
	for ( iter_subflows = regularFlowsList.begin();  iter_subflows !=  regularFlowsList.end(); iter_subflows++ ) {
		(*iter_subflows)->drawSubFlow2(p,false);
	}
}

//new
void FlowGraphicObject::show() {
	std::map<unsigned int, SubFlowGraphicObject *>::iterator iter;
	for ( iter = subflowList.begin(); iter != subflowList.end(); iter++ ) {
		SubFlowGraphicObject *tmp_subflow = dynamic_cast<SubFlowGraphicObject *> (iter->second);
		if ( tmp_subflow->needToShow() )
			tmp_subflow->show();
		else
			tmp_subflow->hide();
	}
}

//new 
void FlowGraphicObject::hide() {
	std::map<unsigned int, SubFlowGraphicObject *>::iterator iter;
	for ( iter = subflowList.begin(); iter != subflowList.end(); iter++ ) {
		SubFlowGraphicObject *tmp_subflow = dynamic_cast<SubFlowGraphicObject *> (iter->second);
		tmp_subflow->hide();
	}
}

//new
void FlowGraphicObject::createSubFlow(unsigned int _id) {
	SubFlowGraphicObject *tmp =  new SubFlowGraphicObject(mapView, this);
	tmp->setSubId(_id);
	addSubFlow(_id, tmp);
}

bool FlowGraphicObject::subFlowExists(unsigned int _id) {
	std::map<unsigned int, SubFlowGraphicObject *>::iterator iter;
	iter = subflowList.find(_id);
	if (iter != subflowList.end()) 
		return true;
	else
		return false;
}

//new not finished!
void FlowGraphicObject::addSubFlow(unsigned int _id, SubFlowGraphicObject *_subflow) {
	if ( !subFlowExists(_id) )
		subflowList[_id] = _subflow;
}

//new not finished!
void FlowGraphicObject::removeSubFlow(unsigned int _id) {
	SubFlowGraphicObject *tmp = getSubFlow(_id);
	if ( tmp ) {
		tmp->hide();
		subflowList.erase(_id);
		mapView->trashGraphicObject(tmp);
	}
}

//new
unsigned int FlowGraphicObject::countSubFlows() {
	return subflowList.size();
}

void FlowGraphicObject::setSubFlowToBreak(SubFlowGraphicObject *sub) {
	subFlowToBreak = sub;
}

void FlowGraphicObject::unsetSubFlowToBreak() {
	subFlowToBreak = NULL;
}

void FlowGraphicObject::setSubFlowToPause(SubFlowGraphicObject *sub) {
	subFlowToPause = sub;
}

void FlowGraphicObject::unsetSubFlowToPause() {
	subFlowToPause = NULL;
}

SubFlowGraphicObject *FlowGraphicObject::getSubFlow(unsigned int _id) {
	std::map<unsigned int, SubFlowGraphicObject *>::iterator iter = subflowList.begin();
	bool found = false;
	SubFlowGraphicObject *result = NULL;
	while( !found && (iter != subflowList.end()) ) {
		result = iter->second;
		if ( result->getSubId() == _id )
			found = true;
		else
			iter++;
	}
	if ( found )
		return result;
	else
		return NULL;
}

void FlowGraphicObject::setSelected(bool s)
{
	if ( selected != s ) {
		AbstractGraphicObject::setSelected(s);
		refresh();
	}
}

void FlowGraphicObject::refresh() {
	std::map<unsigned int, SubFlowGraphicObject *>::iterator iter;
	for ( iter = subflowList.begin(); iter != subflowList.end(); iter++ ) {
		SubFlowGraphicObject *tmp_subflow = dynamic_cast<SubFlowGraphicObject *> (iter->second);
		tmp_subflow->refresh();
	}
}

unsigned int FlowGraphicObject::getId() { return flow->getId(); }

void FlowGraphicObject::showPopupMenu(const QPoint &p) {
	popupMenu.clear();
	if ( (!getModel()->isRunning()) && (getModel()->countOutletInRunning() < 2) ) {
                //We can break if the provider is not running and there is maximum one conso running
		popupMenu.insertItem(tr("&Break link"), this, SLOT(breakLinkAction()));
	}
	else if ( getModel()->isRunning() ) {
		if ( subFlowToBreak == NULL ) {
			//need to check the outlet in is not running
			std::set<FlowOutletIn *> *flowOutletList = getModel()->getOutletsIn();
			std::set<FlowOutletIn *>::iterator iter = flowOutletList->begin();
			bool okToBreak = true;
			while ( iter != flowOutletList->end() && okToBreak ) {
				if ( (*iter)->isRunning() )
					okToBreak = false;
				iter++;
			}
			if ( okToBreak )
				popupMenu.insertItem(tr("&Break link"), this, SLOT(breakLinkAction()));
		}
		else {
			//need to check that the outlet in of the subflow is not running
			std::set<FlowOutletIn *> *flowOutletList = getModel()->getOutletsIn();
			std::set<FlowOutletIn *>::iterator iter = flowOutletList->begin();
			bool found = false;
			while ( iter != flowOutletList->end() && !found ) {
				if ( (*iter)->getId() == subFlowToBreak->getSubId() ) {
					if ( !((*iter)->isRunning()) )
						popupMenu.insertItem(tr("&Break link"), this, SLOT(breakLinkAction()));
					found = true;
				}
				iter++;
			}
		}
	}
	if ( getModel()->isRunning() ) {
		popupMenu.insertItem(tr("&Pause"), this, SLOT(pauseFlowAction()));
		popupMenu.insertItem(tr("&Restart"), this, SLOT(restartFlowAction()));
	}
	popupMenu.insertItem(tr("&Info..."), this, SLOT(infoFlowAction()));
	//popupMenu.insertItem(tr("&View..."), this, SLOT(viewFlowAction()));
	popupMenu.popup(p);

}

void FlowGraphicObject::emitDeleteFlowSignal(Flow *f) {
	emit deleteTheFlow(f);
}

void FlowGraphicObject::breakLinkAction() {
	if ( subFlowToBreak == NULL ) {
		getModel()->disconnect();
	}
	else {
		unsigned int thesubID = subFlowToBreak->getSubId();
		std::set<FlowOutletIn *> *flowOutletList = getModel()->getOutletsIn();
		std::set<FlowOutletIn *>::iterator iter = flowOutletList->begin();
		bool found = false;
		while ( iter != flowOutletList->end() && !found ) {
			if ( (*iter)->getId() == thesubID ) {
				FlowOutletIn *tmp_outlet_in = (*iter);
				hide();
				erasePath();
				unsetSubFlowToBreak();
				tmp_outlet_in->detachFlow();
				tmp_outlet_in->getFlowOutletGraphic()->drawOutletInMap();
				drawPathInMap();
				show();
				mapView->update();
				found = true;
			}
			else
				iter++;
		}
	}
}

void FlowGraphicObject::pauseFlowAction() {
	//need to call unsetSubFlowToPause() in this method
	if ( subFlowToPause == NULL ) {
		//we pause every consumer
		getModel()->pause();
	}
	else {
		//we only pause one consumer
		unsigned int thesubID = subFlowToPause->getSubId();
		std::set<FlowOutletIn *> *flowOutletList = getModel()->getOutletsIn();
		std::set<FlowOutletIn *>::iterator iter = flowOutletList->begin();
		bool found = false;
		while ( iter != flowOutletList->end() && !found ) {
			if ( (*iter)->getId() == thesubID ) {
				FlowOutletIn *tmp_outlet_in = (*iter);
				ClientApp *tmp_client = (ClientApp *) (tmp_outlet_in->getClient());
				unsetSubFlowToPause();
				if ( tmp_client != NULL ) {
					SfControlThread *sfct = MainWindow::instance()->global->getSfControlThread();
					if ( sfct ) {
						SfControl *sfc = sfct->getSfControl();
						if ( sfc ) {
							//sfc->pauseFlow(getModel(), tmp_client);
							sfc->emitPauseFlowSignal(getModel(), tmp_client);
						}
					}
					
					//  mapView->update();
					found = true;
				}
			}
			else
				iter++;
		}
	}
}

void FlowGraphicObject::restartFlowAction() { 
	//need to call unsetSubFlowToPause() in this method
	if ( subFlowToPause == NULL ) {
		//we restart every consumer
		getModel()->restart();
	}
	else {
		//we only restart one consumer
		unsigned int thesubID = subFlowToPause->getSubId();
		std::set<FlowOutletIn *> *flowOutletList = getModel()->getOutletsIn();
		std::set<FlowOutletIn *>::iterator iter = flowOutletList->begin();
		bool found = false;
		while ( iter != flowOutletList->end() && !found ) {
			if ( (*iter)->getId() == thesubID ) {
				FlowOutletIn *tmp_outlet_in = (*iter);
				ClientApp *tmp_client = (ClientApp *) (tmp_outlet_in->getClient());
				unsetSubFlowToPause();
				if ( tmp_client != NULL ) {
					SfControlThread *sfct = MainWindow::instance()->global->getSfControlThread();
					if ( sfct ) {
						SfControl *sfc = sfct->getSfControl();
						if ( sfc ) {
							//sfc->restartFlow(getModel(), tmp_client);
							sfc->emitRestartFlowSignal(getModel(), tmp_client);
						}
					}
					//mapView->update();
					found = true;
				}
			}
			else
				iter++;
		}
	} 
}

void FlowGraphicObject::infoFlowAction() {
	FlowInformation *dialog = new FlowInformation(getModel());
	dialog->exec();
	delete dialog;
}

void FlowGraphicObject::viewFlowAction() {
	cout << "viewFlowAction() : not yet implemented" << endl;
}

/**********************************************************************************************/

SubFlowGraphicObject::SubFlowGraphicObject (MapView *_mapView, FlowGraphicObject *_parent ) : AbstractGraphicObject(_mapView), Q3CanvasPolygon(_mapView->canvas()) {
	parent = _parent;
	setZ(1);
	show();
}

SubFlowGraphicObject::~SubFlowGraphicObject() {
	invalidate();
	hide();
}

void SubFlowGraphicObject::setPointList(Q3ValueList<QPoint> ptlst) {
	pointList = ptlst;
}

bool SubFlowGraphicObject::needToShow() {
	return (pointList.count() >= 2);
}

unsigned int SubFlowGraphicObject::getId() {
	return parent->getId();
}

unsigned int SubFlowGraphicObject::getSubId() {
	return subid;
}

void SubFlowGraphicObject::setSubId(unsigned int _subid) {
	subid = _subid;
}

void SubFlowGraphicObject::drawSubFlow1(QPainter &p) {
	if ( pointList.count() < 2 ) {
		hide();
		return;
	}
	else {
		Q3PointArray tmp(pointList.count());
		int i = 0;
		int shiftx = TILE_SX/2;
		int shifty = TILE_SY/2;
		//was required for Qt3, not needed anymore with Qt4
		/*if ( MAC ) {
			shiftx = (int)(TILE_SX/4 + 1);
		shifty = (int)(TILE_SY/4 + 1);
		}*/
		for (Q3ValueList<QPoint>::const_iterator it = pointList.begin(); it != pointList.end(); it++) {
			tmp[i] = (*it);
			tmp[i].setX( (tmp[i].x() * TILE_SX) + shiftx);
			tmp[i].setY( (tmp[i].y() * TILE_SY) + shifty);
			i++;
		}
		if ( parent->isSelected() ) 
			p.setPen(QPen(Qt::red, 4, Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin));
		//p.setPen(QPen(Qt::red, (int)(mapView->getZoomLevel()*4), Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin));
		else
			p.setPen(QPen(Qt::darkBlue, 4, Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin));
		//p.setPen(QPen(Qt::darkBlue, (int)(mapView->getZoomLevel()*4), Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin));
		p.drawPolyline(tmp);
	} 
}

void SubFlowGraphicObject::drawSubFlow2(QPainter &p, bool isPaused) {
	if ( pointList.count() < 2 )
		return;
	else {
		Q3PointArray tmp(pointList.count());
		int i = 0;
		int shiftx = TILE_SX/2;
		int shifty = TILE_SY/2;
		//was required for Qt3, not needed anymore with Qt4
		/*if ( MAC ) {
			shiftx = (int)(TILE_SX/4 + 2);
		shifty = (int)(TILE_SY/4 + 2);
		}*/
		for (Q3ValueList<QPoint>::const_iterator it = pointList.begin(); it != pointList.end(); it++) {
			tmp[i] = (*it);
			tmp[i].setX( (tmp[i].x() * TILE_SX) + shiftx);
			tmp[i].setY( (tmp[i].y() * TILE_SY) + shifty);
			i++;
		}
		
		if ( isPaused )
			p.setPen(QPen(Qt::gray, 2, Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin));
		//p.setPen(QPen(Qt::gray, (int)(mapView->getZoomLevel()*2), Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin));
		else if ( parent->getModel()->isRunning() ) 
			p.setPen(QPen(Qt::green, 2, Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin));
		//p.setPen(QPen(Qt::green, (int)(mapView->getZoomLevel()*2), Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin));
		else
			p.setPen(QPen(Qt::yellow, 2, Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin));
		//p.setPen(QPen(Qt::yellow, (int)(mapView->getZoomLevel()*2), Qt::SolidLine, Qt::SquareCap, Qt::MiterJoin));
		p.drawPolyline(tmp);
	} 
}

void SubFlowGraphicObject::refresh() {
	update();
}

void SubFlowGraphicObject::drawShape ( QPainter & p ) {
	parent->draw(p);
}

void SubFlowGraphicObject::clearPointList() {
	pointList.clear();
}

int SubFlowGraphicObject::direction(QPoint pt1, QPoint pt2) const {
	if ( pt1.x() == pt2.x() ) {
		if( pt1.y() > pt2.y() )
			return up;
		else if ( pt1.y() < pt2.y() )
			return down;
		else
			return 0;
	}
	else if ( pt1.y() == pt2.y() ) {
		if ( pt1.x() > pt2.x() )
			return left;
		else if ( pt1.x() < pt2.x() )
			return right;
		else 
			return 0;
	}
	else
		return 0;
}

Q3PointArray SubFlowGraphicObject::areaPoints () const {
	if ( pointList.isEmpty() ) {
		Q3PointArray tmp(0);
		return tmp;
	}
	else {
		int pw = (pen().width()+1)/2;
		if ( pw < 1 ) pw = 1;
		if ( pen() == Qt::NoPen ) pw = 0;
		pw = 3;
		//pw = (int) (mapView->getZoomLevel()*3);
		if ( pointList.count() == 1 ) {
			Q3PointArray tmp(4);
			QPoint pt = pointList.first();
			int x = pt.x()*TILE_SX;
			int y = pt.y()*TILE_SY;
			tmp[0] = QPoint(x-pw,y-pw);
			tmp[1] = QPoint(x+pw,y-pw);
			tmp[2] = QPoint(x+pw,y+pw);
			tmp[3] = QPoint(x-pw,y+pw);
			return tmp;
		}
		else {
			
			// there are 2 points or more in the list.
			Q3ValueList<QPoint> go;
			Q3ValueList<QPoint> back;
			int previousDir = 0;
			
			Q3ValueList<QPoint> tmp_pointList(pointList);
			Q3ValueList<QPoint>::iterator it = tmp_pointList.begin();
			int shiftx = TILE_SX/2;
			int shifty = TILE_SY/2;
			for ( it = tmp_pointList.begin(); it != tmp_pointList.end(); it++) {
				(*it).setX(((*it).x() * TILE_SX) + shiftx);
				(*it).setY(((*it).y() * TILE_SY) + shifty);
			}
			
			it = tmp_pointList.begin();
			QPoint pt1 = (*it);
			QPoint pt2;
			it++;
			for (; it != tmp_pointList.end(); it++) {
				pt2 = (*it);
				switch (direction(pt1,pt2)) {
					case right:
						if ( previousDir == 0 ) {
							go.push_back(QPoint(pt1.x()-pw,pt1.y()-pw));
							back.push_front(QPoint(pt1.x()-pw,pt1.y()+pw));
						}
						else {
							switch (previousDir) {
								case right:
									//must never happen
									go.push_back(QPoint(pt1.x()-pw,pt1.y()-pw));
									back.push_front(QPoint(pt1.x()-pw,pt1.y()+pw));
									break;
								case left:
									//must never happen
									go.push_back(QPoint(pt1.x()-pw,pt1.y()-pw));
									back.push_front(QPoint(pt1.x()-pw,pt1.y()+pw));
									break;
								case up:
									go.push_back(QPoint(pt1.x()-pw,pt1.y()-pw));
									back.push_front(QPoint(pt1.x()+pw,pt1.y()+pw));
									break;
								case down:
									go.push_back(QPoint(pt1.x()+pw,pt1.y()-pw));
									back.push_front(QPoint(pt1.x()-pw,pt1.y()+pw));
									break;
							}
						}
						break;
					case left:
						if ( previousDir == 0 ) {
							go.push_back(QPoint(pt1.x()+pw,pt1.y()+pw));
							back.push_front(QPoint(pt1.x()+pw,pt1.y()-pw));
						}
						else {
							switch (previousDir) {
								case right:
									//must never happen
									go.push_back(QPoint(pt1.x()-pw,pt1.y()-pw));
									back.push_front(QPoint(pt1.x()-pw,pt1.y()+pw));
									break;
								case left:
									//must never happen
									go.push_back(QPoint(pt1.x()-pw,pt1.y()-pw));
									back.push_front(QPoint(pt1.x()-pw,pt1.y()+pw));
									break;
								case up:
									go.push_back(QPoint(pt1.x()-pw,pt1.y()+pw));
									back.push_front(QPoint(pt1.x()+pw,pt1.y()-pw));
									break;
								case down:
									go.push_back(QPoint(pt1.x()+pw,pt1.y()+pw));
									back.push_front(QPoint(pt1.x()-pw,pt1.y()-pw));
									break;
							}
						}
						break;
					case up:
						if ( previousDir == 0 ) {
							go.push_back(QPoint(pt1.x()-pw,pt1.y()+pw));
							back.push_front(QPoint(pt1.x()+pw,pt1.y()+pw));
						}
						else {
							switch (previousDir) {
								case right:
									go.push_back(QPoint(pt1.x()-pw,pt1.y()-pw));
									back.push_front(QPoint(pt1.x()+pw,pt1.y()+pw));
									break;
								case left:
									go.push_back(QPoint(pt1.x()-pw,pt1.y()+pw));
									back.push_front(QPoint(pt1.x()+pw,pt1.y()-pw));
									break;
								case up:
									//must never happen
									go.push_back(QPoint(pt1.x()-pw,pt1.y()-pw));
									back.push_front(QPoint(pt1.x()+pw,pt1.y()+pw));
									break;
								case down:
									//must never happen
									go.push_back(QPoint(pt1.x()-pw,pt1.y()-pw));
									back.push_front(QPoint(pt1.x()-pw,pt1.y()+pw));
									break;
							}
						}
						break;
					case down:
						if ( previousDir == 0 ) {
							go.push_back(QPoint(pt1.x()+pw,pt1.y()-pw));
							back.push_front(QPoint(pt1.x()-pw,pt1.y()-pw)); 
						}
						else {
							switch (previousDir) {
								case right:
									go.push_back(QPoint(pt1.x()+pw,pt1.y()-pw));
									back.push_front(QPoint(pt1.x()-pw,pt1.y()+pw));
									break;
								case left:
									go.push_back(QPoint(pt1.x()+pw,pt1.y()+pw));
									back.push_front(QPoint(pt1.x()-pw,pt1.y()-pw));
									break;
								case up:
									//must never happen
									go.push_back(QPoint(pt1.x()-pw,pt1.y()-pw));
									back.push_front(QPoint(pt1.x()-pw,pt1.y()+pw));
									break;
								case down:
									//must never happen
									go.push_back(QPoint(pt1.x()-pw,pt1.y()-pw));
									back.push_front(QPoint(pt1.x()-pw,pt1.y()+pw));
									break;
							}
						}
						break;
				}
				previousDir = direction(pt1,pt2);
				pt1 = pt2;
			}//end for
			switch (previousDir) {
				case right:
					go.push_back(QPoint(pt1.x()+pw,pt1.y()-pw));
					back.push_front(QPoint(pt1.x()+pw,pt1.y()+pw));
					break;
				case left:
					go.push_back(QPoint(pt1.x()-pw,pt1.y()+pw));
					back.push_front(QPoint(pt1.x()-pw,pt1.y()-pw));
					break;
				case up:
					go.push_back(QPoint(pt1.x()-pw,pt1.y()-pw));
					back.push_front(QPoint(pt1.x()+pw,pt1.y()-pw));
					break;
				case down:
					go.push_back(QPoint(pt1.x()+pw,pt1.y()+pw));
					back.push_front(QPoint(pt1.x()-pw,pt1.y()+pw));
					break;
			}
			int nb = go.count() + back.count();
			int i = 0;
			Q3PointArray tmp(nb);
			for (Q3ValueList<QPoint>::const_iterator it2 = go.begin(); it2 != go.end(); it2++) {
				tmp[i] = (*it2);
				i++;
			}
			for (Q3ValueList<QPoint>::const_iterator it3 = back.begin(); it3 != back.end(); it3++) {
				tmp[i] = (*it3);
				i++;
			}
			
			return tmp;
		}//else
	}//else
}

void SubFlowGraphicObject::hide() {
	invalidate();
	Q3CanvasPolygon::hide();
}


/**********************************************************************************************/
//not finished
ClientGraphicObject::ClientGraphicObject(MapView *_mapView, ClientApp *ca) : AbstractGraphicObject(_mapView), Q3CanvasPolygonalItem (_mapView->canvas())
{
	setX(0);
	setY(0);
	setZ(5);
	shown = false;
	size_type = 0;
	clientApp = ca;
	
	coli.clear();
	label = new Q3CanvasText(mapView->canvas());
	label->setTextFlags(Qt::AlignHCenter|Qt::AlignTop);
	label->setZ(4);
	//label->show();
	sprite = new Q3CanvasSprite(_mapView->getPixmap(IDLE, 0), _mapView->canvas());
	setSize(50, 50);
	hide();
	setImage();
	sprite->setFrameAnimation();
	//sprite->show();
	undrawFlows.clear();
	connect( this, SIGNAL(deleteTheClient(ClientApp *)), mapView, SLOT(deleteTheClient(ClientApp *)));
}

//seems Okay
ClientGraphicObject::~ClientGraphicObject()
{
	coli.clear();
	undrawFlows.clear();
}

//seems Okay
void ClientGraphicObject::hide() {
	shown = false;
	Q3CanvasPolygonalItem::hide();
	sprite->hide();
	label->hide();
	eraseInMap();
	
	std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
	std::map<unsigned int, FlowOutletIn*> *fin = getModel()->getInputFlowList();
	for( iter_in = fin->begin(); iter_in != fin->end(); iter_in++) {
		Flow *fl = 0;
		fl = iter_in->second->getFlow();
		if ( fl ) {
			fl->getFlowGraphic()->erasePath();
			fl->getFlowGraphic()->drawOutletsInMap();
			fl->getFlowGraphic()->drawPathInMap();
		}
		else
			(iter_in->second)->getFlowOutletGraphic()->hide();   
	}
	
	std::map<unsigned int, FlowOutletOut *>::iterator iter_out;
	std::map<unsigned int, FlowOutletOut *> *fout = getModel()->getOutputFlowList();
	for( iter_out = fout->begin(); iter_out != fout->end(); iter_out++) {
		Flow *fl = 0;
		fl = iter_out->second->getFlow();
		if ( fl ) {
			fl->getFlowGraphic()->erasePath();
			fl->getFlowGraphic()->drawOutletsInMap();
			fl->getFlowGraphic()->drawPathInMap();
		}
		else
			(iter_out->second)->getFlowOutletGraphic()->hide();
	}
	mapView->update();
	shown = false;
}

//seems Okay
void ClientGraphicObject::show() {
	shown = true;
	Q3CanvasPolygonalItem::show();
	sprite->show();
	label->show();
	drawClient();
	
	std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
	std::map<unsigned int, FlowOutletIn*> *fin = getModel()->getInputFlowList();
	for( iter_in = fin->begin(); iter_in != fin->end(); iter_in++) {
		Flow *fl = 0;
		(iter_in->second)->getFlowOutletGraphic()->show(); 
		fl = iter_in->second->getFlow();
		if ( fl ) {
			fl->getFlowGraphic()->erasePath();
			fl->getFlowGraphic()->drawOutletsInMap();
			fl->getFlowGraphic()->drawPathInMap();
		}
	}
	
	std::map<unsigned int, FlowOutletOut *>::iterator iter_out;
	std::map<unsigned int, FlowOutletOut *> *fout = getModel()->getOutputFlowList();
	for( iter_out = fout->begin(); iter_out != fout->end(); iter_out++) {
		Flow *fl = 0;
		(iter_out->second)->getFlowOutletGraphic()->show();
		fl = iter_out->second->getFlow();
		if ( fl ) {
			fl->getFlowGraphic()->erasePath();
			fl->getFlowGraphic()->drawOutletsInMap();
			fl->getFlowGraphic()->drawPathInMap();
		}
	}
	
	mapView->update();
}

//seems Okay
unsigned int ClientGraphicObject::getSizeType() {
	int tmp =  0;
	std::map<unsigned int, FlowOutletIn*> *fin = getModel()->getInputFlowList();
	std::map<unsigned int, FlowOutletOut*> *fout = getModel()->getOutputFlowList();
	unsigned int fin_size = fin->size();
	unsigned int fout_size = fout->size();
	if ( fin_size <= 5 ) {
		if ( fout_size <= 5 )
			tmp = 0;
		else if ( fout_size <= 7 )
			tmp = 1;
		else
			tmp = 2;
	}
	else if ( fin_size <= 7 ) {
		tmp = 1;
		if ( fout_size > 7 )
			tmp = 2;
	}
	else
		tmp = 2;
	return tmp;
}

//seems Okay
void ClientGraphicObject::updateClient() {
    setImage();
    //we need to update all the input flows
    update();
}

//seems Okay
void ClientGraphicObject::setImage() {
	ClientApp *ca  = getModel();
	if ( ca ) {
		unsigned int s = getSizeType();
		if ( ca->isRunning() )
			if ( ca->hasLog() )
				if ( isSelected() )
					sprite->setSequence(mapView->getPixmap(ON_LOG_S, s));
				else
					sprite->setSequence(mapView->getPixmap(ON_LOG, s));
			else
				if ( isSelected() )
					sprite->setSequence(mapView->getPixmap(ON_S, s));
			else
				sprite->setSequence(mapView->getPixmap(ON, s));
		else if ( ca->isDisable() )
			if ( ca->hasLog() )
				if ( isSelected() )
					sprite->setSequence(mapView->getPixmap(DISABLE_LOG_S, s));
				else
					sprite->setSequence(mapView->getPixmap(DISABLE_LOG, s));
			else
				if ( isSelected() )
					sprite->setSequence(mapView->getPixmap(DISABLE_S, s));
			else
				sprite->setSequence(mapView->getPixmap(DISABLE, s));
		else if ( ca->isStarting() )
			if ( ca->hasLog() )
				if ( isSelected() )
					sprite->setSequence(mapView->getPixmap(START_LOG_S, s));
				else
					sprite->setSequence(mapView->getPixmap(START_LOG, s));
			else
				if ( isSelected() )
					sprite->setSequence(mapView->getPixmap(START_S, s));
			else
				sprite->setSequence(mapView->getPixmap(START, s));
		else if ( ca->isError() )
			if ( ca->hasLog() )
				if ( isSelected() )
					sprite->setSequence(mapView->getPixmap(ERR_LOG_S, s));
				else
					sprite->setSequence(mapView->getPixmap(ERR_LOG, s));
			else
				if ( isSelected() )
					sprite->setSequence(mapView->getPixmap(ERR_S, s));
			else
				sprite->setSequence(mapView->getPixmap(ERR, s));
		else
			if ( ca->hasLog() )
				if ( isSelected() )
					sprite->setSequence(mapView->getPixmap(IDLE_LOG_S, s));
				else
					sprite->setSequence(mapView->getPixmap(IDLE_LOG, s));
		else
			if ( isSelected() )
				sprite->setSequence(mapView->getPixmap(IDLE_S, s));
		else
			sprite->setSequence(mapView->getPixmap(IDLE, s));
		label->move(((2*(int)x())+_height)/2, ((int)y())+_width);
	}
}

//seems Okay
QPoint ClientGraphicObject::pos()
{
	return QPoint((int)x(), (int)y());
}

void ClientGraphicObject::setLabelColor(QColor c) {
	label->setColor(c);
}

void ClientGraphicObject::refreshLabel() {
	Server* s = getModel()->getServer();
	if ( s ) {
		QString tmp_name(getModel()->getName());
		if ( s->getAddress() == "No Server" ) {
			if ( getModel()->getOldServerName().isEmpty() ) {
				tmp_name.append("\n on No Server");
				label->setColor (QColor(Qt::red));
				setLabel(tmp_name);
			}
			else {
				tmp_name.append("\n ");
				tmp_name.append(getModel()->getOldServerName());
				tmp_name.append(" not present");
				label->setColor(QColor(Qt::red));
				setLabel(tmp_name);
			}
		}  
		else {
			QString tmp = "";
			getModel()->setOldServerName(tmp);
			tmp_name.append("\n on ");
			tmp_name.append(s->getName());
			label->setColor(QColor(Qt::black));
			setLabel(tmp_name);
		}
	}
}

void ClientGraphicObject::emitDeleteClientSignal(ClientApp* c) {
	emit deleteTheClient(c);
}

void ClientGraphicObject::startClientAction() {
	SfControlThread *sfct = MainWindow::instance()->global->getSfControlThread();
	if ( sfct ) {
		SfControl *sfc = sfct->getSfControl();
		if ( sfc ) {
			//sfc->startClient(getModel());
			sfc->emitStartClientSignal(getModel());
		}
	}
}

void ClientGraphicObject::stopClientAction() {
	SfControlThread *sfct = MainWindow::instance()->global->getSfControlThread();
	if ( sfct ) {
		SfControl *sfc = sfct->getSfControl();
		if ( sfc ) {
			//sfc->stopClient(getModel());
			sfc->emitStopClientSignal(getModel());
		}
	}
}

void ClientGraphicObject::editClientAction() {
		ClientAppEditor *editor = new ClientAppEditor((ClientApp *)getModel(), getMapView());
		editor->setCaption(tr("Edit Client"));
		if ( editor->exec() == QDialog::Accepted ) {
			Server *s = editor->getServer();
			if ( s ) {
				QString param = editor->getParam();
				getModel()->setParam(param);
				getModel()->setServer(s);
				QString tmp_name(getModel()->getName());
				tmp_name.append(" ");
				if ( s ) {
					if ( s->getAddress() == "No Server" ) {
						if ( getModel()->getOldServerName().isEmpty() ) {
							tmp_name.append("\n on No Server");
							label->setColor (QColor(Qt::red));
							setLabel(tmp_name);
						}
						else {
							tmp_name.append("\n ");
							tmp_name.append(getModel()->getOldServerName());
							tmp_name.append(" not present");
							label->setColor(QColor(Qt::red));
							setLabel(tmp_name);
						}
					}  
					else {
						QString tmp = "";
						getModel()->setOldServerName(tmp);
						tmp_name.append("\n on ");
						tmp_name.append(s->getName());
						label->setColor(QColor(Qt::black));
						setLabel(tmp_name);
					}
				}
				MainWindow::instance()->appliNeedSave = true;
			}
			MainWindow::instance()->appMon->refreshAllView();
		}
}

void ClientGraphicObject::enableDisableClientAction() {
	if ( getModel()->isDisable() )
		getModel()->setIdle();
	else
		getModel()->setDisable();
	updateClient();
}

void ClientGraphicObject::removeClientAction() {
	if ( !isOkayToRemove() ) {
		QMessageBox::warning(MainWindow::instance(), QMessageBox::tr("Attention"), QMessageBox::tr("You cannot remove a producer client having at least one flow connected to several running consumer clients"));
	}
	else {
		//first we hide the client from the map
		hide();
		//then we take care of the output flows
		FlowOutletOut *fo = 0;
		std::map<unsigned int, FlowOutletOut*> *fout = getModel()->getOutputFlowList();
		std::map<unsigned int, FlowOutletOut *>::iterator iter_out;
		while ( fout->size() != 0 ) {
			iter_out = fout->begin(); 
			fo = iter_out->second;
			if(fo) {
				unsigned int tmp_id = fo->getId();
				if ( fo->hasFlow() ) {
					Flow *fl = fo->getFlow();
					fl->disconnect();
					fo->getFlowOutletGraphic()->erase();
					getMapView()->remOutlet(fo->getId());
					MainWindow::instance()->global->getApplication()->removeOutlet(fo);
					delete fo;
					getModel()->removeFlow(tmp_id);
				}
				else {
					fo->getFlowOutletGraphic()->erase();
					getMapView()->remOutlet(fo->getId());
					MainWindow::instance()->global->getApplication()->removeOutlet(fo);
					delete fo;
					getModel()->removeFlow(tmp_id);
				}
			}
			fo = 0; 
		}
		//then we take care of the input flows
		FlowOutletIn *fi = 0;
		std::map<unsigned int, FlowOutletIn*> *fin = getModel()->getInputFlowList();
		std::map<unsigned int, FlowOutletIn*>::iterator iter_in;
		while ( fin->size() != 0 ) {
			iter_in = fin->begin(); 
			fi = iter_in->second;
			if(fi) {
				unsigned int tmp_id = fi->getId();
				if ( fi->hasFlow() ) {
					Flow *fl = fi->getFlow();
					if ( fl->getOutletsIn()->size() == 1 )
						fl->disconnect();
					else
						fi->detachFlow();
					fi->getFlowOutletGraphic()->erase();
					getMapView()->remOutlet(fi->getId());
					MainWindow::instance()->global->getApplication()->removeOutlet(fi);
					delete fi;
					getModel()->removeFlow(tmp_id);
				}
				else {
					fi->getFlowOutletGraphic()->erase();
					getMapView()->remOutlet(fi->getId());
					MainWindow::instance()->global->getApplication()->removeOutlet(fi);
					delete fi;
					getModel()->removeFlow(tmp_id);
				}
			}
			fi = 0; 
		}
		//then we take care of the client itself
		getMapView()->remClient(getId());
		MainWindow::instance()->global->getApplication()->removeClient(getModel());
		//cout << "NB cli=" << MainWindow::instance()->global->getApplication()->getClientList()->size() << endl;
		MainWindow::instance()->global->getApplication()->removeUnusedDuplicators();
		
		emitDeleteClientSignal(getModel());
	}
}

void ClientGraphicObject::showClientLogAction() {
	MainWindow::instance()->viewLog();
	MainWindow::instance()->logView->setClientTab(getModel()->getId());
	//cout << "showClientLogAction() : not yet implemented" << endl;
}

void ClientGraphicObject::clearClientLogAction() {
	MainWindow::instance()->logView->clearClientLog(getModel()->getId());
} 

void ClientGraphicObject::setOutletsPositionAction(){
	//first we erase the flows from the map
	std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
	std::map<unsigned int, FlowOutletIn *> *fin = getModel()->getInputFlowList();
	for(iter_in = fin->begin(); iter_in != fin->end(); iter_in++ ) 
		(iter_in->second)->getFlowOutletGraphic()->eraseFlowAndOutlet();
	std::map<unsigned int, FlowOutletOut *>::iterator iter_out;
	std::map<unsigned int, FlowOutletOut *> *fout = getModel()->getOutputFlowList();
	for(iter_out = fout->begin(); iter_out != fout->end(); iter_out++ ) 
		(iter_out->second)->getFlowOutletGraphic()->eraseFlowAndOutlet();
	
	//then we set the new position of the flows
	resetInputFlows();
	
	//finally we draw the outlets
	for(iter_in = fin->begin(); iter_in != fin->end(); iter_in++ ) 
		(iter_in->second)->getFlowOutletGraphic()->drawOutletInMap();
	for(iter_out = fout->begin(); iter_out != fout->end(); iter_out++ ) 
		(iter_out->second)->getFlowOutletGraphic()->drawOutletInMap();
	
	
	//draw the flow with the new position
	for(iter_in = fin->begin(); iter_in != fin->end(); iter_in++ ) 
		(iter_in->second)->getFlowOutletGraphic()->drawFlowInMap();
	for(iter_out = fout->begin(); iter_out != fout->end(); iter_out++ ) 
		(iter_out->second)->getFlowOutletGraphic()->drawFlowInMap();
}

//need to be moved
void ClientGraphicObject::clearFlows()
{
	//need to clear fin and fout?
	FlowOutletIn *fi = 0;
	std::map<unsigned int, FlowOutletIn*> *fin = getModel()->getInputFlowList();
	std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
	for( iter_in = fin->begin(); iter_in != fin->end(); iter_in++) {
		fi = iter_in->second;
		if(fi)
			fi->detachClient();
		fi = 0;
	}
	
	FlowOutletOut *fo = 0;
	std::map<unsigned int, FlowOutletOut*> *fout = getModel()->getOutputFlowList();
	map<unsigned int, FlowOutletOut *>::iterator iter_out;
	for( iter_out = fout->begin(); iter_out != fout->end(); iter_out++) {
		fo = iter_out->second;
		if(fo) 
			fo->detachClient();
		fo = 0;
	}
	setImage();
	resetFlows();
}

//give an order to flows in order to have a nice display
void ClientGraphicObject::resetFlows()
{
	int basey;
	basey = (int)y()/TILE_SY+(_height/TILE_SY)/2-(getModel()->getInputFlowList()->size()/2);
	int i = 0;
	
	multimap<unsigned int, FlowGraphicOutlet *> flowsOutletOut;
	flowsOutletOut.clear();
	
	multimap<unsigned int, FlowGraphicOutlet *> flowsOutletIn;
	flowsOutletIn.clear();
	
	std::map<unsigned int, FlowOutletIn*> *fin = getModel()->getInputFlowList();
	std::map<unsigned int, FlowOutletIn*>::iterator iter_in;
	for( iter_in = fin->begin(); iter_in != fin->end(); iter_in++) {
		FlowOutletIn *f = iter_in->second;
		if(f) {
			Flow *fo = f->getFlow();
			int tmp = 0; 
			if ( fo )
				tmp = fo->getClientInId();
			if ( tmp != 0 ) {
				int posy = (int) mapView->getClient(tmp)->y();
				flowsOutletIn.insert(make_pair(posy, (f)->getFlowOutletGraphic()));
			}
			else
				flowsOutletIn.insert(make_pair(INT_MAX, (f)->getFlowOutletGraphic()));
		} 
	} 
	
	multimap<unsigned int, FlowGraphicOutlet *>::iterator iter_multi_in;
	basey = ((int)y()/TILE_SY)+(_height/TILE_SY)/2-(fin->size()/2);
	i = 0; 
	for( iter_multi_in = flowsOutletIn.begin(); iter_multi_in != flowsOutletIn.end(); iter_multi_in++) {
		FlowGraphicOutlet *f = iter_multi_in->second;
		if(f) {
			f->setPosition(((int)x()/TILE_SX)-1, basey+i);     
		}
		i++;
	}
	
	std::map<unsigned int, FlowOutletOut*> *fout = getModel()->getOutputFlowList();
	map<unsigned int, FlowOutletOut*>::iterator iter_out;
	for( iter_out = fout->begin(); iter_out != fout->end(); iter_out++) {
		FlowOutletOut *f = iter_out->second;
		if(f) {
			Flow *fo = f->getFlow();
			std::vector<unsigned int> tmp;
			tmp.clear(); 
			if ( fo )
				tmp = fo->getClientOutId();
			if ( !tmp.empty() ) {
				int posy = (int) mapView->getClient(tmp.front())->y();
				flowsOutletOut.insert(make_pair(posy, (f)->getFlowOutletGraphic()));
			}
			else
				flowsOutletOut.insert(make_pair(INT_MAX, (f)->getFlowOutletGraphic()));
		}
	}
	
	multimap<unsigned int, FlowGraphicOutlet *>::iterator iter_multi_out;
	basey = ((int)y()/TILE_SY)+(_height/TILE_SY)/2-(fout->size()/2);
	i = 0; 
	for( iter_multi_out = flowsOutletOut.begin(); iter_multi_out != flowsOutletOut.end(); iter_multi_out++) {
		FlowGraphicOutlet *f = iter_multi_out->second;
		if(f) {
			f->setPosition((((int)x()+_width)/TILE_SX), basey+i);     
		}
		i++;
	}
}


void ClientGraphicObject::resetInputFlows()
{
	int basey;
	basey = (int)y()/TILE_SY+(_height/TILE_SY)/2-(getModel()->getInputFlowList()->size()/2);
	int i = 0;
	
	multimap<unsigned int, FlowGraphicOutlet *> flowsOutletIn;
	flowsOutletIn.clear();
	
	std::map<unsigned int, FlowOutletIn*> *fin = getModel()->getInputFlowList();
	std::map<unsigned int, FlowOutletIn*>::iterator iter_in;
	for( iter_in = fin->begin(); iter_in != fin->end(); iter_in++) {
		FlowOutletIn *f = iter_in->second;
		if(f) {
			Flow *fo = f->getFlow();
			int tmp = 0; 
			if ( fo )
				tmp = fo->getClientInId();
			if ( tmp != 0 ) {
				int posy = (int) mapView->getClient(tmp)->y();
				flowsOutletIn.insert(make_pair(posy, (f)->getFlowOutletGraphic()));
			}
			else
				flowsOutletIn.insert(make_pair(INT_MAX, (f)->getFlowOutletGraphic()));
		} 
	}    
	
	multimap<unsigned int, FlowGraphicOutlet *>::iterator iter_multi_in;
	basey = ((int)y()/TILE_SY)+(_height/TILE_SY)/2-(fin->size()/2);
	i = 0; 
	for( iter_multi_in = flowsOutletIn.begin(); iter_multi_in != flowsOutletIn.end(); iter_multi_in++) {
		FlowGraphicOutlet *f = iter_multi_in->second;
		if(f) {
			f->setPosition(((int)x()/TILE_SX)-1, basey+i);     
		}
		i++;
	}
}

void ClientGraphicObject::refresh() {
	unsigned int s = getSizeType();
	if ( s != size_type ) {
		size_type = s;
		switch(size_type) {
			case 0:
				setSize(50,50);
				break;
			case 1:
				setSize(70,70);
				break;
			case 2:
				setSize(90,90);
				break;
			default:
				break;
		}
		setImage();
	}
	resetFlows();
}

void ClientGraphicObject::setSelected(bool s)
{
	if ( AbstractGraphicObject::selected != s ) {
		AbstractGraphicObject::setSelected(s);
		setImage();
		mapView->update();
	}
}

void ClientGraphicObject::setPos(int _x, int _y) {
	setX(_x*TILE_SX);
	setY(_y*TILE_SY);
	sprite->setX(_x*TILE_SX);
	sprite->setY(_x*TILE_SY);
	label->move(((2*(int)x())+_width)/2, ((int)y())+_height);
}


void ClientGraphicObject::moveRequest(int _x, int _y) {
	Q3CanvasPolygonalItem::move(_x, _y);
	sprite->move(_x, _y);
	label->move(((2*(int)x())+_width)/2, ((int)y())+_height);
}

void ClientGraphicObject::drawClient() {
	move();
}

void ClientGraphicObject::eraseInMap() {
	mapView->getPf()->removeObstacle(this->getId());
}


void ClientGraphicObject::drawClientWithOutlets() 
{
	set<unsigned int> tmp;
	//move the label of the client
	label->move(((2*(int)x())+_height)/2, ((int)y())+_width);
	
	//move the sprite
	sprite->move((int)x(), (int)y());
	
	//erase the client from the Map
	eraseInMap();
	
	//erase all the flow (input and output), which are connected to the client
	std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
	std::map<unsigned int, FlowOutletIn *> *fin = getModel()->getInputFlowList();
	for(iter_in = fin->begin(); iter_in != fin->end(); iter_in++ ) 
		(iter_in->second)->getFlowOutletGraphic()->eraseFlowAndOutlet();
	std::map<unsigned int, FlowOutletOut *>::iterator iter_out;
	std::map<unsigned int, FlowOutletOut *> *fout = getModel()->getOutputFlowList();
	for(iter_out = fout->begin(); iter_out != fout->end(); iter_out++ ) 
		(iter_out->second)->getFlowOutletGraphic()->eraseFlowAndOutlet();
	
	//draw the client
	tmp = drawInMap();
	
	//change the position of the outlet's flow
	resetFlows();
	
	//draw the flow's outlet
	for(iter_in = fin->begin(); iter_in != fin->end(); iter_in++ ) 
		(iter_in->second)->getFlowOutletGraphic()->drawOutletInMap();
	for(iter_out = fout->begin(); iter_out != fout->end(); iter_out++ ) 
		(iter_out->second)->getFlowOutletGraphic()->drawOutletInMap();
}

void ClientGraphicObject::drawClientFlows() { 
	std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
	std::map<unsigned int, FlowOutletIn *> *fin = getModel()->getInputFlowList();
	std::map<unsigned int, FlowOutletOut *>::iterator iter_out;
	std::map<unsigned int, FlowOutletOut *> *fout = getModel()->getOutputFlowList();
	//draw the flow with the new position  
	for(iter_in = fin->begin(); iter_in != fin->end(); iter_in++ ) {
		(iter_in->second)->getFlowOutletGraphic()->drawFlowInMap();
	}
	for(iter_out = fout->begin(); iter_out != fout->end(); iter_out++ ) 
		(iter_out->second)->getFlowOutletGraphic()->drawFlowInMap();
	
	MainWindow::instance()->appliNeedSave = true;
}

void ClientGraphicObject::move() 
{
	set<unsigned int> tmp;
	//move the label of the client
	label->move(((2*(int)x())+_height)/2, ((int)y())+_width);
	
	//move the sprite
	sprite->move((int)x(), (int)y());
	
	//erase the client from the Map
	eraseInMap();
	
	//erase all the flow (input and output), which are connected to the client
	std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
	std::map<unsigned int, FlowOutletIn *> *fin = getModel()->getInputFlowList();
	for(iter_in = fin->begin(); iter_in != fin->end(); iter_in++ ) 
		(iter_in->second)->getFlowOutletGraphic()->eraseFlowAndOutlet();
	std::map<unsigned int, FlowOutletOut *>::iterator iter_out;
	std::map<unsigned int, FlowOutletOut *> *fout = getModel()->getOutputFlowList();
	for(iter_out = fout->begin(); iter_out != fout->end(); iter_out++ ) 
		(iter_out->second)->getFlowOutletGraphic()->eraseFlowAndOutlet();
	
	//draw the client
	tmp = drawInMap();
	
	//change the position of the outlet's flow
	resetFlows();
	
	//draw the flow's outlet
	for(iter_in = fin->begin(); iter_in != fin->end(); iter_in++ ) 
		(iter_in->second)->getFlowOutletGraphic()->drawOutletInMap();
	for(iter_out = fout->begin(); iter_out != fout->end(); iter_out++ ) 
		(iter_out->second)->getFlowOutletGraphic()->drawOutletInMap();
	
	
	//draw the flow with the new position
	for(iter_in = fin->begin(); iter_in != fin->end(); iter_in++ ) {
		(iter_in->second)->getFlowOutletGraphic()->drawFlowInMap();
	}
	for(iter_out = fout->begin(); iter_out != fout->end(); iter_out++ ) 
		(iter_out->second)->getFlowOutletGraphic()->drawFlowInMap();
	
	if (!tmp.empty()) {
		//pb with outlets which disappear
		coli = tmp;
		set<unsigned int>::iterator iter_f;
		FlowGraphicObject *tmp_f;
		tmp_f = 0;
		//erase every flows, which has an intersection with the client
		for(iter_f = tmp.begin(); iter_f != tmp.end(); iter_f++ ) {
			mapView->flowsCol.insert(*iter_f);
			tmp_f = mapView->getFlow(*iter_f);
			if ( tmp_f )
				tmp_f->erasePathWithoutOutlet();
			tmp_f = 0;
		}
		//draw every flows, which had an intersection with the client
		for(iter_f = tmp.begin(); iter_f != tmp.end(); iter_f++ ) {
			tmp_f = mapView->getFlow(*iter_f);
			if ( tmp_f ) {
				if ( !tmp_f->drawPathInMap() )
					undrawFlows.insert(tmp_f);
			}
			tmp_f = 0;
		}
	}
	/* else {
		set<unsigned int>::iterator iter_f;
    FlowGraphicObject *tmp_f;
    tmp_f = 0;
    //erase every flows, which has an intersection with the client
    cout << "coli count=" << coli.size() << endl;
    for(iter_f = coli.begin(); iter_f != coli.end(); iter_f++ ) {
		//mapView->flowsCol.insert(*iter_f);
		tmp_f = mapView->getFlow(*iter_f);
		if ( tmp_f )
			tmp_f->erasePath();
		tmp_f = 0;
    }
    //draw every flows, which had an intersection with the client
    for(iter_f = coli.begin(); iter_f != coli.end(); iter_f++ ) {
		tmp_f = mapView->getFlow(*iter_f);
		if ( tmp_f )
			tmp_f->drawPathInMap();
		tmp_f = 0;
    }
    coli.clear();
	}

std::set<FlowGraphicObject *>::iterator iter_undrawFlows;
cout << "undrawFlow count=" << undrawFlows.size() << endl;
for ( iter_undrawFlows = undrawFlows.begin(); iter_undrawFlows != undrawFlows.end(); iter_undrawFlows++ ) {
    FlowGraphicObject *tmp_f = (FlowGraphicObject *)(*iter_undrawFlows);
    if ( tmp_f )
		if ( tmp_f->drawPathInMap() )
			undrawFlows.erase(tmp_f);
}*/
	MainWindow::instance()->appliNeedSave = true;
}

void ClientGraphicObject::setSize(int _x, int _y) {
	if ( (_x >= 0) && (_y >= 0) ) {
		_width = _x;
		_height = _y;
	}
}

QPoint ClientGraphicObject::getSize() {
	return QPoint(_width, _height);
}

void ClientGraphicObject::setLabel(const QString &l)
{
	label->setText(l);
}

Q3PointArray ClientGraphicObject::areaPoints() const {
	int nbIn = this->getModel()->getInputFlowList()->size();
	int nbOut = this->getModel()->getOutputFlowList()->size();
	if ( (nbIn <= 9) && (nbOut <= 9) ) {
		Q3PointArray tmp(4);
		tmp[0] = QPoint((int)x()-10,(int)y()-3);
		tmp[1] = tmp[0] + QPoint(_width+20,0);
		tmp[2] = tmp[1] + QPoint(0,_height+6);
		tmp[3] = tmp[0] + QPoint(0,_height+6);
		return tmp;
	}
	else {  
		Q3PointArray tmp(12);
		int m = 5;
		tmp[0] = QPoint((int)x()-TILE_SX-m, (nbIn > 9) ? (int)y()+((_height/TILE_SY)/2)*TILE_SY-(nbIn/2)*TILE_SX-m : (int)y()-m);
		tmp[1] = QPoint((int)x(), tmp[0].y());
		tmp[2] = QPoint((int)x(), (int)y()-m);
		tmp[3] = tmp[2] + QPoint(_width,0);
		tmp[4] = QPoint(tmp[3].x(), (nbOut > 9) ? (int)y()+((_height/TILE_SY)/2)*TILE_SY-(nbOut/2)*TILE_SY-m : (int)y()-m);
		tmp[5] = tmp[4] + QPoint(TILE_SX+m, 0);
		tmp[6] = tmp[5] + QPoint(0, (nbOut > 9) ? (nbOut)*TILE_SY+(2*m) : (_height+(2*m)));
		tmp[7] = QPoint((int)x()+_width, tmp[6].y());
		tmp[8] = QPoint((int)x()+_width, (int)y()+_height+m);
		tmp[9] = QPoint((int)x(), (int)y()+_height+m);
		tmp[10] = tmp[1] + QPoint(0,(nbIn > 9) ? (nbIn)*TILE_SY+(2*m) : (_height+(2*m)));
		tmp[11] = QPoint((int)x()-TILE_SX-m, tmp[10].y());
		return tmp;
	}
}

set<unsigned int> ClientGraphicObject::drawInMap(){
	return mapView->getPf()->createObstacle((int)x()/TILE_SX, (int)y()/TILE_SY, _width/TILE_SX, _height/TILE_SY, getId());
}

void ClientGraphicObject::draw ( QPainter & painter ) {
	sprite->draw(painter);
	Q3PointArray tmp = areaPoints();
	tmp.putPoints(tmp.size(), 1, tmp[0].x(), tmp[0].y());
	//painter.drawPolyline(tmp);
}

void ClientGraphicObject::drawShape(QPainter&){}

unsigned int ClientGraphicObject::getId() { return clientApp->getId(); }

bool ClientGraphicObject::isOkayToEdit() {
	ClientApp *capp = getModel();
	if ( capp ) {
		std::map<unsigned int, FlowOutletOut*>* fooutlist = capp->getOutputFlowList();
		std::map<unsigned int, FlowOutletOut*>::iterator fooutlist_iter;
		for ( fooutlist_iter = fooutlist->begin(); fooutlist_iter != fooutlist->end(); fooutlist_iter++ ) {
			FlowOutletOut *foout = (FlowOutletOut*) fooutlist_iter->second;
			if ( foout->hasFlow() ) {
				Flow *fl = foout->getFlow();
				std::set<FlowOutletIn *> *foinlist = fl->getOutletsIn();
				std::set<FlowOutletIn *>::iterator foinlist_iter;
				for ( foinlist_iter = foinlist->begin(); foinlist_iter != foinlist->end(); foinlist_iter++ ) {
					ClientApp *clapp = dynamic_cast<ClientApp *>((*foinlist_iter)->getClient());
					if ( clapp ) {
						if ( clapp->isRunning() || clapp->isStarting() )
							return false;
					}
				}
			}
		}
	}
	else
		return false;
	return true;
}

bool ClientGraphicObject::isOkayToRemove() {
	ClientApp *capp = getModel();
	if ( capp ) {
		std::map<unsigned int, FlowOutletOut*>* fooutlist = capp->getOutputFlowList();
		std::map<unsigned int, FlowOutletOut*>::iterator fooutlist_iter;
		for ( fooutlist_iter = fooutlist->begin(); fooutlist_iter != fooutlist->end(); fooutlist_iter++ ) {
			FlowOutletOut *foout = (FlowOutletOut*) fooutlist_iter->second;
			if ( foout->hasFlow() ) {
				Flow *fl = foout->getFlow();

				if ( fl->countOutletInRunning() > 1 )
					return false;
			}
		}
	}
	else
		return false;
	return true;
}

void ClientGraphicObject::showPopupMenu(const QPoint &p) {
    popupMenu.clear();
	
    SfControlThread *sfct = MainWindow::instance()->global->getSfControlThread(); 
    if ( sfct ) {
		SfControl *sfc = sfct->getSfControl();
		if ( sfc ) {
			if ( sfc->isConnectedToServer() ) {
				if ( getModel()->isIdle() || getModel()->isError() )
					popupMenu.insertItem(tr("&Start Client"), this, SLOT(startClientAction()));
				else if ( getModel()->isStarting() || getModel()->isRunning() )
					popupMenu.insertItem(tr("S&top Client"), this, SLOT(stopClientAction()));
				popupMenu.insertSeparator();
			}
		}
    }
    if ( !(getModel()->isStarting() || getModel()->isRunning()) ) {
		popupMenu.insertItem(tr("&Edit..."), this, SLOT(editClientAction()));
		popupMenu.insertItem(tr("&Remove"), this, SLOT(removeClientAction()));
		popupMenu.insertSeparator();
    }
    if ( !getModel()->isRunning() ) {
		if ( getModel()->isDisable() )	
			popupMenu.insertItem(tr("Ena&ble"), this, SLOT(enableDisableClientAction()));
		else
			popupMenu.insertItem(tr("&Disable"), this, SLOT(enableDisableClientAction()));
    }
    popupMenu.insertItem(tr("Show &Log..."), this, SLOT(showClientLogAction()));
    popupMenu.insertItem(tr("&Clear Log"), this, SLOT(clearClientLogAction()));
	
	popupMenu.popup(p);
}


/*****************************************************************************************************************************/

MapCanvas::MapCanvas() : Q3Canvas(SIZEMAPX*TILE_SX, SIZEMAPY*TILE_SY)//QCanvas(QPixmap::fromMimeSource("Tiles.xpm"), SIZEMAPX, SIZEMAPY, TILE_SX, TILE_SY)
{
	setDoubleBuffering ( true );
	setAdvancePeriod(500);
}

MapCanvas::~MapCanvas()
{
}

void MapCanvas::setZoneRect(const QRect &r)
{
	if(zone_rect.isValid())
		setChanged(zone_rect);
	zone_rect = r;
	if(zone_rect.isValid())
		setChanged(zone_rect);  
}

void MapCanvas::drawTheLine(const QPoint &pt1, const QPoint &pt2, const QRect &r) {
	if(zone_rect2.isValid()) {
		p1 = pt1;
		p2 = pt2;
		setChanged(zone_rect2);
	}
	zone_rect2 = r;
	if(zone_rect2.isValid()) {
		p1 = pt1;
		p2 = pt2;
		setChanged(zone_rect2);
	}
} 

void MapCanvas::drawForeground(QPainter &painter, const QRect &clip)
{
	Q_UNUSED(clip);
	if(zone_rect.isValid()) {
		painter.setPen(Qt::black);
		painter.setPen(Qt::DotLine);
		painter.setBrush(Qt::NoBrush);
		painter.drawRect(zone_rect);
	}
	else if ( zone_rect2.isValid() ) {
		painter.setPen(Qt::black);
		painter.setPen(Qt::DotLine);
		painter.drawLine(p1, p2);
	}
}

/*****************************************************************************************************************************/

MapRoute::MapRoute(){
	size = 0;
	mat = 0;
}

MapRoute::MapRoute(int **m, int s, std::map<unsigned int, unsigned int> &_realId, MapView *map, vector<int> &sClient){
	size = s;
	mat = m;
	realId = _realId;
	mapView = map;
	sizeClients = sClient;
}

MapRoute::~MapRoute(){
	for ( int i = 0; i < size; ++i ) 
		delete [] mat[i];
	delete []mat;
}

void MapRoute::initMatrix() {
	if ( size != 0 ) {
		mat = new int*[size];
		std::fill_n(mat, size, static_cast<int*>(0));
		for ( int i = 0; i < size; ++i) {
			mat[i] = new int[size];
			std::fill_n(mat[i], size, static_cast<int>(0));
		}
	}
}

void MapRoute::setMatrix() {
	size = 6;
	initMatrix();
	mat[0][1] = 1;
	mat[0][2] = 1;
	mat[0][3] = 1;
	mat[0][5] = 1;
	mat[3][4] = 1;
	mat[4][2] = 1;
	//mat[4][5] = 1;
	//mat[5][1] = 1;
}

int** MapRoute::delCircuit() {
	int** myMatrix = NULL;
	if ( size != 0 ) {
		myMatrix = new int*[size];
		std::fill_n(myMatrix, size, static_cast<int*>(0));
		for ( int i = 0; i < size; ++i) {
			myMatrix[i] = new int[size];
			std::fill_n(myMatrix[i], size, static_cast<int>(0));
		}
		for(int i=0; i < size; i++) {
			for (int j=0; j < size; j++) {
				if (mat[i][j] == 1) {
					myMatrix[i][j] = 1;
					if (hasCircuit(myMatrix))
						myMatrix[i][j]=0;
				}
			}
		} 
	}
	return myMatrix;
}

bool MapRoute::hasCircuit(int **new_mat){
	//int intern_degree[size];
	int *intern_degree = new int[size];
	std::vector<int> source; 
	int nbSommets = 0;
	for ( int i = 0; i < size; i++)
		intern_degree[i] = 0;
	for ( int i = 0; i < size; i++)
		for (int j = 0; j < size; j++ )
			if ( new_mat[i][j] == 1 )
				intern_degree[j] += 1;
	source.clear();
	for ( int i = 0; i < size; i++ )
		if ( intern_degree[i] == 0 )
			source.push_back(i);
	int tmp_x;
	while ( !source.empty() ) {
		tmp_x = source.back();
		source.pop_back();
		nbSommets++;
		for ( int j = 0; j < size; j++)
			if ( new_mat[tmp_x][j] == 1 ) {
				intern_degree[j] -= 1;
				if ( intern_degree[j] == 0)
					source.push_back(j);
			}
	}
		return ( nbSommets < size );  
}

void MapRoute::displayLevels() {
	std::vector< vector<int> >::iterator iter1;
	std::vector<int>::iterator iter2;
	int i = 0;
	for ( iter1 = lev.begin(); iter1 != lev.end(); iter1++ ){
		cout << "Niveau " << i << " :";
		for ( iter2 = (*iter1).begin(); iter2 != (*iter1).end(); iter2++ ) {
			cout << " " << (*iter2);
		}
		cout << endl;
		i++;
	}
}

typedef struct {
	unsigned int id;
	int x;
	int y;
} ClientPos;

bool MapRoute::organize(){
	if ( hasCircuit(mat) )
		mat = delCircuit();
	if ( !hasCircuit(mat) && (size != 0) ) {
		int nbSommets = 0;
		//int intern_degree[size];
		int *intern_degree = new int[size];
		lev.resize(size);
		for ( int i = 0; i < size; i++)
			intern_degree[i] = 0;
		for ( int i = 0; i < size; i++)
			for (int j = 0; j < size; j++ )
				if ( mat[i][j] == 1 )
					intern_degree[j] += 1;
		for ( int i = 0; i < size; i++ )
			if ( intern_degree[i] == 0 ) {
				lev[0].push_back(i);
				nbSommets++;
			}
				unsigned int num_lev = 0;
		while ( num_lev < lev.size() ){
			std::vector<int>::iterator iter;
			for ( iter = lev[num_lev].begin(); iter != lev[num_lev].end(); ++iter )
				for ( int j = 0; j < size; j++ ) {
					if ( mat[*iter][j] == 1 ){
						intern_degree[j] -= 1;
						if ( intern_degree[j] == 0 ) {
							lev[num_lev+1].push_back(j);
							nbSommets += 1;
						}
					}
				}//for
					num_lev++;
		}
		
		const std::vector<QPoint> tmp_pt = getPositions();
		
		for ( int k = 0; k < size; k++ ) 
			mapView->getClient(realId[k])->setPos(tmp_pt[k].x(),tmp_pt[k].y());
		
		std::deque<ClientPos> organizedLevel;
		bool insertInFirstPosition = true;
		
		for (std::vector<int>::iterator iter_level = lev[0].begin(); iter_level != lev[0].end(); iter_level++) {
			ClientPos position;
			position.id = (*iter_level);
			bool hasConsumer = false;
			for (int j = 0; j < size; j++ ) {
				if ( mat[*iter_level][j] > 0 ) {
					position.x = mapView->getClient(realId[j])->pos().x();
					position.y = mapView->getClient(realId[j])->pos().y();
					bool inserted = false;
					for (std::deque<ClientPos>::iterator iter_org = organizedLevel.begin(); iter_org != organizedLevel.end(); iter_org++) {
						if ((position.x < (*iter_org).x) || ((position.x == (*iter_org).x) && (position.y < (*iter_org).y))) {
							organizedLevel.insert(iter_org, position);
							inserted = true;
							break;
						}
					}
					if (!inserted)
						organizedLevel.push_back(position);  
					hasConsumer = true;
					break;
				}
			}
			if (!hasConsumer) {
				if (insertInFirstPosition) {
					position.x = -1;
					position.y = -1;
					organizedLevel.push_front(position);
					insertInFirstPosition = false;
				}
				else {
					position.x = 2000;
					position.y = 2000;
					organizedLevel.push_back(position);
					insertInFirstPosition = true;
				}				
			}
		}
		lev[0].clear();
		for (std::deque<ClientPos>::iterator iter_org = organizedLevel.begin(); iter_org != organizedLevel.end(); iter_org++)
			lev[0].push_back((*iter_org).id);
		
		int tiley = 5;
		int tilex = 9;
		int j = 0;
		int size_lev = lev[0].size();
		unsigned int max_lev = getMaxLevel();
		int shift = 0;
		for ( std::vector<int>::iterator iter2 = lev[0].begin(); iter2 != lev[0].end(); iter2++ ) {
			//useless in the process because pos is not used anymore but stay here in order to keep the right info inside the MapRoute Object.
			pos[(*iter2)] = QPoint(tilex, tiley*(1 + max_lev - size_lev + j*2)+shift);
			//usefull
			mapView->getClient(realId[(*iter2)])->setPos(pos[*iter2].x(),pos[*iter2].y());
			shift += sizeClients[*iter2];
			j++;
		}
		
		return true;
	}
	return false;
}

unsigned int MapRoute::getMaxLevel(){
	unsigned int max = 0;
	std::vector< vector<int> >::const_iterator iter;
	for ( iter = lev.begin(); iter != lev.end(); iter++ )
		if ( max < ((*iter).size()) )
			max = ((*iter).size());
	return max;
} 


const std::vector<QPoint>& MapRoute::getPositions(){
	pos.resize(size);
	unsigned int max_lev = getMaxLevel();
	int tiley = 5;
	int tilex = 9;
	std::vector< vector<int> >::iterator iter1;
	std::vector<int>::iterator iter2;
	int i = 0;
	int size_lev = 0;
	for ( iter1 = lev.begin(); iter1 != lev.end(); iter1++ ){
		size_lev = (*iter1).size();
		int j = 0;
		int shift = 0;
		for ( iter2 = (*iter1).begin(); iter2 != (*iter1).end(); iter2++ ) {
			pos[(*iter2)] = QPoint(tilex*(1 + 2*i), tiley*(1 + max_lev - size_lev + j*2)+shift);
			shift += sizeClients[*iter2];
			j++;
		}
		i++;
	}
	return pos;
}

void MapRoute::displayPos() {
	QPoint tmp;
	for (int i = 0; i < size; i++) {
		tmp = pos[i];
		cout << "Num=" << i << " X=" << tmp.x() << " Y=" << tmp.y() << endl;
	}
}

void MapRoute::displayMatrix() {
	if ( mat ) {
		for ( int i = 0; i < size; i++ )
			cout << "**";
		cout << "**" << endl;
		for ( int j = 0; j < size; j++ ){
			cout << "* ";
			for ( int i = 0; i < size; i ++ ) {
				cout << mat[i][j] << " ";
			}
			cout << "*" << endl;
		}
		for ( int i = 0; i < size+1; i++ )
			cout << "**";
		cout << endl;
	}
}

