#include "PathFinding.h"

#include <iostream>
#include <set>
using namespace std;
using std::hex;

PathFinding::PathFinding(){
	graph = new Graph();
	clearMap();
	startXGlobal = startYGlobal = 0;
}

PathFinding::~PathFinding() {
	delete graph;
}

void PathFinding::clearMap(){
	memset(map, 0, sizeof(map));
	memset(idMap, 0, sizeof(idMap));
}

void PathFinding::clearHeap(){
	heap.clear();
}

void PathFinding::setMapCell(int x, int y, int val){
	if ( exist(x,y) )
		map[x][y] = val;
}

int PathFinding::getMapCell(int x, int y){
	if ( exist(x,y) )
		return map[x][y];
	else 
		return -1;
}

void PathFinding::setMapCellId(int x, int y, unsigned int _id){
	if ( exist(x,y) ) 
		idMap[0][x][y] = _id;
}

void PathFinding::setMapCellIdUnder(int x, int y, unsigned int _id){
    idMap[1][x][y] = _id;
}

unsigned int PathFinding::getMapCellId(int x, int y) {
	if ( exist(x,y) )
		return idMap[0][x][y];
	else 
		return 0;
}

unsigned int PathFinding::getMapCellIdUnder(int x, int y) {
	if ( exist(x,y) )
		return idMap[1][x][y];
	else 
		return 0;
}

void PathFinding::insertLeaf(int _x, int _y, int _hcost){
	heap.push_back(leaf(_x, _y, _hcost));
	if ( heap.size() > 1 ) {
		int num_son = heap.size()-1;
		int num_father = (heap.size() - 2)/2;
		while ( num_son != 0 && ( heap[num_father].hcost > heap[num_son].hcost ) ) {
			swap(heap[num_father], heap[num_son]);
			num_son = num_father;
			num_father = (num_father -1)/2;
		}
	}
}

int PathFinding::getMin(unsigned int num1, unsigned int num2){
	if( (num1 < heap.size()) && (num2 < heap.size()) )
		if( heap[num1].hcost < heap[num2].hcost )
			return num1;
		else
			return num2;
	else
		if( num1 < heap.size() ) 
			return num1;
    else 
		if( num2 < heap.size() )
			return num2;
	else
		return -1;
}

void PathFinding::removeMinLeaf(){
	swap(heap.front(), heap.back());
	heap.pop_back();
	int num_father = 0;
	int num_min = getMin(1, 2);
	while( ( num_min != -1 ) && ( heap[num_father].hcost > heap[num_min].hcost ) ) {
		swap(heap[num_father], heap[num_min]);
		num_father = num_min;
		num_min = getMin((2*num_father + 1), (2*num_father + 2));
	}
    
}

//only for debugging
void PathFinding::printHeap() {
	for ( unsigned int i = 0; i < heap.size(); i++)
		cout << i << " " << heap[i].hcost << endl;
}


PathFinding::leaf PathFinding::getMinLeaf(){
	return heap.front();
}

bool PathFinding::graphIsEmpty() {
	if ( heap.size() == 0 )
		return true;
	else 
		return false;
}

bool PathFinding::exist(int x, int y){
	if ( ( x >= 0 ) && ( x < SIZEMAPX ) && ( y >= 0 ) && ( y < SIZEMAPY ) )
		return true;
	else 
		return false;
}

int PathFinding::getHeuristicDistance(int startx, int starty, int endx, int endy){
	return (abs(startx - endx) + abs(starty -endy));
}

Q3ValueList<QPoint> PathFinding::getPath(int startx, int starty, int endx, int endy, unsigned int idFlow) {
	bool find = false;
	startXGlobal = startx;
	startYGlobal = starty;
	Q3ValueList<QPoint> lst;
	//check if the starting tile is in the map
	if ( exist(startx, starty) ) {
		//estimation of the distance between the starting tile and the ending tile
		int hcost = getHeuristicDistance(startx, starty, endx, endy);
		//intern process. For more information, study the A* algorithm
		graph->setTile(startx, starty, 0x00, 0, hcost);
		graph->setOpen(startx, starty);
		//we insert the info corresponding to this tile in the heap
		insertLeaf(startx, starty, hcost);
		int tmpx, tmpy;
		//until we find the ending tile or we explore all possibilities.
		while ( !(graphIsEmpty()) && find == false) {
			//we get the leave with the smallest heuristic distance representing the closest tile (which has been explored) to the ending we explored  
			tmpx = getMinLeaf().x;
			tmpy = getMinLeaf().y;
			//we remove the leave from the heap
			removeMinLeaf();
			//we set the tile corresponding to the leave to close. For more information, have a look at A*
			graph->setClose(tmpx, tmpy);
			//Is this tile the ending tile?
			if ( !(( tmpx == endx ) && ( tmpy == endy )) ){
				//No, we explore then 4 tiles, the one on the left, the one on the right, the one above and the one beneath
				//the one above
				explore(tmpx, tmpy - 1, tmpx, tmpy, endx, endy, idFlow);
				//the one beneath
				explore(tmpx, tmpy + 1, tmpx, tmpy, endx, endy, idFlow);
				//the one on the right
				explore(tmpx + 1, tmpy, tmpx, tmpy, endx, endy, idFlow);
				//the one on the left
				explore(tmpx - 1, tmpy, tmpx, tmpy, endx, endy, idFlow);
			}
			else 
				//yes we find the ending tile
				find = true;
		}// while
		if ( find ) {
			//printPath() is writing the path in the map with the correct base, setting the id for this path. 
			//printPath is doing a reverse parsing from the ending tile to the starting tile using the graph structure.
			lst = printPath(startx, starty, endx, endy, idFlow);
		}
	}
	//work is done, we clear the heap
	clearHeap();
	//and we clear the graph
	graph->clearGraph();
	return lst;
}



void PathFinding::explore(int x, int y, int px, int py, int end_x, int end_y, unsigned int idFlow){
	if ( exist(x,y) && (map[x][y] != C )) {
		int tmp_cost = 0;
		bool ok = false;
		// we have to check if the tile has no obstacle. The checking has to include the previous explored tile
		switch (map[x][y]) {
			case BEF:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case BEF_U:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case BEF_D:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case BEF_L:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case BEF_R:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case NO:ok = true;
				ok = true;
				break;
			case NE:
				ok = true;
				tmp_cost += 3;
				//tmp_cost += 2;
				break;
			case H:
				if ( (px == x) && (idMap[0][x][y] != idFlow) ) {
					tmp_cost += 6;
					ok = true;
				}
				else if ( (px != x) && (idMap[0][x][y] == idFlow) ) 
					ok = true;
				break;
			case V:
				if ( (py == y) && (idMap[0][x][y] != idFlow) ) {
					tmp_cost += 6;
					ok = true;
				}
				else if ( (py != y) && (idMap[0][x][y] == idFlow) )
					ok = true;
				break;
				//not tested yet!!!
			case HonV:
				if ((idMap[0][x][y] == idFlow) || (idMap[1][x][y] == idFlow) ) 
					ok = true;
				break;
				//not tested yet!!!
			case VonH:
				if ((idMap[0][x][y] == idFlow) || (idMap[1][x][y] == idFlow) ) 
					ok = true;
				break;
			case AngleUL:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case AngleUR:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case AngleDL:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case AngleDR:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case RightT:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case LeftT:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case UpT:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case DownT:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
			case Cross:
				if ( idMap[0][x][y] == idFlow )
					ok = true;
				break;
		}
		//okay, the flow may use this tile.
		//so we include this tile in the graph. We are giving the direction we are going. This information is needed to do the reverse parsing of the path if founded.
		if ( ok ) {
			// we come Up or Down
			if ( px == x )
				// we come Down
				if ( y == (py + 1) ){
					// Is there an angle?
					if ( !(graph->fromDown(px, py)) ) {
						//Are we blocking the beginning or the end of a flow?
						if ( ( ((x+1) < SIZEMAPX) && (idMap[0][x+1][y] != idFlow) && (map[x+1][y] == BEF) ) || ( ((x-1) >= 0) && (idMap[0][x-1][y] != idFlow) && (map[x-1][y] == BEF) ) )
							tmp_cost +=50;
						else if ( py == startYGlobal ) {
							//cout << "we are on the abs ord" << endl;
							if ( (abs(x-startXGlobal) < (abs(end_x-startXGlobal)/3) ) || ( abs(x-startXGlobal) > (abs(end_x-startXGlobal)*2/3) ) ) {
								//cout << abs(x-startXGlobal) << " < " << (abs(end_x-startXGlobal)/3) << "    " << abs(x-startXGlobal) << " > " << (abs(end_x-startXGlobal)*2/3) << endl;
								tmp_cost +=5;
							}
							/*  else
							cout << "should be better" << endl;*/
						}
						else
							//we penalise the angle
							tmp_cost += 2;
					}
				}
					else { // we come Up
						   //Is there an angle? 
						if ( !(graph->fromUp(px, py)) ) {
							//Are we blocking the beginning or the end of a flow?
							if ( ( ((x+1) < SIZEMAPX) && (idMap[0][x+1][y] != idFlow) && (map[x+1][y] == BEF) ) || ( ((x-1) >= 0) && (idMap[0][x-1][y] != idFlow) && (map[x-1][y] == BEF) ) )
								tmp_cost +=50;
							else if ( py == startYGlobal ) {
								//cout << "we are on the abs ord" << endl;
								if ( (abs(x-startXGlobal) < (abs(end_x-startXGlobal)/3) ) || ( abs(x-startXGlobal) > (abs(end_x-startXGlobal)*2/3) ) ) {
									//cout << abs(x-startXGlobal) << " < " << (abs(end_x-startXGlobal)/3) << "    " << abs(x-startXGlobal) << " > " << (abs(end_x-startXGlobal)*2/3) << endl;
									tmp_cost +=5;
								}
								/*  else
								cout << "should be better" << endl;*/
							}
							else
								tmp_cost += 2;
						}
					}
					// we come Right or Left
					else 
						// we come from right
						if ( x == (px + 1) ) {
							// cout << "RIGHT" << endl;
							if ( !(graph->fromRight(px, py)) ) {
								//Are we blocking the beginning or the end of a flow?
								if  ( ( ((x+1) < SIZEMAPX) && (idMap[0][x+1][y] != idFlow) && (map[x+1][y] == BEF) ) || ( ((x-1) >= 0) && (idMap[0][x-1][y] != idFlow) && (map[x-1][y] == BEF) ) )
									tmp_cost +=50;
								//we don't penalise the angle if he has the same ordonnee as the starting point and if he is between 1/3 and 2/3 of the x distance
								else {
									//we penalise the angle
									tmp_cost += 2;
									//cout << "else" << endl;
								}
							}
						}
					else { // we come from left
						   // cout << "LEFT" << endl;
						if ( !(graph->fromLeft(px, py)) ) {
							//Are we blocking the beginning or the end of a flow?
							if  ( ( ((x+1) < SIZEMAPX) && (idMap[0][x+1][y] != idFlow) && (map[x+1][y] == BEF) ) || ( ((x-1) >= 0) && (idMap[0][x-1][y] != idFlow) && (map[x-1][y] == BEF) ) )
								tmp_cost +=50;
							//we don't penalise the angle if he has the same ordonnee as the starting point and if he is between 1/3 and 2/3 of the x distance
							else {
								tmp_cost +=2;
								//cout << "else" << endl;
							}
						}
					}
					tmp_cost++;
			//the cost is the actual cost between the starting tile and the current tile + the heuristic distance between the current tile and the ending tile.
			//the path with the lower cost is the better path
			int tmp_hcost = getHeuristicDistance(x, y, end_x, end_y);
			tmp_cost += graph->getCost(px,py);
			//the tile has already been explored and has been used as a point of exploration ( we have already explored the 4 tiles besides it) 
			if ( graph->isClosed(x,y) ) { 
				if( graph->getCost(x,y) > tmp_cost ) {
					//we find a better path to get to this tile
					insertLeaf(x,y, tmp_hcost + tmp_cost);
					graph->setCost(x, y, tmp_cost);
					graph->setHcost(x, y, tmp_cost + tmp_hcost); 
					graph->setOpen(x, y);
					graph->clearDirection(x,y);
					graph->setDirection(x, y, px, py);
				}
				//we find a similar path in a matter of cost to get to this tile, we may need to use it as a point of exploration again, so we set it to open
				if ( graph->getCost(x,y) == tmp_cost ) {
					insertLeaf(x,y, tmp_hcost + tmp_cost);
					graph->setOpen(x, y);
					graph->setDirection(x, y, px, py);
				}
			}
			//this tile has already been explored but not used as a point of exploration
			else if ( graph->isOpen(x,y) ) { 
				if ( graph->getCost(x,y) > tmp_cost ) {
					//we find a better path to get to this tile
					insertLeaf(x,y, tmp_hcost + tmp_cost);
					graph->setCost(x, y, tmp_cost);
					graph->setHcost(x, y, tmp_cost + tmp_hcost);
					graph->clearDirection(x,y);
					graph->setDirection(x, y, px, py);
				}
				if ( graph->getCost(x,y) == tmp_cost ) 
					//we just find an other path to get to this tile
					graph->setDirection(x, y, px, py);
			}
			else {
				//the node is not open nor close
				//this tile has never been explored
				graph->setTile(x, y, 0x00, tmp_cost, tmp_hcost + tmp_cost);
				graph->setOpen(x, y);
				graph->setDirection(x, y, px, py);
				insertLeaf(x, y, tmp_cost + tmp_hcost);
			}
		}
	}
}


Q3ValueList<QPoint> PathFinding::printPath(int startx, int starty, int endx, int endy, unsigned int idFlow){
	int x, y;
	int dir = 0;
	bool ok = false;
	x = endx;
	y = endy;
	previous = 0;
	Q3ValueList<QPoint> lst;
	//we update the map, exploring the graph. We start at the ending tile and we parse the graph until we find the starting tile. It gives us the best path (ie with the lower cost). 
	//each iteration is giving the next tile to the starting point, so we are updating the map
	while ( !(( x == startx ) && ( y == starty )) ){
		updateMap(x, y, dir, startx, starty, endx, endy, idFlow, lst);
		switch (dir) {
			case 1:
				if ( graph->fromUp(x,y) ) {
					ok = true;
					y++;
				}
				else 
					ok = false;
				break;
			case 2:
				if ( graph->fromDown(x,y) ) {
					ok = true;
					y--;
				}
				else 
					ok = false;
				break;
			case 3:
				if ( graph->fromRight(x,y) ) {
					ok = true;
					x--;
				}
				else 
					ok = false;
				break;
			case 4:
				if ( graph->fromLeft(x,y) ) { 
					ok = true;
					x++;
				}
				else 
					ok = false;
				break;
		}
		if ( !ok ) {
			if ( graph->fromUp(x,y) ) {
				y++;
				dir = 1;
			}
			else if ( graph->fromDown(x,y) ) {
				y--;
				dir = 2;
			}
			else if ( graph->fromRight(x,y) ) {
				x--;
				dir = 3;
			}
			else if ( graph->fromLeft(x,y) ) {
				x++;
				dir = 4;
			}
		}
	}//while
	updateMap(x, y, dir, startx, starty, endx, endy, idFlow, lst);
	return lst;
}


void PathFinding::updateMap(int x, int y, int dir, int startx, int starty, int endx, int endy, unsigned int id, Q3ValueList<QPoint> &pt_list) {
	//we update the map regarding what is in the map
	//example, we want to write a vertical flow in the map but there is already an horizontal flow, so we are gonna write a cross in the map.
	//we also update the IdMap in order to remember which flow it is
	if ( (previous != 0)) {  
		switch (dir) {
			case 1:
				if ( previous == 3  ) {
					if ( map[x][y-1] == AngleUR ) { //inutil et verif
													//cout << "LOG: 1-3 A" << endl;
						setMapCell(x, y-1, RightT);
					}
					else if ( map[x][y-1] == AngleDL ) { 
						setMapCell(x, y-1, DownT);
						//cout << "LOG: 1-3 B" << endl;
					}
					else if ( map[x][y-1] == UpT ) { //inutil et verif
						setMapCell(x, y-1, Cross);
						//cout << "LOG: 1-3 C" << endl;
					}
					else if ( map[x][y-1] == NO ) {
						//cout << "LOG: 1-3 D" << endl;
						setMapCell(x, y-1, AngleDR);
					}
					else if ( map[x][y-1] == NE ) {
						//cout << "LOG: 1-3 D" << endl;
						setMapCell(x, y-1, AngleDR);
					}
					else if ( map[x][y-1] == V ) {
						setMapCell(x, y-1, RightT);
						//cout << "LOG: 1-3 E" << endl;
					}
					else if ( map[x][y-1] == LeftT ) {
						setMapCell(x, y-1, Cross);
						//cout << "LOG: 1-3 F" << endl;
					}
					idMap[0][x][y-1] = id;
					pt_list.push_back(QPoint(x, y-1));
				}
				else if ( previous == 4 ) {
					if ( map[x][y-1] == V ) {
						setMapCell(x, y-1, LeftT);
						//cout << "LOG: 1-4 A" << endl;
					}
					else if ( map[x][y-1] == AngleDR ) {
						setMapCell(x, y-1, DownT);
						//cout << "LOG: 1-4 B" << endl;
					}
					else if ( map[x][y-1] == UpT ) { //inutil
						setMapCell(x, y-1, Cross);
						//cout << "LOG: 1-4 C" << endl;
					}
					else if ( map[x][y-1] == NO ) {
						setMapCell(x, y-1, AngleDL);
						//cout << "LOG: 1-4 D" << endl;
					}
					else if ( map[x][y-1] == NE ) {
						setMapCell(x, y-1, AngleDL);
						//cout << "LOG: 1-4 D" << endl;
					}
					else if ( map[x][y-1] == RightT ) {
						setMapCell(x, y-1, Cross);
						//cout << "LOG: 1-4 E" << endl;
					}
					idMap[0][x][y-1] = id;
					pt_list.push_back(QPoint(x, y-1));
				}
				
				else if ( previous == 1 ) {
					if ( map[x][y-1] == H ) { 
						setMapCell(x, y-1, VonH);
						idMap[1][x][y-1] = idMap[0][x][y-1];
						idMap[0][x][y-1] = id;
						//cout << "LOG: 1-1 A" << endl;
					}
					else if ( map[x][y-1] == NO ) {
						setMapCell(x, y-1, V);
						idMap[0][x][y-1] = id;
						//cout << "LOG: 1-1 B" << endl;
					}
					else if ( map[x][y-1] == NE ) {
						setMapCell(x, y-1, V);
						idMap[0][x][y-1] = id;
						//cout << "LOG: 1-1 B" << endl;
					}
					else if ( map[x][y-1] == AngleDL ) {
						setMapCell(x, y-1, LeftT);
						idMap[0][x][y-1] = id;
						//cout << "LOG: 1-1 C" << endl;
					}
					else if ( map[x][y-1] == AngleDR ) {
						setMapCell(x, y-1, RightT);
						idMap[0][x][y-1] = id;
						//cout << "LOG: 1-1 D" << endl;
					}
					else if ( map[x][y-1] == DownT ) {
						setMapCell(x, y-1, Cross);
						idMap[0][x][y-1] = id;
						//cout << "LOG: 1-1 E" << endl;
					}
					/*else if ( map[x][y-1] == BEF ) {
					setMapCell(x, y-1, BEF_D);
					//cout << "LOG: 1-4 E" << endl;
					}*/
				}
				//previous = 0;
				previous = dir;
				break;
			case 2:
				if ( previous == 3 ) {
					if ( map[x][y+1] == AngleDR ) { //inutil
						setMapCell(x, y+1, RightT);
						//cout << "LOG: 2-3 A" << endl;
					}
					else if ( map[x][y+1] == AngleUL ) {
						setMapCell(x, y+1, UpT);
						//cout << "LOG: 2-3 B" << endl;
					}
					else if ( map[x][y+1] == DownT ) { //inutil
						setMapCell(x, y+1, Cross);
						//cout << "LOG: 2-3 C" << endl;
					}
					else if ( map[x][y+1] == NO ) {
						setMapCell(x, y+1, AngleUR);
						//cout << "LOG: 2-3 D" << endl;
					}
					else if ( map[x][y+1] == NE ) {
						setMapCell(x, y+1, AngleUR);
						//cout << "LOG: 2-3 D" << endl;
					}
					else if ( map[x][y+1] == V ) {
						setMapCell(x, y+1, RightT);
						//cout << "LOG: 2-3 E" << endl;
					}
					else if ( map[x][y+1] == LeftT ) {
						setMapCell(x, y+1, Cross);
						//cout << "LOG: 2-3 F" << endl;
					}
					idMap[0][x][y+1] = id;
					pt_list.push_back(QPoint(x, y+1));
				}
				else if ( previous == 4 ){
					if ( map[x][y+1] == V ) { 
						setMapCell(x, y+1, LeftT);
						//cout << "LOG: 2-4 A" << endl;
					}
					else if ( map[x][y+1] == AngleUR ) {
						setMapCell(x, y+1, UpT);
						//cout << "LOG: 2-4 B" << endl;
					}
					else if ( map[x][y+1] == DownT ) { //inutil
						setMapCell(x, y+1, Cross);
						//cout << "LOG: 2-4 C" << endl;
					}
					else if ( map[x][y+1] == NO ) {
						setMapCell(x, y+1, AngleUL);
						//cout << "LOG: 2-4 D" << endl;
					}
					else if ( map[x][y+1] == NE ) {
						setMapCell(x, y+1, AngleUL);
						//cout << "LOG: 2-4 D" << endl;
					}
					else if ( map[x][y+1] == RightT ) {
						setMapCell(x, y+1, Cross);
						//cout << "LOG: 2-4 E" << endl;
					}
					idMap[0][x][y+1] = id;
					pt_list.push_back(QPoint(x, y+1));
				}
				
				else if ( previous == 2 ) {
					if ( map[x][y+1] == H ) { 
						setMapCell(x, y+1, VonH);
						idMap[1][x][y+1] = idMap[0][x][y+1];
						idMap[0][x][y+1] = id;
						//cout << "LOG: 2-2 A" << endl;
					}
					else if ( map[x][y+1] == NO ) {
						setMapCell(x, y+1, V);
						idMap[0][x][y+1] = id;
						//cout << "LOG: 2-2 B" << endl;
					}
					else if ( map[x][y+1] == NE ) {
						setMapCell(x, y+1, V);
						idMap[0][x][y+1] = id;
						//cout << "LOG: 2-2 B" << endl;
					}
					else if ( map[x][y+1] == AngleUL ) {
						setMapCell(x, y+1, LeftT);
						idMap[0][x][y+1] = id;
						//cout << "LOG: 2-2 C" << endl;
					}
					else if ( map[x][y+1] == AngleUR ) {
						setMapCell(x, y+1, RightT);
						idMap[0][x][y+1] = id;
						//cout << "LOG: 2-2 D" << endl;
					}
					else if ( map[x][y+1] == UpT ) {
						setMapCell(x, y+1, Cross);
						idMap[0][x][y+1] = id;
						//cout << "LOG: 2-2 E" << endl;
					}
					else if ( map[x][y+1] == BEF ) {
						//  setMapCell(x, y+1, BEF_U);
						idMap[0][x][y+1] = id;
						//cout << "LOG: 2-2 E" << endl;
					}
				}
				previous = dir;
				//previous = 0;
				break;
			case 3:
				if ( previous == 1 ){
					if ( map[x+1][y] == AngleDL ) {
						setMapCell(x+1, y, LeftT);
						//cout << "LOG: 3-1 A" << endl;
					}
					else if ( map[x+1][y] == H ) {
						setMapCell(x+1, y, UpT);
						//cout << "LOG: 3-1 B" << endl;
					}
					else if ( map[x+1][y] == RightT ) { //inutil
						setMapCell(x+1, y, Cross);
						//cout << "LOG: 3-1 C" << endl;
					}
					else if ( map[x+1][y] == NO ) {
						setMapCell(x+1, y, AngleUL);
						//cout << "LOG: 3-1 D" << endl;
					}
					else if ( map[x+1][y] == NE ) {
						setMapCell(x+1, y, AngleUL);
						//cout << "LOG: 3-1 D" << endl;
					}
					else if ( map[x+1][y] == DownT ) {
						setMapCell(x+1, y, Cross);
						//cout << "LOG: 3-1 E" << endl;
					}
					idMap[0][x+1][y] = id;
					pt_list.push_back(QPoint(x+1, y));
				}
				else if ( previous == 2 ){
					if ( map[x+1][y] == AngleDR ) { //inutil
						setMapCell(x+1, y, DownT);
						//cout << "LOG: 3-2 A" << endl;
					}
					else if ( map[x+1][y] == AngleUL ) {
						setMapCell(x+1, y, LeftT);
						//cout << "LOG: 3-2 B" << endl;
					}
					else if ( map[x+1][y] == H ) { 
						setMapCell(x+1, y, DownT);
						//cout << "LOG: 3-2 C" << endl;
					}
					else if ( map[x+1][y] == RightT ) { //inutil
						setMapCell(x+1, y, Cross);
						//cout << "LOG: 3-2 D" << endl;
					}
					else if ( map[x+1][y] == NO ) {
						setMapCell(x+1, y, AngleDL);
						//cout << "LOG: 3-2 E" << endl;
					}
					else if ( map[x+1][y] == NE ) {
						setMapCell(x+1, y, AngleDL);
						//cout << "LOG: 3-2 E" << endl;
					}
					else if ( map[x+1][y] == UpT ) {
						setMapCell(x+1, y, Cross);
						//cout << "LOG: 3-2 F" << endl;
					}
					idMap[0][x+1][y] = id;
					pt_list.push_back(QPoint(x+1, y));
				}
				else if ( previous == 3 ) {
					if ( map[x+1][y] == V ) {
						setMapCell(x+1, y, HonV);
						idMap[1][x+1][y] = idMap[0][x+1][y];
						idMap[0][x+1][y] = id;
						//cout << "LOG: 3-3 A" << endl;
					}
					else if ( map[x+1][y] == NO ) {
						setMapCell(x+1, y, H);
						idMap[0][x+1][y] = id;
						//cout << "LOG: 3-3 B" << endl;
					}
					else if ( map[x+1][y] == NE ) {
						setMapCell(x+1, y, H);
						idMap[0][x+1][y] = id;
						//cout << "LOG: 3-3 B" << endl;
					}
					else if ( map[x+1][y] == AngleDL ) {
						setMapCell(x+1, y, DownT);
						idMap[0][x+1][y] = id;
						//cout << "LOG: 3-3 C" << endl;
					}
					else if ( map[x+1][y] == AngleUL ) {
						setMapCell(x+1, y, UpT);
						idMap[0][x+1][y] = id;
						//cout << "LOG: 3-3 D" << endl;
					}
					else if ( map[x+1][y] == LeftT ) {
						setMapCell(x+1, y, Cross);
						idMap[0][x+1][y] = id;
						//cout << "LOG: 3-3 E" << endl;
					}
					else if ( map[x+1][y] == BEF ) {
						//setMapCell(x+1, y, BEF_L);
						idMap[0][x+1][y] = id;
						//cout << "LOG: 3-3 E" << endl;
					}
				}
				previous = dir;
				//previous = 0;
				break;
			case 4:
				if ( previous == 1 ) {
					if ( map[x-1][y] == AngleUL ) { //inutil
						setMapCell(x-1, y, UpT);
						//cout << "LOG: 4-1 A" << endl;
					}
					else if ( map[x-1][y] == H ) {
						setMapCell(x-1, y, UpT);
						//cout << "LOG: 4-1 B" << endl;
					}
					else if ( map[x-1][y] == LeftT ) { //inutil
						setMapCell(x-1, y, Cross);
						//cout << "LOG: 4-1 C" << endl;
					}
					else if ( map[x-1][y] == NO ) {
						setMapCell(x-1, y, AngleUR);
						//cout << "LOG: 4-1 D" << endl;
					}
					else if ( map[x-1][y] == NE ) {
						setMapCell(x-1, y, AngleUR);
						//cout << "LOG: 4-1 D" << endl;
					}
					else if ( map[x-1][y] == AngleDR ) {
						setMapCell(x-1, y, RightT);
						//cout << "LOG: 4-1 E" << endl;
					}
					else if ( map[x-1][y] == DownT ) {
						setMapCell(x-1, y, Cross);
						//cout << "LOG: 4-1 F" << endl;
					}
					idMap[0][x-1][y] = id;
					pt_list.push_back(QPoint(x-1, y));
				}
				else if ( previous == 2 ) {
					if ( map[x-1][y] == AngleDL ) { //inutil
						setMapCell(x-1, y, DownT);
						//cout << "LOG: 4-2 A" << endl;
					}
					else if ( map[x-1][y] == H ) {
						setMapCell(x-1, y, DownT);
						//cout << "LOG: 4-2 B" << endl;
					}
					else if ( map[x-1][y] == LeftT ) { //inutil
						setMapCell(x-1, y, Cross);
						//cout << "LOG: 4-2 C" << endl;
					}
					else if ( map[x-1][y] == NO ) {
						setMapCell(x-1, y, AngleDR);
						//cout << "LOG: 4-2 D" << endl;
					}
					else if ( map[x-1][y] == NE ) {
						setMapCell(x-1, y, AngleDR);
						//cout << "LOG: 4-2 D" << endl;
					}
					else if ( map[x-1][y] == AngleUR ) {
						setMapCell(x-1, y, RightT);
						//cout << "LOG: 4-2 E" << endl;
					}
					else if ( map[x-1][y] == UpT ) {
						setMapCell(x-1, y, Cross);
						//cout << "LOG: 4-2 F" << endl;
					}
					idMap[0][x-1][y] = id;
					pt_list.push_back(QPoint(x-1, y));
				}
				
				else if ( previous == 4 ) {
					if ( map[x-1][y] == V ) {
						setMapCell(x-1, y, HonV);
						idMap[1][x-1][y] = idMap[0][x-1][y];
						idMap[0][x-1][y] = id;
						//cout << "LOG: 4-4 A" << endl;
					}
					else if ( map[x-1][y] == NO ) {
						setMapCell(x-1, y, H);
						idMap[0][x-1][y] = id;
						//cout << "LOG: 4-4 B" << endl;
					}
					else if ( map[x-1][y] == NE ) {
						setMapCell(x-1, y, H);
						idMap[0][x-1][y] = id;
						//cout << "LOG: 4-4 B" << endl;
					}
					else if ( map[x-1][y] == AngleDR ) {
						setMapCell(x-1, y, DownT);
						idMap[0][x-1][y] = id;
						//cout << "LOG: 4-4 C" << endl;
					}
					else if ( map[x-1][y] == AngleUR ) {
						setMapCell(x-1, y, UpT);
						idMap[0][x-1][y] = id;
						//cout << "LOG: 4-4 D" << endl;
					}
					else if ( map[x-1][y] == RightT ) {
						setMapCell(x-1, y, Cross);
						idMap[0][x-1][y] = id;
						//cout << "LOG: 4-4 E" << endl;
					}
					else if ( map[x-1][y] == BEF ) {
						//setMapCell(x-1, y, BEF_R);
						idMap[0][x-1][y] = id;
						//cout << "LOG: 4-4 E" << endl;
					}
				}
				previous = dir;
				//previous = 0;
				break;
		}
	}
	if ( ((x == startx) && (y == starty)) || ((x == endx) && (y == endy)) ){
		setMapCell(x, y, BEF);
		idMap[0][x][y] = id;
		/*if ( previous != 0 ) {
			switch (dir) {
				case 1:
					setMapCell(x, y, BEF_U);
					break;
				case 2:
					setMapCell(x, y, BEF_D);
					break;
				case 3:
					setMapCell(x, y, BEF_R);
					break;
				case 4:
					setMapCell(x, y, BEF_L);
					break;
			}
		}*/
		pt_list.push_back(QPoint(x, y));
	}
	else if ( previous == 0) {
		switch (dir) {
			case 1:
				if ( map[x][y-1] == BEF ) {
					//setMapCell(x, y-1, BEF_D);
					idMap[0][x][y-1] = id;
				}
				break;
			case 2:
				if ( map[x][y+1] == BEF ) {
					//setMapCell(x, y+1, BEF_U);
					idMap[0][x][y+1] = id;
				}
				break;
			case 3:
				if ( map[x+1][y] == BEF ) {
					//setMapCell(x+1, y, BEF_L);
					idMap[0][x+1][y] = id;
				}
				break;
			case 4:
				if ( map[x-1][y] == BEF ) {
					//setMapCell(x-1, y, BEF_R);
					idMap[0][x-1][y] = id;
				}
				break;
		} 
		previous = dir;
	}
}

void PathFinding::setMapView(MapView *m) {
	mv = m;
	//  can = m->canvas();
}

void PathFinding::removePath(unsigned int idFlow) {
	for ( int y = 0; y < SIZEMAPY; y++ )
		for ( int x = 0; x < SIZEMAPX; x++ ) {
			if ( idMap[0][x][y] == idFlow ) {
				if ( map[x][y] == HonV ) {
					setMapCell(x, y, V);
					idMap[0][x][y] = idMap[1][x][y];
					idMap[1][x][y] = 0;
				}
				else if ( map[x][y] == VonH ) {
					setMapCell(x, y, H);
					idMap[0][x][y] = idMap[1][x][y];
					idMap[1][x][y] = 0;
				}
				else {
					setMapCell(x, y, 0);
					idMap[0][x][y] = 0;
				}
			}
			else if ( idMap[1][x][y] == idFlow ) {
				if ( map[x][y] == HonV ) {
					setMapCell(x, y, H);
					idMap[1][x][y] = 0;
				}
				else if ( map[x][y] == VonH ) {
					setMapCell(x, y, V);
					idMap[1][x][y] = 0;
				}
			}
		}
}

void PathFinding::removeObstacle(unsigned int id) {
	for ( int y = 0; y < SIZEMAPY; y++ )
		for ( int x = 0; x < SIZEMAPX; x++ ) 
			if ( idMap[0][x][y] == id ) {
				idMap[0][x][y] = 0;
				map[x][y] = NO;
			}
}

void PathFinding::printMap() {
	int i;
	for ( i = 0; i < (SIZEMAPX+2); i++)
		cout << "*";
	cout << endl;
	for ( int j = 0; j < SIZEMAPY; j++ ) {
		cout << "*";
		for ( i = 0; i < SIZEMAPX; i++ )
			cout << hex << map[i][j];
		cout << "*" << endl;
	}
	for ( i = 0; i < (SIZEMAPX+2); i++)
		cout << "*";
	cout << endl;
}

void PathFinding::printIdMap() {
	int i,j;
	cout << "ID MAP 0 : " << endl;
	for(j = 0; j < SIZEMAPY; j++ ) {
		for(i = 0; i < SIZEMAPX; i++ ) 
			cout << idMap[0][i][j] << " ";
		cout << endl;
	}
	cout << endl;
	cout << "ID MAP 1 : " << endl;
	for(j = 0; j < SIZEMAPY; j++ ) {
		for(i = 0; i < SIZEMAPX; i++ ) 
			cout << idMap[1][i][j] << " ";
		cout << endl;
	}
}




set<unsigned int> PathFinding::createObstacle(int x, int y, int width, int height, unsigned int id) {
	set<unsigned int> flows;
	if ( exist(x-1,y-1) && exist((x+width),(y+height)) ) {
		for ( int j = y-1; j < (y+height+1); j++)
			for ( int i = x-1; i < (x+width+1); i++) {
				if ( idMap[0][i][j] != 0 )
					flows.insert(idMap[0][i][j]);
				if ( (i == (x-1)) || (i == (x+width)) || (j == (y-1)) || (j == (y+height))) {
					map[i][j] = NE;
					//can->setTile(i, j, NE);
				}
				else {
					//can->setTile(i, j, C);
					map[i][j] = C;
				}
				idMap[0][i][j] = id;
			}
	}
		return flows;
}

/*void PathFinding::initMap(int val){
memset(map, 0, sizeof(map));
switch(val) {
	case 1:
		map[4][4] = C;
		map[4][5] = C;
		map[4][6] = C;
		map[5][4] = C;
		map[5][5] = C;
		map[5][6] = C;
		map[6][4] = C;
		map[6][5] = C;
		map[6][6] = C;
		break;
	case 2:
		map[4][0] = C;
		map[4][1] = C;
		map[4][2] = C;
		map[4][3] = C;
		map[4][4] = C;
		map[4][5] = C;
		map[4][6] = C;
		map[4][7] = C;
		map[4][8] = C;
		break;
	case 3:
		map[2][4] = C;
		map[2][5] = C;
		map[2][6] = C;
		map[3][4] = C;
		map[3][5] = C;
		map[3][6] = C;
		map[5][4] = C;
		map[5][5] = C;
		map[5][6] = C;
		map[6][4] = C;
		map[6][5] = C;
		map[6][6] = C;
		map[4][0] = C;
		map[4][1] = C;
		map[4][2] = C;
		map[4][3] = C;
		map[4][4] = C;
		map[4][5] = C;
		map[4][6] = C;
		map[4][7] = C;
		map[4][8] = C;
		break;
	case 4:
		map[10][9] = C;
		map[10][10] = C;
		map[10][11] = C;
		map[10][0] = C;
		map[10][1] = C;
		map[10][2] = C;
		map[10][3] = C;
		map[10][4] = C;
		map[10][5] = NE;
		map[10][6] = C;
		map[10][7] = C;
		map[10][8] = C;
		break;
	case 5:
		map[7][7] = C;
		map[7][8] = C;
		map[7][9] = C;
		map[7][10] = C;
		map[7][11] = C;
		map[8][7] = C;
		map[8][8] = C;
		map[8][9] = C;
		map[8][10] = C;
		map[8][11] = C;
		map[9][7] = C;
		map[9][8] = C;
		map[9][9] = C;
		map[9][10] = C;
		map[9][11] = C;
		map[10][7] = C;
		map[10][8] = C;
		map[10][9] = C;
		map[10][10] = C;
		map[10][11] = C;
		map[11][7] = C;
		map[11][8] = C;
		map[11][9] = C;
		map[11][10] = C;
		map[11][11] = C;
		
		map[6][7] = NE;
		map[6][8] = NE;
		map[6][9] = NE;
		map[6][10] = NE;
		map[6][11] = NE;
		map[12][7] = NE;
		map[12][8] = NE;
		map[12][9] = NE;
		map[12][10] = NE;
		map[12][11] = NE;
		map[7][6] = NE;
		map[8][6] = NE;
		map[9][6] = NE;
		map[10][6] = NE;
		map[11][6] = NE;
		map[7][12] = NE;
		map[8][12] = NE;
		map[9][12] = NE;
		map[10][12] = NE;
		map[11][12] = NE;
		map[6][6] = NE;
		map[12][12] = NE;
		map[6][12] = NE;
		map[12][6] = NE;
		break; 
	case 6:
		map[1][1] = C;
		map[2][1] = C;
		map[3][2] = C;
		map[4][3] = C;
		map[5][4] = C;
		map[3][6] = C;
		map[6][5] = C;
		map[7][6] = C;
		map[0][2] = C;
		map[1][3] = C;
		map[2][4] = C;
		map[3][5] = C;
		map[4][6] = C;
		map[5][7] = C;
		map[6][7] = C;
		break;
	case 7:
		map[1][1] = C;
		map[15][1] = C;
		map[1][8] = C;
		map[15][8] = C;
		break;
}
}*/


Graph::Graph(){
	clearGraph();
}

Graph::~Graph(){
}

void Graph::clearGraph(){
	memset(map, 0, sizeof(map));
}

void Graph::setTile( int x, int y, char _val, int _cost, int _hcost ){
	map[x][y] = tile(_val, _cost, _hcost);
}

void Graph::setDirection(int x, int y, int px, int py){
	if ( x == (px + 1) ) {
		//cout << " Right" << endl;
		addRight(x,y);
	}
	else {
		if ( x == (px - 1) ){
			//cout << " Left" << endl;
			addLeft(x,y);
		}
		else {
			if ( y == (py +1) ) {
				//cout << " Down" << endl;
				addDown(x,y);
			}
			else {
				if ( y == (py - 1) ) {
					//cout << " Up" << endl;
					addUp(x,y);
				}
			}
		}
	}
}
