#include "Client.h"
#include "mainwindow.h"

#include <QtGui/QColor>
#include <QtCore/QDateTime>

#include <map>
#include <iostream>
using namespace std;


Client::Client(GlobalCC *g) : GlobalObject(g) {}

Client::Client(GlobalCC *g, QString _name, QString _path) : GlobalObject(g, _name), path(_path) {
	fin.clear();
	fout.clear();
}

Client::~Client() {}

void Client::setPath(QString _path) {
	if ( !_path.isEmpty() )
		path = _path;
}

QString& Client::getPath() { return path; }

void Client::addInputFlow(FlowOutletIn *f) {
	if ( !f->hasClient() ) {
		fin[f->getId()] = f;
		f->attachClient(this);
		refresh();
	}
}

void Client::addOutputFlow(FlowOutletOut *f) {
	if ( !f->hasClient() ) {
		fout[f->getId()] = f;
		f->attachClient(this);
		refresh();
	}
}

void Client::removeFlow(unsigned int id) {
	std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
	iter_in = fin.find(id);
	if ( iter_in != fin.end() ) {
		iter_in->second->detachClient();
		fin.erase(id);
	}
	
	std::map<unsigned int, FlowOutletOut *>::iterator iter_out;
	iter_out = fout.find(id);
	if ( iter_out != fout.end() ) {
		iter_out->second->detachClient();
		fout.erase(id);
	}
	refresh();
}

/*************************************************************/

ClientConf::ClientConf(GlobalCC *g) : Client(g) {}

ClientConf::ClientConf(GlobalCC *g, QString _name, QString _path) : Client(g, _name, _path) {}

ClientConf::ClientConf(ClientConf *cc) : Client(cc->getGlobal(), cc->getName(), cc->getPath()) {}

ClientConf::~ClientConf() {}

void ClientConf::print() {
	cout << "Client #" << getId() << endl << "  Path=" << getPath().toStdString() << endl;
}

void ClientConf::initTreeWidgetItem(QTreeWidget *parent) { 
	QStringList theName(getName());
    clientConfViewItem = new ClientConfItem(this, parent, theName);
}

ClientConfItem* ClientConf::getClientConfViewItem() { return clientConfViewItem; }

/*************************************************************/

ClientApp::ClientApp(ClientConf *cc, Server *_server, bool keepGroupId ) : ClientConf(cc) {
    server = _server;
    running = has_log = starting = disable = error = false;
    idle = true;
    clientGraphic = new ClientGraphicObject(getGlobal()->getApplication()->getMap(), this);
    QString tmp_name(getName());
    tmp_name.append(" ");
    if ( server ) {
		if ( server->getAddress() == "No Server" ) {
			tmp_name.append("\n on No Server ");
			clientGraphic->setLabelColor(QColor(Qt::red));
		}
		else {
			tmp_name.append("\n on ");
			tmp_name.append(server->getName());
			clientGraphic->setLabelColor(QColor(Qt::black));
		}
    }
    clientGraphic->setLabel(tmp_name);
    setAppId(getGlobal()->getApplication()->getId());
    if ( !keepGroupId )
		copyFlows(cc);
    else
		copyFlowsWithGroupId(cc);
    registerFlowsAndClientInMap();
    setGroup(GlobalCC::getNewGroup());
    param = "";
    oldServerName = "";
}

ClientApp::ClientApp(ClientApp *ca, bool keepGroupId ) : ClientConf(ca->getGlobal()) {
	setName(ca->getName());
	setPath(ca->getPath());
	setServer(ca->getServer());
	running = has_log = starting = disable = error = false;
	idle = true;
	clientGraphic = new ClientGraphicObject(getGlobal()->getApplication()->getMap(), this);
	QString tmp_name(getName());
	tmp_name.append(" ");
	if ( server ) {
		if ( server->getAddress() == "No Server" ) {
			tmp_name.append("\n on No Server ");
			clientGraphic->setLabelColor(QColor(Qt::red));
		}
		else {
			tmp_name.append("\n on ");
			tmp_name.append(server->getName());
			clientGraphic->setLabelColor(QColor(Qt::black));
		}
	}
	clientGraphic->setLabel(tmp_name);
	setAppId(getGlobal()->getApplication()->getId());
	if ( !keepGroupId )
		copyFlows(ca);
    else
		copyFlowsWithGroupId(ca);
	registerFlowsAndClientInMap();
	setGroup(GlobalCC::getNewGroup());
	param = "";
	oldServerName = "";
}

ClientApp::~ClientApp() {}

void ClientApp::setServer(Server *s) {
	server = s;
}

Server* ClientApp::getServer() { return server; }

void ClientApp::setAppId(unsigned int _id) {
	appId = _id;
}

unsigned int ClientApp::getAppId() { return appId; }

void ClientApp::setParam(QString p) {
	if ( !p.isEmpty() )  
		param = p;
	else 
		param = "";
}

QString& ClientApp::getParam() {
	return param;
}

void ClientApp::setGroup(QString g) {
    if ( !g.isEmpty() )
		grp = g;
    else 
		grp = "";
}

QString& ClientApp::getGroup() {
    return grp;
}

void ClientApp::addLogLine(QString l) {
    MainWindow::instance()->logView->addLogForClient(getId(),l);
}

void ClientApp::setRunning() {
	if ( !running ) {
		running = true;
		starting = disable = error = idle = false;
		//for ( std::map<unsigned int, FlowOutletIn*>::iterator iter = fin.begin(); iter != fin.end(); iter++ ) {
		//iter->second->setRunning();
		//}
		updateClient();
	} 
}

void ClientApp::setDisable() {
	if ( !disable ) {
		disable = true;
		running = starting = error = idle = false;
		for ( std::map<unsigned int, FlowOutletIn*>::iterator iter = fin.begin(); iter != fin.end(); iter++ ) {
			iter->second->setDisable();
		}
		for ( std::map<unsigned int, FlowOutletOut*>::iterator iter = fout.begin(); iter != fout.end(); iter++ ) {
			iter->second->setDisable();
		}
		updateClient();
	}
}

void ClientApp::setStarting() {
	if ( !starting ) {
		starting = true;
		running = disable = error = idle = false;
		updateClient();
	}
}

void ClientApp::setError() {
	if ( !error ) {
		error = true;
		running = disable = starting = idle = false;
		for ( std::map<unsigned int, FlowOutletIn*>::iterator iter = fin.begin(); iter != fin.end(); iter++ ) {
			iter->second->setError();
		}
		for ( std::map<unsigned int, FlowOutletOut*>::iterator iter = fout.begin(); iter != fout.end(); iter++ ) {
			iter->second->setError();
		}
		updateClient();
	}
}  

void ClientApp::setIdle() {
	if ( !idle ) {
		idle = true;
		running = disable = starting = error = false;
		for ( std::map<unsigned int, FlowOutletIn*>::iterator iter = fin.begin(); iter != fin.end(); iter++ ) {
			iter->second->setIdle();
		}
		for ( std::map<unsigned int, FlowOutletOut*>::iterator iter = fout.begin(); iter != fout.end(); iter++ ) {
			iter->second->setIdle();
		}
		updateClient();
	}
}

void ClientApp::setLog(bool l)
{
	if ( has_log != l ) {
		has_log = l;
		updateClient();
	}
}

void ClientApp::setOldServerName(QString &name) {
	if ( !name.isEmpty() )
		oldServerName = name;
	else
		oldServerName = "";
}

QString& ClientApp::getOldServerName() { return oldServerName; }

void ClientApp::updateClient(){
	getClientGraphic()->refreshLabel();
	getClientGraphic()->refresh();
	getClientGraphic()->updateClient();
	getClientGraphic()->getMapView()->update();
}

void ClientApp::refresh() { getClientGraphic()->refresh(); }

void ClientApp::copyFlows(Client *c) {
	std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
	std::map<unsigned int, FlowOutletIn *> *fin_tmp = c->getInputFlowList();
	for( iter_in = fin_tmp->begin(); iter_in != fin_tmp->end(); iter_in++ ) {
		FlowOutletIn *tmp_in = new FlowOutletIn(iter_in->second);
		addInputFlow(tmp_in);
	}  
	std::map<unsigned int, FlowOutletOut *>::iterator iter_out;  
	std::map<unsigned int, FlowOutletOut *> *fout_tmp = c->getOutputFlowList();
	for( iter_out = fout_tmp->begin(); iter_out != fout_tmp->end(); iter_out++ ) {
		FlowOutletOut *tmp_out = new FlowOutletOut(iter_out->second);
		addOutputFlow(tmp_out);
	}
}

void ClientApp::copyFlowsWithGroupId(Client *c) {
	std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
	std::map<unsigned int, FlowOutletIn *> *fin_tmp = c->getInputFlowList();
	for( iter_in = fin_tmp->begin(); iter_in != fin_tmp->end(); iter_in++ ) {
		FlowOutletIn *tmp_in = new FlowOutletIn(iter_in->second, true);
		addInputFlow(tmp_in);
	}  
	std::map<unsigned int, FlowOutletOut *>::iterator iter_out;  
	std::map<unsigned int, FlowOutletOut *> *fout_tmp = c->getOutputFlowList();
	for( iter_out = fout_tmp->begin(); iter_out != fout_tmp->end(); iter_out++ ) {
		FlowOutletOut *tmp_out = new FlowOutletOut(iter_out->second, true);
		addOutputFlow(tmp_out);
	}
}

void ClientApp::registerFlowsAndClientInMap() {
	MapView *mv = getGlobal()->getApplication()->getMap();
	if ( mv ) {
		mv->addClient(getId(), getClientGraphic());
		std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
		std::map<unsigned int, FlowOutletOut *>::iterator iter_out;
		
		for( iter_in = fin.begin(); iter_in != fin.end(); iter_in++ ) {
			FlowOutletIn *tmp_in = iter_in->second;
			mv->addOutlet(tmp_in->getId(), tmp_in->getFlowOutletGraphic());
		}
		
		for( iter_out = fout.begin(); iter_out != fout.end(); iter_out++ ) {
			FlowOutletOut *tmp_out = iter_out->second;
			mv->addOutlet(tmp_out->getId(), tmp_out->getFlowOutletGraphic());
		}
	}
}

void ClientApp::printInfo() {
	cout << "Client" << endl;
	cout << "   Name: " << getName().toStdString() << endl;
	cout << "   ID: " << getId() << endl;
	cout << "   Group: " << getGroup().toStdString() << endl;
	cout << "   Server: " << getServer()->getName().toStdString() << endl;
	cout << "   Status: ";
	if ( isRunning() )
		cout << "Running";
	else if ( isDisable() )
		cout << "Disable";
	else if ( isIdle()  )
		cout << "Idle";
	else if ( isError() )
		cout << "Error";
	else if ( isStarting() )
		cout << "Starting";
	else
		cout << "No Status";
	cout << endl;
}
