#ifndef _CLIENT_H
#define _CLIENT_H

#include "GlobalCC.h"
#include "Server.h"
#include "Flow.h"
#include "map/MapView.h"
#include "dialog/ConfigurationClientList.h"

#include <QtCore/QString>
#include <QtGui/QTreeWidget>

#include <iostream>
#include <set>
#include <map>

class Client;
class ClientConf;
class ClientApp;
class ClientConfItem;

class Client : public GlobalObject {
public:
	Client(GlobalCC *g);
	Client(GlobalCC *g, QString _name, QString _path = QString::null);
	~Client();
	
	//need to refresh graphic view
	void setPath(QString _path);
	QString &getPath();
	
	void addInputFlow(FlowOutletIn *f);
	void addOutputFlow(FlowOutletOut *f);
	void removeFlow(unsigned int id);
	
	std::map<unsigned int, FlowOutletIn*>* getInputFlowList() { return &fin; };
	std::map<unsigned int, FlowOutletOut*>* getOutputFlowList() { return &fout; };
	
	virtual void refresh() = 0;
	
protected:
	QString path;
	std::map<unsigned int, FlowOutletIn *> fin;
	std::map<unsigned int, FlowOutletOut *> fout;
	
};

class ClientConf : public Client {
public:
	ClientConf(GlobalCC *g);
	ClientConf(GlobalCC *g, QString _name, QString _path = QString::null);
	ClientConf(ClientConf *cc);
	~ClientConf();
	
	void refresh() {}
	
	void print();
	
	void initTreeWidgetItem(QTreeWidget *parent);
	ClientConfItem* getClientConfViewItem();
	
private:
	ClientConfItem* clientConfViewItem;
};


class ClientApp : public ClientConf {
public:
	ClientApp(ClientConf *cc, Server *_server = NULL, bool keepGroupId = false);
	ClientApp(ClientApp *ca, bool keepGroupId = false);
	virtual ~ClientApp();
	
	//need to refresh graphic view
	void setServer(Server *s);
	
	Server* getServer();
	
	void setAppId(unsigned int _id);
	unsigned int getAppId();
	
	void setParam(QString p);
	QString &getParam();
	
	void setGroup(QString g);
	QString &getGroup();
	
	void addLogLine(QString l);

	bool isRunning() { return running;}
	bool isDisable() { return disable;}
	bool isIdle() { return idle;}
	bool isError() { return error;}
	bool isStarting() { return starting;}
	bool hasLog() { return has_log; }
	
	void setRunning();
	void setDisable();
	void setStarting();
	void setError();
	void setIdle();
	void setLog(bool l);
	
	void setOldServerName(QString &name);
	QString& getOldServerName();
	
	virtual void updateClient();
	
	void refresh();
	
	ClientGraphicObject *getClientGraphic() { return clientGraphic; }
	
	void copyFlows(Client *c);
	void copyFlowsWithGroupId(Client *c);
	void registerFlowsAndClientInMap();
	
	void printInfo();
	
private:
	bool running, has_log, starting, disable, error, idle;
	ClientGraphicObject *clientGraphic;
	Server *server;
	QString oldServerName;
	QString grp;
	unsigned int appId;
	QString param;

};


#endif
