#include "Flow.h"
#include "GlobalCC.h"
#include "Client.h"
#include "mainwindow.h"

#include <QtCore/QString>

#include <iostream>
using namespace std;

AbstractFlow::AbstractFlow(GlobalCC *g, QString _name, QString _dataType) : GlobalObject(g,_name) {
	dataType = _dataType;
}

AbstractFlow::~AbstractFlow() {}

void AbstractFlow::setDataType(QString _dataType) {
	if ( !_dataType.isEmpty() )
		dataType = _dataType;
}

QString& AbstractFlow::getDataType() { return dataType; }

/*************************************************************/

Flow::Flow(GlobalCC *g, QString _name, QString _dataType) : AbstractFlow(g, _name, _dataType) {
	ooutlet = 0;
	ioutlets.clear();
	running = false;
	flowGraphic = new FlowGraphicObject(getGlobal()->getApplication()->getMap(), this);
}

Flow::Flow(GlobalCC *g, QString _dataType) : AbstractFlow(g, "", _dataType) {
	ooutlet = 0;
	ioutlets.clear();
	running = false;
	flowGraphic = new FlowGraphicObject(getGlobal()->getApplication()->getMap(), this);
}

Flow::~Flow() {
	for(std::set<FlowOutletIn *>::iterator i = ioutlets.begin(); i != ioutlets.end(); i++)
		(*i)->detachFlow();
	ioutlets.clear();
	if ( ooutlet ) {		
		ooutlet->detachFlow();
		ooutlet = 0;
	}
	global->getApplication()->getMap()->remFlow(getId());
	delete flowGraphic;
}

void Flow::setIn(FlowOutletIn *in) {
	if ( in != 0 ) {
		for(std::set<FlowOutletIn *>::iterator i = ioutlets.begin(); i != ioutlets.end(); i++) {
			if ( (*i) == in ) {
				return;
			}
		}
		ioutlets.insert(in);
		flowGraphic->createSubFlow(in->getId());
	}
}

void Flow::setOut(FlowOutletOut *out) {
	if ( out != 0 ) {
		ooutlet = out;
	}
}

void Flow::removeIn(FlowOutletIn *in) {
       if ( in != NULL ) {
               for(std::set<FlowOutletIn *>::iterator i = ioutlets.begin(); i!=ioutlets.end(); i++) {
	               if( (*i) == in) {
		               ioutlets.erase(i);
			       flowGraphic->removeSubFlow(in->getId());
			       if ( !in->isRunning() ) {
			               in->setGroup(GlobalCC::getNewGroup());
			       }
			       else {
				       //we update group but don't check if there is an outlet running in the group....
				       //It is okay to do so because we cant be here if 2 outlets In are running.
				       updateGroup(GlobalCC::getNewGroup());
			       }
			       return;
		       }
	       }
       }
}

void Flow::removeOut() {
	if ( ooutlet && !ooutlet->isRunning() )
		ooutlet->setGroup(GlobalCC::getNewGroup());
	ooutlet = 0;
}

bool Flow::hasOutput() {
	return ( ooutlet != 0 );
}

unsigned int Flow::getClientInId() {
	if (ooutlet)
		return ooutlet->getClientId();
	else
		return 0;
}

std::vector<unsigned int> Flow::getClientOutId() {
	std::vector<unsigned int> tmp;
	tmp.clear();
	std::set<FlowOutletIn *>::iterator iter;
	for ( iter = ioutlets.begin(); iter != ioutlets.end(); ++iter )
		if ( (*iter) )
			tmp.push_back((*iter)->getClientId());
	return tmp;
}

bool Flow::isRunning() { 
	if ( ooutlet )
		return ooutlet->isRunning();
	else
		return false;
}	

int Flow::countOutletInRunning() {
	int nbOutlet = 0;
	for ( std::set<FlowOutletIn *>::iterator i = ioutlets.begin(); i != ioutlets.end();  i++ ){
		if( (*i) && (*i)->isRunning() ) {
			nbOutlet++;
		}
	}
	return nbOutlet;
}

QString& Flow::getGroupOfRunningOutletIn() {
	bool found = false;
	QString *theID = new QString("");
	std::set<FlowOutletIn *>::iterator i = ioutlets.begin();
	while ( (found == false) && (i != ioutlets.end()) ) {
		if( (*i) && (*i)->isRunning() ) {
			theID->append((*i)->getGroup());
			found = true;
		}
		i++;
	}
	return *theID;
}

void Flow::updateGroup(QString &newGroup) {
	if ( ooutlet )
		ooutlet->setGroup(newGroup);
	std::set<FlowOutletIn *>::iterator iter;
	for ( iter = ioutlets.begin(); iter != ioutlets.end(); ++iter ) {
		if ( (*iter) )
			(*iter)->setGroup(newGroup);
	}
}
  
unsigned int Flow::nbOutletsIn() {
	return ioutlets.size();
}

void Flow::refresh() {
	getFlowGraphic()->refresh();
}

void Flow::disconnect() {
	getFlowGraphic()->erasePath();
	FlowOutletIn *tmp_in = 0;
	FlowOutletOut *tmp_out = ooutlet;	
	ooutlet->detachFlow();
	tmp_out->getFlowOutletGraphic()->drawOutletInMap();
	std::set<FlowOutletIn *>::iterator iter;
	iter = ioutlets.begin();
	while ( iter != ioutlets.end() ) {
		tmp_in = (*iter);
		tmp_in->detachFlow();
		tmp_in->getFlowOutletGraphic()->drawOutletInMap();
		iter = ioutlets.begin();
	}
	getFlowGraphic()->getMapView()->remFlow(getId());
	getFlowGraphic()->getMapView()->removeSelected(getFlowGraphic());
	global->getApplication()->removeFlow(this);
	getFlowGraphic()->getMapView()->update();
	getFlowGraphic()->emitDeleteFlowSignal(this);
}

void Flow::pause() {
	//we need to launch a pause on all the flowgraphic outletin
	std::set<FlowOutletIn *>::iterator iter;
	for ( iter = ioutlets.begin(); iter != ioutlets.end(); ++iter )
		if ( (*iter) ) {
			FlowOutletIn *tmp_outlet =  (*iter);
			SfControlThread *sfct = MainWindow::instance()->global->getSfControlThread();
			if ( sfct ) {
				SfControl *sfc = sfct->getSfControl();
				if ( sfc ) {
					ClientApp *tmp_client = (ClientApp *) (tmp_outlet->getClient());
					if ( tmp_client ) {
						//cout << "we pause a client" << endl;
						//sfc->pauseFlow(this, tmp_client);
						sfc->emitPauseFlowSignal(this, tmp_client);
					}
				}
			}
		}
}

void Flow::restart() {
	std::set<FlowOutletIn *>::iterator iter;
	for ( iter = ioutlets.begin(); iter != ioutlets.end(); ++iter )
		if ( (*iter) ) {
			FlowOutletIn *tmp_outlet =  (*iter);
			SfControlThread *sfct = MainWindow::instance()->global->getSfControlThread();
			if ( sfct ) {
				SfControl *sfc = sfct->getSfControl();
				if ( sfc ) {
					ClientApp *tmp_client = (ClientApp *) (tmp_outlet->getClient());
					if ( tmp_client )
						//cout << "we restart a client" << endl;
						//sfc->restartFlow(this, tmp_client);
						sfc->emitRestartFlowSignal(this, tmp_client);
				}
			}
		}
}

void Flow::printInfo() {
	cout << "Flow" << endl;
	cout << "   Name: " << getName().data() << endl;
	cout << "   ID: " << getId() << endl;
	cout << "   Provider: " << ooutlet->getName().data() << endl;
	cout << "   Status: ";
	if ( isRunning() )
		cout << "Running";
	else
		cout << "No Status";
	cout << endl;
}

/*************************************************************/

FlowOutlet::FlowOutlet(GlobalCC *g, QString _name, QString _dataType) : AbstractFlow(g, _name, _dataType) {
	//need of define the client here, and maybe the flow
	running = disable = error = false;
	idle = true;
	hasAFlow = false;
	flow = 0;
	client = 0;
}

FlowOutlet::FlowOutlet(FlowOutlet *f) : AbstractFlow(f->getGlobal(), f->getName(), f->getDataType()) {
	running = false;
}

FlowOutlet::~FlowOutlet() {
	
}

Client* FlowOutlet::getClient() { 
	if ( client )
		return client; 
	else
		return 0;
}

void FlowOutlet::attachClient(Client *c){
	if ( c != 0 ) 
		client = c;
}

void FlowOutlet::detachClient(){
	client = 0;
}

unsigned int FlowOutlet::getClientId(){
	if ( client != 0 )
		return client->getId();
	else
		return 0;
}

bool FlowOutlet::hasClient(){
	return (client != 0);
}

void FlowOutlet::setFlow(Flow *f){
	if ( f ) {
		flow = f;
		hasAFlow = true;
	}
}

void FlowOutlet::unsetFlow(){
	flow = 0;
	hasAFlow = false;
}

Flow* FlowOutlet::getFlow(){ 
	if ( hasAFlow )
		return flow;
	else 
		return 0;
}

unsigned int FlowOutlet::getFlowId(){
	if ( flow )
		return flow->getId();
	else
		return 0;
}

bool FlowOutlet::hasFlow() {
	return ( hasAFlow );
}

bool FlowOutlet::compatible(FlowOutlet *f) {
	if ( f && (getType() != f->getType()) && ( getDataType() == f->getDataType() ) && ( f->getClient() != getClient() ))
		return true;
	else
		return false;
}

bool FlowOutlet::compatible(Flow *f) {
    if ( f && (getDataType() == f->getDataType())  &&  (f->getClientInId() != getClient()->getId()) )
		return true;
    else
		return false;
}

/*************************************************************/

FlowOutletIn::FlowOutletIn(GlobalCC *g, QString _name, QString _dataType) : FlowOutlet(g,  _name, _dataType ) {
	paused = false;
	flowGraphic = new FlowGraphicOutletIn(this, getGlobal()->getApplication()->getMap());
}

FlowOutletIn::FlowOutletIn(FlowOutletIn *f, bool keepGroup) : FlowOutlet(f->getGlobal(), f->getName(), f->getDataType()) {
	paused = false;
	flowGraphic = new FlowGraphicOutletIn(this, getGlobal()->getApplication()->getMap());
	if ( !keepGroup )
		setGroup(GlobalCC::getNewGroup());
	else
		setGroup(f->getGroup());
}

FlowOutletIn::~FlowOutletIn() {
	if ( flowGraphic )
		delete flowGraphic;
	flowGraphic = NULL;
}

void FlowOutletIn::attachFlow(Flow *f) {
	if ( f != 0 ) {
		setFlow(f);
		f->setIn(this);
	}
}

void FlowOutletIn::detachFlow() {
	getFlow()->removeIn(this);
	unsetFlow(); 
}

Flow* FlowOutletIn::connect(FlowOutlet *f, bool force) {
        QString consoID;
	if ( f )
	        if ( connectable(f, force, consoID) ) {
			if ( !f->hasFlow() ) {
				//need to create a new flow and a graphic flow and insert it in the map and in the app
				Flow *fl = new Flow(getGlobal(), f->getName(), getDataType());
			
				f->attachFlow(fl);
				attachFlow(fl);

				if ( consoID == "" )
				        setGroup(f->getGroup());
				else
				        f->setGroup(consoID);

		    
				fl->getGlobal()->getApplication()->getMap()->addFlow(fl->getId(), fl->getFlowGraphic());
				getGlobal()->getApplication()->addFlow(fl);
				
				dynamic_cast<ClientApp *>(getClient())->getClientGraphic()->drawClient();
				dynamic_cast<ClientApp *> (f->getClient())->getClientGraphic()->drawClient();
				getFlowOutletGraphic()->drawFlowAndOutlet();
				return fl;
			}
			//the output outlet has already a flow
			else {
				attachFlow(f->getFlow());
				if ( consoID == "" )
				        setGroup(f->getGroup());
				else
				        f->getFlow()->updateGroup(consoID);
				getFlowOutletGraphic()->drawFlowAndOutlet();
				return getFlow();
			}
		}
	return NULL; 
}

Flow* FlowOutletIn::connect(Flow *fo, bool force) {
        QString consoID;
        if ( connectable(fo, force, consoID) ) {
		attachFlow(fo);
		getFlowOutletGraphic()->drawFlowAndOutlet();
		
		if ( consoID == "" ) 
		        setGroup(fo->getOutletOut()->getGroup());
		else 
		        fo->updateGroup(consoID);
		getFlowOutletGraphic()->drawFlowAndOutlet();
		return getFlow();
	}
	else
		return NULL;
}

bool FlowOutletIn::connectable(FlowOutlet *f, bool force, QString &consoID) {
	consoID = "";
	if ( f ) {
		ClientApp *capp = dynamic_cast<ClientApp *>(getClient());
		if ( force ) {
			if ( compatible(f) && !hasFlow() ) {
				return true;
			}
		}
		else { 
			if ( !hasFlow() && compatible(f) && (capp != NULL) ) {
				ClientApp *capp2 = dynamic_cast<ClientApp *>(f->getClient());
				if ( !isRunning() && !capp->isRunning() && !capp->isStarting() )
					return true;
				else if ( !f->isRunning() && (capp2 != NULL) && !capp2->isRunning() && !capp2->isStarting() ) {
					if ( !f->hasFlow() ) {
						consoID = getGroup();
						return true;
					}
					else if (f->getFlow()->countOutletInRunning() == 0) {
						consoID = getGroup();
						return true;
					}
				} 
			}
		}
	}
	return false;
}

bool FlowOutletIn::connectable(Flow *fo, bool force, QString &consoID) {
	consoID = "";
	if ( fo ) {
		ClientApp *capp = dynamic_cast<ClientApp *>(getClient());
		if ( force ) {
			return ( compatible(fo) && !hasFlow());
		}
		else {
			if ( compatible(fo) && !hasFlow() && (capp != NULL) ) {
				if ( !(capp->isRunning() || capp->isStarting()) ) 
					return true;
				else if ( isRunning() && !fo->isRunning() && fo->countOutletInRunning() == 0 ) {
					consoID = getGroup();
					return true;
				}
			}
		}
	}
	return false;
}

void FlowOutletIn::setGroup(QString g) {
	if ( !g.isEmpty() )
		grp = g;
}

QString& FlowOutletIn::getGroup() {
	return grp;
}

void FlowOutletIn::setRunning() { 
	if ( !running ) {
		running = true;
		idle = disable = error = false;
		refresh();
	}
}

void FlowOutletIn::setDisable() {
	if ( !disable ) {
		disable = true;
		idle = running = error = paused = false;
		refresh();
	}
}

void FlowOutletIn::setError() {
	if ( !error ) {
		error = true;
		idle = running = disable = paused = false;
		refresh();
	}
}

void FlowOutletIn::setIdle() {
	if ( !idle ) {
		idle = true;
		error = running = disable = paused = false;
		refresh();
	}
}

bool FlowOutletIn::isPaused() {
	return paused;
}

void FlowOutletIn::setPaused(bool _pause) {
	if ( _pause != paused ) {
		paused = _pause;
		refresh();
	}
}

void FlowOutletIn::refresh() {
	getFlowOutletGraphic()->refresh();
	//cout << "REFRESH IN" << endl;
}

/*************************************************************/  

FlowOutletOut::FlowOutletOut(GlobalCC *g, QString _name, QString _dataType) : FlowOutlet(g, _name, _dataType) {
	flowGraphic = new FlowGraphicOutletOut(this, getGlobal()->getApplication()->getMap());
}

FlowOutletOut::FlowOutletOut(FlowOutletOut *f, bool keepGroup) : FlowOutlet(f->getGlobal(), f->getName(), f->getDataType()) {
	flowGraphic = new FlowGraphicOutletOut(this, getGlobal()->getApplication()->getMap());
	if ( !keepGroup )
		setGroup(GlobalCC::getNewGroup());
	else
		setGroup(f->getGroup());
}


FlowOutletOut::~FlowOutletOut() {
	if ( flowGraphic )
		delete flowGraphic;
	flowGraphic = NULL;
}

void FlowOutletOut::attachFlow(Flow *f) {
	if ( f != 0 ) {
		setFlow(f);
		f->setOut(this);
	}
}

void FlowOutletOut::detachFlow() {
	getFlow()->removeOut();
	unsetFlow();
}

Flow* FlowOutletOut::connect(FlowOutlet *f, bool force) {
	QString consoID;
	if ( f && connectable(f, force, consoID) )
		return f->connect(this, force);
	return NULL;
}

Flow* FlowOutletOut::connect(Flow *fo, bool force) {
	Q_UNUSED(fo);
	Q_UNUSED(force);
	return NULL;
}

bool FlowOutletOut::connectable(FlowOutlet *f, bool force, QString &consoID) {
	consoID = "";
	if ( f ) {
		ClientApp *capp = dynamic_cast<ClientApp *>(getClient());
		if ( force ) {
			if ( compatible(f) && !f->hasFlow() ) {
				return true;
			}
		}
		else {
			if ( compatible(f) && !f->hasFlow() && (capp != NULL) ) {
				ClientApp *capp2 = dynamic_cast<ClientApp *>(f->getClient());
				if ( !f->isRunning() && (capp2 != NULL) && !capp2->isRunning() && !capp2->isStarting() )
					return true;
				else if ( !isRunning() && !capp->isRunning() && !capp->isStarting()) { //the id used here should be the one from the consumer
					if ( !hasFlow() ) {
						consoID = f->getGroup();
						return true;
					}
					else if ( getFlow()->countOutletInRunning() == 0 ) {
						consoID = f->getGroup();
						return true;
					}
				}
			}    
		}
	}
	return false;
}

bool FlowOutletOut::connectable(Flow *fo, bool force, QString &consoID) {  
	Q_UNUSED(fo);
	Q_UNUSED(force);
        consoID = "";
	//Always returns false no matter what
	return false;
}

void FlowOutletOut::setGroup(QString g) {
	if ( !g.isEmpty() )
		grp = g;
}

QString& FlowOutletOut::getGroup() {
	return grp;
}

void FlowOutletOut::setRunning() { 
	if ( !running ) {
		running = true;
		idle = disable = error = false;
		refresh();
	}
}

void FlowOutletOut::setDisable() {
	if ( !disable ) {
		disable = true;
		idle = running = error = false;
		refresh();
	}
}

void FlowOutletOut::setError() {
	if ( !error ) {
		error = true;
		idle = running = disable = false;
		refresh();
	}
}

void FlowOutletOut::setIdle() {
	if ( !idle ) {
		idle = true;
		error = running = disable = false;
		refresh();
	}
}

bool FlowOutletOut::isPaused() {
	return false;
}

void FlowOutletOut::setPaused(bool _pause) {
	Q_UNUSED(_pause);
}

void FlowOutletOut::refresh() {
	getFlowOutletGraphic()->refresh();
	//cout << "REFRESH OUT" << endl;
	if ( flow )
		flow->refresh();
}
