#ifndef _FLOW_H
#define _FLOW_H

#include "GlobalCC.h"
#include "map/MapView.h"

#include <QtCore/QString>

#include <list>

class AbstractFlow;
class Flow;
class FlowOutlet;
class FlowOutletIn;
class FlowOutletOut;

class Client;

class AbstractFlow : public GlobalObject {
public:
	AbstractFlow(GlobalCC *g, QString _name, QString _dataType);
	~AbstractFlow();
	
	void setDataType(QString _type);
	QString& getDataType();
	
protected:
	QString dataType;
	
};

class Flow : public AbstractFlow {
public:
	Flow(GlobalCC *g, QString _name, QString _dataType);
	Flow(GlobalCC *g, QString _dataType);
	~Flow();
	
	void setIn(FlowOutletIn *in);
	void setOut(FlowOutletOut *out);
	void removeIn(FlowOutletIn *in);
	void removeOut();
	
	bool hasOutput();
	unsigned int getClientInId();
	std::vector<unsigned int> getClientOutId();
	
	bool isRunning();
	int countOutletInRunning();
	QString& getGroupOfRunningOutletIn();
	//void setRunning(bool r);
	void updateGroup(QString &newGroup);
	unsigned int nbOutletsIn();
	
	std::set<FlowOutletIn *> *getOutletsIn() { return &ioutlets; } 
	FlowOutletOut *getOutletOut() { return ooutlet; }
	
	FlowGraphicObject *getFlowGraphic() { return flowGraphic; }
	
	void refresh();
	
	void disconnect();
	
	void pause();
	void restart();
	
	void printInfo();
	
private:
	std::set<FlowOutletIn *> ioutlets;  
	FlowOutletOut *ooutlet;
	bool running;
	FlowGraphicObject *flowGraphic;
};

class FlowOutlet : public AbstractFlow {
public:
	FlowOutlet(GlobalCC *g, QString _name, QString _dataType);
	FlowOutlet(FlowOutlet *f);
	~FlowOutlet();
	
	virtual void attachFlow(class Flow *f) = 0;
	virtual void detachFlow() = 0;
	
	void attachClient(Client *c);
	void detachClient(); 
	
	Client* getClient();
	unsigned int getClientId();
	bool hasClient();
	
	void setFlow(Flow *f);
	void unsetFlow();
	
	Flow* getFlow();
	unsigned int getFlowId();
	bool hasFlow();
	
	virtual void setGroup(QString g) = 0;
	virtual QString &getGroup() = 0;
	
	enum { FLOW_IN, FLOW_OUT };
	
	bool compatible(FlowOutlet *f);
	bool compatible(Flow *f);
	
	virtual Flow *connect(FlowOutlet *f, bool force) = 0;
	virtual Flow *connect(Flow *fo, bool force) = 0;
	virtual bool connectable(FlowOutlet *f, bool force, QString &consoID) = 0;
	virtual bool connectable(Flow *fo, bool force, QString &consoID) = 0;
	virtual int getType() = 0;
	
	bool isRunning() { return running; }
	bool isDisable() { return disable; }
	bool isError() { return error; }
	bool isIdle() { return idle; }
	//need to refresh graphic view
	virtual void setRunning() = 0;
	virtual void setDisable() = 0;
	virtual void setError() = 0;
	virtual void setIdle() = 0;
	virtual void refresh() =0;
	
	virtual bool isPaused() = 0;
	virtual void setPaused(bool _pause) = 0;
	
protected:
	bool hasAFlow;
	bool running;
	bool disable;
	bool error;
	bool idle;
	bool paused;
	Client *client;
	Flow *flow;
};

class FlowOutletIn : public FlowOutlet {
public:
	FlowOutletIn(GlobalCC *g, QString _name, QString _dataType);
	FlowOutletIn(FlowOutletIn *f, bool keepGroup = false);
	~FlowOutletIn();
	
	//need to refresh graphic view
	void attachFlow(class Flow *f);
	void detachFlow();
	
	//need to refresh graphic view
	Flow *connect(FlowOutlet *f, bool force);
	Flow *connect(Flow *fo, bool force);
	
	bool connectable(FlowOutlet *f, bool force, QString &consoID);
	bool connectable(Flow *fo, bool force, QString &consoID);
	
	int getType() { return FLOW_IN; }
	
	void setGroup(QString g);
	QString &getGroup();
	
	void setRunning();
	void setDisable();
	void setError();
	void setIdle();
	void refresh();
	bool isPaused();
	void setPaused(bool _pause);
	
	
	FlowGraphicOutletIn *getFlowOutletGraphic() { return flowGraphic; }
	
private:
	FlowGraphicOutletIn *flowGraphic;
	QString grp;
	bool paused;
};

class FlowOutletOut : public FlowOutlet {
public:
	FlowOutletOut(GlobalCC *g, QString _name, QString _dataType);
	FlowOutletOut(FlowOutletOut *f, bool keepGroup = false);
	~FlowOutletOut();
	
	//need to refresh graphic view
	void attachFlow(class Flow *f);
	void detachFlow();
	
	//need to refresh graphic view
	Flow *connect(FlowOutlet *f, bool force);
	Flow *connect(Flow *fo, bool force);
	
	bool connectable(FlowOutlet *f, bool force, QString &consoID);
	bool connectable(Flow *fo, bool force, QString &consoID);
	
	int getType() { return FLOW_OUT; }
	
	void setGroup(QString g);
	QString &getGroup();
	
	void setRunning();
	void setDisable();
	void setError();
	void setIdle();
	void refresh();
	bool isPaused();
	void setPaused(bool _pause);
	
	FlowGraphicOutletOut *getFlowOutletGraphic() { return flowGraphic; }
	
private:
	FlowGraphicOutletOut *flowGraphic;
	QString grp; 
};

#endif
