#include "GlobalCC.h"
#include "SfControl.h"
#include "Server.h"
#include "Client.h"
#include "Flow.h"
#include "Duplicator.h"
#include "map/MapView.h"
#include "mainwindow.h"

#include <QtCore/QFile>
#include <QtCore/QRegExp>
#include <QtCore/QPoint>
#include <QtCore/QStringList>
#include <QtGui/QWidget>
#include <QtXml/QDomDocument>

#include <set>
#include <iostream>
using namespace std;

GlobalCC::GlobalCC() {
	appli = NULL;
	sfct = NULL;
	serverList = NULL;
	clientConfList = NULL;
}

GlobalCC::~GlobalCC() {
	if ( sfct ) {
		sfct->quit();
		sfct->wait();
		delete sfct;
	}
	sfct = NULL;
	if ( appli )
		delete appli;
	appli = NULL;
	if ( serverList )
		delete serverList;
	serverList = NULL;
	if ( clientConfList )
		delete clientConfList;
	clientConfList = NULL;
}

void GlobalCC::createServerList() {
	serverList = new ServerList(this);
}

void GlobalCC::createClientConfList() {
	clientConfList = new ClientConfList(this);
}

void GlobalCC::createApplication() {
	appli = new ApplicationCC(this);
}

void GlobalCC::createSfControlThread(QSemaphore *s) {
	//cout << "Creating SfControlThread" << endl;
	sfct = new SfControlThread(this, s);
	//cout << "SfControlThread created" << endl;
	sfct->start();
	//cout << "SfControlThread started" << endl;
}

void GlobalCC::setServerList(ServerList *_serverList) {
	if ( _serverList )
		serverList =  _serverList;
} 

void GlobalCC::setClientConfList(ClientConfList *_clientList) {
	if ( _clientList )
		clientConfList = _clientList;
}

void GlobalCC::setApplication(ApplicationCC *_appli) {
	if ( _appli )
		appli = _appli;
}

ApplicationCC* GlobalCC::getApplication() { return appli; }

ServerList* GlobalCC::getServerList() { return serverList; }

ClientConfList* GlobalCC::getClientConfList() { return clientConfList; }

SfControlThread* GlobalCC::getSfControlThread() { return sfct; }

void GlobalCC::clearServerList() {
	if ( serverList )
		delete serverList;
	createServerList();
}

void GlobalCC::clearClientConfList() {
	if ( clientConfList )
		delete clientConfList;
	createClientConfList();
}

void GlobalCC::clearApplication() {
    if ( appli )
		delete appli;
    createApplication();
}

bool GlobalCC::hasServerList() { return ( serverList != NULL ); }

bool GlobalCC::hasClientConfList() { return ( clientConfList != NULL ); }

bool GlobalCC::hasApplication() { return ( appli != NULL ); }

int GlobalCC::theGroupId;

unsigned int GlobalCC::getNewId() {
	static int theID = 1;
	return ++theID;
}

QString& GlobalCC::getNewGroup() {
    ++theGroupId;
    QString *grp = new QString("ID");
    QString nb = QString::number(theGroupId);
    grp->append(nb);
    return *grp;
}

void GlobalCC::setInitialGroupId(int val) {
    theGroupId = val;
}

/***************************************************************/

DuplicatorList::DuplicatorList(GlobalCC *_g) {
	g = _g;
	initList();
}

DuplicatorList::~DuplicatorList() {
	g = NULL;
	clear();
}

void DuplicatorList::addDuplicator(Duplicator *dup) {
	if ( dup ) {
		if ( !duplicatorExists(dup->getId()) ) {
			duplicators[dup->getId()] = dup;
			MainWindow::instance()->logView->addDuplicatorTab(dup->getId());
		}
	}
}

void DuplicatorList::removeDuplicator(Duplicator *dup) {
	if ( dup ) {
		if ( duplicatorExists(dup->getId()) ) {
			duplicators.erase(dup->getId());
			MainWindow::instance()->logView->removeDuplicatorTab(dup->getId());
			delete dup;
		}
	}
}

void DuplicatorList::removeDuplicator(unsigned int _id) {
	if ( duplicatorExists(_id) ) {
	    Duplicator *tmp_dup = getDuplicator(_id);
		duplicators.erase(_id);
		MainWindow::instance()->logView->removeDuplicatorTab(_id);
		delete tmp_dup;
	}
}

bool DuplicatorList::duplicatorExists(unsigned int _id) {
	std::map<unsigned int, Duplicator *>::iterator iter;
	iter = duplicators.find(_id);
	if (iter != duplicators.end()) 
		return true;
	else
		return false;
}

unsigned int DuplicatorList::duplicatorExists(QString &flowID, QString &serverName) {
	std::map<unsigned int, Duplicator *>::iterator iter;	
    iter = duplicators.begin();
    bool found = false;
    while ( iter != duplicators.end() && !found  ) {
		if ( (iter->second->getFlowID() == flowID) && (iter->second->getServerName() == serverName) ) {
			found = true;
		}
		else 
			iter++;
	}
	if ( found )
		return iter->first;
	else 
		return -1;
}

Duplicator* DuplicatorList::getDuplicator(unsigned int _id) {
	if ( duplicatorExists(_id) )
		return duplicators[_id];
	else 
		return NULL;
}

Duplicator* DuplicatorList::getDuplicator(QString &flowID, QString &serverName) {
	int  res = duplicatorExists(flowID, serverName);
	if ( res != -1 )
		return duplicators[res];
    else
		return NULL;
}

std::map<unsigned int, Duplicator*>* DuplicatorList::getDuplicatorList() {
	return &duplicators;
}

void DuplicatorList::printList() {
	cout << "Duplicator List" << endl;
	std::map<unsigned int, Duplicator *>::iterator iter;
	for ( iter = duplicators.begin(); iter != duplicators.end(); iter++ )
		iter->second->print();
}

void DuplicatorList::initList() {
	duplicators.clear();
}

void DuplicatorList::clear() {
	//we need to delete every duplicator
	//and also their log tab and clear the list
	MainWindow::instance()->logView->removeAllDuplicatorTab();
	std::map<unsigned int, Duplicator *>::iterator iter;
	Duplicator *tmp_dup = NULL;
	for ( iter = duplicators.begin(); iter != duplicators.end(); iter++ ) {
		tmp_dup = iter->second;
		delete tmp_dup;
	}
	duplicators.clear();
}

/***************************************************************/

ServerList::ServerList(GlobalCC *_g) {
	id = GlobalCC::getNewId();
	g = _g;
	initList();
}

ServerList::~ServerList() {}

void ServerList::addServer(Server *s) {
	if ( s ) {
		if ( !serverExists(s->getId()) ) {
			servers[s->getId()] = s;
			MainWindow::instance()->logView->addServerTab(this, s->getId());
		}
	}
}

void ServerList::removeServer(Server *s) {
	if ( s ) {
		if ( serverExists(s->getId()) )
			servers.erase(s->getId()); 
	}
}

void ServerList::removeServer(unsigned int _id) {
	if ( serverExists(_id) )
		servers.erase(_id); 
}

bool ServerList::serverExists(unsigned int _id) {
	std::map<unsigned int, Server *>::iterator iter;
	iter = servers.find(_id);
	if (iter != servers.end()) 
		return true;
	else
		return false;
} 

int ServerList::serverNameExist(QString s) {
    std::map<unsigned int, Server *>::iterator iter;	
    iter = servers.begin();
    bool found = false;
    while ( iter != servers.end() && !found  ) {
		if ( iter->second->getName() == s ) {
			found = true;
		}
		else 
			iter++;
    }
    if ( !found ) 
		return -1;
    else 
		return iter->second->getId();
}

int ServerList::serverPathExist(QString s) {
    std::map<unsigned int, Server *>::iterator iter;	
    iter = servers.begin();
    bool found = false;
    while ( iter != servers.end() && !found  ) {
		if ( iter->second->getAddress() == s ) {
			found = true;
		}
		else 
			iter++;
    }
    if ( !found ) 
		return -1;
    else 
		return iter->second->getId();
}

Server* ServerList::getServer(unsigned int _id) {
	if ( serverExists(_id) )
		return servers[_id];
	else 
		return NULL;
}

Server* ServerList::getServer(QString name) {
    int  res = serverNameExist(name);
    if ( res != -1 )
		return servers[res];
    else
		return NULL;
}

Server* ServerList::getServerWithPath(QString _path) {
	int  res = serverPathExist(_path);
    if ( res != -1 )
		return servers[res];
    else
		return NULL;
}

std::map<unsigned int, Server *>* ServerList::getServerList() {
    return &servers;
}

void ServerList::printServerList() {
	cout << "Server List" << endl;
	std::map<unsigned int, Server *>::iterator iter;
	for ( iter = servers.begin(); iter != servers.end(); iter++ )
		iter->second->print();
}

void ServerList::initList() {
	Server *serv = new Server(g, "No Server", "No Server", 0, "");
	if ( serverPathExist(serv->getAddress()) == -1 )
		addServer(serv);
}

bool ServerList::readServerList(QString serverList) {
	QDomDocument myDoc;
	if ( !myDoc.setContent(serverList) )
		return false;
	//time to parse the document
	QString tagname = "";
	QDomElement root = myDoc.documentElement(); 
	QDomElement child = root.firstChild().toElement();
	while( !child.isNull() ) {
		tagname = child.tagName();
		if ( tagname == "Serverlist" ) {
			QDomElement smallchild = child.firstChild().toElement();
			QString tagname2 = "";
			while ( !smallchild.isNull() ) {
				tagname2 = smallchild.tagName();
				if ( tagname2 == "server" ) {
					QDomElement smallsmallchild = smallchild.firstChild().toElement();
					QString tagname3 = "";
					int i = 0;
					QString theName = "";
					QString thePort = "";
					QString theOS = "";
					while ( !smallsmallchild.isNull() ) {
						tagname3 = smallsmallchild.tagName();
						if ( tagname3 == "name" ) {
							theName = smallsmallchild.text();
							i++;
						}
						if ( tagname3 == "port" ) {
							thePort = smallsmallchild.text();
							i++;
						}
						if ( tagname3 == "os" ) {
							theOS = smallsmallchild.text();
							i++;
						}
						if ( !theName.isEmpty() && !thePort.isEmpty() && !theOS.isEmpty() && ( i==3 ) ) {
							Server *s = new Server(g, theName, theName, thePort.toUInt(), theOS);
							if ( s ) {
								if ( serverPathExist(s->getAddress()) == -1 ) 
									addServer(s);
							}
							theName = "";
							thePort = "";
							theOS = "";
							i = 0;
						}
						smallsmallchild = smallsmallchild.nextSibling().toElement();
					}
				}
				smallchild = smallchild.nextSibling().toElement();
			}
		} // if ( tagname == "ServerList" ) 
		child = child.nextSibling().toElement();
	} // while( !child.isNull() )
	return true;
}

/***************************************************************/

ClientConfList::ClientConfList(GlobalCC *_g) {
	id = GlobalCC::getNewId();
	g = _g;
}

ClientConfList::~ClientConfList() {}

void ClientConfList::addClient(ClientConf *c) {
	if ( c ) {
		if ( !clientExists(c->getId()) )
			clients[c->getId()] = c;
	}
}

void ClientConfList::removeClient(ClientConf *c) {
	if ( c ) {
		if ( clientExists(c->getId()) )
			clients.erase(c->getId()); 
	}
}

void ClientConfList::removeClient(unsigned int _id) {
	if ( clientExists(_id) ) 
		clients.erase(_id);
}

bool ClientConfList::clientExists(unsigned int _id) {
	std::map<unsigned int, ClientConf *>::iterator iter;
	iter = clients.find(_id);
	if (iter != clients.end()) 
		return true;
	else
		return false;
}

ClientConf* ClientConfList::clientExists(ClientConf *cc) {
	if ( cc == NULL )
		return NULL;
	std::map<unsigned int, ClientConf *>::iterator iter;	
	iter = clients.begin();
	ClientConf *found = NULL;
	while ( iter != clients.end() && !found  ) {
		if ( ( iter->second->getName() == cc->getName() ) && ( iter->second->getPath() == cc->getPath() ) )
			found = iter->second;
		iter++;
	}
	return found;  
}

bool ClientConfList::clientNameExist(ClientConf *cc) {
    std::map<unsigned int, ClientConf *>::iterator iter;	
    iter = clients.begin();
    bool found = false;
    while ( iter != clients.end() && !found  ) {
		if ( iter->second->getName() == cc->getName() )
			found = true;
		iter++;
    }
    return found;  
}

int ClientConfList::clientNameExist(QString s) {
	std::map<unsigned int, ClientConf *>::iterator iter;	
	iter = clients.begin();
	bool found = false;
	while ( iter != clients.end() && !found  ) {
		if ( iter->second->getName() == s ) {
			found = true;
		}		
		else
			iter++;
	}
	if ( !found ) 
		return -1;
	else 
		return iter->second->getId();
}

bool ClientConfList::clientPathExists(QString thePath) {
	std::map<unsigned int, ClientConf *>::iterator iter;
	iter = clients.begin();
	bool found = false;
	while ( iter != clients.end() && !found  ) {
		if ( iter->second->getPath() == thePath )
			found = true;
		iter++;
	}
	return found;  
}

ClientConf* ClientConfList::getClientWithPath(QString thePath) {
	std::map<unsigned int, ClientConf *>::iterator iter;
	iter = clients.begin();
	bool found = false;
	while ( iter != clients.end() && !found  ) {
		if ( iter->second->getPath() == thePath )
			found = true;
		else
			iter++;
	}
	if ( iter == clients.end() )
		return NULL;
	else
		return iter->second;   
}

ClientConf* ClientConfList::getClient(unsigned int _id) {
	if ( clientExists(_id) )
		return clients[_id];
	else 
		return NULL;
}

std::map<unsigned int, ClientConf *>* ClientConfList::getClientList() {
    return &clients;
}

void ClientConfList::printClientList() {
	cout << "Client List" << endl;
	std::map<unsigned int, ClientConf *>::iterator iter;
	for ( iter = clients.begin(); iter != clients.end(); iter++ )
		iter->second->print();
}

ClientConf* ClientConfList::readClientFromXMLFile(QString filename) {
	if ( filename.isEmpty() ) {
		return NULL;
	}	
	QFile f(filename);
	QDomDocument myDoc;
	if ( !f.open( QIODevice::ReadOnly ) ) { 
		return NULL;
	}	
	if ( !myDoc.setContent( &f ) ) {
		f.close(); 	
		return NULL;
	}
	f.close();
	QString type = myDoc.doctype().name();
	if ( type != "clientDescription" ) {
		return NULL;
	}
	else {
		QString tagname = "";
		QDomElement root = myDoc.documentElement(); 
		QDomElement child = root.firstChild().toElement();
		tagname = child.tagName();
		if ( tagname == "client" ) {
			QDomElement smallchild = child.firstChild().toElement();
			QString tagname3 = "";
			int i = 0;
			QString theName = "";
			QString thePath = "";
			std::set<FlowOutletIn *> foin;
			foin.clear();
			std::set<FlowOutletOut *> foout;
			foout.clear();
			while( !smallchild.isNull() ) {
				tagname3 = smallchild.tagName();
				if ( tagname3 == "name" ) {
					theName = smallchild.text();
					i++;
				}
				if ( tagname3 == "path" ) {
					thePath = smallchild.text();
					i++;
				}
				if ( tagname3 == "flows" ) {
					i++;
					QDomElement smallsmallchild = smallchild.firstChild().toElement();
					QString tagname4 = "";
					while( !smallsmallchild.isNull() ) {
						tagname4 = smallsmallchild.tagName();
						if ( tagname4 == "input" ) {
							QDomElement smallsmallsmallchild = smallsmallchild.firstChild().toElement();
							QString tagname5 = "";
							int j = 0;
							QString flowName = "";
							QString flowType = "";
							while( !smallsmallsmallchild.isNull() ) {
								tagname5 = smallsmallsmallchild.tagName();
								if ( tagname5 == "name" ) {
									flowName = smallsmallsmallchild.text();
									j++;
								}
								if ( tagname5 == "type" ) {
									flowType = smallsmallsmallchild.text();
									j++;
								}
								if ( !flowName.isEmpty() && !flowType.isEmpty() && (j == 2) ) {
									j = 0;
									FlowOutletIn *f = new FlowOutletIn(g, flowName, flowType);
									if ( f ) {
										foin.insert(f);
									}
									flowName = "";
									flowType = "";
								}
								smallsmallsmallchild = smallsmallsmallchild.nextSibling().toElement();
							}
						}
						if ( tagname4 == "output" ) {
							QDomElement smallsmallsmallchild = smallsmallchild.firstChild().toElement();
							QString tagname5 = "";
							int j = 0;
							QString flowName = "";
							QString flowType = "";
							while( !smallsmallsmallchild.isNull() ) {
								tagname5 = smallsmallsmallchild.tagName();
								if ( tagname5 == "name" ) {
									flowName = smallsmallsmallchild.text();
									j++;
								}
								if ( tagname5 == "type" ) {
									flowType = smallsmallsmallchild.text();
									j++;
								}
								if ( !flowName.isEmpty() && !flowType.isEmpty() && (j == 2) ) {
									FlowOutletOut *f = new FlowOutletOut(g, flowName, flowType);
									j = 0;
									if ( f ) {
										foout.insert(f);
									}
									flowName = "";
									flowType = "";
								}
								smallsmallsmallchild = smallsmallsmallchild.nextSibling().toElement();
							}
						}
						smallsmallchild = smallsmallchild.nextSibling().toElement(); 
					}
				}// if ( tagname3 == "flows" ) {
				if ( !theName.isEmpty() && !thePath.isEmpty() && ( i > 2 ) ) {
					//we connect the flows to the client and add the client to the configuration
					i = 0;
					//we have to take care of the path
					/*thePath = filename;
					QRegExp rxp( "\\.client$", true, false );
					int nb = thePath.contains( rxp );
					if ( nb != 0 ) 
					thePath = thePath.remove(rxp);*/
					ClientConf *cl = new ClientConf(g, theName, thePath);
					cout << "thePath: " << cl->getPath().toStdString() << endl;
					if ( cl ) {
						std::set<FlowOutletIn *>::iterator iter_in;
						for ( iter_in = foin.begin(); iter_in != foin.end(); iter_in++ ) {
							cl->addInputFlow((*iter_in));
						}
						std::set<FlowOutletOut *>::iterator iter_out;
						for ( iter_out = foout.begin(); iter_out != foout.end(); iter_out++ ) {
							cl->addOutputFlow((*iter_out));
						}
						//connect the flows to the client
						return cl;
						foin.clear();
						foout.clear();
						theName = "";
						thePath = "";
					}
				}
				smallchild = smallchild.nextSibling().toElement();
				}//while( !smallchild.isNull() ) {
				}
			}
	return NULL;
		}

bool ClientConfList::readClientListFromXMLFile(QString &filename) {
	if ( filename.isEmpty() ) {
		return false;
	}	
	QFile f(filename);
	QDomDocument myDoc;
	if ( !f.open( QIODevice::ReadOnly ) ) { 
		return false;
	}	
	if ( !myDoc.setContent( &f ) ) {
		f.close(); 	
		return false;
	}
	f.close();
	QString type = myDoc.doctype().name();
	if ( type != "clientListCC" ) {
		return false;
	}
	else {
		QString tagname = "";
		QDomElement root = myDoc.documentElement(); 
		QDomElement child = root.firstChild().toElement();
		while( !child.isNull() ) {
			tagname = child.tagName();
			/*if ( tagname == "servers" ) {
			QDomElement smallchild = child.firstChild().toElement();
			QString tagname2 = "";
			while( !smallchild.isNull() ) {
				tagname2 = smallchild.tagName();
				if ( tagname2 == "server" ) {
					QDomElement smallsmallchild = smallchild.firstChild().toElement();
					QString tagname3 = "";
					int i = 0;
					QString theName = "";
					QString theHost = "";
					QString thePort = "";
					QString theComment = "";
					while( !smallsmallchild.isNull() ) {
						tagname3 = smallsmallchild.tagName();
						if ( tagname3 == "name" ) {
							theName = smallsmallchild.text();
							i++;
						}
						if ( tagname3 == "host" ) {
							theHost = smallsmallchild.text();
							i++;
						}
						if ( tagname3 == "port" ) {
							thePort = smallsmallchild.text();
							i++;
						}
						if ( tagname3 == "comment" ) {
							theComment = smallsmallchild.text();
							i++;
						}
						if ( !theName.isEmpty() && !theHost.isEmpty() &&  !thePort.isEmpty() && (i == 4) ) {
							i = 0;
							Server *s = new Server(g, theName, theHost, thePort.toUInt(), theComment);
							theName = "";
							theHost = "";
							thePort = "";
							theComment = "";
							if ( s ) {
								addServer(s);
							}
						}
						smallsmallchild = smallsmallchild.nextSibling().toElement();  
					}
				}
				smallchild = smallchild.nextSibling().toElement(); 
			}
			} */
			if ( tagname == "clients" ) {
				QDomElement smallchild = child.firstChild().toElement();
				QString tagname2 = "";
				while( !smallchild.isNull() ) {
					tagname2 = smallchild.tagName();
					if ( tagname2 == "client" ) {
						QDomElement smallsmallchild = smallchild.firstChild().toElement();
						QString tagname3 = "";
						int i = 0;
						QString theName = "";
						QString thePath = "";
						std::set<FlowOutletIn *> foin;
						foin.clear();
						std::set<FlowOutletOut *> foout;
						foout.clear();
						while( !smallsmallchild.isNull() ) {
							tagname3 = smallsmallchild.tagName();
							if ( tagname3 == "name" ) {
								theName = smallsmallchild.text();
								i++;
							}
							if ( tagname3 == "path" ) {
								thePath = smallsmallchild.text();
								i++;
							}
							if ( tagname3 == "flows" ) {
								i++;
								QDomElement smallsmallsmallchild = smallsmallchild.firstChild().toElement();
								QString tagname4 = "";
								while( !smallsmallsmallchild.isNull() ) {
									tagname4 = smallsmallsmallchild.tagName();
									if ( tagname4 == "input" ) {
										QDomElement foursmallchild = smallsmallsmallchild.firstChild().toElement();
										QString tagname5 = "";
										int j = 0;
										QString flowName = "";
										QString flowType = "";
										while( !foursmallchild.isNull() ) {
											tagname5 = foursmallchild.tagName();
											if ( tagname5 == "name" ) {
												flowName = foursmallchild.text();
												j++;
											}
											if ( tagname5 == "type" ) {
												flowType = foursmallchild.text();
												j++;
											}
											if ( !flowName.isEmpty() && !flowType.isEmpty() && (j == 2) ) {
												j = 0;
												FlowOutletIn *f = new FlowOutletIn(g, flowName, flowType);
												if ( f ) {
													foin.insert(f);
												}
												flowName = "";
												flowType = "";
											}
											foursmallchild = foursmallchild.nextSibling().toElement();
										}
									}
									if ( tagname4 == "output" ) {
										QDomElement foursmallchild = smallsmallsmallchild.firstChild().toElement();
										QString tagname5 = "";
										int j = 0;
										QString flowName = "";
										QString flowType = "";
										while( !foursmallchild.isNull() ) {
											tagname5 = foursmallchild.tagName();
											if ( tagname5 == "name" ) {
												flowName = foursmallchild.text();
												j++;
											}
											if ( tagname5 == "type" ) {
												flowType = foursmallchild.text();
												j++;
											}
											if ( !flowName.isEmpty() && !flowType.isEmpty() && (j == 2) ) {
												FlowOutletOut *f = new FlowOutletOut(g, flowName, flowType);
												j = 0;
												if ( f ) {
													foout.insert(f);
												}
												flowName = "";
												flowType = "";
											}
											foursmallchild = foursmallchild.nextSibling().toElement();
										}
									}
									smallsmallsmallchild = smallsmallsmallchild.nextSibling().toElement(); 
								}
							}// if ( tagname3 == "flows" ) {
							if ( !theName.isEmpty() && !thePath.isEmpty() && ( i > 2 ) ) {
								//we connect the flows to the client and add the client to the configuration
								i = 0;
								ClientConf *cl = new ClientConf(g, theName, thePath);
								if ( cl ) {
									std::set<FlowOutletIn *>::iterator iter_in;
									for ( iter_in = foin.begin(); iter_in != foin.end(); iter_in++ ) {
										cl->addInputFlow((*iter_in));
									}
									std::set<FlowOutletOut *>::iterator iter_out;
									for ( iter_out = foout.begin(); iter_out != foout.end(); iter_out++ ) {
										cl->addOutputFlow((*iter_out));
									}
									//connect the flows to the client
									if ( !clientPathExists(cl->getPath()) )
										addClient(cl);
									foin.clear();
									foout.clear();
									theName = "";
									thePath = "";
								}
							}
							smallsmallchild = smallsmallchild.nextSibling().toElement();
							}//while( !smallsmallchild.isNull() ) {
							}
					smallchild = smallchild.nextSibling().toElement();
						}
					}
			child = child.nextSibling().toElement();
				}
			}
	return true;
		}

bool ClientConfList::writeClientListFromXMLFile(QString &filename) {
	QFile file(filename);
	if ( !file.open( QIODevice::WriteOnly ) ) {
		return false;
	}
	else {
		XmlWriter *xw;
		xw = new  XmlWriter(&file);
		xw->setAutoNewLine(true);
		xw->writeRaw("<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>");
		xw->newLine();
		xw->writeRaw("<!DOCTYPE clientListCC >");
		xw->newLine();
		xw->writeRaw("<!--This file describe a List of Clients in the NSDFS Control Center-->");
		xw->newLine();
		xw->writeOpenTag("config");
		/*xw->writeOpenTag("servers");
		std::map<unsigned int, Server *>::iterator iter;
		for ( iter = servers.begin(); iter != servers.end(); iter++ )
			writeXmlServer(xw, iter->second);
		xw->writeCloseTag("servers");*/
		xw->writeOpenTag("clients");
		std::map<unsigned int, ClientConf *>::iterator iter2;
		for ( iter2 = clients.begin(); iter2 != clients.end(); iter2++ )
			writeXmlClient(xw, iter2->second);
		//we write every clients in the xml file
		xw->writeCloseTag("clients");
		xw->writeCloseTag("config");
	}
	return true;
}

void ClientConfList::writeXmlClient(XmlWriter *xw, ClientConf *c) {
	if ( c ) {
		xw->writeOpenTag("client");
		xw->writeTaggedString("name", c->getName());
		xw->writeTaggedString("path", c->getPath());
		xw->writeOpenTag("flows");
		
		std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
		std::map<unsigned int, FlowOutletIn *> *fin_tmp = c->getInputFlowList();
		for( iter_in = fin_tmp->begin(); iter_in != fin_tmp->end(); iter_in++ ) {
			FlowOutletIn *tmp_in = (iter_in->second);
			writeXmlFlow(xw, tmp_in);
		}
		
		std::map<unsigned int, FlowOutletOut *>::iterator iter_out;  
		std::map<unsigned int, FlowOutletOut *> *fout_tmp = c->getOutputFlowList();
		for( iter_out = fout_tmp->begin(); iter_out != fout_tmp->end(); iter_out++ ) {
			FlowOutletOut *tmp_out = (iter_out->second);
			writeXmlFlow(xw, tmp_out);
		}
		
		xw->writeCloseTag("flows");
		xw->writeCloseTag("client");
	}
}

void ClientConfList::writeXmlFlow(XmlWriter *xw, FlowOutlet *f) {
	if ( f ) {
		if ( f->getType() == FlowOutlet::FLOW_IN )
			xw->writeOpenTag("input");
		else
			xw->writeOpenTag("output");
		
		xw->writeTaggedString("name", f->getName());
		xw->writeTaggedString("type", f->getDataType());
		
		
		if ( f->getType() == FlowOutlet::FLOW_IN )
			xw->writeCloseTag("input");
		else 
			xw->writeCloseTag("output"); 
	}
}

/***************************************************************/

ApplicationCC::ApplicationCC(GlobalCC *_g) {
	id = GlobalCC::getNewId();
	g = _g;
	//  mutex = new QMutex();
	map = NULL;
	dupList = NULL;
	dupList = new DuplicatorList(_g);
	
}

ApplicationCC::~ApplicationCC() {
    if ( map ) {
		delete map;
		map = NULL;
    }
	if ( dupList )
		delete dupList;
}

void ApplicationCC::initMap(QWidget* parent) {
    map = new MapView(parent);
}

void ApplicationCC::createClientApp(ClientConf *cc, Server *s, const QPoint &p, QString &param) {
    if ( cc && s ) {
		ClientApp *ca = new ClientApp(cc, s);
		if ( ca ) {
			int numClients = getNumberOfClientsWithSamePath(ca);
			if ( numClients > 0 )
				ca->setName(ca->getName() + "_(" + QString::number(numClients+1) + ")");
			ca->setParam(param);
			ca->getClientGraphic()->hide();
			ca->getClientGraphic()->setPos(p.x()/TILE_SX, p.y()/TILE_SY);
			ca->getClientGraphic()->move();
			ca->getClientGraphic()->refreshLabel();
			ca->getClientGraphic()->show();
			addClient(ca);
			std::map<unsigned int, FlowOutletIn*>* foin = ca->getInputFlowList();
			std::map<unsigned int, FlowOutletOut*>* foout = ca->getOutputFlowList(); 
			std::map<unsigned int, FlowOutletIn*>::iterator iter_in;
			std::map<unsigned int, FlowOutletOut*>::iterator iter_out;
			for ( iter_in = foin->begin(); iter_in != foin->end(); iter_in++) {
				addOutlet( iter_in->second );
			}
			for ( iter_out = foout->begin(); iter_out != foout->end(); iter_out++) {
				addOutlet( iter_out->second );
			}
			MainWindow::instance()->appliNeedSave = true;
		}	
    }
}

void ApplicationCC::addClient(ClientApp *c) {
	if ( c ) {
		if ( !clientExists(c->getId()) ) {
			clients[c->getId()] = c;
			MainWindow::instance()->appliNeedSave = true;
			MainWindow::instance()->logView->addClientTab(c->getId());
			refreshAllViews();
		}
	}
}

void ApplicationCC::addFlow(Flow *f) {
	if ( f ) {
		if ( !flowExists(f->getId()) ) {
			flows[f->getId()] = f;
			MainWindow::instance()->appliNeedSave = true;
			refreshAllViews();
		}
	}
}

void ApplicationCC::addOutlet(FlowOutlet *fo) {
	if ( fo ) {
		if ( !flowOutletExists(fo->getId()) ) {
			flowOutlets[fo->getId()] = fo;
			MainWindow::instance()->appliNeedSave = true;
			refreshAllViews();
		}
	}    
}

void ApplicationCC::removeClient(ClientApp *c) {
	if ( c ) {
		if ( clientExists(c->getId()) ) {
			MainWindow::instance()->logView->removeClientTab(c->getId());
			clients.erase(c->getId());
			MainWindow::instance()->appliNeedSave = true;
			refreshAllViews();
		}
	}
}

void ApplicationCC::removeClient(unsigned int _id) {
    if ( clientExists(_id) )  {
		clients.erase(_id);
		MainWindow::instance()->appliNeedSave = true;
		refreshAllViews();
    }
}

void ApplicationCC::removeFlow(Flow *f) {
	if ( f ) {
		if ( flowExists(f->getId()) ) {
			flows.erase(f->getId()); 
			MainWindow::instance()->appliNeedSave = true;
			refreshAllViews();
		}
	}
}

void ApplicationCC::removeFlow(unsigned int _id) {
    if ( flowExists(_id) ) {
		flows.erase(_id);
		MainWindow::instance()->appliNeedSave = true;
		refreshAllViews();
    }
}

void ApplicationCC::removeOutlet(FlowOutlet *fo) {
	if ( fo ) {
		if ( flowOutletExists(fo->getId()) ) {
			flowOutlets.erase(fo->getId()); 
			MainWindow::instance()->appliNeedSave = true;
		}	
	}
}

void ApplicationCC::removeOutlet(unsigned int _id) {
    if ( flowOutletExists(_id) ) {
		flowOutlets.erase(_id);
		MainWindow::instance()->appliNeedSave = true;
    }
}

bool ApplicationCC::clientExists(unsigned int _id) {
	std::map<unsigned int, ClientApp *>::iterator iter;
	iter = clients.find(_id);
	if (iter != clients.end()) 
		return true;
	else
		return false;
}

bool ApplicationCC::flowExists(unsigned int _id) {
	std::map<unsigned int, Flow *>::iterator iter;
	iter = flows.find(_id);
	if (iter != flows.end()) 
		return true;
	else
		return false;
}

bool ApplicationCC::flowOutletExists(unsigned int _id) {
	std::map<unsigned int, FlowOutlet *>::iterator iter;
	iter = flowOutlets.find(_id);
	if (iter != flowOutlets.end()) 
		return true;
	else
		return false;
}

ClientApp* ApplicationCC::getClient(unsigned int _id) {
	if ( clientExists(_id) )
		return clients[_id];
	else 
		return NULL;
}

ClientApp* ApplicationCC::getClient(QString &_group, QString &_name) {
	std::map<unsigned int, ClientApp *>::iterator iter;	
	iter = clients.begin();
	bool found = false;
	while ( iter != clients.end() && !found  ) {
		if ( (iter->second->getGroup() == _group) && (iter->second->getName() == _name) ) {
			found = true;
		}		
		else
			iter++;
	}
	if ( !found ) 
		return NULL;
	else 
		return iter->second;
}

Flow* ApplicationCC::getFlow(unsigned int _id) {
	if ( flowExists(_id) )
		return flows[_id];
	else 
		return NULL;
}

/*Flow* ApplicationCC::getFlow(QString &_group) {
std::map<unsigned int, Flow *>::iterator iter;	
iter = flows.begin();
bool found = false;
while ( iter != flows.end() && !found  ) {
    if ( iter->second->getGroup() == _group ) {
		found = true;
    }		
    else
		iter++;
}
if ( !found ) 
return NULL;
else 
return iter->second;

}*/

FlowOutlet* ApplicationCC::getFlowOutlet(unsigned int _id) {
	if ( flowOutletExists(_id) )
		return flowOutlets[_id];
	else 
		return NULL;
}

FlowOutlet* ApplicationCC::getFlowOutlet(QString &_group) {
	std::map<unsigned int, FlowOutlet *>::iterator iter;	
	iter = flowOutlets.begin();
	bool found = false;
	while ( iter != flowOutlets.end() && !found  ) {
		if ( iter->second->getGroup() == _group ) {
			FlowOutletOut *fout = dynamic_cast<FlowOutletOut *>(iter->second);
			if ( fout )
				found = true;
			else
				iter++;
		}		
		else
			iter++;
	}
	if ( !found ) 
		return NULL;
	else 
		return iter->second;
}

unsigned int ApplicationCC::getNumberOfClientsWithSamePath(ClientApp *c) {
	if ( c ) {
		unsigned int numClients = 0;
		std::map<unsigned int, ClientApp *>::iterator iter;	
		QString thePath = c->getPath();
		for ( iter = clients.begin(); iter != clients.end(); iter++ ) 
			if ( iter->second->getPath() == thePath ) 
				numClients++; 
		return numClients;
	}
	else
		return 0;
}


struct consumerFlow {
    QString givenFlowName;
    QString status;
};

class FlowInfos {
public:
	QString group; 
	QString type; 
	QString name; 
	QString desc; 
	QString producer;
	FlowInfos();
	~FlowInfos() {
		if ( consumerNameList )
			delete consumerNameList;
	}
	FlowInfos(QString _group, QString _type, QString _name, QString _desc, QString _producer) {
		group = _group;
		type = _type;
		name = _name;
		desc = _desc;
		producer = _producer;
		consumerNameList = NULL;
		//iter_consumerNameList = NULL;
		initIter = false;
	}
	void setConsumerList(std::map<QString, consumerFlow> *thelist) { consumerNameList = thelist; }
	void initIterator() { 
		if ( consumerNameList != NULL ) {
			initIter = true;
			iter_consumerNameList = consumerNameList->begin(); 
		}
	}
	
	void getNextConsumer(QString &consName, QString &flowName, QString &flowStatus) { 
		if ( initIter ) {
			if ( iter_consumerNameList != consumerNameList->end() ) {
				consName = iter_consumerNameList->first;
				flowName = iter_consumerNameList->second.givenFlowName;
				flowStatus = iter_consumerNameList->second.status;
				iter_consumerNameList++;
			}
			else {
				consName = "";
				flowName = "";
				flowStatus = "";
			}
		} 
		else {
			consName = "";
			flowName = ""; 
			flowStatus = "";
		}
	}
	
private:
		bool initIter;
	std::map<QString, consumerFlow> *consumerNameList;
	std::map<QString, consumerFlow>::iterator iter_consumerNameList;
};

bool ApplicationCC::readApplicationDescription(QString appliDesc) {
	std::map<QString, ClientConf *> clientConfList;
	clientConfList.clear();
	std::map<QString, FlowInfos *> flowInfosList;
	flowInfosList.clear();
	std::map<QString, ClientApp *> clientAppList;
	clientAppList.clear();
	QDomDocument myDoc;
	if ( !myDoc.setContent(appliDesc) ) {
		//cout << "APPLI DESC\n " << appliDesc.toStdString() << endl;
		return false;
	}
	
	//time to parse the document
	QString tagname = "";
	QDomElement root = myDoc.documentElement(); 
	QDomElement child = root.firstChild().toElement();
	while( !child.isNull() ) {
		tagname = child.tagName();
		if ( tagname == "GlobalList" ) {
			QDomElement smallchild = child.firstChild().toElement();
			QString tagname2 = "";
			while ( !smallchild.isNull() ) {
				tagname2 = smallchild.tagName();
				if ( tagname2 == "ClientList" ) {
					QDomElement smallsmallchild = smallchild.firstChild().toElement();
					QString tagname3 = "";
					while ( !smallsmallchild.isNull() ) {
						tagname3 = smallsmallchild.tagName();
						QString theID = "";
						if ( tagname3 == "Client" ) {
							theID = smallsmallchild.attribute("ID");
							QDomElement smallsmallsmallchild = smallsmallchild.firstChild().toElement();
							QString tagname4 = "";
							int i = 0;
							QString theGroup = "";
							QString theName = "";
							QString theDesc = "";
							QString theServer = "";
							QString theParams = "";
							QString thePath = "";
							QString theFlowInfo = "";
							while ( !smallsmallsmallchild.isNull() ) {
								tagname4 = smallsmallsmallchild.tagName();
								if ( tagname4 == "Group" ) {
									theGroup = smallsmallsmallchild.text();
									i++;
								}
								if ( tagname4 == "Name" ) {
									theName = smallsmallsmallchild.text();
									i++;
								}
								if ( tagname4 == "Description" ) {
									theDesc = smallsmallsmallchild.text();
									i++;
								}
								if ( tagname4 == "Server" ) {
									theServer = smallsmallsmallchild.text();
									i++;
								}
								if ( tagname4 == "Params" ) {
									theParams = smallsmallsmallchild.text();
									i++;
								}
								if ( tagname4 == "Path" ) {
									thePath = smallsmallsmallchild.text();
									i++;
								}
								if ( tagname4 == "FlowInfo" ) {
									theFlowInfo = smallsmallsmallchild.text();
									i++;
								}
								if ( (i == 7) && ( !theID.isEmpty() && !theGroup.isEmpty() && !theName.isEmpty() && !theServer.isEmpty() ) ) {
									// we process the information here
									if ( theGroup != "admin" ) {
										QRegExp rxp( "_\\([0-9]+\\)$", true, false );
										QString theConfName = theName;
										bool nb = theConfName.contains( rxp );
										if ( nb == true ) 
											theConfName = theConfName.remove(rxp);
										if ( thePath.isEmpty() )
											thePath = theConfName;
										ClientConf *cc = new ClientConf(g, theConfName, thePath);
										
										QStringList tmp_flowList = QStringList::split( QRegExp("%%"), theFlowInfo);
										for ( QStringList::Iterator it = tmp_flowList.begin(); it != tmp_flowList.end(); ++it ) {
											//cout << "tmp_flowList is not NULL" << endl;
											QString tmp_flowInfo = (*it);
											QStringList tmp_flowInfoList = QStringList::split( QRegExp("%"), tmp_flowInfo);
											int i = 0;
											int direction = -1;
											QString fname = "";
											QString ftype = "";
											for ( QStringList::Iterator it2 = tmp_flowInfoList.begin(); it2 != tmp_flowInfoList.end(); ++it2 ) {
												if ( i == 0 ) {
													i++;
													if ( (*it2) == "I" )
														direction = 0;
													else if ( (*it2) == "O" )
														direction = 1;
													else
														direction = -1;
												}
												else if ( i == 1) {
													i++;
													fname = (*it2);
												}
												else if ( i == 2 ) {
													i++;
													ftype = (*it2);
												}
											}
											//cout << "Flow found: " << "dir= " << direction << " " << "name= " << fname << " " << "type= " << ftype << endl;
											if ( direction == 0 ) {
												FlowOutletIn *fin = new FlowOutletIn(g, fname, ftype);
												if ( fin )
													cc->addInputFlow(fin);
											}
											else if ( direction == 1 ) {
												FlowOutletOut *fout = new FlowOutletOut(g, fname, ftype);
												if ( fout )
													cc->addOutputFlow(fout);
											}
										}
										
										ClientConf *res = g->getClientConfList()->clientExists(cc);
										if ( res == NULL ) {
											//cout << "The clientConf does NOT exist in the list, so we insert it in the list" << endl;  
											g->getClientConfList()->addClient(cc);
											clientConfList.insert(std::make_pair(theID, cc));
										}
										else {
											if ( cc )
												delete cc;
											clientConfList.insert(std::make_pair(theID, res));
											cc = res;
										}
										
										Server *s = g->getServerList()->getServer(theServer);
										if ( s ) {
											//cout << "we create a clientapp " << endl;
											ClientApp *capp = new ClientApp(cc, s, true);
											if ( capp ) {
												capp->setName(theName);
												capp->setParam(theParams);
												capp->setGroup(theGroup);
												capp->setRunning();
												//cout << "the client App ID=" << theID << endl;
												clientAppList.insert(std::make_pair(theID, capp));
											}
										}
										//else 
										//	cout << "server NULL" << endl;
									}
									
									theGroup = "";
									theID = "";
									theName = "";
									theDesc = "";
									theServer = "";
									thePath = "";
									theParams = "";
									theFlowInfo = "";
									i = 0;
								}		
								smallsmallsmallchild = smallsmallsmallchild.nextSibling().toElement();
							}
						}
						smallsmallchild = smallsmallchild.nextSibling().toElement();
					}
				}
				else if ( tagname2 == "FlowList" ) {
					QDomElement smallsmallchild = smallchild.firstChild().toElement();
					QString tagname3 = "";
					while ( !smallsmallchild.isNull() ) {
						tagname3 = smallsmallchild.tagName();
						QString theID = "";
						if ( tagname3 == "Flow" ) {
							theID = smallsmallchild.attribute("ID");
							QDomElement smallsmallsmallchild = smallsmallchild.firstChild().toElement();
							QString tagname4 = "";
							int i = 0;
							QString theGroup = "";
							QString theType = "";
							QString theName = "";
							QString theDesc = "";
							QString theProducer = "";
							std::map<QString, consumerFlow> *consumerNameList = new std::map<QString, consumerFlow>();
							consumerNameList->clear();
							while ( !smallsmallsmallchild.isNull() ) {
								tagname4 = smallsmallsmallchild.tagName();
								if ( tagname4 == "Group" ) {
									theGroup = smallsmallsmallchild.text();
									i++;
								}
								if ( tagname4 == "Type" ) {
									theType = smallsmallsmallchild.text();
									i++;
								}
								if ( tagname4 == "Name" ) {
									theName = smallsmallsmallchild.text();
									i++;
								}
								if ( tagname4 == "Description" ) {
									theDesc = smallsmallsmallchild.text();
									i++;
								}
								if ( tagname4 == "Producer" ) {
									theProducer = smallsmallsmallchild.attribute("RefID");
									i++;
								}
								if ( tagname4 == "ConsumerList" ) {
									i++;
									QDomElement foursmallchild = smallsmallsmallchild.firstChild().toElement();
									QString tagname5 = "";
									while ( !foursmallchild.isNull() ) {
										tagname5 = foursmallchild.tagName();
										if ( tagname5 == "Consumer" ) {
											consumerFlow tmp_consFlow;
											tmp_consFlow.givenFlowName = foursmallchild.attribute("GivenFlowName");
											tmp_consFlow.status = foursmallchild.attribute("Status");
											consumerNameList->insert(std::make_pair(foursmallchild.attribute("RefID"), tmp_consFlow));
										}
										foursmallchild = foursmallchild.nextSibling().toElement();
									}
								}
								if ( ( i == 6 ) && ( !theGroup.isEmpty() && !theType.isEmpty() && !theID.isEmpty() && !theName.isEmpty() ) ) {//producer || consumer list
									FlowInfos *flowInf = new FlowInfos(theGroup, theType, theName, theDesc, theProducer);
									flowInf->setConsumerList(consumerNameList);
									flowInfosList.insert(std::make_pair(theID,flowInf));
									
									theGroup = "";
									theID = "";
									theName = "";
									theDesc = "";
									theType = "";
									theProducer = "";
									i = 0;
								}
								smallsmallsmallchild = smallsmallsmallchild.nextSibling().toElement();
							}
						}
						smallsmallchild = smallsmallchild.nextSibling().toElement();
					}
					
				}
				smallchild = smallchild.nextSibling().toElement();
			}
		}// if ( tagname == "GlobalList" )
		child = child.nextSibling().toElement();
	} // while( !child.isNull() )
	  //time to process all the infos
	
	std::map<QString, FlowInfos *>::iterator iter_flow;
	/*cout << "Flow List:" << endl;
	for ( iter_flow = flowInfosList.begin(); iter_flow != flowInfosList.end(); iter_flow++ ) {
		FlowInfos *fi = (FlowInfos *) iter_flow->second;
		cout << "Flow ID: " << iter_flow->first << endl;
		cout << "   Name: " << fi->name << endl;
		cout << "   Type: " << fi->type << endl;
		cout << "   Group: " << fi->group << endl;
		cout << "   Producer: " << fi->producer << endl;
		cout << "   ConsumerList: " << endl;
		fi->initIterator();
		QString consName = "";
		QString flowName = "";
		fi->getNextConsumer(consName, flowName);
		while ( consName != "" ) {
			cout << "     " << consName << "  " << flowName << endl;
			fi->getNextConsumer(consName, flowName);
		}
    }*/
	
	
	std::map<QString, ClientApp *>::iterator iter_clientAppList;
	for ( iter_clientAppList = clientAppList.begin(); iter_clientAppList != clientAppList.end(); iter_clientAppList++ ) {
		ClientApp *ca =  (ClientApp *) iter_clientAppList->second;
		if ( ca ) {
			
			std::map<unsigned int, FlowOutletIn*>* foin_tmp = ca->getInputFlowList();
			std::map<unsigned int, FlowOutletOut*>* foout_tmp = ca->getOutputFlowList(); 
			std::map<unsigned int, FlowOutletIn*>::iterator iter_in;
			std::map<unsigned int, FlowOutletOut*>::iterator iter_out;
			for ( iter_in = foin_tmp->begin(); iter_in != foin_tmp->end(); iter_in++) {
				addOutlet( iter_in->second );
			}
			for ( iter_out = foout_tmp->begin(); iter_out != foout_tmp->end(); iter_out++) {
				addOutlet( iter_out->second );
			}
			addClient(ca);
			
			ca->getClientGraphic()->show();
			
		}
	}
	
	for ( iter_flow = flowInfosList.begin(); iter_flow != flowInfosList.end(); iter_flow++ ) {
		FlowInfos *fi = (FlowInfos *) iter_flow->second;
		iter_clientAppList = clientAppList.find(fi->producer);
		if ( iter_clientAppList != clientAppList.end() ) {
			// cout << "THE PRODUCER=" << iter_clientAppList->second->getName() << endl;
			//cout << "Producer found" << endl;
			ClientApp *cl_app_tmp = iter_clientAppList->second;
			std::map<unsigned int, FlowOutletOut*>* foout_tmp = cl_app_tmp->getOutputFlowList(); 
			std::map<unsigned int, FlowOutletOut*>::iterator iter_out;
			iter_out = foout_tmp->begin();
			bool found = false;
			while ( iter_out != foout_tmp->end() && !found) {
				FlowOutletOut *foout = iter_out->second;
				if ( foout->getName() == fi->name ) {
					//cout << "flow Outlet out founded" << endl;
					found = true;
					foout->setGroup(fi->group);
					foout->setRunning();
				} 
				else
					iter_out++;
			}
			if ( !found ) {
				//the outlet out is not in the static list of flow outlets. We create it anyway
				ClientConf *tmp_clientconf = g->getClientConfList()->getClientWithPath(cl_app_tmp->getPath());
				if ( tmp_clientconf ) {
					// cout << "in the produccer loop" << endl;
					//cout << "ClientConf Name=" << tmp_clientconf->getName() << endl;
					//cout << "ClientConf Path=" << tmp_clientconf->getPath() << endl;
					//cout << "ClientApp Path=" << cl_app_tmp->getPath() << endl;
					tmp_clientconf->addOutputFlow(new FlowOutletOut(g, fi->name, fi->type));
					FlowOutletOut *tmp_foout_app = new FlowOutletOut(g, fi->name, fi->type);
					tmp_foout_app->setGroup(fi->group);
					tmp_foout_app->setRunning();
					addOutlet(tmp_foout_app);
					cl_app_tmp->addOutputFlow(tmp_foout_app);
					MapView *mv = getMap();
					if ( mv ) {
						mv->addOutlet(tmp_foout_app->getId(), tmp_foout_app->getFlowOutletGraphic());   
						tmp_foout_app->getFlowOutletGraphic()->show();
					}
				}
			}
		}
		fi->initIterator();
		QString consName = "";
		QString flowName = "";
		QString flowStatus = "";
		fi->getNextConsumer(consName, flowName, flowStatus);
		while ( consName != "" ) {
			iter_clientAppList = clientAppList.find(consName);
			//cout << "THE CONSUMER=" << iter_clientAppList->second->getName() << endl;
			if ( iter_clientAppList != clientAppList.end() ) {
				ClientApp *cl_app_tmp = iter_clientAppList->second;
				std::map<unsigned int, FlowOutletIn*>* foin_tmp = cl_app_tmp->getInputFlowList(); 
				std::map<unsigned int, FlowOutletIn*>::iterator iter_in;
				iter_in = foin_tmp->begin();
				bool found = false;
				while ( iter_in != foin_tmp->end() && !found) {
					FlowOutletIn *foin = iter_in->second;
					//  cout << "IOP flow Outlet in founded foin->getName()=" << foin->getName() << " flowName=" << flowName <<endl;
					if ( foin->getName() == flowName ) {
						//cout << "flow Outlet in founded foin->getName()=" << foin->getName() << " flowName=" << flowName <<endl;
						found = true;
						foin->setRunning();
						foin->setGroup(fi->group);
						if ( flowStatus == "3" ) {
							foin->setPaused(true);
						}
					}
					else
						iter_in++;
				}
				if ( !found ) {
					//the outlet in is not in the static list of flow outlets. We create it anyway
					if ( iter_clientAppList != clientAppList.end() ) {
						ClientConf *tmp_clientconf = g->getClientConfList()->getClientWithPath(cl_app_tmp->getPath());
						if ( tmp_clientconf ) {
							//cout << "in the consumer loop" << endl;
							//cout << "ClientConf Name=" << tmp_clientconf->getName() << endl;
							//cout << "ClientConf Path=" << tmp_clientconf->getPath() << endl;
							//cout << "ClientApp Name=" << cl_app_tmp->getName() << endl;
							tmp_clientconf->addInputFlow(new FlowOutletIn(g, flowName, fi->type));
							FlowOutletIn *tmp_foin_app = new FlowOutletIn(g, flowName, fi->type);
							tmp_foin_app->setGroup(fi->group);
							tmp_foin_app->setRunning();
							if ( flowStatus == "3" ) {
								tmp_foin_app->setPaused(true);
							}
							addOutlet(tmp_foin_app);
							cl_app_tmp->addInputFlow(tmp_foin_app);
							MapView *mv = getMap();
							if ( mv ) {
								mv->addOutlet(tmp_foin_app->getId(), tmp_foin_app->getFlowOutletGraphic()); 
								tmp_foin_app->getFlowOutletGraphic()->show();
							}  
						} 
					}
				}
				//cout << "Consumer found" << endl;
			}
			fi->getNextConsumer(consName, flowName, flowStatus);
		}
	}
	getMap()->setClientsPosition(); 
	std::map<unsigned int, FlowOutlet *>::iterator iter_flowOutletOut;
	std::map<unsigned int, FlowOutlet *>::iterator iter_flowOutletIn;
	GlobalCC::setInitialGroupId(getMaxGroupId()+1);
	for ( iter_flowOutletOut = flowOutlets.begin(); iter_flowOutletOut != flowOutlets.end(); iter_flowOutletOut++) {
		if ( iter_flowOutletOut->second->getType() == FlowOutlet::FLOW_OUT ) {
			QString groupOut = iter_flowOutletOut->second->getGroup();
			if ( groupOut.isEmpty() )
				iter_flowOutletOut->second->setGroup(GlobalCC::getNewGroup());
			for ( iter_flowOutletIn = flowOutlets.begin(); iter_flowOutletIn != flowOutlets.end(); iter_flowOutletIn++) {
				if ( iter_flowOutletIn->second->getType() == FlowOutlet::FLOW_IN ) {
					if ( iter_flowOutletIn->second->getGroup().isEmpty() )
						iter_flowOutletIn->second->setGroup(GlobalCC::getNewGroup());
					if ( iter_flowOutletIn->second->getGroup() == groupOut ) {
						if ( !iter_flowOutletOut->second->hasFlow() ) {
							//we connect the two outlets
							iter_flowOutletOut->second->connect(iter_flowOutletIn->second, true);
						}
						else {
							iter_flowOutletIn->second->connect(iter_flowOutletOut->second->getFlow(), true);
							//we connect the outlet to the flow 
						}
					}
				}
			}
		}
	}
	
	getMap()->setClientsPosition(); 
	getMap()->update();
	//cout << " end readApplicationDescription" << endl;
	return true;
}

std::map<unsigned int, ClientApp *>* ApplicationCC::getClientList() {
    return &clients;
}

std::map<unsigned int, FlowOutlet *>* ApplicationCC::getFlowList() {
    return &flowOutlets;
}

int ApplicationCC::getMaxGroupId() {
    int max = -1;
    std::map<unsigned int, ClientApp *>::iterator iter;
    for ( iter = clients.begin(); iter != clients.end(); iter++ )	{
		max = getMax(max, iter->second->getGroup());
		std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
		std::map<unsigned int, FlowOutletIn *> *fin_tmp = iter->second->getInputFlowList();
		for( iter_in = fin_tmp->begin(); iter_in != fin_tmp->end(); iter_in++ ) {
			max = getMax(max, iter_in->second->getGroup());	
		}
		std::map<unsigned int, FlowOutletOut *>::iterator iter_out;  
		std::map<unsigned int, FlowOutletOut *> *fout_tmp = iter->second->getOutputFlowList();
		for( iter_out = fout_tmp->begin(); iter_out != fout_tmp->end(); iter_out++ ) {
			max = getMax(max, iter_out->second->getGroup());
		}
    }
    return max;
}

bool ApplicationCC::readXmlApplication(QString &filename) {
	if ( filename.isEmpty() ) {
		return false;
	}	
	QFile f(filename);
	QDomDocument myDoc;
	if ( !f.open( QIODevice::ReadOnly ) ) { 
		return false;
	}	
	if ( !myDoc.setContent( &f ) ) {
		f.close(); 	
		return false;
	}
	f.close();
	QString type = myDoc.doctype().name();
	if ( type != "appliCC" ) {
		return false;
	}   
	else {
		QString tagname = "";
		QDomElement root = myDoc.documentElement(); 
		QDomElement child = root.firstChild().toElement();
		while( !child.isNull() ) {
			tagname = child.tagName();
			if ( tagname == "clients" ) {
				QDomElement smallchild = child.firstChild().toElement();
				QString tagname2 = "";
				while( !smallchild.isNull() ) {
					tagname2 = smallchild.tagName();
					if ( tagname2 == "client" ) {
						QDomElement smallsmallchild = smallchild.firstChild().toElement();
						QString tagname3 = "";
						QString theName = "";
						QString thePath = "";
						QString theGroup = "";
						QString isEnable = "";
						QString posX = "";
						QString posY = "";
						QString theParams = "";
						std::set<FlowOutletIn *> foin;
						foin.clear();
						std::set<FlowOutletOut *> foout;
						foout.clear();
						Server *s = NULL;
						QString theServerHost = "";
						int i = 0;
						while( !smallsmallchild.isNull() ) {
							tagname3 = smallsmallchild.tagName();
							if ( tagname3 == "name" ) {
								theName = smallsmallchild.text();
								i++;
							}
							if ( tagname3 == "path" ) {
								thePath = smallsmallchild.text();
								i++;
							}
							if ( tagname3 == "group" ) {
								theGroup = smallsmallchild.text();
								i++;
							}
							if ( tagname3 == "enable" ) {
								isEnable = smallsmallchild.text();
								i++;
							}
							if ( tagname3 == "server" ) {
								QDomElement smallsmallsmallchild = smallsmallchild.firstChild().toElement();
								QString tagname4 = "";
								int j = 0;
								while( !smallsmallsmallchild.isNull() ) {
									tagname4 = smallsmallsmallchild.tagName();
									if ( tagname4 == "host" ) {
										theServerHost = smallsmallsmallchild.text();
										j++;
									}
									if ( !theServerHost.isEmpty() && ( j == 1) ) {
										//we create a server if needed
										int res = g->getServerList()->serverPathExist(theServerHost);
										if ( res == -1 ) {
											//the server doesn't exist, so we link the client to the Server "No Server" and we keep the old host
											//TODO!
											s = g->getServerList()->getServer("No Server");
											//if ( s != NULL )
											//	cout << "server don't exist" << endl;
										}
										else {
											//the server already exist, we put the address of the server in "s"
											s = g->getServerList()->getServer(res);
										}
									}
									smallsmallsmallchild = smallsmallsmallchild.nextSibling().toElement();
								}
								i++;
							}
							if ( tagname3 == "position" ) {
								QDomAttr thePosX = smallsmallchild.attributeNode ( "x" );
								QDomAttr thePosY = smallsmallchild.attributeNode ( "y" );
								if ( !thePosX.isNull() ) 
									posX = thePosX.value();
								if ( !thePosY.isNull() )
									posY = thePosY.value();
								i++;
							}
							if ( tagname3 == "param" ) {
								theParams = smallsmallchild.text();
								i++;
							}
							if ( tagname3 == "flows" ) {
								QDomElement smallsmallsmallchild = smallsmallchild.firstChild().toElement();
								QString tagname4 = "";
								while( !smallsmallsmallchild.isNull() ) {
									tagname4 = smallsmallsmallchild.tagName();
									if ( tagname4 == "input" ) {
										QDomElement foursmallchild = smallsmallsmallchild.firstChild().toElement();
										QString tagname5 = "";
										int j = 0;
										QString flowName = "";
										QString flowType = "";
										QString flowGroup = "";
										while( !foursmallchild.isNull() ) {
											tagname5 = foursmallchild.tagName();
											if ( tagname5 == "name" ) {
												flowName = foursmallchild.text();
												j++;
											}
											if ( tagname5 == "type" ) {
												flowType = foursmallchild.text();
												j++;
											}
											if ( tagname5 == "group" ) {
												flowGroup = foursmallchild.text();
												j++;
											}
											if ( !flowName.isEmpty() && !flowType.isEmpty() && !flowGroup.isEmpty() && (j == 3) ) {
												j = 0;
												FlowOutletIn *f = new FlowOutletIn(g, flowName, flowType);
												
												if ( f ) {
													f->setGroup(flowGroup);
													foin.insert(f);
												}
												flowName = "";
												flowType = "";
												flowGroup = "";
											}
											foursmallchild = foursmallchild.nextSibling().toElement();
										}
									}
									if ( tagname4 == "output" ) {
										QDomElement foursmallchild = smallsmallsmallchild.firstChild().toElement();
										QString tagname5 = "";
										int j = 0;
										QString flowName = "";
										QString flowType = "";
										QString flowGroup = "";
										while( !foursmallchild.isNull() ) {
											tagname5 = foursmallchild.tagName();
											if ( tagname5 == "name" ) {
												flowName = foursmallchild.text();
												j++;
											}
											if ( tagname5 == "type" ) {
												flowType = foursmallchild.text();
												j++;
											}
											if ( tagname5 == "group" ) {
												flowGroup = foursmallchild.text();
												j++;
											}
											if ( !flowName.isEmpty() && !flowType.isEmpty() && (j == 3) ) {
												FlowOutletOut *f = new FlowOutletOut(g, flowName, flowType);
												j = 0;
												if ( f ) {
													f->setGroup(flowGroup);
													foout.insert(f);
												}
												flowName = "";
												flowType = "";
												flowGroup = "";
											}
											foursmallchild = foursmallchild.nextSibling().toElement();
										}
									}
									smallsmallsmallchild = smallsmallsmallchild.nextSibling().toElement(); 
								}
								i++;
							} // if ( tagname3 == flow )
							
							if ( !theName.isEmpty() && !thePath.isEmpty() && !theGroup.isEmpty() && !isEnable.isEmpty() && !posX.isEmpty() && !posY.isEmpty() && !posY.isEmpty() && s  && (i==8) ) {
								ClientConf *cc = NULL;
								cc = new ClientConf(g, theName, thePath);
								if ( cc ) {
									std::set<FlowOutletIn *>::iterator iter_in;
									for ( iter_in = foin.begin(); iter_in != foin.end(); iter_in++ ) {
										cc->addInputFlow((*iter_in));
									}
									std::set<FlowOutletOut *>::iterator iter_out;
									for ( iter_out = foout.begin(); iter_out != foout.end(); iter_out++ ) {
										cc->addOutputFlow((*iter_out));
									}
									foin.clear();
									foout.clear();
								}
								bool res = g->getClientConfList()->clientPathExists(thePath);
								if ( !res ) {
									g->getClientConfList()->addClient(cc);
								}
								if ( s && cc ) {
									ClientApp *ca = new ClientApp(cc, s, true);
									if ( ca ) {
										ca->setParam(theParams);
										ca->setGroup(theGroup);
										if ( isEnable == "false")
											ca->setDisable();
										ca->getClientGraphic()->hide();
										ca->getClientGraphic()->setPos(posX.toUInt(), posY.toUInt());
										ca->getClientGraphic()->move();
										ca->getClientGraphic()->show();
										addClient(ca);
										if ( s->getAddress() == "No Server" && theServerHost != "No Server" ) {
											ca->setOldServerName(theServerHost);
											ca->getClientGraphic()->refreshLabel();
										} 
										std::map<unsigned int, FlowOutletIn*>* foin_tmp = ca->getInputFlowList();
										std::map<unsigned int, FlowOutletOut*>* foout_tmp = ca->getOutputFlowList(); 
										std::map<unsigned int, FlowOutletIn*>::iterator iter_in;
										std::map<unsigned int, FlowOutletOut*>::iterator iter_out;
										for ( iter_in = foin_tmp->begin(); iter_in != foin_tmp->end(); iter_in++) {
											addOutlet( iter_in->second );
										}
										for ( iter_out = foout_tmp->begin(); iter_out != foout_tmp->end(); iter_out++) {
											addOutlet( iter_out->second );
										}
										MainWindow::instance()->appliNeedSave = true;
										ca->getClientGraphic()->updateClient();
									} //if ( ca )
								} //if ( s && cc )
								i = 0;
							}
							
							smallsmallchild = smallsmallchild.nextSibling().toElement();
						}
					}
					smallchild = smallchild.nextSibling().toElement();
				}
			}
			child = child.nextSibling().toElement();
		}
	}
	//there is an error before, when we read it. writing is fine!
	std::map<unsigned int, FlowOutlet *>::iterator iter_flowOutletOut;
	std::map<unsigned int, FlowOutlet *>::iterator iter_flowOutletIn;
	for ( iter_flowOutletOut = flowOutlets.begin(); iter_flowOutletOut != flowOutlets.end(); iter_flowOutletOut++) {
		if ( iter_flowOutletOut->second->getType() == FlowOutlet::FLOW_OUT ) {
			QString groupOut = iter_flowOutletOut->second->getGroup();
			for ( iter_flowOutletIn = flowOutlets.begin(); iter_flowOutletIn != flowOutlets.end(); iter_flowOutletIn++) {
				if ( iter_flowOutletIn->second->getType() == FlowOutlet::FLOW_IN ) {
					if ( iter_flowOutletIn->second->getGroup() == groupOut ) {
						if ( !iter_flowOutletOut->second->hasFlow() ) {
							//we connect the two outlets
							iter_flowOutletOut->second->connect(iter_flowOutletIn->second, false);
						}
						else {
							iter_flowOutletIn->second->connect(iter_flowOutletOut->second->getFlow(), false);
							//we connect the outlet to the flow 
						}
					}
				}
			}
		}
	}
	getMap()->update();
	return true;
}

//new
bool ApplicationCC::writeXmlApplication(QString &filename) {
	QFile file(filename);
	if ( !file.open( QIODevice::WriteOnly ) ) {
		return false;
	}
	else {
		XmlWriter *xw;
		xw = new  XmlWriter(&file);
		xw->setAutoNewLine(true);
		xw->writeRaw("<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>");
		xw->newLine();
		xw->writeRaw("<!DOCTYPE appliCC >");
		xw->newLine();
		xw->writeRaw("<!--This file describe an application of the NSDFS Control Center-->");
		xw->newLine();
		xw->writeOpenTag("application");
		xw->writeOpenTag("clients");
		std::map<unsigned int, ClientApp *>::iterator iter;
		for ( iter = clients.begin(); iter != clients.end(); iter++ )
			writeXmlClient(xw, iter->second);
		xw->writeCloseTag("clients");
		xw->writeCloseTag("application");
	}
	return true;
}

void ApplicationCC::writeXmlClient(XmlWriter *xw, ClientApp *c) {
	if ( c ) {
		xw->writeOpenTag("client");
		xw->writeTaggedString("name", c->getName());
		xw->writeTaggedString("path", c->getPath());
		xw->writeTaggedString("group", c->getGroup());
		QString tmp_enable = "true";
		if  ( c->isDisable() )
			tmp_enable = "false";
		xw->writeTaggedString("enable", tmp_enable);
		//we need to give a server
		if ( c->getOldServerName().isEmpty() )
			writeXmlServer(xw, c->getServer()->getAddress());
		else
			writeXmlServer(xw, c->getOldServerName());
		//we need to set up the position
		AttrMap attrs;
		QPoint p = c->getClientGraphic()->pos();
		attrs.insert("x", QString::number(p.x()/10));
		attrs.insert("y", QString::number(p.y()/10));
		xw->writeAtomTag("position", attrs);
		
		xw->writeTaggedString("param", c->getParam());
		
		xw->writeOpenTag("flows");
		
		std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
		std::map<unsigned int, FlowOutletIn *> *fin_tmp = c->getInputFlowList();
		for( iter_in = fin_tmp->begin(); iter_in != fin_tmp->end(); iter_in++ ) {
			FlowOutletIn *tmp_in = (iter_in->second);
			writeXmlFlow(xw, tmp_in);
		}
		
		std::map<unsigned int, FlowOutletOut *>::iterator iter_out;  
		std::map<unsigned int, FlowOutletOut *> *fout_tmp = c->getOutputFlowList();
		for( iter_out = fout_tmp->begin(); iter_out != fout_tmp->end(); iter_out++ ) {
			FlowOutletOut *tmp_out = (iter_out->second);
			writeXmlFlow(xw, tmp_out);
		}
		
		xw->writeCloseTag("flows");
		xw->writeCloseTag("client");
	}
}

void ApplicationCC::writeXmlFlow(XmlWriter *xw, FlowOutlet *f) {
	if ( f ) {
		if ( f->getType() == FlowOutlet::FLOW_IN )
			xw->writeOpenTag("input");
		else
			xw->writeOpenTag("output");
		
		xw->writeTaggedString("name", f->getName());
		xw->writeTaggedString("type", f->getDataType());
		xw->writeTaggedString("group", f->getGroup());
		
		if ( f->getType() == FlowOutlet::FLOW_IN )
			xw->writeCloseTag("input");
		else 	
			xw->writeCloseTag("output"); 
	}
}

void ApplicationCC::writeXmlServer(XmlWriter *xw, QString& serverName) {
    xw->writeOpenTag("server");
    xw->writeTaggedString("host", serverName);
    xw->writeCloseTag("server");
}

void ApplicationCC::refreshAllViews() {
    MainWindow::instance()->appMon->refreshAllView();
}

unsigned int ApplicationCC::numberOfClients() {
	return clients.size();
}

bool ApplicationCC::parseLastModificationsInApplication(QString &modif) {
	modif.replace ("<?xml version=\'1.0\'?>", "");
	QDomDocument myDoc;
	//cout << endl << endl << modif.toStdString()  << endl << endl;
	if ( !myDoc.setContent(modif) ) {
		cout << "BEGIN document non valide" << endl << modif.toStdString() <<  endl << "END" << endl;
		return false;
	}
	
	//time to parse the document
	//cout << modif << endl;
	QString tagname = "";
	QDomElement root = myDoc.documentElement(); 
	QDomElement child = root.firstChild().toElement();
	while( !child.isNull() ) {
		tagname = child.tagName();
		if ( tagname == "ModificationsList" ) {
			QDomElement smallchild = child.firstChild().toElement();
			QString tagname2 = "";
			while ( !smallchild.isNull() ) {
				tagname2 = smallchild.tagName();
				//cout << "This is the new tagname" << tagname2 << endl << endl;;
				if ( tagname2 == "modif" ) {
					QDomElement smallsmallchild = smallchild.firstChild().toElement();
					QString tagname3 = "";
					while ( !smallsmallchild.isNull() ) {
						tagname3 = smallsmallchild.tagName();
						if ( tagname3 == "smartspace" ) {
							QDomElement smallsmallsmallchild = smallsmallchild.firstChild().toElement();
							QString tagname4 = "";
							QString theType = "";
							QString theClientName = "";
							QString theClientGroup = "";
							QString theFlowGroup = "";
							QString theFlowProvider = "";
							QString theFlowType = "";
							QString theServerName = "";
							QString theTarget = "";
							QString theClientID = "";
							QString theFlowID = "";
							QString theServerOS = "";
							QString theClientServer = "";
							std::vector<QString> logMessages;
							logMessages.clear();
							int i = 0;
							int j = 0;
							int k = 0;
							int l = 0;
							int m = 0;
							int n = 0;
							int o = 0;
							int p = 0;
							int q = 0;
							int r = 0;
							int s = 0;
							while ( !smallsmallsmallchild.isNull() ) {
								tagname4 = smallsmallsmallchild.tagName();
								if ( tagname4 == "type" ) {
									theType = smallsmallsmallchild.text();
									i++;
									j++;
									k++;
									l++;
									m++;
									n++;
									o++;
									p++;
									q++;
									r++;
									s++;
								}
								if ( tagname4 == "target" ) {
									theTarget = smallsmallsmallchild.text();
									j++;
									k++;
									l++;
									m++;
									n++;
									o++;
									p++;
									q++;
									r++;
								}
								if ( tagname4 == "clientgroup" ) {
									theClientGroup = smallsmallsmallchild.text();
									i++;
									l++;
									m++;
									o++;
									s++;
								}
								if ( tagname4 == "clientname" ) {
									theClientName = smallsmallsmallchild.text();
									i++;
									l++;
									m++;
									o++;
									s++;
								}
								if ( tagname4 == "group" ) {
									theFlowGroup = smallsmallsmallchild.text();
									j++;
									r++;
								}
								if ( tagname4 == "flowgroup" ) {
									theFlowGroup = smallsmallsmallchild.text();
									m++;
									o++;
								}
								if ( tagname4 == "flowtype" ) {
									theFlowType = smallsmallsmallchild.text();
									m++;
									o++;
								}
								if ( tagname4 == "typeflow" ) {
									theFlowType = smallsmallsmallchild.text();
									j++;
									r++;
								}
								if ( tagname4 == "provider" ) {
									theFlowProvider = smallsmallsmallchild.text();
									j++;
									r++;
								}
								if ( tagname4 == "clientserver" ) {
								        theClientServer = smallsmallsmallchild.text();
									i++;
								}
								if ( tagname4 == "servername" ) {
									theServerName = smallsmallsmallchild.text();
									k++;
									n++;
									p++;
									q++;
								}
								if ( tagname4 == "clientid" ) {
									theClientID = smallsmallsmallchild.text();
									r++;
								}
								if ( tagname4 == "flowID" ) {
									theFlowID = smallsmallsmallchild.text();
									q++;
								}
								if ( tagname4 == "serveros" ) {
									theServerOS = smallsmallsmallchild.text();
									k++;
								}
								if ( tagname4 == "messages" ) {
									QDomElement smallsmallsmallsmallchild = smallsmallsmallchild.firstChild().toElement();
									QString tagname5 = "";
									while ( !smallsmallsmallsmallchild.isNull() ) {
										tagname5 = smallsmallsmallsmallchild.tagName();
										if ( tagname5 == "message" ) {
											logMessages.push_back(smallsmallsmallsmallchild.text());
										}
										smallsmallsmallsmallchild = smallsmallsmallsmallchild.nextSibling().toElement();
									}
									o++;
									p++;
									q++;
								}
								smallsmallsmallchild = smallsmallsmallchild.nextSibling().toElement();
							} // while ( !smallsmallsmallchild.isNull() ) 
							if ( !theClientName.isEmpty() && !theClientGroup.isEmpty() && !theType.isEmpty() && !theTarget.isEmpty() && !theFlowGroup.isEmpty() && !theFlowType.isEmpty() && ( m==6 ) ) {
								//cout << "CHECK1" << endl;
								if ( theType == "5" ) {
									if ( theTarget == "21" ) {
										//we pause the flow
										ClientApp * ca = getClient(theClientGroup, theClientName);
										if ( ca != NULL ) {
											std::map<unsigned int, FlowOutletIn *>::iterator iter_in;  
											std::map<unsigned int, FlowOutletIn *> *fin_tmp = ca->getInputFlowList();
											for( iter_in = fin_tmp->begin(); iter_in != fin_tmp->end(); iter_in++ ) {
												if ( ( iter_in->second->getGroup() == theFlowGroup ) &&  ( iter_in->second->getDataType() == theFlowType ) ) {
													iter_in->second->setPaused(true);
													if ( iter_in->second->hasFlow() ) 
														iter_in->second->getFlow()->refresh();
													QString messsage4Log = "Flow \"";
													messsage4Log.append(iter_in->second->getName());
													messsage4Log.append("\" is paused");
													ca->addLogLine(messsage4Log);
												}
											}
										}
									}
									else if ( theTarget == "22" ) {
										//we unpause the flow
										ClientApp * ca = getClient(theClientGroup, theClientName);
										if ( ca != NULL ) {
											std::map<unsigned int, FlowOutletIn *>::iterator iter_in;  
											std::map<unsigned int, FlowOutletIn *> *fin_tmp = ca->getInputFlowList();
											for( iter_in = fin_tmp->begin(); iter_in != fin_tmp->end(); iter_in++ ) {
												if ( ( iter_in->second->getGroup() == theFlowGroup ) &&  ( iter_in->second->getDataType() == theFlowType ) ) {
													iter_in->second->setPaused(false);
													if ( iter_in->second->hasFlow() ) 
														iter_in->second->getFlow()->refresh();
													QString messsage4Log = "Flow \"";
													messsage4Log.append(iter_in->second->getName());
													messsage4Log.append("\" is resumed");
													ca->addLogLine(messsage4Log);
												}
											}
										}
									}
								}
								else if ( theType == "1" && theTarget == "12" ) {
									//a flow from a client has been removed
									ClientApp * ca = getClient(theClientGroup, theClientName);
									if ( ca != NULL ) {
										std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
										std::map<unsigned int, FlowOutletIn *> *fin_tmp = ca->getInputFlowList();
										iter_in = fin_tmp->begin();
										bool found = false;
										while( iter_in != fin_tmp->end() && found == false ) {
											if ( ( iter_in->second->getGroup() == theFlowGroup ) &&  ( iter_in->second->getDataType() == theFlowType ) ) {
												found = true;
												iter_in->second->setIdle();
												if ( iter_in->second->hasFlow() ) 
													iter_in->second->getFlow()->refresh();
											}
											iter_in++;
										}
										if ( found == false ) {
											std::map<unsigned int, FlowOutletOut *>::iterator iter_out; 
											std::map<unsigned int, FlowOutletOut *> *fout_tmp = ca->getOutputFlowList();
											iter_out = fout_tmp->begin();
											while( iter_out != fout_tmp->end() && found == false ) {
												if ( ( iter_out->second->getGroup() == theFlowGroup ) &&  ( iter_out->second->getDataType() == theFlowType ) ) {
													found = true;
													iter_out->second->setIdle();
													if ( iter_out->second->hasFlow() ) 
														iter_out->second->getFlow()->refresh();
												}
												iter_out++;
											}
										}	
									}
								}
								theType = "";
								theClientName = "";
								theClientGroup = "";
								theFlowGroup = "";
								theFlowProvider = "";
								theFlowType = "";
								theServerName = "";
								theTarget = "";
								theClientID = "";
								theFlowID = "";
								theServerOS = "";
								theClientServer = "";
								logMessages.clear();
								i = 0;
								j = 0;
								k = 0;
								l = 0;
								m = 0;
								n = 0;
								o = 0;
								p = 0;
								q = 0;
								r = 0;
								s = 0;
								
							}
							if ( !theClientName.isEmpty() && !theClientGroup.isEmpty() && !theType.isEmpty() && !theClientServer.isEmpty() && (i == 4) && theClientGroup != "admin" && theType == "3" && theTarget.isEmpty() ) {
								//cout << "CHECK2" << endl;
								//cout << "Name:" << theClientName << "Group:" << theClientGroup << "Type:" << theType  << endl;
							        //a client is connected
							        //at this point we assume the client exists in the map
								ClientApp * ca = getClient(theClientGroup, theClientName);
								if ( ca != NULL  ) {
								      //cout << "client found :connect " << ca->getName() << " Group=" << ca->getGroup() << endl;
								      if ( ca->getServer()->getName() != theClientServer ) {
									   Server *s = g->getServerList()->getServerWithPath(theClientServer);  
									   if ( s != NULL ) {
									     ca->setServer(s);
									     ca->setRunning();
									     ca->addLogLine("Client started");
									   }
								      }
								      else {
									  ca->setRunning();
									  ca->addLogLine("Client started");
								      }
								}
								theType = "";
								theClientName = "";
								theClientGroup = "";
								theFlowGroup = "";
								theFlowProvider = "";
								theFlowType = "";
								theServerName = "";
								theTarget = "";
								theClientID = "";
								theFlowID = "";
								theServerOS = "";
								theClientServer = "";
								logMessages.clear();
								i = 0;
								j = 0;
								k = 0;
								l = 0;
								m = 0;
								n = 0;
								o = 0;
								p = 0;
								q = 0;
								r = 0;
								s = 0;
							}
							if ( !theClientName.isEmpty() && !theClientGroup.isEmpty() && !theType.isEmpty() && (s == 3) && theClientGroup != "admin" && theType == "4" && theTarget.isEmpty() ) {
								//cout << "CHECK2B" << endl;
								//cout << "Name:" << theClientName << "Group:" << theClientGroup << "Type:" << theType  << endl;
							        //a client is disconnected
								//at this point we assume the client exists in the map
								ClientApp * ca = getClient(theClientGroup, theClientName);
								if ( ca != NULL ) {
								       	//cout << "client found :disconnect " << ca->getName() << " Group=" << ca->getGroup()  << endl;
									ca->setIdle();
									ca->addLogLine("Client stopped");
									//we need to set the flow outlet flow to setRunning(false)
									std::map<unsigned int, FlowOutletOut *>::iterator iter_out;  
									std::map<unsigned int, FlowOutletOut *> *fout_tmp = ca->getOutputFlowList();
									for( iter_out = fout_tmp->begin(); iter_out != fout_tmp->end(); iter_out++ ) {
									   	iter_out->second->setIdle();
									}
								}
								theType = "";
								theClientName = "";
								theClientGroup = "";
								theFlowGroup = "";
								theFlowProvider = "";
								theFlowType = "";
								theServerName = "";
								theTarget = "";
								theClientID = "";
								theFlowID = "";
								theServerOS = "";
								theClientServer = "";
								logMessages.clear();
								i = 0;
								j = 0;
								k = 0;
								l = 0;
								m = 0;
								n = 0;
								o = 0;
								p = 0;
								q = 0;
								r = 0;
								s = 0;
							}
							if ( !theType.isEmpty() && !theClientName.isEmpty() && !theClientGroup.isEmpty() && !theTarget.isEmpty() && theType == "5" && theTarget == "28" && l == 4 ) {
								//cout << "CHECK3" << endl;
								//cout << "PB Lancement client" << endl;
								ClientApp *ca = getClient(theClientGroup, theClientName);
								//cout << "PB Lancement client" << endl;
								if ( ca != NULL ) {
									ca->setError();
									ca->addLogLine("Client executable not found");
								}
								theType = "";
								theClientName = "";
								theClientGroup = "";
								theFlowGroup = "";
								theFlowProvider = "";
								theFlowType = "";
								theServerName = "";
								theTarget = "";
								theClientID = "";
								theFlowID = "";
								theServerOS = "";
								theClientServer = "";
								logMessages.clear();
								i = 0;
								j = 0;
								k = 0;
								l = 0;
								m = 0;
								n = 0;
								o = 0;
								p = 0;
								q = 0;
								r = 0;
								s = 0;
							}
							if ( !theType.isEmpty() && !theServerName.isEmpty() && !theServerOS.isEmpty() &&  theType == "3" && k == 4 ) {
								//cout << "CHECK4" << endl;
								//a new server is connected
								//we check the server is not already present in the ServerList
								Server *s = NULL;
								s = g->getServerList()->getServerWithPath(theServerName);
								if ( s == NULL ) {
									//temporary change!!!!
									int tmp_port = 1714;
									s = new Server(g, theServerName, theServerName, tmp_port, theServerOS);
									g->getServerList()->addServer(s);
									MainWindow::instance()->refreshConfigurationViews();
									checkIfClientsHaveServerInApplication();
								}	   
								theType = "";
								theClientName = "";
								theClientGroup = "";
								theFlowGroup = "";
								theFlowProvider = "";
								theFlowType = "";
								theServerName = "";
								theTarget = "";
								theClientID = "";
								theFlowID = "";
								theServerOS = "";
								theClientServer = "";
								logMessages.clear();
								i = 0;
								j = 0;
								k = 0;
								l = 0;
								m = 0;
								n = 0;
								o = 0;
								p = 0;
								q = 0;
								r = 0;
								s = 0;
							}
							if ( !theType.isEmpty() && !theServerName.isEmpty() && theType == "4" && n == 3 ) {
								//cout << "CHECK5" << endl;
								//a server is disconnected
								//we have to remove the server and refresh all the views and the app
								Server *s = g->getServerList()->getServerWithPath(theServerName);
								if ( s ) {
									checkIfClientsUseServer(s);
									g->getServerList()->removeServer(s);
									MainWindow::instance()->refreshConfigurationViews();
									checkIfClientsHaveServerInApplication();
								} 
								
								theType = "";
								theClientName = "";
								theClientGroup = "";
								theFlowGroup = "";
								theFlowProvider = "";
								theFlowType = "";
								theServerName = "";
								theTarget = "";
								theClientID = "";
								theFlowID = "";
								theServerOS = "";
								theClientServer = "";
								logMessages.clear();
								i = 0;
								j = 0;
								k = 0;
								l = 0;
								m = 0;
								n = 0;
								o = 0;
								p = 0;
								q = 0;
								r = 0;
								s = 0;	 
							}
							if ( !theType.isEmpty() && !theTarget.isEmpty() && !theFlowGroup.isEmpty() && !theFlowProvider.isEmpty() && !theFlowType.isEmpty() && theType == "1" && theTarget == "11" && theFlowProvider == "2" && j == 5 ) {
								//cout << "CHECK6" << endl;
								//a provider is running a flow
								//the flow outlet and its associated flow (if any) must be set to the running state
								FlowOutlet *fo = getFlowOutlet(theFlowGroup);
								if ( fo != NULL ) {
									fo->setRunning();
									//cout << "Flow Outlet found: " << theFlowGroup << " is running" << endl;  
								}
								theType = "";
								theClientName = "";
								theClientGroup = "";
								theFlowGroup = "";
								theFlowProvider = "";
								theFlowType = "";
								theServerName = "";
								theTarget = "";
								theClientID = "";
								theFlowID = "";
								theServerOS = "";
								theClientServer = "";
								logMessages.clear();
								i = 0;
								j = 0;
								k = 0;
								l = 0;
								m = 0;
								n = 0;
								o = 0;
								p = 0;
								q = 0;
								r = 0;
								s = 0;
							}
							else if ( !theType.isEmpty() && !theTarget.isEmpty() && !theFlowGroup.isEmpty() && !theFlowProvider.isEmpty() && !theFlowType.isEmpty() && !theClientID.isEmpty() && theType == "1" && theTarget == "11" && theFlowProvider == "1" && r == 6 ) {
								//cout << "CHECK6b" << endl;
								//a consumer is running a flow
								
								QStringList tmp_clientID = QStringList::split( QRegExp(":"), theClientID);
								int i = 0;
								QString cname = "";
								QString cgroup = "";
								for ( QStringList::Iterator it = tmp_clientID.begin(); it != tmp_clientID.end(); ++it ) {
									if ( i == 0 ) {
										i++;
										cgroup = (*it);
									}
									else if ( i == 1) {
										i++;
										cname = (*it);
									}
								}
								if ( !cname.isEmpty() && !cgroup.isEmpty() ) {
									ClientApp * ca = getClient(cgroup, cname);
									if ( ca != NULL ) {
										std::map<unsigned int, FlowOutletIn *>::iterator iter_in;
										std::map<unsigned int, FlowOutletIn *> *fin_tmp = ca->getInputFlowList();
										iter_in = fin_tmp->begin();
										bool found = false;
										while( iter_in != fin_tmp->end() && found == false ) {
											if ( ( iter_in->second->getGroup() == theFlowGroup ) &&  ( iter_in->second->getDataType() == theFlowType ) ) {
												found = true;
												iter_in->second->setRunning();
												if ( iter_in->second->hasFlow() ) 
													iter_in->second->getFlow()->refresh();
											}
											iter_in++;
										}
									}
									
								}
								theType = "";
								theClientName = "";
								theClientGroup = "";
								theFlowGroup = "";
								theFlowProvider = "";
								theFlowType = "";
								theServerName = "";
								theTarget = "";
								theClientID = "";
								theFlowID = "";
								theServerOS = "";
								theClientServer = "";
								logMessages.clear();
								i = 0;
								j = 0;
								k = 0;
								l = 0;
								m = 0;
								n = 0;
								o = 0;
								p = 0;
								q = 0;	
								r = 0; 
								s = 0;
							}
							else if ( !theType.isEmpty() && !theTarget.isEmpty() && !theClientName.isEmpty() && !theClientGroup.isEmpty()  && theType == "5" && theTarget == "35" && o == 5 ) {
								//cout << "CHECK7" << endl;
								//log from client
								ClientApp *ca = getClient(theClientGroup, theClientName);
								//cout << "PB Lancement client" << endl;
								if ( ca != NULL ) {
									std::vector<QString>::iterator iter_log;
									for ( iter_log = logMessages.begin(); iter_log != logMessages.end(); iter_log++ ) {
										QString tmp_log = (QString)(*iter_log);
										if ( tmp_log.endsWith('\n') ) {
											int index_char = tmp_log.lastIndexOf('\n');
											tmp_log.data()[index_char] = ' ';
										}
										ca->addLogLine(tmp_log);
									}
								}
								theType = "";
								theClientName = "";
								theClientGroup = "";
								theFlowGroup = "";
								theFlowProvider = "";
								theFlowType = "";
								theServerName = "";
								theTarget = "";
								theClientID = "";
								theFlowID = "";
								theServerOS = "";
								theClientServer = "";
								logMessages.clear();
								i = 0;
								j = 0;
								k = 0;
								l = 0;
								m = 0;
								n = 0;
								o = 0;
								p = 0;
								q = 0;
								r = 0;
								s = 0;
							}
							else if ( !theType.isEmpty() && !theTarget.isEmpty() && !theServerName.isEmpty() && theType == "5" && theTarget == "37" &&  p == 4 ) {
								//cout << "CHECK8" << endl;
								//log from server
								//cout << "We have a log from a SERVER" << endl;
								Server *tmp_s = NULL;
								tmp_s = g->getServerList()->getServerWithPath(theServerName);
								if ( tmp_s != NULL ) {
									std::vector<QString>::iterator iter_log;
									for ( iter_log = logMessages.begin(); iter_log != logMessages.end(); iter_log++ ) {
										QString tmp_log = (QString)(*iter_log);
										//cout << "LOG=" << tmp_log.toStdString() << endl;
										if ( tmp_log.endsWith('\n') ) {
											int index_char = tmp_log.lastIndexOf('\n');
											tmp_log.data()[index_char] = ' ';
										}
										tmp_s->addLogLine(tmp_log);
									}
								}
								//else {
								//	cout << "SERVER NOT FOUND" << endl;
								//}
								theType = "";
								theClientName = "";
								theClientGroup = "";
								theFlowGroup = "";
								theFlowProvider = "";
								theFlowType = "";
								theServerName = "";
								theTarget = "";
								theClientID = "";
								theFlowID = "";
								theServerOS = "";
								theClientServer = "";
								logMessages.clear();
								i = 0;
								j = 0;
								k = 0;
								l = 0;
								m = 0;
								n = 0;
								o = 0;
								p = 0;
								q = 0;
								r = 0;
								s = 0;
							}
							else if ( !theType.isEmpty() && !theTarget.isEmpty() && !theServerName.isEmpty() && !theFlowID.isEmpty() && theType == "5" && theTarget == "36" && q == 5 ) {
								//cout << "CHECK9" << endl;
								//log from dup
								//cout << "---------------------->We have a log from a DUPLICATOR" << endl;
								//1 Find the dup if it exists, otherwise create it.
								Duplicator *dup = getDuplicatorList()->getDuplicator(theFlowID, theServerName);
								//if ( dup != NULL )
								//	cout << "Duplicator already exists" << endl;
								if ( dup == NULL ) {
									//duplicator not found, we need to add it in the list if necessary
									//found means, we have a client on a the dup server consuming or producing a flow
									dup = createDuplicatorIfNeeded(theFlowID, theServerName);
								}
								if ( dup != NULL ) {
									std::vector<QString>::iterator iter_log;
									for ( iter_log = logMessages.begin(); iter_log != logMessages.end(); iter_log++ ) {
										QString tmp_log = (QString)(*iter_log);
										//cout << "LOG=" << tmp_log.toStdString() << endl;
										if ( tmp_log.endsWith('\n') ) {
											int index_char = tmp_log.lastIndexOf('\n');
											tmp_log.data()[index_char] = ' ';
										}
										dup->addLogLine(tmp_log);
									}
								}
								theType = "";
								theClientName = "";
								theClientGroup = "";
								theFlowGroup = "";
								theFlowProvider = "";
								theFlowType = "";
								theServerName = "";
								theTarget = "";
								theClientID = "";
								theFlowID = "";
								theServerOS = "";
								theClientServer = "";
								logMessages.clear();
								i = 0;
								j = 0;
								k = 0;
								l = 0;
								m = 0;
								n = 0;
								o = 0;
								p = 0;
								q = 0;
								r = 0;
								s = 0;
							}
							
							
						}
						smallsmallchild = smallsmallchild.nextSibling().toElement();
					}
				}
				smallchild = smallchild.nextSibling().toElement();
			} //while ( !smallchild.isNull() ) 
		} //if ( tagname == "ModificationList" )
		child = child.nextSibling().toElement();
	} //while( !child.isNull() )
	return false;
}

Duplicator* ApplicationCC::createDuplicatorIfNeeded(QString flowID, QString serverName) {
	Duplicator *dup = NULL;
	bool isFound = false;
	std::map<unsigned int, ClientApp *> *cliList = getClientList();
	if ( cliList ) {
		std::map<unsigned int, ClientApp *>::iterator iter_cli;	
		iter_cli = cliList->begin();
		while ( iter_cli != cliList->end() && !isFound  ) {
			ClientApp *tmp_cli = iter_cli->second;
			if ( tmp_cli && tmp_cli->getServer()->getName() == serverName ) {
				std::map<unsigned int, FlowOutletOut *>* outlet_out = tmp_cli->getOutputFlowList();
				std::map<unsigned int, FlowOutletOut *>::iterator iter_flowOutletOut;
				if ( outlet_out != NULL ) {
					iter_flowOutletOut = outlet_out->begin();
					while ( iter_flowOutletOut != outlet_out->end() && !isFound ) {
						FlowOutletOut *out = iter_flowOutletOut->second;
						if ( out != NULL ) {
							QString tmp_id = out->getGroup();
							tmp_id.append(":");
							tmp_id.append(out->getDataType());
							if ( tmp_id == flowID )
								isFound = true;
						}
						iter_flowOutletOut++;
					}
				}
				if ( !isFound ) {
					std::map<unsigned int, FlowOutletIn *>* outlet_in = tmp_cli->getInputFlowList();
					std::map<unsigned int, FlowOutletIn *>::iterator iter_flowOutletIn;
					if ( outlet_in != NULL ) {
						iter_flowOutletIn = outlet_in->begin();
						while ( iter_flowOutletIn != outlet_in->end() && !isFound ) {
							FlowOutletIn *in = iter_flowOutletIn->second;
							if ( in != NULL ) {
								QString tmp_id = in->getGroup();
								tmp_id.append(":");
								tmp_id.append(in->getDataType());
								if ( tmp_id == flowID )
									isFound = true;
							}
							iter_flowOutletIn++;
						}
					}
				}
			}
			iter_cli++;
		}
	}
	if ( isFound ) {
		dup = new Duplicator(g, flowID, serverName);
		getDuplicatorList()->addDuplicator(dup);
	//	cout << "Duplicator created " << endl;
	}
	//else
	//	cout << "Duplicator not found " << endl;
	return dup;
}

void ApplicationCC::removeUnusedDuplicators() {
	std::map<unsigned int, Duplicator *>* dlist = dupList->getDuplicatorList();
	std::map<unsigned int, Duplicator *>::iterator iter_dup;
	//for ( iter_dup = dlist->begin(); iter_dup != dlist->end(); iter_dup++ ) {
	iter_dup = dlist->begin();
	while (  iter_dup != dlist->end() ) {
		QString serverName = iter_dup->second->getServerName();
		QString flowID = iter_dup->second->getFlowID();
		bool isFound = false;
		std::map<unsigned int, ClientApp *> *cliList = getClientList();
		if ( cliList ) {
			std::map<unsigned int, ClientApp *>::iterator iter_cli;	
			iter_cli = cliList->begin();
			while ( iter_cli != cliList->end() && !isFound  ) {
				ClientApp *tmp_cli = iter_cli->second;
				if ( tmp_cli && tmp_cli->getServer()->getName() == serverName ) {
					std::map<unsigned int, FlowOutletOut *>* outlet_out = tmp_cli->getOutputFlowList();
					std::map<unsigned int, FlowOutletOut *>::iterator iter_flowOutletOut;
					if ( outlet_out != NULL ) {
						iter_flowOutletOut = outlet_out->begin();
						while ( iter_flowOutletOut != outlet_out->end() && !isFound ) {
							FlowOutletOut *out = iter_flowOutletOut->second;
							if ( out != NULL ) {
								QString tmp_id = out->getGroup();
								tmp_id.append(":");
								tmp_id.append(out->getDataType());
								if ( tmp_id == flowID ) 
									isFound = true;
							}
							iter_flowOutletOut++;
						}
					}
					if ( !isFound ) {
						std::map<unsigned int, FlowOutletIn *>* outlet_in = tmp_cli->getInputFlowList();
						std::map<unsigned int, FlowOutletIn *>::iterator iter_flowOutletIn;
						if ( outlet_in != NULL ) {
							iter_flowOutletIn = outlet_in->begin();
							while ( iter_flowOutletIn != outlet_in->end() && !isFound ) {
								FlowOutletIn *in = iter_flowOutletIn->second;
								if ( in != NULL ) {
									QString tmp_id = in->getGroup();
									tmp_id.append(":");
									tmp_id.append(in->getDataType());
									if ( tmp_id == flowID )
										isFound = true;
								}
								iter_flowOutletIn++;
							}
						}
					}
				}
				iter_cli++;
			}
		}
		if ( !isFound ) {
			getDuplicatorList()->removeDuplicator(iter_dup->second);
			iter_dup = dlist->begin();
		}
		else
			iter_dup++;
		isFound = false;
	}
	}

DuplicatorList* ApplicationCC::getDuplicatorList() {
	return dupList;
}

void ApplicationCC::updateAfterMainServerDisconnection() {
	//we need to clear the server list, for each server, we update the map => all clients are idle without a server
	//start and stop button must be disabled, connect to server must be enabled.
	//cout << "[ApplicationCC::updateAfterMainServerDisconnection()]" << endl;
	if ( g ) {
		ServerList* server_list = g->getServerList();
		if ( server_list ) {
			std::map<unsigned int, Server *> *theList = server_list->getServerList();
			if ( theList ) {
				//iterate on every server, for each one, check if there is any client associated with it
				for (std::map<unsigned int, Server *>::iterator iter_server = theList->begin(); iter_server != theList->end(); iter_server++ ) {
					if ( iter_server->second ) {
						checkIfClientsUseServer(iter_server->second);
					}
				}
				//then we clear the server list;
				g->clearServerList();
				//we update all clients in the map
				checkIfClientsHaveServerInApplication();
			}
		}
	}
}


void ApplicationCC::checkIfClientsHaveServerInApplication() {
	std::map<unsigned int, ClientApp *>::iterator iter;
	for ( iter = clients.begin(); iter != clients.end(); iter++ ) {
		ClientApp *ca = (ClientApp *) iter->second;
		Server *s = ca->getServer();
		if ( s ) {
			if ( (s->getAddress() == "No Server") && (ca->getOldServerName() != "") ) {
				QString serverPath = ca->getOldServerName();
				Server *s_new = g->getServerList()->getServerWithPath(serverPath);
				if ( s_new != NULL ) {
					ca->setServer(s_new);
					QString tmp_serverName = "";
					ca->setOldServerName(tmp_serverName);
					ca->updateClient();
				}
			}
		}
	}
}  

void ApplicationCC::checkIfClientsUseServer(Server *s) {
	std::map<unsigned int, ClientApp *>::iterator iter;
	for ( iter = clients.begin(); iter != clients.end(); iter++ ) {
		ClientApp *ca = (ClientApp *) iter->second;
		Server *s_tmp = ca->getServer();
		if ( s == s_tmp ) {
			ca->setOldServerName(s->getAddress());
			Server *s_new = g->getServerList()->getServerWithPath("No Server");
			ca->setServer(s_new);
			if ( !ca->isDisable() )
				ca->setIdle();
			ca->updateClient();
		}
	}
}

int ApplicationCC::getMax(int val, QString& grpId) {
    QString tmp_grpId = grpId;
    tmp_grpId.replace( "ID", "" );
    int tmp_val = tmp_grpId.toInt();
    if ( tmp_val > val )
		return tmp_val;
    else
		return val;
}
/***************************************************************/

//GlobalObject::GlobalObject(GlobalCC *g, QString _name) {
GlobalObject::GlobalObject(GlobalCC *g, QString _name) : global(g) {
	//global = g;
	name = _name;
	setNewId();
}

GlobalObject::~GlobalObject() {}

void GlobalObject::setName(QString _name) {
	if ( !_name.isEmpty() )
		name = _name;
}

void GlobalObject::setId(unsigned int _id) {
	id = _id;
}

QString& GlobalObject::getName() { return name; }

unsigned int GlobalObject::getId() { return id; }

unsigned int GlobalObject::setNewId() { 
	id = GlobalCC::getNewId();
	return id;
}

void GlobalObject::setGlobal(GlobalCC *g) {
	if ( g )
		global = g;
}

