#ifndef _GLOBALCC_H
#define _GLOBALCC_H

#include "xmlwriter/xmlwriter.h"

#include <QtCore/QString>
#include <QtCore/QPoint>
#include <QtCore/QMutex>
#include <QtCore/QSemaphore>

//The log per item (dup, server, client) can contain 1000 lines
#define MAXLOGSIZE 1000

#include <iostream>
#include <map>

class GlobalCC;
class ServerList;
class ClientConfList;
class ApplicationCC;
class GlobalObject;

class Server;
class ClientConf;
class ClientApp;
class Flow;
class FlowOutlet;
class Duplicator;
class MapView;
class QWidget;
class SfControlThread;

class GlobalCC {
public:
	GlobalCC();
	~GlobalCC();
	
	void createServerList();
	void createClientConfList();
	void createApplication();
	void createSfControlThread(QSemaphore *s);
	
	void setServerList(ServerList *_serverList);
	void setClientConfList(ClientConfList *_clientList);
	void setApplication(ApplicationCC *_appli);
	
	ApplicationCC* getApplication();
	ServerList* getServerList();
	ClientConfList* getClientConfList();
	SfControlThread* getSfControlThread();
	
	void clearServerList();
	void clearClientConfList();
	void clearApplication();
	
	bool hasServerList();
	bool hasClientConfList();
	bool hasApplication();
	
	static unsigned int getNewId();
	static QString& getNewGroup();
	static void setInitialGroupId(int val);  
	
private:
		ApplicationCC *appli;
	ServerList *serverList;
	ClientConfList *clientConfList; 
	SfControlThread *sfct;
	static int theGroupId;	
};

class DuplicatorList {
public:
	DuplicatorList(GlobalCC *_g);
	~DuplicatorList();
	
	void addDuplicator(Duplicator *dup);
	void removeDuplicator(Duplicator *dup);
	void removeDuplicator(unsigned int _id);
	
	bool duplicatorExists(unsigned int _id);
	unsigned int duplicatorExists(QString &flowID, QString &serverName);
	
	Duplicator *getDuplicator(unsigned int _id);
	Duplicator *getDuplicator(QString &flowID, QString &serverName);
	
	std::map<unsigned int, Duplicator*>* getDuplicatorList();
	
	void printList();
	
	void initList();
	void clear();
	
private:
		std::map<unsigned int, Duplicator*> duplicators;
    GlobalCC *g;
};

class ServerList {
public:
	ServerList(GlobalCC *_g);
	~ServerList();
	
	unsigned int getId() { return id; }
	
	void addServer(Server *s);
	void removeServer(Server *s);
	void removeServer(unsigned int _id);
	
	bool serverExists(unsigned int _id);
	int serverNameExist(QString s);
	int serverPathExist(QString s);
	
	Server *getServer(unsigned int _id);
	Server *getServer(QString name);
	Server *getServerWithPath(QString _path);
	
	std::map<unsigned int, Server *>* getServerList();
	void printServerList();
	
	void initList();
	bool readServerList(QString serverList);
	
private:
		unsigned int id;
	std::map<unsigned int, Server *> servers;
	GlobalCC *g; 
};

class ClientConfList {
public:
	ClientConfList(GlobalCC *_g);
	~ClientConfList();
		
	unsigned int getId() { return id; }
	
	void addClient(ClientConf *c);
	void removeClient(ClientConf *c);
	void removeClient(unsigned int _id);
	
	bool clientExists(unsigned int _id);
	ClientConf* clientExists(ClientConf *cc);
	bool clientNameExist(ClientConf *cc);
	int clientNameExist(QString s);
	bool clientPathExists(QString thePath);
	ClientConf* getClientWithPath(QString thePath);
	
	ClientConf *getClient(unsigned int _id);
	std::map<unsigned int, ClientConf *>* getClientList();
	
	void printClientList();
	
	ClientConf* readClientFromXMLFile(QString filename);
	
	bool readClientListFromXMLFile(QString &filename);
	bool writeClientListFromXMLFile(QString &filename);
	
private:
		unsigned int id;
	std::map<unsigned int, ClientConf *> clients;
	GlobalCC *g;
	
	void writeXmlClient(XmlWriter *xw, ClientConf *c);
	void writeXmlFlow(XmlWriter *xw, FlowOutlet *f);
};

class ApplicationCC {
public:
	ApplicationCC(GlobalCC *_g);
	~ApplicationCC();
	
	unsigned int getId() { return id; }
	
	void setMap(MapView *m) { map = m; }
	MapView *getMap() { return map; }
	void initMap(QWidget* parent = 0);
	void createClientApp(ClientConf *cc, Server *s, const QPoint &p, QString &param);
	
	void addClient(ClientApp *c);
	void addFlow(Flow *f);
	void addOutlet(FlowOutlet *fo);
	
	void removeClient(ClientApp *c);
	void removeClient(unsigned int _id);
	void removeFlow(Flow *f);
	void removeFlow(unsigned int _id);
	void removeOutlet(FlowOutlet *fo);
	void removeOutlet(unsigned int id);
	
	bool clientExists(unsigned int _id);
	bool flowExists(unsigned int _id);
	bool flowOutletExists(unsigned int _id);
	
	ClientApp *getClient(unsigned int _id);
	ClientApp *getClient(QString &_group, QString &_name);
	Flow *getFlow(unsigned int _id);
	//Flow *getFlow(QString &_group);
	FlowOutlet *getFlowOutlet(unsigned int _id);
	FlowOutlet *getFlowOutlet(QString &_group);
	
	unsigned int getNumberOfClientsWithSamePath(ClientApp *c);
	bool readApplicationDescription(QString appliDesc);
	
	std::map<unsigned int, ClientApp *>* getClientList();
	std::map<unsigned int, FlowOutlet *>* getFlowList();
	
	int getMaxGroupId();
	
	bool readXmlApplication(QString &filename);
	bool writeXmlApplication(QString &filename);
	
	void refreshAllViews();
	
	unsigned int numberOfClients();
	
	bool parseLastModificationsInApplication(QString &modif);
	
	void updateAfterMainServerDisconnection();
	
	//not finished, we have to test if some server are gone!!!
	//if the client has the server No server, we cannot run it!!
	void checkIfClientsHaveServerInApplication();
	void checkIfClientsUseServer(Server *s);
	
	DuplicatorList *getDuplicatorList();
	
	void removeUnusedDuplicators();
	
private:
		unsigned int id;
	
	std::map<unsigned int, Flow *> flows;
	std::map<unsigned int, ClientApp *> clients;
	std::map<unsigned int, FlowOutlet *> flowOutlets;
	
	DuplicatorList *dupList;
	
	MapView *map;
	
	GlobalCC *g;
	QMutex *mutex;
	
	void writeXmlClient(XmlWriter *xw, ClientApp *c);
	void writeXmlFlow(XmlWriter *xw, FlowOutlet *f);
	void writeXmlServer(XmlWriter *xw, QString &serverName);
	
	int getMax(int val, QString& grpId);
	Duplicator* createDuplicatorIfNeeded(QString flowID, QString serverName);
	
};

class GlobalObject {
public:
	GlobalObject(GlobalCC *g, QString _name = QString::null);
	virtual ~GlobalObject();
	
	void setName(QString _name);
	void setId(unsigned int _id); 
	
	QString& getName();
	unsigned int getId();
	
	unsigned int setNewId();
	
	void setGlobal(GlobalCC *g);
	GlobalCC *getGlobal() { return global; }
	
protected:
		GlobalCC *global;
	QString name;
	unsigned int id;
};

#endif
