#ifndef _SFCONTROL_H
#define _SFCONTROL_H

#include <QtCore/QObject>
#include <QtCore/QMutex>

#include "Smartflow.h"

class GlobalCC;
class ClientApp;
class Flow;
class CheckSmartflowModifications;
class SfControlThread;

class SfControl : public QObject {
	Q_OBJECT
	
public:
	SfControl(GlobalCC *_global, SfControlThread *_sfct);
	~SfControl();
	
	static void disconnectFromServerHandler(void *arg);
	
	void setGlobalCC(class GlobalCC *_global);
	void unsetGlobalCC();
	GlobalCC* getGlobalCC();
	
	bool isConnectedToServer();
	//bool isApplicationStarted();
	void launchSmartflowModificationChecker();
	void stopSmartflowModificationChecker();
	
	//public: so the static method can change this boolean 
	bool connectedToServer;  

	void emitConnectToServerSignal();
	void emitDisconnectFromServerSignal();
	void emitGetServerListSignal();
	void emitStartClientSignal(unsigned int id);
	void emitStartClientSignal(ClientApp *client);
	void emitStopClientSignal(unsigned int id);
	void emitStopClientSignal(ClientApp* client);
	void emitStartAllClientsSignal();
	void emitStopAllClientsSignal();
	void emitDiscoverApplicationSignal();
	
	void emitPauseFlowSignal(Flow *f, ClientApp *cl);
	void emitRestartFlowSignal(Flow *f, ClientApp *cl);

signals:
	void connectToServerSignal();
	void disconnectFromServerSignal();
	void getServerListSignal();
	void startClientSignal(unsigned int id);
	void startClientSignal(ClientApp *client);
	void stopClientSignal(unsigned int id);
	void stopClientSignal(ClientApp* client);
	void startAllClientsSignal();
	void stopAllClientsSignal();
	void discoverApplicationSignal();
	
	void pauseFlowSignal(Flow *f, ClientApp *cl);
	void restartFlowSignal(Flow *f, ClientApp *cl);

public slots:
	bool connectToServer();
	void disconnectFromServer();
	void getServerList();
	void startClient(unsigned int id);
	void startClient(ClientApp *client);
	void stopClient(unsigned int id);
	void stopClient(ClientApp* client);
	void startAllClients();
	void stopAllClients();
	void discoverApplication();
	
	void pauseFlow(Flow *f, ClientApp *cl);
	void restartFlow(Flow *f, ClientApp *cl);
	
private:
	smartflow::Smartflow *sf;
	CheckSmartflowModifications *smartflowModificationsChecker;
	QMutex *mutex;
	SfControlThread *sfct;
	GlobalCC *global;
};

#endif
